/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "ADLSProperties.h"
#include "../client/ADLSearch.h"
#include "WinUtil.h"
#include "WinApiUtil.h"

INT_PTR CALLBACK ADLSProperties::ADLSPropertiesDlgProc(HWND hWndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam){
	ADLSProperties * pParent;

	if(uMsg == WM_INITDIALOG) {
		pParent = (ADLSProperties *)lParam;
		SetWindowLongPtr(hWndDlg, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWndDlg;
	} else {
		pParent = (ADLSProperties *)GetWindowLongPtr(hWndDlg, GWLP_USERDATA);
		if(pParent == NULL) {
			return FALSE;
		}
	}

    SetWindowLongPtr(hWndDlg, DWLP_MSGRESULT, 0);

	return pParent->ADLSPropertiesProc(hWndDlg, uMsg, wParam, lParam);
}

LRESULT ADLSProperties::ADLSPropertiesProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == WM_INITDIALOG) {
        OnInitDialog();
        return TRUE;
    } else if(uMsg == WM_COMMAND && (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)) {
        OnCloseCmd(LOWORD(wParam));
        return TRUE;
    }

	return FALSE;
}

WinUtil::TextItem ADLSProperties::texts[] = {
	{ IDD_ADLS_PROPERTIES, ResourceManager::ADLSEARCH_PROPERTIES },
	{ IDC_ADLSEARCH_SEARCH_STRING, ResourceManager::ADLSEARCH_SEARCH_STRING },
	{ IDC_ADLSEARCH_SOURCE_TYPE, ResourceManager::ADLSEARCH_SOURCE_TYPE },
	{ IDC_ADLSEARCH_DESTINATION_DIRECTORY, ResourceManager::ADLSEARCH_DESTINATION_DIRECTORY },
	{ IDC_ADLSEARCH_MIN_SIZE, ResourceManager::ADLSEARCH_MIN_SIZE },
	{ IDC_ADLSEARCH_MAX_SIZE, ResourceManager::ADLSEARCH_MAX_SIZE }, 
	{ IDC_IS_ACTIVE, ResourceManager::ADLSEARCH_ACTIVE_SEARCH },
	{ IDC_ADLSEARCH_SIZE_TYPE, ResourceManager::ADLSEARCH_SIZE_TYPE },
	{ IDOK, ResourceManager::OK },
	{ IDCANCEL, ResourceManager::CANCEL },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

// Initialize dialog
void ADLSProperties::OnInitDialog() {
	WinUtil::translate(m_hWnd, texts);

	// Initialize dialog items
	::SendMessage(::GetDlgItem(m_hWnd, IDC_SOURCE_TYPE), CB_ADDSTRING, 0, (LPARAM)CTSTRING(FILENAME));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_SOURCE_TYPE), CB_ADDSTRING, 0, (LPARAM)CTSTRING(DIRECTORY));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_SOURCE_TYPE), CB_ADDSTRING, 0, (LPARAM)CTSTRING(ADL_FULL_PATH));

	::SendMessage(::GetDlgItem(m_hWnd, IDC_SIZE_TYPE), CB_ADDSTRING, 0, (LPARAM)CTSTRING(B));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_SIZE_TYPE), CB_ADDSTRING, 0, (LPARAM)CTSTRING(KB));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_SIZE_TYPE), CB_ADDSTRING, 0, (LPARAM)CTSTRING(MB));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_SIZE_TYPE), CB_ADDSTRING, 0, (LPARAM)CTSTRING(GB));

	// Load search data
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_SEARCH_STRING), Text::toT(search->searchString).c_str());
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_DEST_DIR), Text::toT(search->destDir).c_str());
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_MIN_FILE_SIZE), Text::toT(search->minFileSize > 0 ? Util::toString(search->minFileSize) : "").c_str());
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_MAX_FILE_SIZE), Text::toT(search->maxFileSize > 0 ? Util::toString(search->maxFileSize) : "").c_str());
	::SendMessage(::GetDlgItem(m_hWnd, IDC_IS_ACTIVE), BM_SETCHECK, (search->isActive == true ? BST_CHECKED : BST_UNCHECKED), 0L);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_AUTOQUEUE), BM_SETCHECK, (search->isAutoQueue == true ? BST_CHECKED : BST_UNCHECKED), 0L);

	::SendMessage(::GetDlgItem(m_hWnd, IDC_SOURCE_TYPE), CB_SETCURSEL, search->sourceType, 0L);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_SIZE_TYPE), CB_SETCURSEL, search->typeFileSize, 0L);

	// Center dialog
	CenterWindow(m_hWnd, ::GetParent(m_hWnd));
}

// Exit dialog
void ADLSProperties::OnCloseCmd(WORD wID) {
	if(wID == IDOK) {
		// Update search
		TCHAR buf[256];

		::GetWindowText(::GetDlgItem(m_hWnd, IDC_SEARCH_STRING), buf, 256);
		search->searchString = Text::fromT(buf);
		::GetWindowText(::GetDlgItem(m_hWnd, IDC_DEST_DIR), buf, 256);
		search->destDir = Text::fromT(buf);

		::GetWindowText(::GetDlgItem(m_hWnd, IDC_MIN_FILE_SIZE), buf, 256);
		search->minFileSize = (_tcslen(buf) == 0 ? -1 : Util::toInt64(Text::fromT(buf)));
		::GetWindowText(::GetDlgItem(m_hWnd, IDC_MAX_FILE_SIZE), buf, 256);
		search->maxFileSize = (_tcslen(buf) == 0 ? -1 : Util::toInt64(Text::fromT(buf)));

		search->isActive = ((int)::SendMessage(::GetDlgItem(m_hWnd, IDC_IS_ACTIVE), BM_GETCHECK, 0, 0L) == 1);
		search->isAutoQueue = ((int)::SendMessage(::GetDlgItem(m_hWnd, IDC_AUTOQUEUE), BM_GETCHECK, 0, 0L) == 1);

		search->sourceType = (ADLSearch::SourceType)((int)::SendMessage(::GetDlgItem(m_hWnd, IDC_SOURCE_TYPE), CB_GETCURSEL, 0, 0L));
		search->typeFileSize = (ADLSearch::SizeType)((int)::SendMessage(::GetDlgItem(m_hWnd, IDC_SIZE_TYPE), CB_GETCURSEL, 0, 0L));
	}

	::EndDialog(m_hWnd, wID);
}

INT_PTR ADLSProperties::DoModal(HWND hWndParent) {
	return ::DialogBoxParam(g_hInstance, MAKEINTRESOURCE(IDD_ADLS_PROPERTIES), hWndParent, ADLSPropertiesDlgProc, (LPARAM)this);
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
