/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"
#include "../client/ResourceManager.h"
#include "../client/UserCommand.h"
#include "../client/NmdcHub.h"

#include "WinUtil.h"
#include "CommandDlg.h"
#include "WinApiUtil.h"

INT_PTR CALLBACK CommandDlg::CommandDlgProc(HWND hWndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam){
	CommandDlg * pParent;

	if(uMsg == WM_INITDIALOG) {
		pParent = (CommandDlg *)lParam;
		SetWindowLongPtr(hWndDlg, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWndDlg;
	} else {
		pParent = (CommandDlg *)GetWindowLongPtr(hWndDlg, GWLP_USERDATA);
		if(pParent == NULL) {
			return FALSE;
		}
	}

    SetWindowLongPtr(hWndDlg, DWLP_MSGRESULT, 0);

	return pParent->CommandProc(hWndDlg, uMsg, wParam, lParam);
}

LRESULT CommandDlg::CommandProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_INITDIALOG:
            OnInitDialog();
            return TRUE;
        case WM_SETFOCUS:
            onFocus();
            return FALSE;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDOK:
                case IDCANCEL:
                    OnCloseCmd(LOWORD(wParam));
                    return TRUE;
                case IDC_SETTINGS_SEPARATOR:
                case IDC_SETTINGS_RAW:
                case IDC_SETTINGS_CHAT:
                case IDC_SETTINGS_PM:
                    onType();
                    return TRUE;
                case IDC_COMMAND:
                case IDC_NICK:
                    if(HIWORD(wParam) == EN_CHANGE) {
                        onChange();
                        return TRUE;
                    }
                    break;
            }
            break;
    }

	return FALSE;
}

void CommandDlg::OnInitDialog() {
	// Translate
	::SetWindowText(m_hWnd, CTSTRING(USER_CMD_WINDOW));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_TYPE, CTSTRING(USER_CMD_TYPE));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_SEPARATOR, CTSTRING(SEPARATOR));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_RAW, CTSTRING(USER_CMD_RAW));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_CHAT, CTSTRING(USER_CMD_CHAT));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_PM, CTSTRING(USER_CMD_PM));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_CONTEXT, CTSTRING(USER_CMD_CONTEXT));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_HUB_MENU, CTSTRING(USER_CMD_HUB_MENU));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_USER_MENU, CTSTRING(USER_CMD_USER_MENU));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_SEARCH_MENU, CTSTRING(USER_CMD_SEARCH_MENU));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_FILELIST_MENU, CTSTRING(USER_CMD_FILELIST_MENU));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_IP_MENU, CTSTRING(USER_CMD_IP_MENU));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_PARAMETERS, CTSTRING(USER_CMD_PARAMETERS));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_NAME, CTSTRING(HUB_NAME));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_COMMAND, CTSTRING(USER_CMD_COMMAND));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_HUB, CTSTRING(USER_CMD_HUB));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_TO, CTSTRING(USER_CMD_TO));
	::SetDlgItemText(m_hWnd, IDC_SETTINGS_ONCE, CTSTRING(USER_CMD_ONCE));
	::SetDlgItemText(m_hWnd, IDC_USER_CMD_EXAMPLE, CTSTRING(USER_CMD_PREVIEW));

	::SetDlgItemText(m_hWnd, IDC_COMMAND_DESCRIPTION, _T("\
Command Types:\r\n\
Separator: Adds a separator to the menu\r\n\
Raw: Sends raw command to the hub (experts only, end it with '|'!)\r\n\
Chat: Sends command as if you were typing it in the chat\r\n\
PM: Sends command as if you sent it by pm\r\n\
Contexts determine where the command is shown:\r\n\
Hub Menu: Hub tab (at the bottom of the screen) right-click menu\r\n\
Chat Menu: User right-click menu in chat and PM tab menu\r\n\
Search Menu: Search right-click menu\r\n\
Parameters:\r\n\
Name: Name (use '\\' to create submenus)\r\n\
Command: Command text (may contain parameters)\r\n")
_T("Hub: Hub ip as typed when connecting (empty = all hubs, \"op\" = hubs where you're op)\r\n\
To: PM recipient\r\n\
Only once: Send only once per user from search frame\r\n\
In the parameters, you can use %[xxx] variables and date/time specifiers (%Y, %m, ...). The following are available:\r\n\
%[mynick]: your own nick\r\n\
%[nick]: the users nick (user && search context only)\r\n\
%[tag]: user tag (user && search context only)\r\n\
%[description]: user description (user && search context only)\r\n\
%[connection: user connection (user && search context only)\r\n\
%[email]: user email (user && search context only)\r\n\
%[share]: user shared bytes (exact) (user && search context only)\r\n\
%[shareshort]: user shared bytes (formatted) (user && search context only)\r\n\
%[ip]: user ip (if supported by hub) or clicked ip (for ip menu)\r\n\
%[fileFN]: filename (search context only)\r\n\
%[line:reason]: opens up a window asking for \"reason\"\
"));

	if(type == UserCommand::TYPE_SEPARATOR) {
		::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_SEPARATOR), BM_SETCHECK, BST_CHECKED, 0L);
	} else {
		// More difficult, determine type by what it seems to be...
		if((_tcsncmp(command.c_str(), _T("$To: "), 5) == 0) &&
		  (command.find(_T(" From: %[myNI] $<%[myNI]> ")) != string::npos ||
		  command.find(_T(" From: %[mynick] $<%[mynick]> ")) != string::npos) &&
		  command.find(_T('|')) == command.length() - 1) { // if it has | anywhere but the end, it is raw
			string::size_type i = command.find(_T(' '), 5);
			dcassert(i != string::npos);
			tstring to = command.substr(5, i-5);
			string::size_type cmd_pos = command.find(_T('>'), 5) + 2;
			tstring cmd = Text::toT(Util::validateMessage(Text::fromT(command.substr(cmd_pos, command.length()-cmd_pos-1)), true, false));
			::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_PM), BM_SETCHECK, BST_CHECKED, 0L);
			::SetWindowText(::GetDlgItem(m_hWnd, IDC_NICK), to.c_str());
			::SetWindowText(::GetDlgItem(m_hWnd, IDC_COMMAND), cmd.c_str());
		} else if(((_tcsncmp(command.c_str(), _T("<%[mynick]> "), 12) == 0) ||
		  (_tcsncmp(command.c_str(), _T("<%[myNI]> "), 10) == 0)) &&
		  command[command.length()-1] == '|') {
			// Looks like a chat thing...
			string::size_type cmd_pos = command.find(_T('>')) + 2;
			tstring cmd = Text::toT(Util::validateMessage(Text::fromT(command.substr(cmd_pos, command.length()-cmd_pos-1)), true, false));
			::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_CHAT), BM_SETCHECK, BST_CHECKED, 0L);
			::SetWindowText(::GetDlgItem(m_hWnd, IDC_COMMAND), cmd.c_str());
		} else {
			tstring cmd = command;
			::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_RAW), BM_SETCHECK, BST_CHECKED, 0L);
			::SetWindowText(::GetDlgItem(m_hWnd, IDC_COMMAND), cmd.c_str());
		}
		if(type == UserCommand::TYPE_RAW_ONCE) {
			::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_ONCE), BM_SETCHECK, BST_CHECKED, 0L);
			type = 1;
		}
	}

	::SetWindowText(::GetDlgItem(m_hWnd, IDC_HUB), hub.c_str());
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_NAME), name.c_str());

	if(ctx & UserCommand::CONTEXT_HUB)
		::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_HUB_MENU), BM_SETCHECK, BST_CHECKED, 0L);
	if(ctx & UserCommand::CONTEXT_CHAT)
		::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_USER_MENU), BM_SETCHECK, BST_CHECKED, 0L);
	if(ctx & UserCommand::CONTEXT_SEARCH)
		::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_SEARCH_MENU), BM_SETCHECK, BST_CHECKED, 0L);
	if(ctx & UserCommand::CONTEXT_FILELIST)
		::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_FILELIST_MENU), BM_SETCHECK, BST_CHECKED, 0L);
	if(ctx & UserCommand::CONTEXT_IP)
		::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_IP_MENU), BM_SETCHECK, BST_CHECKED, 0L);

	updateControls();
	updateCommand();
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_RESULT), command.c_str());

	::SetFocus(::GetDlgItem(m_hWnd, IDC_SETTINGS_SEPARATOR));

	CenterWindow(m_hWnd, ::GetParent(m_hWnd));
}

void CommandDlg::onFocus() {
	::SetFocus(::GetDlgItem(m_hWnd, IDC_NAME));
}

void CommandDlg::OnCloseCmd(WORD wID) {
	updateContext();
	if(wID == IDOK) {
		TCHAR buf[256];

		if((type != 0) &&
			((::GetWindowTextLength(::GetDlgItem(m_hWnd, IDC_NAME)) == 0) || (::GetWindowTextLength(::GetDlgItem(m_hWnd, IDC_COMMAND)) == 0))) {
			  ::MessageBox(m_hWnd, _T("Name and command must not be empty"), _T(""), MB_OK);
			return;
		}

#define GET_TEXT(id, var) \
	::GetDlgItemText(m_hWnd, id, buf, 256); \
	var = buf;

		GET_TEXT(IDC_NAME, name);
		GET_TEXT(IDC_HUB, hub);

		if(type != 0) {
			type = ((int)::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_ONCE), BM_GETCHECK, 0, 0L) == BST_CHECKED) ? 2 : 1;
		}
	}
	::EndDialog(m_hWnd, wID);
}

void CommandDlg::onChange() {
	updateCommand();
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_RESULT), command.c_str());
}

void CommandDlg::onType() {
	updateType();
	updateCommand();
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_RESULT), command.c_str());
	updateControls();
}

void CommandDlg::updateContext() {
	ctx = 0;
	if((int)::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_HUB_MENU), BM_GETCHECK, 0, 0L) == BST_CHECKED)
		ctx |= UserCommand::CONTEXT_HUB;
	if((int)::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_USER_MENU), BM_GETCHECK, 0, 0L) == BST_CHECKED)
		ctx |= UserCommand::CONTEXT_CHAT;
	if((int)::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_SEARCH_MENU), BM_GETCHECK, 0, 0L) == BST_CHECKED)
		ctx |= UserCommand::CONTEXT_SEARCH;
	if((int)::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_FILELIST_MENU), BM_GETCHECK, 0, 0L) == BST_CHECKED)
		ctx |= UserCommand::CONTEXT_FILELIST;
	if((int)::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_IP_MENU), BM_GETCHECK, 0, 0L) == BST_CHECKED)
		ctx |= UserCommand::CONTEXT_IP;
}

void CommandDlg::updateType() {
	if((int)::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_SEPARATOR), BM_GETCHECK, 0, 0L) == BST_CHECKED) {
		type = 0;
	} else if((int)::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_RAW), BM_GETCHECK, 0, 0L) == BST_CHECKED) {
		type = 1;
	} else if((int)::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_CHAT), BM_GETCHECK, 0, 0L) == BST_CHECKED) {
		type = 2;
	} else if((int)::SendMessage(::GetDlgItem(m_hWnd, IDC_SETTINGS_PM), BM_GETCHECK, 0, 0L) == BST_CHECKED) {
		type = 3;
	}
}

void CommandDlg::updateCommand() {
	static const size_t BUF_LEN = 1024;
	TCHAR buf[BUF_LEN];
	if(type == 0) {
		command.clear();
	} else if(type == 1) {
		::GetWindowText(::GetDlgItem(m_hWnd, IDC_COMMAND), buf, BUF_LEN-1);
		command = buf;
	} else if(type == 2) {
		::GetWindowText(::GetDlgItem(m_hWnd, IDC_COMMAND), buf, BUF_LEN - 1);
		command = Text::toT("<%[myNI]> " + Util::validateMessage(Text::fromT(buf), false) + "|");
	} else if(type == 3) {
		::GetWindowText(::GetDlgItem(m_hWnd, IDC_NICK), buf, BUF_LEN - 1);
		tstring to(buf);
		::GetWindowText(::GetDlgItem(m_hWnd, IDC_COMMAND), buf, BUF_LEN - 1);
		command = _T("$To: ") + to + _T(" From: %[myNI] $<%[myNI]> ") + Text::toT(Util::validateMessage(Text::fromT(buf), false)) + _T("|");
	}
}

void CommandDlg::updateControls() {
	switch(type) {
		case 0:
			::EnableWindow(::GetDlgItem(m_hWnd, IDC_NAME), FALSE);
			::EnableWindow(::GetDlgItem(m_hWnd, IDC_COMMAND), FALSE);
			::EnableWindow(::GetDlgItem(m_hWnd, IDC_NICK), FALSE);
			break;
		case 1:
		case 2:
			::EnableWindow(::GetDlgItem(m_hWnd, IDC_NAME), TRUE);
			::EnableWindow(::GetDlgItem(m_hWnd, IDC_COMMAND), TRUE);
			::EnableWindow(::GetDlgItem(m_hWnd, IDC_NICK), FALSE);
			break;
		case 3:
			::EnableWindow(::GetDlgItem(m_hWnd, IDC_NAME), TRUE);
			::EnableWindow(::GetDlgItem(m_hWnd, IDC_COMMAND), TRUE);
			::EnableWindow(::GetDlgItem(m_hWnd, IDC_NICK), TRUE);
			break;
	}
}

INT_PTR CommandDlg::DoModal(HWND hWndParent) {
	return ::DialogBoxParam(g_hInstance, MAKEINTRESOURCE(IDD_USER_COMMAND), hWndParent, CommandDlgProc, (LPARAM)this);
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
