/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2009 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "CzDCPage.h"
#include "../client/SettingsManager.h"
#include "WinUtil.h"
#include "MainFrm.h"
#include "WinApiUtil.h"

CzDCPage * pCzDCPage = NULL;

CzDCPage::CzDCPage(SettingsManager *s) : PropPage(s, IDD_CZDCPAGE) {
    pCzDCPage = this;
	SetTitle(CTSTRING(SETTINGS_CZDC));
	m_psp.pfnDlgProc = (DLGPROC)CzDCPageProc;
}

CzDCPage::~CzDCPage() {
    pCzDCPage = NULL;
}

BOOL CALLBACK CzDCPage::CzDCPageProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_INITDIALOG:
            pCzDCPage->m_hWnd = hWnd;
            pCzDCPage->onInitDialog();
            return TRUE;
        case WM_COMMAND:
            if(HIWORD(wParam) == BN_CLICKED) {
                switch(LOWORD(wParam)) {
                    case IDC_BEEPBROWSE:
                        pCzDCPage->onBrowse();
                        return TRUE;
                    case IDC_SETTINGS_DOWNLOAD_BAR_COLOR:
                        pCzDCPage->onClickedProgressDown();
                        return TRUE;
                    case IDC_SETTINGS_UPLOAD_BAR_COLOR:
                        pCzDCPage->onClickedProgressUp();
                        return TRUE;
                    case IDC_PROGRESS_TEXT_COLOR_DOWN:
                        pCzDCPage->onClickedProgressTextDown();
                        return TRUE;
                    case IDC_PROGRESS_TEXT_COLOR_UP:
                        pCzDCPage->onClickedProgressTextUp();
                        return TRUE;
                    case IDC_WINAMP_HELP:
                        pCzDCPage->onClickedWinampHelp();
                        return TRUE;
                    case IDC_CLEARKICKLINE:
                        pCzDCPage->onClearKickline();
                        return TRUE;
                    case IDC_CLEARSEARCHES:
                        pCzDCPage->onClearSearches();
                        return TRUE;
                }
            }
            break;
    }

	return FALSE;
}

PropPage::TextItem CzDCPage::texts[] = {
	{ IDC_CZDC_FEAT, ResourceManager::SETCZDC_FEAT },
	{ IDC_CZDC_PM_SOUND, ResourceManager::SETCZDC_PRIVATE_SOUND },
	{ IDC_BEEPBROWSE, ResourceManager::BROWSE },
	{ IDC_CZDC_PROGRESS_COLOR, ResourceManager::SETCZDC_PROGRESSBAR_COLORS },
	{ IDC_SETTINGS_DOWNLOAD_BAR_COLOR, ResourceManager::DOWNLOAD },
	{ IDC_PROGRESS_TEXT_COLOR_DOWN, ResourceManager::DOWNLOAD },
	{ IDC_SETTINGS_UPLOAD_BAR_COLOR, ResourceManager::SETCZDC_UPLOAD },
	{ IDC_PROGRESS_TEXT_COLOR_UP, ResourceManager::SETCZDC_UPLOAD },
	{ IDC_CZDC_PROGRESS_TEXT, ResourceManager::SETCZDC_PROGRESSBAR_TEXT },
	{ IDC_CZDC_WINAMP, ResourceManager::SETCZDC_WINAMP },
	{ IDC_SET_SHUTDOWNTIMEOUT, ResourceManager::SETCZDC_SHUTDOWNTIMEOUT },
	{ IDC_CLEARKICKLINE, ResourceManager::CLEARKICKLINE },
	{ IDC_CLEARSEARCHES, ResourceManager::CLEARSEARCHES },
	{ IDC_USERLISTDBLCLICKACTION, ResourceManager::USERLISTDBLCLICKACTION },
	{ IDC_TRANSFERLISTDBLCLICKACTION, ResourceManager::TRANSFERLISTDBLCLICKACTION },	
	{ IDC_CHATDBLCLICKACTION, ResourceManager::CHATDBLCLICKACTION },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
}; 

PropPage::Item CzDCPage::items[] = {
	{ IDC_WINAMP, SettingsManager::WINAMP_FORMAT, PropPage::T_STR },
	{ IDC_SHUTDOWNTIMEOUT, SettingsManager::SHUTDOWN_TIMEOUT, PropPage::T_INT },
	{ 0, 0, PropPage::T_END }
};

void CzDCPage::onInitDialog() {
	PropPage::translate(m_hWnd, texts);
	PropPage::read(m_hWnd, items);

	crProgressDown = SETTING(DOWNLOAD_BAR_COLOR);
	crProgressUp = SETTING(UPLOAD_BAR_COLOR);
	crProgressTextDown = SETTING(PROGRESS_TEXT_COLOR_DOWN);
	crProgressTextUp = SETTING(PROGRESS_TEXT_COLOR_UP);

	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(GET_FILE_LIST));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(SEND_PUBLIC_MESSAGE));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(SEND_PRIVATE_MESSAGE));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(MATCH_QUEUE));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(GRANT_EXTRA_SLOT));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(ADD_TO_FAVORITES));

	::SendMessage(::GetDlgItem(m_hWnd, IDC_TRANSFERLIST_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(SEND_PRIVATE_MESSAGE));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_TRANSFERLIST_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(GET_FILE_LIST));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_TRANSFERLIST_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(MATCH_QUEUE));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_TRANSFERLIST_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(GRANT_EXTRA_SLOT));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_TRANSFERLIST_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(ADD_TO_FAVORITES));

	::SendMessage(::GetDlgItem(m_hWnd, IDC_CHAT_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(SELECT_USER_LIST));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_CHAT_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(SEND_PUBLIC_MESSAGE));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_CHAT_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(SEND_PRIVATE_MESSAGE));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_CHAT_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(GET_FILE_LIST));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_CHAT_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(MATCH_QUEUE));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_CHAT_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(GRANT_EXTRA_SLOT));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_CHAT_DBLCLICK), CB_ADDSTRING, 0, (LPARAM)CTSTRING(ADD_TO_FAVORITES));

	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_DBLCLICK), CB_SETCURSEL, SETTING(USERLIST_DBLCLICK), 0L);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_TRANSFERLIST_DBLCLICK), CB_SETCURSEL, SETTING(TRANSFERLIST_DBLCLICK), 0L);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_CHAT_DBLCLICK), CB_SETCURSEL, SETTING(CHAT_DBLCLICK), 0L);
	
	// Do specialized reading here
	::SetDlgItemText(m_hWnd, IDC_BEEPFILE, Text::toT(SETTING(BEEPFILE)).c_str());
}

void CzDCPage::write() {
	PropPage::write(m_hWnd, items);

	// Do specialized writing here
	TCHAR buf[256];
	::GetDlgItemText(m_hWnd, IDC_BEEPFILE, buf, 256);
	settings->set(SettingsManager::BEEPFILE, Text::fromT(buf));

	SettingsManager::getInstance()->set(SettingsManager::DOWNLOAD_BAR_COLOR, (int)crProgressDown);
	SettingsManager::getInstance()->set(SettingsManager::UPLOAD_BAR_COLOR, (int)crProgressUp);
	SettingsManager::getInstance()->set(SettingsManager::PROGRESS_TEXT_COLOR_DOWN, (int)crProgressTextDown);
	SettingsManager::getInstance()->set(SettingsManager::PROGRESS_TEXT_COLOR_UP, (int)crProgressTextUp);

	settings->set(SettingsManager::USERLIST_DBLCLICK, (int)::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_DBLCLICK), CB_GETCURSEL, 0, 0L));
	settings->set(SettingsManager::TRANSFERLIST_DBLCLICK, (int)::SendMessage(::GetDlgItem(m_hWnd, IDC_TRANSFERLIST_DBLCLICK), CB_GETCURSEL, 0, 0L));
	settings->set(SettingsManager::CHAT_DBLCLICK, (int)::SendMessage(::GetDlgItem(m_hWnd, IDC_CHAT_DBLCLICK), CB_GETCURSEL, 0, 0L));
}

void CzDCPage::onBrowse() {
	TCHAR buf[MAX_PATH];

	::GetDlgItemText(m_hWnd, IDC_BEEPFILE, buf, MAX_PATH);
	tstring x = buf;

	if(WinUtil::browseFile(x, m_hWnd, false) == IDOK) {
		::SetDlgItemText(m_hWnd, IDC_BEEPFILE, x.c_str());
	}
}

void CzDCPage::onClickedProgressDown() {
    CHOOSECOLOR cc;
    InitializeChooseColor(cc, crProgressDown, m_hWnd);

	if(::ChooseColor(&cc) == TRUE) {
		crProgressDown = cc.rgbResult;
	}
}

void CzDCPage::onClickedProgressUp() {
    CHOOSECOLOR cc;
    InitializeChooseColor(cc, crProgressUp, m_hWnd);

	if(::ChooseColor(&cc) == TRUE) {
		crProgressUp = cc.rgbResult;
	}
}

void CzDCPage::onClickedProgressTextDown() {
    CHOOSECOLOR cc;
    InitializeChooseColor(cc, crProgressTextDown, m_hWnd);

	if(::ChooseColor(&cc) == TRUE) {
		crProgressTextDown = cc.rgbResult;
	}
}

void CzDCPage::onClickedProgressTextUp() {
    CHOOSECOLOR cc;
    InitializeChooseColor(cc, crProgressTextUp, m_hWnd);

	if(::ChooseColor(&cc) == TRUE) {
		crProgressTextUp = cc.rgbResult;
	}
}

void CzDCPage::onClickedWinampHelp() {
	::MessageBox(m_hWnd, CTSTRING(WINAMP_HELP), CTSTRING(WINAMP_HELP_DESC), MB_OK | MB_ICONINFORMATION);
}

void CzDCPage::onClearKickline() {
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_01, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_02, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_03, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_04, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_05, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_06, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_07, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_08, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_09, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_10, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_11, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_12, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_13, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_14, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_15, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_16, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_17, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_18, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_19, "");
	SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_20, "");
}

void CzDCPage::onClearSearches() {
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_01, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_02, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_03, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_04, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_05, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_06, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_07, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_08, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_09, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_10, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_11, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_12, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_13, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_14, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_15, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_16, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_17, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_18, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_19, "");
	SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_20, "");
	MainFrame::clearSearches();
}
