#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "DebugFrame.h"
#include "WinUtil.h"
#include "../client/File.h"
#include "WinApiUtil.h"
#include "FlatTabCtrl.h"

ATOM DebugFrame::m_aAtom = 0;
UINT DebugFrame::m_uiResourceID = IDR_DEBUG_FRAME;
static WNDPROC DefButtonProc = NULL;
static WNDPROC DefStatusBarProc = NULL;

DebugFrame::DebugFrame() : closed(false), showHubCommands(false), showClientCommands(false), bFilterIp(false) {
	DebugManager::getInstance()->addListener(this);

    RegisterWindowClass(m_aAtom, DebugFrameWndProc, _T("DebugFrame"), m_uiResourceID);
}

LRESULT CALLBACK DebugFrame::DebugFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	DebugFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (DebugFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (DebugFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->DebugFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT DebugFrame::DebugFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_SETFOCUS:
            OnFocus();
            break;
        case WM_CREATE:
            OnCreate();
            return 0;
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
            break;
        case WM_CTLCOLOREDIT:
        case WM_CTLCOLORSTATIC: {
            LRESULT lResult = onCtlColor(wParam, lParam);
            if(lResult != 0) {
                return lResult;
            }
            break;
        }
        case WM_SPEAKER:
            onSpeaker(lParam);
            return 0;
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

    LRESULT lResult = 0;
    if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
    	return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK DebugFrame::StatusBarWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == WM_COMMAND && LOWORD(wParam) == IDC_CLEAR) {
		DebugFrame * pParent = (DebugFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
			pParent->onClear();
        }
    } else if(uMsg == WM_COMMAND && LOWORD(wParam) == IDC_DEBUG_FILTER_TEXT && HIWORD(wParam) == EN_CHANGE) {
		DebugFrame * pParent = (DebugFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
            pParent->onChange();
        }
    }

    return DefStatusBarProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK DebugFrame::HubCommandsWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == BM_SETCHECK) {
		DebugFrame * pParent = (DebugFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
			pParent->showHubCommands = (wParam == BST_CHECKED);
        }
    }

    return DefButtonProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK DebugFrame::ClientCommandsWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == BM_SETCHECK) {
		DebugFrame * pParent = (DebugFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
			pParent->showClientCommands = (wParam == BST_CHECKED);
        }
    }

    return DefButtonProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK DebugFrame::FilterWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == BM_SETCHECK) {
		DebugFrame * pParent = (DebugFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
			pParent->onSetCheckFilter(wParam);
        }
    }

    return DefButtonProc(hWnd, uMsg, wParam, lParam);
}

void DebugFrame::OnCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(0, 0, 0), IDR_DEBUG_FRAME);
    created = true;

	hWndPad = CreateWindowEx(WS_EX_CLIENTEDGE, _T("EDIT"), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
		WS_VSCROLL | ES_AUTOVSCROLL | ES_MULTILINE | ES_NOHIDESEL | ES_READONLY,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
    ::SendMessage(hWndPad, EM_LIMITTEXT, 0, 0L);
	::SendMessage(hWndPad, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

    m_hWndStatusBar = CreateWindowEx(0, STATUSCLASSNAME, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | SBARS_SIZEGRIP,
      0, 0, 0, 0, m_hWnd, (HMENU)NULL, g_hInstance, NULL);
    ::SetWindowLongPtr(m_hWndStatusBar, GWLP_USERDATA, (LONG_PTR)this);
    DefStatusBarProc = (WNDPROC)::SetWindowLongPtr(m_hWndStatusBar, GWLP_WNDPROC, (LONG_PTR)StatusBarWndProc);

    hWndClear = CreateWindowEx(0, _T("BUTTON"), _T("Clear"), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | BS_PUSHBUTTON,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWndStatusBar, (HMENU)IDC_CLEAR, g_hInstance, NULL);
    ::SendMessage(hWndClear, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

    hWndHubCommands = CreateWindowEx(WS_EX_STATICEDGE, _T("BUTTON"), _T("HubCommands"), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWndStatusBar, 0U, g_hInstance, NULL);
	::SendMessage(hWndHubCommands, BM_SETSTYLE, BS_AUTOCHECKBOX, (LPARAM)FALSE);
	::SendMessage(hWndHubCommands, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
	::SendMessage(hWndHubCommands, BM_SETCHECK, (showHubCommands == true ? BST_CHECKED : BST_UNCHECKED), 0L);
    ::SetWindowLongPtr(hWndHubCommands, GWLP_USERDATA, (LONG_PTR)this);
    DefButtonProc = (WNDPROC)::SetWindowLongPtr(hWndHubCommands, GWLP_WNDPROC, (LONG_PTR)HubCommandsWndProc);

    hWndClientCommands = CreateWindowEx(WS_EX_STATICEDGE, _T("BUTTON"), _T("ClientCommands"), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWndStatusBar, 0U, g_hInstance, NULL);
	::SendMessage(hWndClientCommands, BM_SETSTYLE, BS_AUTOCHECKBOX, (LPARAM)FALSE);
	::SendMessage(hWndClientCommands, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
	::SendMessage(hWndClientCommands, BM_SETCHECK, (showClientCommands == true ? BST_CHECKED : BST_UNCHECKED), 0L);
    ::SetWindowLongPtr(hWndClientCommands, GWLP_USERDATA, (LONG_PTR)this);
    DefButtonProc = (WNDPROC)::SetWindowLongPtr(hWndClientCommands, GWLP_WNDPROC, (LONG_PTR)ClientCommandsWndProc);

    hWndFilter = CreateWindowEx(WS_EX_STATICEDGE, _T("BUTTON"), _T("Filter"), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWndStatusBar, 0U, g_hInstance, NULL);
	::SendMessage(hWndFilter, BM_SETSTYLE, BS_AUTOCHECKBOX, (LPARAM)FALSE);
	::SendMessage(hWndFilter, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
	::SendMessage(hWndFilter, BM_SETCHECK, (bFilterIp == true ? BST_CHECKED : BST_UNCHECKED), 0L);
    ::SetWindowLongPtr(hWndFilter, GWLP_USERDATA, (LONG_PTR)this);
    DefButtonProc = (WNDPROC)::SetWindowLongPtr(hWndFilter, GWLP_WNDPROC, (LONG_PTR)FilterWndProc);
	
    hWndFilterText = CreateWindowEx(WS_EX_STATICEDGE, _T("EDIT"), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | ES_NOHIDESEL,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWndStatusBar, (HMENU)IDC_DEBUG_FILTER_TEXT, g_hInstance, NULL);
	::SendMessage(hWndFilterText, EM_LIMITTEXT, 0, 0L);
	::SendMessage(hWndFilterText, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));
}

bool DebugFrame::onClose() {
	if(!closed) {
		DebugManager::getInstance()->removeListener(this);
		closed = true;

		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
		return true;
	} else {
		return false;
	}
}

void DebugFrame::UpdateLayout() {
	RECT rect = { 0 };
	::GetClientRect(m_hWnd, &rect);

	// position bars and offset their dimensions
	UpdateBarsPosition(rect);
	
	if(::IsWindow(m_hWndStatusBar)) {
		RECT sr;
		int w[6];
		::GetClientRect(m_hWndStatusBar, &sr);

		//int clearButtonWidth = 50;
		int tmp = (((sr.right - sr.left) - 50) / 5) - 4;
		w[0] = 50;
		w[1] = w[0] + tmp;
		w[2] = w[1] + tmp;
		w[3] = w[2] + tmp;
		w[4] = w[3] + tmp;
		w[5] = w[4] + tmp;
		
		::SendMessage(m_hWndStatusBar, SB_SETPARTS, 6, (LPARAM)w);

        ::SendMessage(m_hWndStatusBar, SB_GETRECT, 0, (LPARAM)&sr);
		::MoveWindow(hWndClear, sr.left, sr.top, sr.right - sr.left, sr.bottom - sr.top, TRUE);
		::SendMessage(m_hWndStatusBar, SB_GETRECT, 1, (LPARAM)&sr);
		::MoveWindow(hWndHubCommands, sr.left, sr.top, sr.right - sr.left, sr.bottom - sr.top, TRUE);
		::SendMessage(m_hWndStatusBar, SB_GETRECT, 2, (LPARAM)&sr);
		::MoveWindow(hWndClientCommands, sr.left, sr.top, sr.right - sr.left, sr.bottom - sr.top, TRUE);
		::SendMessage(m_hWndStatusBar, SB_GETRECT, 3, (LPARAM)&sr);
		::MoveWindow(hWndFilter, sr.left, sr.top, sr.right - sr.left, sr.bottom - sr.top, TRUE);
		::SendMessage(m_hWndStatusBar, SB_GETRECT, 4, (LPARAM)&sr);
		::MoveWindow(hWndFilterText, sr.left, sr.top, sr.right - sr.left, sr.bottom - sr.top, TRUE);
	}
	
	// resize client window
	if(hWndPad != NULL)
		::SetWindowPos(hWndPad, NULL, rect.left, rect.top,
			rect.right - rect.left, rect.bottom - rect.top,
			SWP_NOZORDER | SWP_NOACTIVATE);
}

void DebugFrame::addLine(const tstring& aLine) {
	if(::GetWindowTextLength(hWndPad) > 131072) {
		::SendMessage(hWndPad, WM_SETREDRAW, (WPARAM)FALSE, 0);

		::SendMessage(hWndPad, EM_SETSEL, 0, (int)::SendMessage(hWndPad, EM_LINEINDEX, (int)::SendMessage(hWndPad, EM_LINEFROMCHAR, 2000, 0L), 0L));
		::SendMessage(hWndPad, EM_SCROLLCARET, 0, 0L);
		::SendMessage(hWndPad, EM_REPLACESEL, (WPARAM)FALSE, (LPARAM)_T(""));

		::SendMessage(hWndPad, WM_SETREDRAW, (WPARAM)TRUE, 0);
	}
	BOOL noscroll = TRUE;

	DWORD dwRet = (DWORD)::SendMessage(hWndPad, EM_POSFROMCHAR, ::GetWindowTextLength(hWndPad) - 1, 0);
	POINT p = { (LONG)LOWORD(dwRet), (LONG)HIWORD(dwRet) };

	RECT r;
	::GetClientRect(hWndPad, &r);
		
	if(::PtInRect(&r, p) || (HWND)::SendMessage(WinUtil::mdiClient, WM_MDIGETACTIVE, 0, (LPARAM)NULL) != m_hWnd)
		noscroll = FALSE;
	else {
		::SendMessage(hWndPad, WM_SETREDRAW, (WPARAM)FALSE, 0); // Strange!! This disables the scrolling...????
	}

	::SendMessage(hWndPad, EM_SETSEL, ::GetWindowTextLength(hWndPad), ::GetWindowTextLength(hWndPad));
	::SendMessage(hWndPad, EM_SCROLLCARET, 0, 0L);
	::SendMessage(hWndPad, EM_REPLACESEL, (WPARAM)FALSE, (LPARAM)(aLine + _T("\r\n")).c_str());

	if(noscroll) {
		::SendMessage(hWndPad, WM_SETREDRAW, (WPARAM)TRUE, 0);
	}
	setDirty();
}

void DebugFrame::onClear() {
	::SetWindowText(hWndPad, _T(""));
	::SetFocus(hWndPad);
}

void DebugFrame::onSpeaker(LPARAM lParam) {
	DbgItem* i = (DbgItem*)lParam;

	switch(i->iType) {
		case DebugManager::HUB_IN:
			if((!bFilterIp || i->tsIp == sFilterIp) && showHubCommands) {
				addLine(_T("Hub:\t[") + i->tsIp + _T("]\t <<  \t") + i->tsLine + _T("|"));
			}
			break;
		case DebugManager::HUB_OUT:
			if((!bFilterIp || i->tsIp == sFilterIp) && showHubCommands) {
				addLine(_T("Hub:\t[") + i->tsIp + _T("]\t   >>\t") + i->tsLine);
			}
			break;
		case DebugManager::CLIENT_IN:
			if((!bFilterIp || i->tsIp == sFilterIp) && showClientCommands) {
				addLine(_T("Client:\t[") + i->tsIp + _T("]\t <<  \t") + i->tsLine + _T("|"));
			}
			break;
		case DebugManager::CLIENT_OUT:
			if((!bFilterIp || i->tsIp == sFilterIp) && showClientCommands) {
				addLine(_T("Client:\t[") + i->tsIp + _T("]\t   >>\t") + i->tsLine);
			}
			break;
		default:
            break;
	}

	delete i;
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
