#if !defined __DEBUGFRAME_H
#define __DEBUGFRAME_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MDIChildWindow.h"
#include "WinUtil.h"

#include "../client/DebugManager.h"

class DebugFrame : private DebugManagerListener, public MDITabChildWindow, public StaticFrame<DebugFrame, ResourceManager::DEBUG_FRAME> {
public:
	DebugFrame();
	~DebugFrame() { }

    static ATOM m_aAtom;
    static UINT m_uiResourceID;

    static LRESULT CALLBACK DebugFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    LRESULT DebugFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void OnCreate();
	bool onClose();
	void onClear();
	void onSpeaker(LPARAM lParam);
	
	void UpdateLayout();

	LRESULT onCtlColor(WPARAM wParam, LPARAM lParam) {
		HWND hWnd = (HWND)lParam;
		HDC hDC = (HDC)wParam;
		if(hWnd == hWndPad) {
			::SetBkColor(hDC, WinUtil::bgColor);
			::SetTextColor(hDC, WinUtil::textColor);
			return (LRESULT)WinUtil::bgBrush;
		}

		return 0;
	}

	void OnFocus() {
		::SetFocus(hWndPad);
	}

	void onSetCheckFilter(WPARAM wParam) {
		bFilterIp = (wParam == BST_CHECKED);

		if(bFilterIp == true) {
            ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (5 | 0), (LPARAM)(_T("Watching IP: ") + sFilterIp).c_str());
		} else {
            ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (5 | 0), (LPARAM)_T("Watching all IPs"));
		}
	}

	void onChange() {
		AutoArray<TCHAR> buf(::GetWindowTextLength(hWndFilterText) + 1);
		::GetWindowText(hWndFilterText, buf, ::GetWindowTextLength(hWndFilterText) + 1);
		sFilterIp = buf;

		if(bFilterIp == true) {
            ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (5 | 0), (LPARAM)(_T("Watching IP: ") + sFilterIp).c_str());
		} else {
            ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (5 | 0), (LPARAM)_T("Watching all IPs"));
		}
	}	
private:
	HWND hWndPad, hWndFilterText, hWndClear, hWndHubCommands, hWndClientCommands, hWndFilter;

	bool showHubCommands, showClientCommands, bFilterIp;
	tstring sFilterIp;

	bool closed;
	
	void addLine(const tstring& aLine);
		
	struct DbgItem {
		DbgItem(int typeDir, const string& line, const string& ip) : iType(typeDir), tsLine(Text::toT(line)), tsIp(Text::toT(ip)) { }
		int iType;
		tstring tsLine;
		tstring tsIp;
	};
	
	void on(DebugManagerListener::DebugCommand, const string& aLine, int typeDir, const string& ip) throw() {
		::PostMessage(m_hWnd, WM_SPEAKER, NULL, (LPARAM)new DbgItem(typeDir, aLine, ip));		
	}

    static LRESULT CALLBACK StatusBarWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    static LRESULT CALLBACK HubCommandsWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    static LRESULT CALLBACK ClientCommandsWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    static LRESULT CALLBACK FilterWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
};

#endif // __DEBUGFRAME_H

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
