/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef DIRECTORY_LISTING_FRM_H
#define DIRECTORY_LISTING_FRM_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "../client/User.h"

#include "MDIChildWindow.h"
#include "TypedListViewCtrl.h"
#include "WinUtil.h"
#include "UCHandler.h"
#include "SearchFrm.h"
#include "WinApiUtil.h"
#include "SplitterWindow.h"

#include "../client/DirectoryListing.h"
#include "../client/StringSearch.h"
#include "../client/FavoriteManager.h"
#include "../client/ShareManager.h"

class DirectoryListingFrame : public MDITabChildWindow, private CSplitter, private UCHandler<DirectoryListingFrame>, private SettingsManagerListener {
public:
    static ATOM m_aAtom;
    static UINT m_uiResourceID;

	static void openWindow(const tstring& aFile, const UserPtr& aUser, int64_t aSpeed);
	static void openWindow(const UserPtr& aUser, const string& txt, int64_t aSpeed);

	enum {
		COLUMN_FILENAME,
		COLUMN_TYPE,
		COLUMN_SIZE,
		COLUMN_EXACT_SIZE,
		COLUMN_TTH,
		COLUMN_LAST
	};

	enum {
		STATUS_TEXT,
		STATUS_SPEED,
		STATUS_TOTAL_FILES,
		STATUS_TOTAL_SIZE,
		STATUS_SELECTED_FILES,
		STATUS_SELECTED_SIZE,
		STATUS_FILE_LIST_DIFF,
		STATUS_MATCH_QUEUE,
		STATUS_FIND,
		STATUS_NEXT,
		STATUS_DUMMY,
		STATUS_LAST
	};

	DirectoryListingFrame(const UserPtr& aUser, int64_t aSpeed);
	~DirectoryListingFrame();

    static LRESULT CALLBACK DirectoryListingFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    LRESULT DirectoryListingFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void OnCreate();
	void onDownloadWithPrio(WORD wID);
	void onDownloadDir();
	void onDownloadDirWithPrio(WORD wID);
	void onDownloadDirTo();
	void onDownloadTo();
	void onCopy(WORD wID);
	void onAddToFavorites();
	void onGoToDirectory();
	void onDownloadTarget(WORD wID);
	void onDownloadTargetDir(WORD wID);
	void onDoubleClickFiles(LPNMHDR pnmh);
	void onSelChangedDirectories(LPNMHDR pnmh);
	bool onContextMenu(WPARAM wParam, LPARAM lParam);
	bool onXButtonUp(WPARAM wParam);
	void onDownloadFavoriteDirs(WORD wID);
	void onDownloadWholeFavoriteDirs(WORD wID);
	void onSearchAlternates();
	bool onClose();
	void onRefreshSettings();
	bool onKeyDown(LPNMHDR pnmh);
	
	void downloadList(const tstring& aTarget, bool view = false,  QueueItem::Priority prio = QueueItem::DEFAULT);
	void updateTree(DirectoryListing::Directory* tree, HTREEITEM treeItem);
	void UpdateLayout(bool bResizeBars = true);
	void findFile(bool findNext);
	void runUserCommand(UserCommand& uc);
	void loadFile(const tstring& name);
	void loadXML(const string& txt);
	void refreshTree(const tstring& root);

	HTREEITEM findItem(HTREEITEM ht, const tstring& name);
	void selectItem(const tstring& name);

	void setWindowTitle() {
		if(error.empty()) {
			::SetWindowText(m_hWnd, (WinUtil::getNicks(dl->getUser()) + _T(" - ") + WinUtil::getHubNames(dl->getUser()).first).c_str());
		} else {
			::SetWindowText(m_hWnd, error.c_str());	
		}
	}

	void clearList() {
		int j = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMCOUNT, 0, 0L);
		for(int i = 0; i < j; i++) {
			delete ctrlList.getItemData(i);
		}
		::SendMessage(ctrlList.m_hWnd, LVM_DELETEALLITEMS, 0, 0L);
	}

	void onFind() {
		searching = true;
		findFile(false);
		searching = false;
		updateStatus();
	}

	void onNext() {
		searching = true;
		findFile(true);
		searching = false;
		updateStatus();
	}

	void onMatchQueue();
	void onListDiff();

	bool onKeyDownDirs(LPNMHDR pnmh) {
		NMTVKEYDOWN* kd = (NMTVKEYDOWN*)pnmh;

		if(kd->wVKey == VK_TAB) {
			onTab();
			return true;
		}

		return false;
	}

	void onTab() {
		HWND focus = ::GetFocus();
		if(focus == hWndTree) {
			::SetFocus(ctrlList.m_hWnd);
		} else if(focus == ctrlList.m_hWnd) {
			::SetFocus(hWndTree);
		}
	}

	static void closeAll();
private:
	void changeDir(DirectoryListing::Directory* d, BOOL enableRedraw);
	HTREEITEM findFile(const StringSearch& str, HTREEITEM root, int &foundFile, int &skipHits);
	void updateStatus();
	void initStatus();
	void addHistory(const string& name);
	void up();
	void back();
	void forward();

	class ItemInfo {
	public:
		enum ItemType {
			FILE,
			DIRECTORY
		} type;

		union {
			DirectoryListing::File* file;
			DirectoryListing::Directory* dir;
		};

		ItemInfo(DirectoryListing::File* f, bool utf8) : type(FILE), file(f) {
			if(utf8) {
				filename = Text::toT(f->getName());
			} else {
				filename = Text::toT(Text::acpToUtf8(f->getName()));
			}
		}
		ItemInfo(DirectoryListing::Directory* d, bool utf8) : type(DIRECTORY), dir(d) {
			if(utf8) {
				filename = Text::toT(d->getName());
			} else {
				filename = Text::toT(Text::acpToUtf8(d->getName()));
			}
		}

		const TCHAR* getText(const int &col, TCHAR* buf) const;

		const tstring& getFileName() const {
            return filename;
        }

		struct TotalSize {
			TotalSize() : total(0) { }
			void operator()(ItemInfo* a) { total += a->type == DIRECTORY ? a->dir->getTotalSize() : a->file->getSize(); }
			int64_t total;
		};

		static int compareItems(ItemInfo* a, ItemInfo* b, int col) {
			if(a->type == DIRECTORY) {
				if(b->type == DIRECTORY) {
					switch(col) {
                        case COLUMN_SIZE:
                        case COLUMN_EXACT_SIZE:
                            return compare(a->dir->getTotalSize(), b->dir->getTotalSize());
                        case COLUMN_FILENAME:
                            return lstrcmpi(a->filename.c_str(), b->filename.c_str());
                        default:
                            return 0;
					}
				} else {
					return -1;
				}
			} else if(b->type == DIRECTORY) {
				return 1;
			} else {
				switch(col) {
				    case COLUMN_FILENAME:
                        return lstrcmp(a->filename.c_str(), b->filename.c_str());
                    case COLUMN_TYPE: {
                        tstring ext1 = Util::getFileExt(a->filename);
                        tstring ext2 = Util::getFileExt(b->filename);
                        return lstrcmp(ext1.c_str(), ext2.c_str());
                    }
				    case COLUMN_SIZE: 
				    case COLUMN_EXACT_SIZE: 
					   return compare(a->file->getSize(), b->file->getSize());
					case COLUMN_TTH:
                        return lstrcmp(a->file->getTTH() != NULL ? Text::toT(a->file->getTTH()->toBase32()).c_str() : Util::emptyStringT.c_str(),
                            b->file->getTTH() != NULL ? Text::toT(b->file->getTTH()->toBase32()).c_str() : Util::emptyStringT.c_str());
				    default:
                        return 0;
				}
			}
		}
		int imageIndex() {
			if(type == DIRECTORY) {
				return WinUtil::getDirIconIndex();
			} else {
				return WinUtil::getIconIndex(filename);
			}
		}

	private:
		tstring filename;
	};

	HMENU targetMenu;
	HMENU targetDirMenu;
	HMENU fileMenu;
	HMENU directoryMenu;
	HMENU copyMenu;

	StringList targets;

	deque<string> history;
	size_t historyIndex;

	HWND hWndTree;
	TypedListViewCtrl<ItemInfo, IDC_FILES> ctrlList;
	HTREEITEM treeRoot;

	HWND hWndFind, hWndFindNext, hWndListDiff, hWndMatchQueue;

	string findStr;
	tstring error;
	string size;

	int skipHits;

	size_t files;
	int64_t speed;		/**< Speed at which this file list was downloaded */

	bool updating;
	bool searching;
	bool closed;

	int statusSizes[10];

	auto_ptr<DirectoryListing> dl;

	StringMap ucLineParams;

	typedef HASH_MAP_X(UserPtr, DirectoryListingFrame*, User::HashFunction, equal_to<UserPtr>, less<UserPtr>) UserMap;
	typedef UserMap::iterator UserIter;

	static UserMap lists;

	static int columnIndexes[COLUMN_LAST];
	static int columnSizes[COLUMN_LAST];

	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();

	typedef map< HWND , DirectoryListingFrame* > FrameMap;
	typedef pair< HWND , DirectoryListingFrame* > FramePair;
	typedef FrameMap::iterator FrameIter;
	static FrameMap frames;

    static LRESULT CALLBACK TreeWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    static LRESULT CALLBACK StatusWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
};

#endif // DIRECTORY_LISTING_FRM_H

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
