/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"
#include "WinUtil.h"
#include "WinApiUtil.h"

#include "FavHubProperties.h"

#include "../client/FavoriteManager.h"
#include "../client/ResourceManager.h"

INT_PTR CALLBACK FavHubProperties::FavHubPropertiesDlgProc(HWND hWndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam){
	FavHubProperties * pParent;

	if(uMsg == WM_INITDIALOG) {
		pParent = (FavHubProperties *)lParam;
		SetWindowLongPtr(hWndDlg, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWndDlg;
	} else {
		pParent = (FavHubProperties *)GetWindowLongPtr(hWndDlg, GWLP_USERDATA);
		if(pParent == NULL) {
			return FALSE;
		}
	}

    SetWindowLongPtr(hWndDlg, DWLP_MSGRESULT, 0);

	return pParent->FavHubPropertiesProc(hWndDlg, uMsg, wParam, lParam);
}

LRESULT FavHubProperties::FavHubPropertiesProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_INITDIALOG:
            OnInitDialog();
            return TRUE;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDOK:
                case IDCANCEL:
                    OnCloseCmd(LOWORD(wParam));
                    return TRUE;
                case IDC_HUBNICK:
                case IDC_HUBPASS:
                case IDC_HUBUSERDESCR:
                    if(HIWORD(wParam) == EN_CHANGE) {
                        OnTextChanged(LOWORD(wParam), (HWND)lParam);
                        return TRUE;
                    }
                    break;
            }
            break;
    }

	return FALSE;
}

void FavHubProperties::OnInitDialog() {
	// Translate dialog
	::SetWindowText(m_hWnd, CTSTRING(FAVORITE_HUB_PROPERTIES));
	::SetDlgItemText(m_hWnd, IDC_FH_HUB, CTSTRING(HUB));
	::SetDlgItemText(m_hWnd, IDC_FH_IDENT, CTSTRING(FAVORITE_HUB_IDENTITY));
	::SetDlgItemText(m_hWnd, IDC_FH_NAME, CTSTRING(HUB_NAME));
	::SetDlgItemText(m_hWnd, IDC_FH_ADDRESS, CTSTRING(HUB_ADDRESS));
	::SetDlgItemText(m_hWnd, IDC_FH_HUB_DESC, CTSTRING(DESCRIPTION));
	::SetDlgItemText(m_hWnd, IDC_FH_NICK, CTSTRING(NICK));
	::SetDlgItemText(m_hWnd, IDC_FH_PASSWORD, CTSTRING(PASSWORD));
	::SetDlgItemText(m_hWnd, IDC_FH_USER_DESC, CTSTRING(DESCRIPTION));

	// Fill in values
	::SetDlgItemText(m_hWnd, IDC_HUBNAME, Text::toT(entry->getName()).c_str());
	::SetDlgItemText(m_hWnd, IDC_HUBDESCR, Text::toT(entry->getDescription()).c_str());
	::SetDlgItemText(m_hWnd, IDC_HUBADDR, Text::toT(entry->getServer()).c_str());
	::SetDlgItemText(m_hWnd, IDC_HUBNICK, Text::toT(entry->getNick(false)).c_str());
	::SetDlgItemText(m_hWnd, IDC_HUBPASS, Text::toT(entry->getPassword()).c_str());
	::SetDlgItemText(m_hWnd, IDC_HUBUSERDESCR, Text::toT(entry->getUserDescription()).c_str());
	::CheckDlgButton(m_hWnd, IDC_STEALTH, entry->getStealth() ? BST_CHECKED : BST_UNCHECKED);

	::SetFocus(::GetDlgItem(m_hWnd, IDC_HUBNAME));

	::SendMessage(::GetDlgItem(m_hWnd, IDC_HUBNAME), EM_SETSEL, 0, (int)-1);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_HUBNAME), EM_SCROLLCARET, 0, 0L);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_HUBNICK), EM_LIMITTEXT, 35, 0L);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_HUBUSERDESCR), EM_LIMITTEXT, 50, 0L);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_HUBPASS), EM_SETPASSWORDCHAR, '*', 0L);

	CenterWindow(m_hWnd, ::GetParent(m_hWnd));
}

void FavHubProperties::OnCloseCmd(WORD wID) {
	if(wID == IDOK) {
		TCHAR buf[256];
		::GetDlgItemText(m_hWnd, IDC_HUBADDR, buf, 256);
		if(buf[0] == '\0') {
			::MessageBox(m_hWnd, CTSTRING(INCOMPLETE_FAV_HUB), _T(""), MB_ICONWARNING | MB_OK);
			return;
		}
		entry->setServer(Text::fromT(buf));
		::GetDlgItemText(m_hWnd, IDC_HUBNAME, buf, 256);
		entry->setName(Text::fromT(buf));
		::GetDlgItemText(m_hWnd, IDC_HUBDESCR, buf, 256);
		entry->setDescription(Text::fromT(buf));
		::GetDlgItemText(m_hWnd, IDC_HUBNICK, buf, 256);
		entry->setNick(Text::fromT(buf));
		::GetDlgItemText(m_hWnd, IDC_HUBPASS, buf, 256);
		entry->setPassword(Text::fromT(buf));
		::GetDlgItemText(m_hWnd, IDC_HUBUSERDESCR, buf, 256);
		entry->setUserDescription(Text::fromT(buf));
		entry->setStealth(::IsDlgButtonChecked(m_hWnd, IDC_STEALTH) == 0 ? false : true);
		FavoriteManager::getInstance()->save();
	}
	::EndDialog(m_hWnd, wID);
}

void FavHubProperties::OnTextChanged(WORD wID, HWND hWndCtl) {
	TCHAR buf[256];

	::GetDlgItemText(m_hWnd, wID, buf, 256);
	tstring old = buf;

	// Strip '$', '|' and ' ' from text
	TCHAR *b = buf, *f = buf, c;
	while((c = *b++) != 0) {
        switch(c) {
            case '|':
                break;
            case '$':
                if(wID == IDC_HUBPASS) {
                    *f++ = c;
                    continue;
                }

                break;
            case ' ':
            case '<':
            case '>':
                if(wID == IDC_HUBNICK) {
                    break;
                }
            default:
                *f++ = c;
                continue;
        }

        break;
	}

	*f = '\0';

	if(old != buf) {
		// Something changed; update window text without changing cursor pos
		int start, end;

        ::SendMessage(hWndCtl, EM_GETSEL, (WPARAM)&start, (LPARAM)&end);

		::SetWindowText(hWndCtl, buf);

		if(start > 0) start--;
		if(end > 0) end--;

		::SendMessage(hWndCtl, EM_SETSEL, start, end);
		::SendMessage(hWndCtl, EM_SCROLLCARET, 0, 0L);
	}
}

INT_PTR FavHubProperties::DoModal(HWND hWndParent) {
	return ::DialogBoxParam(g_hInstance, MAKEINTRESOURCE(IDD_FAVORITEHUB), hWndParent, FavHubPropertiesDlgProc, (LPARAM)this);
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
