/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "FavoriteDirsPage.h"
#include "WinUtil.h"
#include "LineDlg.h"

#include "../client/Util.h"
#include "../client/SettingsManager.h"
#include "../client/FavoriteManager.h"

FavoriteDirsPage * pFavoriteDirsPage = NULL;

FavoriteDirsPage::FavoriteDirsPage(SettingsManager *s) : PropPage(s, IDD_FAVORITE_DIRSPAGE) {
    pFavoriteDirsPage = this;
	title = _tcsdup((TSTRING(SETTINGS_DOWNLOADS) + _T('\\') + TSTRING(SETTINGS_FAVORITE_DIRS_PAGE)).c_str());
	SetTitle(title);
	m_psp.pfnDlgProc = (DLGPROC)FavoriteDirsPageProc;
}

FavoriteDirsPage::~FavoriteDirsPage() {
    pFavoriteDirsPage = NULL;
	ctrlDirectories.m_hWnd = NULL;
	free(title);
}

BOOL CALLBACK FavoriteDirsPage::FavoriteDirsPageProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_INITDIALOG:
            pFavoriteDirsPage->m_hWnd = hWnd;
            pFavoriteDirsPage->onInitDialog();
            return TRUE;
        case WM_DROPFILES:
            pFavoriteDirsPage->onDropFiles(wParam);
            return TRUE;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDC_ADD:
                    pFavoriteDirsPage->onClickedAdd();
                    return TRUE;
                case IDC_REMOVE:
                    pFavoriteDirsPage->onClickedRemove();
                    return TRUE;
                case IDC_RENAME:
                    pFavoriteDirsPage->onClickedRename();
                    return TRUE;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_FAVORITE_DIRECTORIES) {
                switch(((LPNMHDR)lParam)->code) {
                    case LVN_ITEMCHANGED:
                        pFavoriteDirsPage->onItemchangedDirectories((LPNMHDR)lParam);
                        return TRUE;
                    case LVN_KEYDOWN:
                        if(pFavoriteDirsPage->onKeyDown((LPNMHDR)lParam) == true) {
                            return TRUE;
                        }

                        return FALSE;
                    case NM_DBLCLK:
                        pFavoriteDirsPage->onDoubleClick((LPNMHDR)lParam);
                        return TRUE;
                }
            }

            break;
    }

	return FALSE;
}

PropPage::TextItem FavoriteDirsPage::texts[] = {
	{ IDC_SETTINGS_FAVORITE_DIRECTORIES, ResourceManager::SETTINGS_FAVORITE_DIRS },
	{ IDC_REMOVE, ResourceManager::REMOVE },
	{ IDC_ADD, ResourceManager::SETTINGS_ADD_FOLDER },
	{ IDC_RENAME, ResourceManager::SETTINGS_RENAME_FOLDER },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

void FavoriteDirsPage::onInitDialog() {
	PropPage::translate(m_hWnd, texts);
	ctrlDirectories.m_hWnd = ::GetDlgItem(m_hWnd, IDC_FAVORITE_DIRECTORIES);
	::SendMessage(ctrlDirectories.m_hWnd, LVM_SETEXTENDEDLISTVIEWSTYLE, 0, LVS_EX_LABELTIP | LVS_EX_FULLROWSELECT);

	// Prepare shared dir list
	ListViewInsertColumn(ctrlDirectories.m_hWnd, 0, CTSTRING(FAVORITE_DIR_NAME), LVCFMT_LEFT, 80, 0);
	ListViewInsertColumn(ctrlDirectories.m_hWnd, 1, CTSTRING(DIRECTORY), LVCFMT_LEFT, 197, 1);
	StringPairList directories = FavoriteManager::getInstance()->getFavoriteDirs();
	for(StringPairIter j = directories.begin(); j != directories.end(); j++) {
		int i = ctrlDirectories.insert((int)::SendMessage(ctrlDirectories.m_hWnd, LVM_GETITEMCOUNT, 0, 0L), Text::toT(j->second));
		ListViewSetItem(ctrlDirectories.m_hWnd, i, 1, LVIF_TEXT, Text::toT(j->first).c_str(), 0);
	}
}


void FavoriteDirsPage::write() {
//	PropPage::write((HWND)*this, items);
}

void FavoriteDirsPage::onDropFiles(WPARAM wParam) {
	HDROP drop = (HDROP)wParam;
	AutoArray<TCHAR> buf(MAX_PATH);
	UINT nrFiles;

	nrFiles = DragQueryFile(drop, (UINT)-1, NULL, 0);

	for(UINT i = 0; i < nrFiles; ++i){
		if(DragQueryFile(drop, i, buf, MAX_PATH)){
			if(PathIsDirectory(buf))
				addDirectory(tstring(buf));
		}
	}

	DragFinish(drop);
}

void FavoriteDirsPage::onItemchangedDirectories(LPNMHDR pnmh) {
	NM_LISTVIEW* lv = (NM_LISTVIEW*) pnmh;
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_REMOVE), (lv->uNewState & LVIS_FOCUSED));
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_RENAME), (lv->uNewState & LVIS_FOCUSED));
}

bool FavoriteDirsPage::onKeyDown(LPNMHDR pnmh) {
	NMLVKEYDOWN* kd = (NMLVKEYDOWN*)pnmh;
    switch(kd->wVKey) {
        case VK_INSERT:
			::PostMessage(m_hWnd, WM_COMMAND, IDC_ADD, 0);
            return true;
        case VK_DELETE:
			::PostMessage(m_hWnd, WM_COMMAND, IDC_REMOVE, 0);
            return true;
        default:
            return false;
	}
}

void FavoriteDirsPage::onDoubleClick(LPNMHDR pnmh) {
	NMITEMACTIVATE* item = (NMITEMACTIVATE*)pnmh;
	if(item->iItem >= 0) {
		::PostMessage(m_hWnd, WM_COMMAND, IDC_RENAME, 0);
	} else if(item->iItem == -1) {
		::PostMessage(m_hWnd, WM_COMMAND, IDC_ADD, 0);
	}
}

void FavoriteDirsPage::onClickedAdd() {
	tstring target;
	if(WinUtil::browseDirectory(target, m_hWnd)) {
		addDirectory(target);
	}
}

void FavoriteDirsPage::onClickedRemove() {
	TCHAR buf[MAX_PATH];
	LVITEM item;
	::ZeroMemory(&item, sizeof(item));
	item.mask = LVIF_TEXT;
	item.cchTextMax = sizeof(buf);
	item.pszText = buf;

	int i = -1;
	while((i = (int)::SendMessage(ctrlDirectories.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_SELECTED, 0))) != -1) {
		item.iItem = i;
		item.iSubItem = 1;
		::SendMessage(ctrlDirectories.m_hWnd, LVM_GETITEM, 0, (LPARAM)&item);
		if(FavoriteManager::getInstance()->removeFavoriteDir(Text::fromT(buf))) {
			::SendMessage(ctrlDirectories.m_hWnd, LVM_DELETEITEM, i, 0L);
		}
	}
}

void FavoriteDirsPage::onClickedRename() {
	TCHAR buf[MAX_PATH];
	LVITEM item;
	::ZeroMemory(&item, sizeof(item));
	item.mask = LVIF_TEXT;
	item.cchTextMax = sizeof(buf);
	item.pszText = buf;

	int i = -1;
	while((i = (int)::SendMessage(ctrlDirectories.m_hWnd, LVM_GETNEXTITEM, i, MAKELPARAM(LVNI_SELECTED, 0))) != -1) {
		item.iItem = i;
		item.iSubItem = 0;
		::SendMessage(ctrlDirectories.m_hWnd, LVM_GETITEM, 0, (LPARAM)&item);

		LineDlg virt;
		virt.title = TSTRING(FAVORITE_DIR_NAME);
		virt.description = TSTRING(FAVORITE_DIR_NAME_LONG);
		virt.line = tstring(buf);
		if(virt.DoModal(m_hWnd) == IDOK) {
			if (FavoriteManager::getInstance()->renameFavoriteDir(Text::fromT(buf), Text::fromT(virt.line))) {
				ListViewSetItem(ctrlDirectories.m_hWnd, i, 0, LVIF_TEXT, virt.line.c_str(), 0);
			} else {
				::MessageBox(m_hWnd, CTSTRING(DIRECTORY_ADD_ERROR), _T(""), MB_OK);
			}
		}
	}
}

void FavoriteDirsPage::addDirectory(const tstring& aPath){
	tstring path = aPath;
	if( path[ path.length() -1 ] != PATH_SEPARATOR )
		path += PATH_SEPARATOR;

	LineDlg virt;
	virt.title = TSTRING(FAVORITE_DIR_NAME);
	virt.description = TSTRING(FAVORITE_DIR_NAME_LONG);
	virt.line = Util::getLastDir(path);
	if(virt.DoModal(m_hWnd) == IDOK) {
		if (FavoriteManager::getInstance()->addFavoriteDir(Text::fromT(path), Text::fromT(virt.line))) {
			int j = ctrlDirectories.insert((int)::SendMessage(ctrlDirectories.m_hWnd, LVM_GETITEMCOUNT, 0, 0L), virt.line );
			ListViewSetItem(ctrlDirectories.m_hWnd, j, 1, LVIF_TEXT, path.c_str(), 0);
		} else {
			::MessageBox(m_hWnd, CTSTRING(DIRECTORY_ADD_ERROR), _T(""), MB_OK);
		}
	}
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
