/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef FAVORITE_HUBS_FRM_H
#define FAVORITE_HUBS_FRM_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MDIChildWindow.h"
#include "ExListViewCtrl.h"

#include "../client/FavoriteManager.h"

class FavoriteHubsFrame : public MDITabChildWindow, public StaticFrame<FavoriteHubsFrame, ResourceManager::FAVORITE_HUBS, IDC_FAVORITES>,
	private FavoriteManagerListener, private SettingsManagerListener {
public:
	FavoriteHubsFrame();
	~FavoriteHubsFrame();

    static ATOM m_aAtom;
    static UINT m_uiResourceID;

    static LRESULT CALLBACK FavoriteHubsFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    LRESULT FavoriteHubsFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void onCreate();
	bool onClose();
	void onDoubleClickHublist(LPNMHDR pnmh);
	bool onKeyDown(LPNMHDR pnmh);
	void onEdit();
	void onRemove();
	void onNew();
	void onItemChanged(LPNMHDR pnmh);
	void onMoveUp();
	void onMoveDown();
	bool onContextMenu(WPARAM wParam, LPARAM lParam);
	void onOpenHubLog();
	void onColumnClickHublist(LPNMHDR pnmh);
	void onRefreshSettings();

	bool checkNick();
	void UpdateLayout();

	void onSetFocus() {
		::SetFocus(ctrlHubs.m_hWnd);
	}

private:
	enum {
		COLUMN_FIRST,
		COLUMN_NAME = COLUMN_FIRST,
		COLUMN_DESCRIPTION,
		COLUMN_NICK,
		COLUMN_PASSWORD,
		COLUMN_SERVER,
		COLUMN_USERDESCRIPTION,
		COLUMN_LAST
	};

	HWND hWndConnect, hWndRemove, hWndNew, hWndProperties, hWndUp, hWndDown;
	HMENU hubsMenu;
	
	ExListViewCtrl ctrlHubs;

	bool nosave;
	bool closed;

	static int columnSizes[COLUMN_LAST];
	static int columnIndexes[COLUMN_LAST];

	void openSelected();

	void updateList(const FavoriteHubEntry::List& fl) {
		::SendMessage(ctrlHubs.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
		for(FavoriteHubEntry::List::const_iterator i = fl.begin(); i != fl.end(); ++i) {
			addEntry(*i, (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMCOUNT, 0, 0L));
		}
		::SendMessage(ctrlHubs.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
		::InvalidateRect(ctrlHubs.m_hWnd, NULL, TRUE);
	}

	void addEntry(const FavoriteHubEntry* entry, int pos);
	void on(FavoriteAdded, const FavoriteHubEntry* e) throw() { addEntry(e, (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMCOUNT, 0, 0L)); }
	void on(FavoriteRemoved, const FavoriteHubEntry* e) throw() { ::SendMessage(ctrlHubs.m_hWnd, LVM_DELETEITEM, ctrlHubs.find((LPARAM)e), 0L); }
	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();
};

#endif // FAVORITE_HUBS_FRM_H

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
