/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "../client/SettingsManager.h"
#include "../client/ResourceManager.h"

#include "WinApiUtil.h"
#include "WinUtil.h"

#include "FlatTabCtrl.h"
#include "../client/version.h"

FlatTabCtrl::FlatTabCtrl() : closing(NULL), rows(1), height(0), active(NULL), moving(NULL), inTab(false), mnu(NULL) {
	black = ::CreatePen(PS_SOLID, 1, RGB(0, 0, 0));
}

FlatTabCtrl::~FlatTabCtrl() {
    if(mnu != NULL) {
        ::DestroyMenu(mnu);
    }
    ::DeleteObject(black);
}

LRESULT CALLBACK FlatTabCtrl::FlatTabWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	FlatTabCtrl * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (FlatTabCtrl *)((LPCREATESTRUCT)lParam)->lpCreateParams;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (FlatTabCtrl *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->FlatTabProc(hWnd, uMsg, wParam, lParam);
}

LRESULT FlatTabCtrl::FlatTabProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_SIZE:
            onSize(lParam);
            return 0;
        case WM_CREATE:
            onCreate();
            return 0;
        case WM_PAINT:
            onPaint();
            return 0;
        case WM_LBUTTONDOWN:
            onLButtonDown(wParam, lParam);
            return 0;
        case WM_LBUTTONUP:
            onLButtonUp(lParam);
            return 0;
        case WM_CONTEXTMENU:
            onContextMenu(lParam);
            return 0;
        case WM_MOUSEMOVE:
            onMouseMove(lParam);
            return 0;
        case WM_MBUTTONUP:
            onCloseTab(lParam);
            return 0;
        case WM_COMMAND:
            if(LOWORD(wParam) == IDC_CLOSE_WINDOW) {
                onCloseWindow();
                return 0;
            } else if(LOWORD(wParam) == IDC_CHEVRON) {
                onChevron();
                return 0;
            } else if(LOWORD(wParam) >= IDC_SELECT_WINDOW  && LOWORD(wParam) <= IDC_SELECT_WINDOW+tabs.size()) {
                onSelectWindow(LOWORD(wParam));
                return 0;
            }

            break;
    }

    return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
}

void FlatTabCtrl::addTab(HWND hWnd, COLORREF color/* = RGB(0, 0, 0)*/, UINT icon/* = 0*/, UINT stateIcon/* = 0*/) {
	TabInfo* i = new TabInfo(hWnd, color, icon, (stateIcon != 0) ? stateIcon : icon);
	dcassert(getTabInfo(hWnd) == NULL);
	tabs.push_back(i);
	viewOrder.push_back(hWnd);
	active = i;
	calcRows(false);
	::InvalidateRect(m_hWnd, NULL, TRUE);
}

void FlatTabCtrl::removeTab(HWND aWnd) {
	TabInfo::ListIter i;
	for(i = tabs.begin(); i != tabs.end(); ++i) {
		if((*i)->hWnd == aWnd)
			break;
	}

	dcassert(i != tabs.end());
	TabInfo* ti = *i;
	if(active == ti)
		active = NULL;
	if(moving == ti)
		moving = NULL;

	delete ti;
	tabs.erase(i);
	viewOrder.remove(aWnd);

	calcRows(false);
	::InvalidateRect(m_hWnd, NULL, TRUE);
}

void FlatTabCtrl::startSwitch() {
	if(viewOrder.empty() == false) {
        inTab = true;
    }
}

void FlatTabCtrl::endSwitch() {
	inTab = false;
	if(active)
        setTop(active->hWnd);
}

void FlatTabCtrl::setActive(HWND aWnd) {
	if(!inTab)
		setTop(aWnd);

	TabInfo* ti = getTabInfo(aWnd);
	dcassert(ti != NULL);
	active = ti;
	ti->dirty = false;
	calcRows(false);
	::InvalidateRect(m_hWnd, NULL, TRUE);
}

void FlatTabCtrl::setTop(HWND aWnd) {
	viewOrder.remove(aWnd);
	viewOrder.push_back(aWnd);
}

void FlatTabCtrl::setDirty(HWND aWnd) {
	TabInfo* ti = getTabInfo(aWnd);
	dcassert(ti != NULL);
	bool inval = ti->update();

	if(active != ti) {
		if(!ti->dirty) {
			ti->dirty = true;
			inval = true;
		}
	}

	if(inval) {
		calcRows(false);
		::InvalidateRect(m_hWnd, NULL, TRUE);
	}
}

void FlatTabCtrl::setIconState(HWND aWnd) {
	TabInfo* ti = getTabInfo(aWnd);
	if(ti != NULL) {
		ti->bState = true;
		::InvalidateRect(m_hWnd, NULL, TRUE);
	}
}

void FlatTabCtrl::unsetIconState(HWND aWnd) {
	TabInfo* ti = getTabInfo(aWnd);
	if(ti != NULL) {
		ti->bState = false;
		::InvalidateRect(m_hWnd, NULL, TRUE);
	}
}

void FlatTabCtrl::setColor(HWND aWnd, COLORREF color) {
	TabInfo* ti = getTabInfo(aWnd);
	if(ti != NULL) {
		::DeleteObject(ti->pen);
		ti->pen = ::CreatePen(PS_SOLID, 1, color);
		::InvalidateRect(m_hWnd, NULL, TRUE);
	}
}

void FlatTabCtrl::updateText(HWND aWnd, LPCTSTR text) {
	TabInfo* ti = getTabInfo(aWnd);
	if(ti != NULL) {
		ti->updateText(text);
		calcRows(false);
		::InvalidateRect(m_hWnd, NULL, TRUE);
	}
}

void FlatTabCtrl::onLButtonDown(WPARAM wParam, LPARAM lParam) {
	int xPos = (LONG)LOWORD(lParam);
	int yPos = (LONG)HIWORD(lParam);
	int row = getRows() - ((yPos / getTabHeight()) + 1);

	for(TabInfo::ListIter i = tabs.begin(); i != tabs.end(); ++i) {
		TabInfo* t = *i;
		if((row == t->row) && (xPos >= t->xpos) && (xPos < (t->xpos + t->getWidth())) ) {
			// Bingo, this was clicked
			HWND hWnd = ::GetParent(m_hWnd);
			if(hWnd) {
				if(wParam & MK_SHIFT)
					::SendMessage(t->hWnd, WM_CLOSE, 0, 0);
				else
					moving = t;
			}

			break;
		}
	}
}

void FlatTabCtrl::onLButtonUp(LPARAM lParam) {
	if (moving) {
		int xPos = (LONG)LOWORD(lParam);
		int yPos = (LONG)HIWORD(lParam);
		int row = getRows() - ((yPos / getTabHeight()) + 1);

		bool moveLast = true;

		for(TabInfo::ListIter i = tabs.begin(); i != tabs.end(); ++i) {
			TabInfo* t = *i;
			if((row == t->row) && (xPos >= t->xpos) && (xPos < (t->xpos + t->getWidth())) ) {
				// Bingo, this was clicked
				HWND hWnd = ::GetParent(m_hWnd);
				if(hWnd) {
					if(t == moving) {
						::SendMessage(hWnd, FTM_SELECTED, (WPARAM)t->hWnd, 0);
				    } else {
					   //check if the pointer is on the left or right half of the tab
						//to determine where to insert the tab
						moveTabs(t, xPos > (t->xpos + (t->getWidth()/2)));
					}
				}
				moveLast = false;
				break;
			}
		}
		if(moveLast && tabs.size() != 0)
			moveTabs(tabs.back(), true);

		moving = NULL;
	}
}

void FlatTabCtrl::onContextMenu(LPARAM lParam) {
	POINT pt = { (LONG)LOWORD(lParam), (LONG)HIWORD(lParam) };        // location of mouse click

	::ScreenToClient(m_hWnd, &pt);
	int xPos = pt.x;
	int row = getRows() - ((pt.y / getTabHeight()) + 1);

	for(TabInfo::ListIter i = tabs.begin(); i != tabs.end(); ++i) {
		TabInfo* t = *i;
		if((row == t->row) && (xPos >= t->xpos) && (xPos < (t->xpos + t->getWidth())) ) {
			// Bingo, this was clicked, check if the owner wants to handle it...
			if(::SendMessage(t->hWnd, FTM_CONTEXTMENU, 0, lParam) != TRUE) {
    			closing = t->hWnd;
				::ClientToScreen(m_hWnd, &pt);

    			HMENU tabMenu = ::CreatePopupMenu();
    			::AppendMenu(tabMenu, MF_STRING, IDC_CLOSE_WINDOW, CTSTRING(CLOSE));
    			::TrackPopupMenu(tabMenu, TPM_LEFTALIGN | TPM_BOTTOMALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, 0, m_hWnd, NULL);
    			::DestroyMenu(tabMenu);
			}

			break;
		}
	}
}

void FlatTabCtrl::onMouseMove(LPARAM lParam) {
	POINT pt = { (LONG)LOWORD(lParam), (LONG)HIWORD(lParam) };        // location of mouse click

	int xPos = pt.x;
	int row = rows - ((pt.y / height) + 1);

	for(TabInfo::ListIter i = tabs.begin(); i != tabs.end(); ++i) {
		TabInfo* t = *i;
		if((row == t->row) && (xPos >= t->xpos) && (xPos < (t->xpos + t->getWidth()))) {
			// Bingo, the mouse was over this one
			int len = ::GetWindowTextLength(t->hWnd) + 1;
			if(len >= SETTING(MAX_TAB_TXT_LEN)) {
				TCHAR* buf = new TCHAR[len];
				::GetWindowText(t->hWnd, buf, len);
				if(buf != current_tip) {
                    TOOLINFO ti;
                    ToolInfoInit(ti, 0, m_hWnd, 0, NULL, NULL);
					::SendMessage(hWndTabToolTip, TTM_DELTOOL, 0, (LPARAM)&ti);
					ToolInfoInit(ti, TTF_SUBCLASS, m_hWnd, 0, NULL, buf);
					::SendMessage(hWndTabToolTip, TTM_ADDTOOL, 0, (LPARAM)&ti);
					::SendMessage(hWndTabToolTip, TTM_ACTIVATE, TRUE, 0L);
					current_tip = buf;
				}
				delete[] buf;
				return;
			}
		}
	}

	::SendMessage(hWndTabToolTip, TTM_ACTIVATE, FALSE, 0L);
	current_tip = _T("");
}

void FlatTabCtrl::onCloseWindow() {
	if(::IsWindow(closing)) {
		::SendMessage(closing, WM_CLOSE, 0, 0);
    }
}

int FlatTabCtrl::getTabHeight() {
    return height;
}

int FlatTabCtrl::getHeight() {
    return (getRows() * getTabHeight())+1;
}

int FlatTabCtrl::getRows() {
    return rows;
}

void FlatTabCtrl::calcRows(bool inval/* = true*/) {
	RECT rc;
	::GetClientRect(m_hWnd, &rc);
	int r = 1;
	int w = 0;
	bool notify = false;
	bool needInval = false;

	for(TabInfo::ListIter i = tabs.begin(); i != tabs.end(); ++i) {
		TabInfo* ti = *i;
		if( (r != 0) && ((w + ti->getWidth()) > (rc.right - rc.left)) ) {
			if(r >= SETTING(MAX_TAB_ROWS)) {
				notify |= (rows != r);
				rows = r;
				r = 0;
				::EnableWindow(hWndChevron, TRUE);
			} else {
				r++;
				w = 0;
			}
		}
		ti->xpos = w;
		needInval |= (ti->row != (r-1));
		ti->row = r-1;
		w += ti->getWidth();
	}

	if(r != 0) {
		::EnableWindow(hWndChevron, FALSE);
		notify |= (rows != r);
		rows = r;
	}

	if(notify) {
		::SendMessage(::GetParent(m_hWnd), FTM_ROWS_CHANGED, 0, 0);
	}

	if(needInval && inval) {
		::InvalidateRect(m_hWnd, NULL, TRUE);
	}
}

void FlatTabCtrl::onCreate() {
    hWndChevron = CreateWindowEx(0, _T("BUTTON"), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | BS_PUSHBUTTON,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_CHEVRON, g_hInstance, NULL);
	::SetWindowText(hWndChevron, _T("\u00bb"));

	mnu = ::CreatePopupMenu();

	hWndTabToolTip = CreateWindowEx(0, TOOLTIPS_CLASS, NULL, TTS_ALWAYSTIP | TTS_NOPREFIX,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);

	HDC dc = ::GetDC(m_hWnd);
	HFONT oldfont = (HFONT)::SelectObject(dc, WinUtil::font);
	height = WinUtil::getTextHeight(dc) + 2;
	if(height < 16)
		height = 16;

	::SelectObject(dc, oldfont);
	::ReleaseDC(m_hWnd, dc);
}

void FlatTabCtrl::onSize(LPARAM lParam) {
	calcRows();
	SIZE sz = { LOWORD(lParam), HIWORD(lParam) };
	::MoveWindow(hWndChevron, sz.cx-14, 1, 14, getHeight(), TRUE);
}

void FlatTabCtrl::onPaint() {
	RECT rc;
	bool drawActive = false;
	RECT crc;
	::GetClientRect(m_hWnd, &crc);

	if(::GetUpdateRect(m_hWnd, &rc, FALSE)) {
        PAINTSTRUCT m_ps;
        HDC dc = ::BeginPaint(m_hWnd, &m_ps);

		HFONT oldfont = (HFONT)::SelectObject(dc, WinUtil::font);

		for(TabInfo::ListIter i = tabs.begin(); i != tabs.end(); ++i) {
			TabInfo* t = *i;

			if(t->row != -1 && t->xpos < rc.right && t->xpos + t->getWidth() >= rc.left ) {
				if(t != active) {
					drawTab(dc, t, t->xpos, t->row);
				} else {
					drawActive = true;
				}
			}
		}
		HPEN oldpen = (HPEN)::SelectObject(dc, black);
		for(int r = 0; r < rows; r++) {
			::MoveToEx(dc, rc.left, r*getTabHeight(), NULL);
			::LineTo(dc, rc.right, r*getTabHeight());
		}

		if(drawActive) {
			dcassert(active);
			drawTab(dc, active, active->xpos, active->row, true);
			::SelectObject(dc, active->pen);
			int y = (rows - active->row -1) * getTabHeight();
			::MoveToEx(dc, active->xpos, y, NULL);
			::LineTo(dc, active->xpos + active->getWidth(), y);
		}
		::SelectObject(dc, oldpen);
		::SelectObject(dc, oldfont);

		::EndPaint(m_hWnd, &m_ps);
	}
}

void FlatTabCtrl::onChevron() {
	while(::GetMenuItemCount(mnu) > 0) {
        ::RemoveMenu(mnu, 0, MF_BYPOSITION);
	}

	int n = 0;
	RECT rc;
	::GetClientRect(m_hWnd, &rc);
	MENUITEMINFO mi;
	memset(&mi, 0, sizeof(MENUITEMINFO));
	mi.cbSize = sizeof(MENUITEMINFO);
	mi.fMask = MIIM_ID | MIIM_TYPE | MIIM_DATA | MIIM_STATE;
	mi.fType = MFT_STRING | MFT_RADIOCHECK;

	for(TabInfo::ListIter i = tabs.begin(); i != tabs.end(); ++i) {
		TabInfo* ti = *i;
		if(ti->row == -1) {
			mi.dwTypeData = (LPTSTR)ti->name.c_str();
			mi.dwItemData = (ULONG_PTR)ti->hWnd;
			mi.fState = MFS_ENABLED | (ti->dirty ? MFS_CHECKED : 0);
			mi.wID = IDC_SELECT_WINDOW + n;
			::InsertMenuItem(mnu, n++, TRUE, &mi);
		}
	}

	POINT pt;
	::GetClientRect(hWndChevron, &rc);
	pt.x = rc.right - rc.left;
	pt.y = 0;
	::ClientToScreen(hWndChevron, &pt);

    ::TrackPopupMenu(mnu, TPM_RIGHTALIGN | TPM_BOTTOMALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, 0, m_hWnd, NULL);
}

void FlatTabCtrl::onSelectWindow(WORD wID) {
	MENUITEMINFO mi;
	memset(&mi, 0, sizeof(MENUITEMINFO));
	mi.cbSize = sizeof(MENUITEMINFO);
	mi.fMask = MIIM_DATA;

	::GetMenuItemInfo(mnu, wID, FALSE, &mi);

	HWND hWnd = ::GetParent(m_hWnd);
	if(hWnd) {
		::SendMessage(hWnd, FTM_SELECTED, (WPARAM)mi.dwItemData, 0);
	}
}

void FlatTabCtrl::SwitchTo(bool next/* = true*/) {
	TabInfo::ListIter i = tabs.begin();
	for(; i != tabs.end(); ++i){
		if((*i)->hWnd == active->hWnd){
			if(next) {
				++i;
				if(i == tabs.end())
					i = tabs.begin();
			} else{
				if(i == tabs.begin())
					i = tabs.end();
				--i;
			}
			setActive((*i)->hWnd);
			::SetWindowPos((*i)->hWnd, HWND_TOP, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE);
			break;
		}
	}
}

void FlatTabCtrl::onCloseTab(LPARAM lParam) {
	int xPos = (LONG)LOWORD(lParam);
	int yPos = (LONG)HIWORD(lParam);
	int row = getRows() - ((yPos / getTabHeight()) + 1);

	for(TabInfo::ListIter i = tabs.begin(); i != tabs.end(); ++i) {
		TabInfo* t = *i;
		if((row == t->row) && (xPos >= t->xpos) && (xPos < (t->xpos + t->getWidth())) ) {
			// Bingo, this was clicked
			HWND hWnd = ::GetParent(m_hWnd);
			if(hWnd) {
				::SendMessage(t->hWnd, WM_CLOSE, 0, 0);
			}
			break;
		}
	}
}

void FlatTabCtrl::Create(HWND hWndParent) {
	WNDCLASSEX m_wc;
	memset(&m_wc, 0, sizeof(WNDCLASSEX));
	m_wc.cbSize = sizeof(WNDCLASSEX);
	m_wc.lpfnWndProc = FlatTabWndProc;
	m_wc.hInstance = g_hInstance;
	m_wc.hCursor = ::LoadCursor(m_wc.hInstance, IDC_ARROW);
	m_wc.hbrBackground = (HBRUSH)(COLOR_3DFACE + 1);
	m_wc.lpszClassName = _T(CzDC_VERSION_STRING) _T("_") _T("FlatTabCtrl");

	ATOM atom = ::RegisterClassEx(&m_wc);

	m_hWnd = ::CreateWindowEx(0, MAKEINTATOM(atom), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, CW_USEDEFAULT,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, hWndParent, 0U, g_hInstance, this);
}

FlatTabCtrl::TabInfo::TabInfo(HWND aWnd, COLORREF c, UINT icon, UINT stateIcon) : hWnd(aWnd), xpos(0), row(0), dirty(false), hIcon(NULL), hStateIcon(NULL), bState(false) {
    if(icon != 0) {
		hIcon = (HICON)::LoadImage((HINSTANCE)::GetWindowLongPtr(aWnd, GWLP_HINSTANCE), MAKEINTRESOURCE(icon), IMAGE_ICON, 16, 16, LR_DEFAULTSIZE);
    }
	if(stateIcon != 0) {
		hStateIcon = (HICON)::LoadImage((HINSTANCE)::GetWindowLongPtr(aWnd, GWLP_HINSTANCE), MAKEINTRESOURCE(stateIcon), IMAGE_ICON, 16, 16, LR_DEFAULTSIZE);
    }
	pen = ::CreatePen(PS_SOLID, 1, c);
	memset(&size, 0, sizeof(size));
	memset(&boldSize, 0, sizeof(boldSize));
	name = Util::emptyStringT;
	update();
}

FlatTabCtrl::TabInfo::~TabInfo() {
	if(hStateIcon != NULL) {
		::DestroyIcon(hStateIcon);
	}
	::DestroyIcon(hIcon);
	::DeleteObject(pen);
}

bool FlatTabCtrl::TabInfo::update() {
	TCHAR *name2 = new TCHAR[SETTING(MAX_TAB_TXT_LEN)+1];
	if(::GetWindowTextLength(hWnd) > SETTING(MAX_TAB_TXT_LEN)) {
		::GetWindowText(hWnd, name2, SETTING(MAX_TAB_TXT_LEN) - 2);
		name2[SETTING(MAX_TAB_TXT_LEN) - 3] = _T('.');
		name2[SETTING(MAX_TAB_TXT_LEN) - 2] = _T('.');
		name2[SETTING(MAX_TAB_TXT_LEN) - 1] = _T('.');
		name2[SETTING(MAX_TAB_TXT_LEN)] = 0;
	} else {
		::GetWindowText(hWnd, name2, SETTING(MAX_TAB_TXT_LEN)+1);
	}
	if(_tcscmp(name.c_str(), name2) == 0) {
        delete[] name2;
		return false;
   }
	name = tstring(name2);
	delete[] name2;
	HDC dc = ::GetDC(hWnd);
	HFONT f = (HFONT)::SelectObject(dc, WinUtil::font);
	::GetTextExtentPoint32(dc, name.c_str(), (int)name.size(), &size);
	::SelectObject(dc, WinUtil::boldFont);
	::GetTextExtentPoint32(dc, name.c_str(), (int)name.size(), &boldSize);
	::SelectObject(dc, f);
	::ReleaseDC(hWnd, dc);
	return true;
}

bool FlatTabCtrl::TabInfo::updateText(LPCTSTR text) {
	if(_tcslen(text) > (size_t)SETTING(MAX_TAB_TXT_LEN)) {
		name = tstring(text, SETTING(MAX_TAB_TXT_LEN));
		name[SETTING(MAX_TAB_TXT_LEN) - 3] = '.';
		name[SETTING(MAX_TAB_TXT_LEN) - 2] = '.';
		name[SETTING(MAX_TAB_TXT_LEN) - 1] = '.';
	} else {
		name = tstring(text);
	}
	HDC dc = ::GetDC(hWnd);
	HFONT f = (HFONT)::SelectObject(dc, WinUtil::font);
	::GetTextExtentPoint32(dc, name.c_str(), (int)name.size(), &size);
	::SelectObject(dc, WinUtil::boldFont);
	::GetTextExtentPoint32(dc, name.c_str(), (int)name.size(), &boldSize);
	::SelectObject(dc, f);
	::ReleaseDC(hWnd, dc);
	return true;
}

int FlatTabCtrl::TabInfo::getWidth() {
	return (dirty ? boldSize.cx : size.cx) + FT_EXTRA_SPACE + (BOOLSETTING(ICONS_ON_TABS) && hIcon != NULL ? 18 : 0);
}

void FlatTabCtrl::moveTabs(TabInfo* aTab, bool after){
	if(moving == NULL)
		return;

	TabInfo::ListIter i, j;
	//remove the tab we're moving
	for(j = tabs.begin(); j != tabs.end(); ++j){
		if((*j) == moving){
			tabs.erase(j);
			break;
		}
	}

	//find the tab we're going to insert before or after
	for(i = tabs.begin(); i != tabs.end(); ++i){
		if((*i) == aTab){
			if(after)
				++i;

			break;
		}
	}

	tabs.insert(i, moving);
	moving = NULL;

	calcRows(false);
	::InvalidateRect(m_hWnd, NULL, TRUE);
}

FlatTabCtrl::TabInfo* FlatTabCtrl::getTabInfo(HWND aWnd) {
	for(TabInfo::ListIter i	= tabs.begin(); i != tabs.end(); ++i) {
		if((*i)->hWnd == aWnd)
			return *i;
	}

	return NULL;
}

void FlatTabCtrl::drawTab(HDC& dc, TabInfo* tab, int pos, int row, bool aActive/* = false*/) {
	int ypos = (getRows() - row - 1) * getTabHeight();

	HPEN oldpen = (HPEN)::SelectObject(dc, black);

	POINT p[4];
	::BeginPath(dc);
	::MoveToEx(dc, pos, ypos, NULL);
	p[0].x = pos + tab->getWidth();
	p[0].y = ypos;
	p[1].x = pos + tab->getWidth();
	p[1].y = ypos + getTabHeight();
	p[2].x = pos;
	p[2].y = ypos + getTabHeight();
	p[3].x = pos;
	p[3].y = ypos;

	::PolylineTo(dc, p, 4);
	::CloseFigure(dc);
	::EndPath(dc);

	HBRUSH hBr = GetSysColorBrush(aActive ? COLOR_WINDOW : COLOR_BTNFACE);
	HBRUSH oldbrush = (HBRUSH)::SelectObject(dc, hBr);
	::FillPath(dc);

	::MoveToEx(dc, p[1].x + 1, p[1].y, NULL);
	::LineTo(dc, p[0].x + 1, p[0].y);
	::MoveToEx(dc, p[2].x, p[2].y, NULL);
	::LineTo(dc, p[3].x, p[3].y);
	if(!active || (tab->row != (rows - 1))) {
		::LineTo(dc, p[0].x, p[0].y);
    }

	::SelectObject(dc, tab->pen);
	::MoveToEx(dc, p[1].x, p[1].y, NULL);
	::LineTo(dc, p[0].x, p[0].y);
	::MoveToEx(dc, p[1].x, p[1].y, NULL);
	::LineTo(dc, p[2].x, p[2].y);

	::SelectObject(dc, oldpen);
	::SelectObject(dc, oldbrush);

	::SetBkMode(dc, TRANSPARENT);

	pos += (FT_EXTRA_SPACE / 2)+1;
	if(BOOLSETTING(ICONS_ON_TABS) && tab->hIcon != NULL) {
		pos += 18;
    }

    COLORREF oldclr = ::SetTextColor(dc, GetSysColor(COLOR_BTNTEXT));
	if(tab->dirty) {
		HFONT f = (HFONT)::SelectObject(dc, WinUtil::boldFont);
		::TextOut(dc, pos, ypos + 2, tab->name.c_str(), (int)tab->name.size());
        ::SelectObject(dc, f);
	} else {
        ::TextOut(dc, pos, ypos + 2, tab->name.c_str(), (int)tab->name.size());
	}

	if(BOOLSETTING(ICONS_ON_TABS) && tab->hIcon != NULL && (tab->bState == false || tab->hStateIcon != NULL)) {
		DrawIconEx(dc, pos - 19, ypos, tab->bState ? tab->hStateIcon : tab->hIcon, 16, 16, NULL, hBr, DI_NORMAL | DI_COMPAT);
	}

	::SetTextColor(dc, oldclr);
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
