/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef FLAT_TAB_CTRL_H
#define FLAT_TAB_CTRL_H

enum {
	FT_FIRST = WM_APP + 700,
	/** This will be sent when the user presses a tab. WPARAM = HWND */
	FTM_SELECTED,
	/** The number of rows changed */
	FTM_ROWS_CHANGED,
	/** Set currently active tab to the HWND pointed by WPARAM */
	FTM_SETACTIVE,
	/** Display context menu and return TRUE, or return FALSE for the default one */
	FTM_CONTEXTMENU,
};

class FlatTabCtrl {
public:
	enum { FT_EXTRA_SPACE = 6 };

	FlatTabCtrl();
	~FlatTabCtrl();

	static LRESULT CALLBACK FlatTabWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

    LRESULT FlatTabProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void addTab(HWND hWnd, COLORREF color = RGB(128, 128, 128), UINT icon = 0, UINT stateIcon = 0);
	void removeTab(HWND aWnd);
	void startSwitch();
	void endSwitch();
	void setActive(HWND aWnd);
	void setTop(HWND aWnd);
	void setDirty(HWND aWnd);
	void setIconState(HWND aWnd);
	void unsetIconState(HWND aWnd);
	void setColor(HWND aWnd, COLORREF color);
	void updateText(HWND aWnd, LPCTSTR text);
	void onLButtonDown(WPARAM wParam, LPARAM lParam);
	void onLButtonUp(LPARAM lParam);
	void onContextMenu(LPARAM lParam);
	void onMouseMove(LPARAM lParam);
	void onCloseWindow();

	int getTabHeight();
	int getHeight();
	int getRows();

	void calcRows(bool inval = true);
	void onCreate();
	void onSize(LPARAM lParam);
	void onPaint();
	void onChevron();
	void onSelectWindow(WORD wID);
	void SwitchTo(bool next = true);
	void onCloseTab(LPARAM lParam);
	void Create(HWND hWndParent);

	HWND m_hWnd;
private:
	class TabInfo {
	public:
		typedef vector<TabInfo*> List;
		typedef List::iterator ListIter;

		TabInfo(HWND aWnd, COLORREF c, UINT icon, UINT stateIcon);
		~TabInfo();
	
		HWND hWnd;
		HPEN pen;
		tstring name;
		SIZE size;
		SIZE boldSize;
		int xpos;
		int row;
		bool dirty;
		HICON hIcon;
		HICON hStateIcon;
		bool bState;

		bool update();
		bool updateText(LPCTSTR text);

		int getWidth();
	};

	void moveTabs(TabInfo* aTab, bool after);

	HWND closing;
	HWND hWndChevron, hWndTabToolTip;
	HMENU mnu;
	
	int rows;
	int height;

	tstring current_tip;

	TabInfo* active;
	TabInfo* moving;
	TabInfo::List tabs;
	HPEN black;

	typedef list<HWND> WindowList;
	typedef WindowList::iterator WindowIter;

	WindowList viewOrder;

	bool inTab;

	TabInfo* getTabInfo(HWND aWnd);

	void drawTab(HDC& dc, TabInfo* tab, int pos, int row, bool aActive = false);
};

#endif // FLAT_TAB_CTRL_H

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
