/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2009 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "../client/SettingsManager.h"
#include "Resource.h"
#include "LineDlg.h"
#include "WinApiUtil.h"

INT_PTR CALLBACK LineDlg::LineDlgProc(HWND hWndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam){
	LineDlg * pParent;

	if(uMsg == WM_INITDIALOG) {
		pParent = (LineDlg *)lParam;
		SetWindowLongPtr(hWndDlg, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWndDlg;
	} else {
		pParent = (LineDlg *)GetWindowLongPtr(hWndDlg, GWLP_USERDATA);
		if(pParent == NULL) {
			return FALSE;
		}
	}

    SetWindowLongPtr(hWndDlg, DWLP_MSGRESULT, 0);

	return pParent->LineProc(hWndDlg, uMsg, wParam, lParam);
}

LRESULT LineDlg::LineProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_INITDIALOG:
            OnInitDialog();
            return TRUE;
        case WM_SETFOCUS:
            onFocus();
            return TRUE;
        case WM_COMMAND:
            if(LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
                OnCloseCmd(LOWORD(wParam));
                return TRUE;
            }
            break;
    }

	return FALSE;
}

void LineDlg::onFocus() {
	::SetFocus(::GetDlgItem(m_hWnd, IDC_LINE));
}

void LineDlg::OnInitDialog() {
	::SetFocus(::GetDlgItem(m_hWnd, IDC_LINE));
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_LINE), line.c_str());

	::SendMessage(::GetDlgItem(m_hWnd, IDC_LINE), EM_SETSEL, 0, (int)-1);

	if(password) {
		::SetWindowLong(::GetDlgItem(m_hWnd, IDC_LINE), GWL_STYLE, ::GetWindowLong(::GetDlgItem(m_hWnd, IDC_LINE), GWL_STYLE) | ES_PASSWORD);
		::SendMessage(::GetDlgItem(m_hWnd, IDC_LINE), EM_SETPASSWORDCHAR, '*', 0L);
	}

	::SetWindowText(::GetDlgItem(m_hWnd, IDC_DESCRIPTION), description.c_str());

	::SetWindowText(m_hWnd, title.c_str());

	CenterWindow(m_hWnd, ::GetParent(m_hWnd));
}

void LineDlg::OnCloseCmd(WORD wID) {
	if(wID == IDOK) {
		int len = ::GetWindowTextLength(::GetDlgItem(m_hWnd, IDC_LINE)) + 1;
		AutoArray<TCHAR> buf(len);
		::GetDlgItemText(m_hWnd, IDC_LINE, buf, len);
		line = buf;
	}
	::EndDialog(m_hWnd, wID);
}

INT_PTR LineDlg::DoModal(HWND hWndParent) {
	return ::DialogBoxParam(g_hInstance, MAKEINTRESOURCE(IDD_LINE), hWndParent, LineDlgProc, (LPARAM)this);
}

INT_PTR CALLBACK KickDlg::KickDlgProc(HWND hWndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam){
	KickDlg * pParent;

	if(uMsg == WM_INITDIALOG) {
		pParent = (KickDlg *)lParam;
		SetWindowLongPtr(hWndDlg, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWndDlg;
	} else {
		pParent = (KickDlg *)GetWindowLongPtr(hWndDlg, GWLP_USERDATA);
		if(pParent == NULL) {
			return FALSE;
		}
	}

    SetWindowLongPtr(hWndDlg, DWLP_MSGRESULT, 0);

	return pParent->KickProc(hWndDlg, uMsg, wParam, lParam);
}

LRESULT KickDlg::KickProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_INITDIALOG:
            OnInitDialog();
            return TRUE;
        case WM_SETFOCUS:
            onFocus();
            return TRUE;
        case WM_COMMAND:
            if(LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
                OnCloseCmd(LOWORD(wParam));
                return TRUE;
            }
            break;
    }

	return FALSE;
}

tstring KickDlg::m_sLastMsg = _T("");

void KickDlg::onFocus() {
	::SetFocus(::GetDlgItem(m_hWnd, IDC_LINE));
}

void KickDlg::OnInitDialog() {
	Msgs[0] = Text::toT(SETTING(KICK_MSG_RECENT_01));
	Msgs[1] = Text::toT(SETTING(KICK_MSG_RECENT_02));
	Msgs[2] = Text::toT(SETTING(KICK_MSG_RECENT_03));
	Msgs[3] = Text::toT(SETTING(KICK_MSG_RECENT_04));
	Msgs[4] = Text::toT(SETTING(KICK_MSG_RECENT_05));
	Msgs[5] = Text::toT(SETTING(KICK_MSG_RECENT_06));
	Msgs[6] = Text::toT(SETTING(KICK_MSG_RECENT_07));
	Msgs[7] = Text::toT(SETTING(KICK_MSG_RECENT_08));
	Msgs[8] = Text::toT(SETTING(KICK_MSG_RECENT_09));
	Msgs[9] = Text::toT(SETTING(KICK_MSG_RECENT_10));
	Msgs[10] = Text::toT(SETTING(KICK_MSG_RECENT_11));
	Msgs[11] = Text::toT(SETTING(KICK_MSG_RECENT_12));
	Msgs[12] = Text::toT(SETTING(KICK_MSG_RECENT_13));
	Msgs[13] = Text::toT(SETTING(KICK_MSG_RECENT_14));
	Msgs[14] = Text::toT(SETTING(KICK_MSG_RECENT_15));
	Msgs[15] = Text::toT(SETTING(KICK_MSG_RECENT_16));
	Msgs[16] = Text::toT(SETTING(KICK_MSG_RECENT_17));
	Msgs[17] = Text::toT(SETTING(KICK_MSG_RECENT_18));
	Msgs[18] = Text::toT(SETTING(KICK_MSG_RECENT_19));
	Msgs[19] = Text::toT(SETTING(KICK_MSG_RECENT_20));

	::SetFocus(::GetDlgItem(m_hWnd, IDC_LINE));

	line = Util::emptyStringT;
	for(int i = 0; i < 20; i++) {
		if(Msgs[i] != Util::emptyStringT)
			::SendMessage(::GetDlgItem(m_hWnd, IDC_LINE), CB_ADDSTRING, 0, (LPARAM)Msgs[i].c_str());
	}
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_LINE), m_sLastMsg.c_str());

	::SetWindowText(::GetDlgItem(m_hWnd, IDC_DESCRIPTION), description.c_str());
	
	::SetWindowText(m_hWnd, title.c_str());
	
	CenterWindow(m_hWnd, ::GetParent(m_hWnd));
}
        	
void KickDlg::OnCloseCmd(WORD wID) {
	if(wID == IDOK) {
		int len = ::GetWindowTextLength(::GetDlgItem(m_hWnd, IDC_LINE)) + 1;
        AutoArray<TCHAR> buf(len);
		::GetDlgItemText(m_hWnd, IDC_LINE, buf, len);
		line = buf;
		m_sLastMsg = line;
		int i, j;

		for(i = 0; i < 20; i++) {
			if(line == Msgs[i]) {
				for(j = i; j > 0; j--) {
					Msgs[j] = Msgs[j-1];
				}
				Msgs[0] = line;
				break;
			}
		}

		if(i >= 20) {
			for(j = 19; j > 0; j--) {
				Msgs[j] = Msgs[j-1];
			}
			Msgs[0] = line;
		}

		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_01, Text::fromT(Msgs[0]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_02, Text::fromT(Msgs[1]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_03, Text::fromT(Msgs[2]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_04, Text::fromT(Msgs[3]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_05, Text::fromT(Msgs[4]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_06, Text::fromT(Msgs[5]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_07, Text::fromT(Msgs[6]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_08, Text::fromT(Msgs[7]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_09, Text::fromT(Msgs[8]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_10, Text::fromT(Msgs[9]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_11, Text::fromT(Msgs[10]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_12, Text::fromT(Msgs[11]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_13, Text::fromT(Msgs[12]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_14, Text::fromT(Msgs[13]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_15, Text::fromT(Msgs[14]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_16, Text::fromT(Msgs[15]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_17, Text::fromT(Msgs[16]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_18, Text::fromT(Msgs[17]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_19, Text::fromT(Msgs[18]));
		SettingsManager::getInstance()->set(SettingsManager::KICK_MSG_RECENT_20, Text::fromT(Msgs[19]));
	}
  
	::EndDialog(m_hWnd, wID);
}

INT_PTR KickDlg::DoModal(HWND hWndParent) {
	return ::DialogBoxParam(g_hInstance, MAKEINTRESOURCE(IDD_KICK), hWndParent, KickDlgProc, (LPARAM)this);
}
