/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "ListViewArrows.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------

HBITMAP ListViewArrows::upArrow = NULL;
HBITMAP ListViewArrows::downArrow = NULL;
//----------------------------------------------------------------------------------------------------------------------------------------------------

void ListViewArrows::Init() {
	rebuildArrows();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void ListViewArrows::Uninit() {
    if(upArrow != NULL) {
        ::DeleteObject(upArrow);
    }
    if(downArrow != NULL) {
        ::DeleteObject(downArrow);
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void ListViewArrows::rebuildArrows() {
	POINT pathArrowLong[9] = { {0L,7L}, {7L,7L}, {7L,6L}, {6L,6L}, {6L,4L}, {5L,4L}, {5L,2L}, {4L,2L}, {4L,0L} };
	POINT pathArrowShort[7] = { {0L,6L}, {1L,6L}, {1L,4L}, {2L,4L}, {2L,2L}, {3L,2L}, {3L,0L} };

	const int bitmapWidth = 8;
	const int bitmapHeight = 8;
	const RECT rect = {0, 0, bitmapWidth, bitmapHeight};

    HDC hDcWindow = ::CreateCompatibleDC(NULL);
	if(hDcWindow == NULL) {
		return;
	}

    HDC dc = ::CreateCompatibleDC(hDcWindow);
	if(dc == NULL) {
        ::DeleteDC(hDcWindow);
		return;
	}

    HBRUSH brush = ::GetSysColorBrush(COLOR_3DFACE);
	if(brush == NULL) {
		::DeleteDC(dc);
		::DeleteDC(hDcWindow);
		return;
	}

    HPEN penLight = ::CreatePen(PS_SOLID, 1, ::GetSysColor(COLOR_3DHIGHLIGHT));
	if(penLight == NULL) {
		::DeleteDC(dc);
		::DeleteDC(hDcWindow);
		return;
	}

    HPEN penShadow = ::CreatePen(PS_SOLID, 1, ::GetSysColor(COLOR_3DSHADOW));
	if(penShadow == NULL) {
		::DeleteObject(penLight);
		::DeleteDC(dc);
		::DeleteDC(hDcWindow);
		return;
	}

	if(upArrow == NULL) {
        upArrow = ::CreateBitmap(bitmapWidth, bitmapHeight, 1, 32, NULL);
    }

	if(downArrow == NULL) {
        downArrow = ::CreateBitmap(bitmapWidth, bitmapHeight, 1, 32, NULL);
    }

	// create up arrow
	HBITMAP oldBitmap = (HBITMAP)::SelectObject(dc, upArrow);
	::FillRect(dc, &rect, brush);
	HPEN oldPen = (HPEN)::SelectObject(dc, penLight);
	::Polyline(dc, pathArrowLong, sizeof(pathArrowLong)/sizeof(pathArrowLong[0]));
	::SelectObject(dc, penShadow);
	::Polyline(dc, pathArrowShort, sizeof(pathArrowShort)/sizeof(pathArrowShort[0]));

	// create down arrow
	::SelectObject(dc, downArrow);
	::FillRect(dc, &rect, brush);
	for(int i=0; i < sizeof(pathArrowShort)/sizeof(pathArrowShort[0]); ++i) {
		POINT& pt = pathArrowShort[i];
		pt.x = bitmapWidth - pt.x;
		pt.y = bitmapHeight - pt.y;
	}
	::SelectObject(dc, penLight);
	::Polyline(dc, pathArrowShort, sizeof(pathArrowShort)/sizeof(pathArrowShort[0]));
	for(int i=0; i < sizeof(pathArrowLong)/sizeof(pathArrowLong[0]); ++i) {
		POINT& pt = pathArrowLong[i];
		pt.x = bitmapWidth - pt.x;
		pt.y = bitmapHeight - pt.y;
	}
	::SelectObject(dc, penShadow);
	::Polyline(dc, pathArrowLong, sizeof(pathArrowLong)/sizeof(pathArrowLong[0]));

    ::SelectObject(dc, oldPen);
    ::SelectObject(dc, oldBitmap);
	::DeleteObject(penLight);
	::DeleteObject(penShadow);
	::DeleteDC(dc);
    ::DeleteDC(hDcWindow);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void ListViewArrows::updateArrow(HWND hWnd, const bool &isAscending, const int iSortColumn) {
	if(upArrow == NULL) {
		return;
    }

	HBITMAP bitmap = (isAscending ? upArrow : downArrow);

	HWND headerCtrl = (HWND)::SendMessage(hWnd, LVM_GETHEADER, 0, 0L);
	const int itemCount = (int)::SendMessage(headerCtrl, HDM_GETITEMCOUNT, 0, 0L);

	for(int i = 0; i < itemCount; ++i) {
		HDITEM item;
		item.mask = HDI_FORMAT;
		::SendMessage(headerCtrl, HDM_GETITEM, i, (LPARAM)&item);
		item.mask = HDI_FORMAT | HDI_BITMAP;
		if(i == iSortColumn) {
			item.fmt |= HDF_BITMAP | HDF_BITMAP_ON_RIGHT;
			item.hbm = bitmap;
		} else {
			item.fmt &= ~(HDF_BITMAP | HDF_BITMAP_ON_RIGHT);
			item.hbm = 0;
		}
		::SendMessage(headerCtrl, HDM_SETITEM, i, (LPARAM)&item);
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
