/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "LogPage.h"
#include "../client/SettingsManager.h"
#include "../client/LogManager.h"
#include "../client/File.h"
#include "WinUtil.h"

LogPage * pLogPage = NULL;

LogPage::LogPage(SettingsManager *s) : PropPage(s, IDD_LOGPAGE) {
    pLogPage = this;
	title = _tcsdup((TSTRING(SETTINGS_ADVANCED) + _T('\\') + TSTRING(SETTINGS_LOGS)).c_str());
	SetTitle(title);
	m_psp.pfnDlgProc = (DLGPROC)LogPageProc;
}

LogPage::~LogPage() {
    pLogPage = NULL;
    free(title);
}

BOOL CALLBACK LogPage::LogPageProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_INITDIALOG:
            pLogPage->m_hWnd = hWnd;
            pLogPage->onInitDialog();
            return TRUE;
        case WM_COMMAND:
            if(LOWORD(wParam) == IDC_BROWSE_LOG) {
                pLogPage->onClickedBrowseDir();
                return TRUE;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_LOG_OPTIONS && ((LPNMHDR)lParam)->code == LVN_ITEMCHANGED) {
        		pLogPage->onItemChanged();
        		return TRUE;
            }
    }

	return FALSE;
}

PropPage::TextItem LogPage::texts[] = {
	{ IDC_SETTINGS_LOGGING, ResourceManager::SETTINGS_LOGGING },
	{ IDC_SETTINGS_LOG_DIR, ResourceManager::DIRECTORY},
	{ IDC_BROWSE_LOG, ResourceManager::BROWSE_ACCEL },
	{ IDC_SETTINGS_FORMAT, ResourceManager::SETTINGS_FORMAT },
	{ IDC_SETTINGS_FILE_NAME, ResourceManager::SETTINGS_FILE_NAME },
	{ IDC_SETTINGS_BOLD_CONTENTS, ResourceManager::SETTINGS_BOLD_OPTIONS },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

PropPage::Item LogPage::items[] = {
	{ IDC_LOG_DIRECTORY, SettingsManager::LOG_DIRECTORY, PropPage::T_STR },
	{ 0, 0, PropPage::T_END }
};

PropPage::ListItem LogPage::listItems[] = {
	{ SettingsManager::LOG_MAIN_CHAT, ResourceManager::SETTINGS_LOG_MAIN_CHAT },
	{ SettingsManager::LOG_PRIVATE_CHAT, ResourceManager::SETTINGS_LOG_PRIVATE_CHAT },
	{ SettingsManager::LOG_DOWNLOADS, ResourceManager::SETTINGS_LOG_DOWNLOADS }, 
	{ SettingsManager::LOG_UPLOADS, ResourceManager::SETTINGS_LOG_UPLOADS },
	{ SettingsManager::LOG_SYSTEM, ResourceManager::SETTINGS_LOG_SYSTEM_MESSAGES },
	{ SettingsManager::LOG_STATUS_MESSAGES, ResourceManager::SETTINGS_LOG_STATUS_MESSAGES },
	{ SettingsManager::LOG_FILELIST_TRANSFERS, ResourceManager::SETTINGS_LOG_FILELIST_TRANSFERS },
	{ SettingsManager::LOG_PARTIAL_UPLOADS, ResourceManager::LOG_PARTIAL_UPLOADS },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

PropPage::ListItem LogPage::listItems1[] = {
	{ SettingsManager::BOLD_FINISHED_DOWNLOADS, ResourceManager::FINISHED_DOWNLOADS },
	{ SettingsManager::BOLD_FINISHED_UPLOADS, ResourceManager::FINISHED_UPLOADS },
	{ SettingsManager::BOLD_QUEUE, ResourceManager::DOWNLOAD_QUEUE },
	{ SettingsManager::BOLD_HUB, ResourceManager::HUB },
	{ SettingsManager::BOLD_PM, ResourceManager::PRIVATE_MESSAGE },
	{ SettingsManager::BOLD_SEARCH, ResourceManager::SEARCH },
	{ SettingsManager::BOLD_WAITING_USERS, ResourceManager::WAITING_USERS },
	{ SettingsManager::BOLD_SYSTEM_LOG, ResourceManager::SYSTEM_LOG },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

void LogPage::onInitDialog() {
	PropPage::translate(m_hWnd, texts);
	PropPage::read(m_hWnd, items, listItems, ::GetDlgItem(m_hWnd, IDC_LOG_OPTIONS));
	PropPage::read(m_hWnd, items, listItems1, ::GetDlgItem(m_hWnd, IDC_BOLD_BOOLEANS));

	for(int i = 0; i < LogManager::LAST; ++i) {
		TStringPair pair;
		pair.first = Text::toT(LogManager::getInstance()->getSetting(i, LogManager::FILE));
		pair.second = Text::toT(LogManager::getInstance()->getSetting(i, LogManager::FORMAT));
		options.push_back(pair);
	}

	::EnableWindow(::GetDlgItem(m_hWnd, IDC_LOG_FORMAT), false);
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_LOG_FILE), false);

	oldSelection = -1;
}

void LogPage::onItemChanged() {
	logOptions.m_hWnd = ::GetDlgItem(m_hWnd, IDC_LOG_OPTIONS);

	getValues();

	int sel = (int)::SendMessage(logOptions.m_hWnd, LVM_GETNEXTITEM, (WPARAM)-1, MAKELPARAM(LVNI_ALL | LVNI_SELECTED, 0));

	if(sel >= 0 && sel < LogManager::LAST) {
		BOOL checkState = (((UINT)::SendMessage(logOptions.m_hWnd, LVM_GETITEMSTATE, sel, LVIS_STATEIMAGEMASK)>> 12) - 1) == BST_CHECKED ? TRUE : FALSE;

		::EnableWindow(::GetDlgItem(m_hWnd, IDC_LOG_FORMAT), checkState);
		::EnableWindow(::GetDlgItem(m_hWnd, IDC_LOG_FILE), checkState);

		::SetDlgItemText(m_hWnd, IDC_LOG_FILE, options[sel].first.c_str());
		::SetDlgItemText(m_hWnd, IDC_LOG_FORMAT, options[sel].second.c_str());

		//save the old selection so we know where to save the values
		oldSelection = sel;
	} else {
		::EnableWindow(::GetDlgItem(m_hWnd, IDC_LOG_FORMAT), FALSE);
		::EnableWindow(::GetDlgItem(m_hWnd, IDC_LOG_FILE), FALSE);

		::SetDlgItemText(m_hWnd, IDC_LOG_FILE, _T(""));
		::SetDlgItemText(m_hWnd, IDC_LOG_FORMAT, _T(""));
	}

	logOptions.m_hWnd = NULL;
}

void LogPage::getValues() {
	if(oldSelection >= 0) {
		TCHAR buf[512];

		if(::GetDlgItemText(m_hWnd, IDC_LOG_FILE, buf, 512) > 0)
			options[oldSelection].first = buf;
		if(::GetDlgItemText(m_hWnd, IDC_LOG_FORMAT, buf, 512) > 0)
			options[oldSelection].second = buf;
	}
}

void LogPage::write() {
	PropPage::write(m_hWnd, items, listItems, ::GetDlgItem(m_hWnd, IDC_LOG_OPTIONS));
	PropPage::write(m_hWnd, items, listItems1, ::GetDlgItem(m_hWnd, IDC_BOLD_BOOLEANS));

	const string& s = SETTING(LOG_DIRECTORY);
	if(s.length() > 0 && s[s.length() - 1] != '\\') {
		SettingsManager::getInstance()->set(SettingsManager::LOG_DIRECTORY, s + '\\');
	}
	File::ensureDirectory(SETTING(LOG_DIRECTORY));

	//make sure we save the last edit too, the user
	//might not have changed the selection
	getValues();

	for(int i = 0; i < LogManager::LAST; ++i) {
		string tmp = Text::fromT(options[i].first);
		if(Util::stricmp(Util::getFileExt(tmp), ".log") != 0)
			tmp += ".log";

		LogManager::getInstance()->saveSetting(i, LogManager::FILE, tmp);
		LogManager::getInstance()->saveSetting(i, LogManager::FORMAT, Text::fromT(options[i].second));
	}
}

void LogPage::onClickedBrowseDir() {
	tstring dir = Text::toT(SETTING(LOG_DIRECTORY));
	if(WinUtil::browseDirectory(dir, m_hWnd)) {
		// Adjust path string
		if(dir.size() > 0 && dir[dir.size() - 1] != '\\')
			dir += '\\';

		::SetDlgItemText(m_hWnd, IDC_LOG_DIRECTORY, dir.c_str());
	}
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
