/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "../client/SettingsManager.h"

#include "FlatTabCtrl.h"
#include "WinUtil.h"

#include "MDIChildWindow.h"

static const UINT WM_REALLY_CLOSE = WM_APP + 705;

MDITabChildWindow::MDITabChildWindow() : created(false), m_hWnd(NULL), m_hWndStatusBar(NULL) {
    m_hAccel = NULL;
}

bool MDITabChildWindow::MDIWindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, LRESULT& lResult) {
    switch(uMsg) {
        case WM_CLOSE:
            ::PostMessage(m_hWnd, WM_REALLY_CLOSE, 0, 0);
            return true;
        case WM_SYSCOMMAND:
            if(onSysCommand(wParam) == true) {
                return true;
            }
            break;
        case WM_FORWARDMSG:
            onForwardMsg(lParam);
            return true;
        case WM_MDIACTIVATE:
            onMDIActivate(lParam);
            return false;
        case WM_DESTROY:
            onDestroy();
            return false;
        case WM_SETTEXT:
            onSetText(lParam);
            return false;
        case WM_REALLY_CLOSE:
            ::SendMessage(WinUtil::mdiClient, WM_MDIDESTROY, (WPARAM)m_hWnd, 0);
            return true;
        case WM_NOTIFYFORMAT:
            lResult = NFR_UNICODE;
            return true;
        case WM_WINDOWPOSCHANGED:
            onWindowPosChanged(lParam);
            break;
        case WM_MOUSEACTIVATE:
            lResult = onMouseActivate(hWnd, uMsg, wParam, lParam);
            return true;
		case WM_GETMINMAXINFO: {
			MINMAXINFO *mminfo = (MINMAXINFO*)lParam;
			mminfo->ptMinTrackSize.x = 300;
			mminfo->ptMinTrackSize.y = 150;
			if(mminfo->ptMaxTrackSize.x < mminfo->ptMaxSize.x) {
				mminfo->ptMaxTrackSize.x = mminfo->ptMaxSize.x;
			}
			if(mminfo->ptMaxTrackSize.y < mminfo->ptMaxSize.y) {
				mminfo->ptMaxTrackSize.y = mminfo->ptMaxSize.y;
			}
			break;
		}
    }

    return false;
}

void MDITabChildWindow::onForwardMsg(LPARAM lParam) {
	if(m_hAccel != NULL) {
        ::TranslateAccelerator(m_hWnd, m_hAccel, (LPMSG)lParam);
    }
}

bool MDITabChildWindow::onSysCommand(WPARAM wParam) {
	if(wParam == SC_NEXTWINDOW) {
		WinUtil::tabCtrl->SwitchTo(true);
		return true;
	} else if(wParam == SC_PREVWINDOW) {
		WinUtil::tabCtrl->SwitchTo(false);
		return true;
	}

	return false;
}

void MDITabChildWindow::onMDIActivate(LPARAM lParam) {
	if((m_hWnd == (HWND)lParam)) {
		WinUtil::tabCtrl->setActive(m_hWnd);
    }
}

void MDITabChildWindow::onDestroy() {
	WinUtil::tabCtrl->removeTab(m_hWnd);

	BOOL bMaximized = FALSE;
	if(::SendMessage(WinUtil::mdiClient, WM_MDIGETACTIVE, 0, (LPARAM)&bMaximized) != NULL) {
		SettingsManager::getInstance()->set(SettingsManager::MDI_MAXIMIZED, (bMaximized>0));
    }

    UpdateClientEdge();
}

void MDITabChildWindow::onSetText(LPARAM lParam) {
	if(created == true) {
		WinUtil::tabCtrl->updateText(m_hWnd, (LPCTSTR)lParam);
	}
}

LRESULT MDITabChildWindow::onMouseActivate(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	LRESULT lRes = ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);

	// Activate this MDI window if needed
	if(lRes == MA_ACTIVATE || lRes == MA_ACTIVATEANDEAT) {
		if((HWND)::SendMessage(WinUtil::mdiClient, WM_MDIGETACTIVE, 0, (LPARAM)NULL) != m_hWnd) {
			::SendMessage(WinUtil::mdiClient, WM_MDIACTIVATE, (WPARAM)m_hWnd, 0);
        }
	}

	return lRes;
}

void MDITabChildWindow::CreateEx(void * pOwner, ATOM aAtom, UINT uiResourceID, LPRECT rect/* = NULL*/, LPCTSTR lpcstrWindowName/* = NULL*/) {
	// If the currently active MDI child is maximized, we want to create this one maximized too
	BOOL bMaximized = FALSE;

	if((HWND)::SendMessage(WinUtil::mdiClient, WM_MDIGETACTIVE, 0, (LPARAM)&bMaximized) == NULL) {
		bMaximized = BOOLSETTING(MDI_MAXIMIZED);
	}

	if(bMaximized) {
        ::SendMessage(WinUtil::mdiClient, WM_SETREDRAW, (WPARAM)FALSE, 0);
    }

	if(rect != NULL) {
		m_hWnd = ::CreateWindowEx(WS_EX_MDICHILD, MAKEINTATOM(aAtom), lpcstrWindowName, WS_OVERLAPPEDWINDOW | WS_CHILD | WS_VISIBLE |
            WS_CLIPCHILDREN | WS_CLIPSIBLINGS, rect->left, rect->top, rect->right - rect->left, rect->bottom - rect->top, WinUtil::mdiClient, 0U,
            g_hInstance, pOwner);
	} else {
		m_hWnd = ::CreateWindowEx(WS_EX_MDICHILD, MAKEINTATOM(aAtom), lpcstrWindowName, WS_OVERLAPPEDWINDOW | WS_CHILD | WS_VISIBLE |
			WS_CLIPCHILDREN | WS_CLIPSIBLINGS, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, WinUtil::mdiClient, 0U, g_hInstance,
            pOwner);
	}

	if(bMaximized) {
		// Maximize and redraw everything
		if(m_hWnd != NULL) {
			::SendMessage(WinUtil::mdiClient, WM_MDIMAXIMIZE, (WPARAM)m_hWnd, 0);
        }
		::SendMessage(WinUtil::mdiClient, WM_SETREDRAW, (WPARAM)TRUE, 0);
		::RedrawWindow(WinUtil::mdiClient, NULL, NULL, RDW_INVALIDATE | RDW_ALLCHILDREN);
		::SetFocus(::GetParent(WinUtil::mdiClient));   // focus will be set back to this window
	} else if(m_hWnd != NULL && WinUtil::isAppActive && ::IsWindowVisible(m_hWnd) && !::IsChild(m_hWnd, ::GetFocus())) {
		::SetFocus(m_hWnd);
	}

	if(m_hWnd != NULL) {
		m_hAccel = ::LoadAccelerators(g_hInstance, MAKEINTRESOURCE(uiResourceID));
	}
}

void MDITabChildWindow::setDirty() {
	WinUtil::tabCtrl->setDirty(m_hWnd);
}

void MDITabChildWindow::setTabColor(COLORREF color) {
	WinUtil::tabCtrl->setColor(m_hWnd, color);
}

void MDITabChildWindow::setIconState() {
	WinUtil::tabCtrl->setIconState(m_hWnd);
}

void MDITabChildWindow::unsetIconState() {
	WinUtil::tabCtrl->unsetIconState(m_hWnd);
}

void MDITabChildWindow::UpdateBarsPosition(RECT& rect, bool bResizeBars/* = true*/) {
	if(m_hWndStatusBar != NULL && ((DWORD)::GetWindowLong(m_hWndStatusBar, GWL_STYLE) & WS_VISIBLE)) {
		if(bResizeBars == true) {
			::SendMessage(m_hWndStatusBar, WM_SIZE, 0, 0);
        }

		RECT rectSB = { 0 };
		::GetWindowRect(m_hWndStatusBar, &rectSB);
		rect.bottom -= rectSB.bottom - rectSB.top;
	}
}


bool MDITabChildWindow::UpdateClientEdge(LPRECT lpRect/* = NULL*/) {
	HWND hWndChild = (HWND)::SendMessage(WinUtil::mdiClient, WM_MDIGETACTIVE, 0, (LPARAM)NULL);
	if(hWndChild != NULL && hWndChild != m_hWnd) {
		return false;
    }

	DWORD dwStyle = ::GetWindowLong(WinUtil::mdiClient, GWL_EXSTYLE);
	DWORD dwNewStyle = dwStyle;
	if(hWndChild != NULL && ((::GetWindowLong(m_hWnd, GWL_EXSTYLE) & WS_EX_CLIENTEDGE) == 0) && ((::GetWindowLong(m_hWnd, GWL_STYLE) & WS_MAXIMIZE) != 0)) {
		dwNewStyle &= ~(WS_EX_CLIENTEDGE);
	} else {
		dwNewStyle |= WS_EX_CLIENTEDGE;
    }

	if(dwStyle != dwNewStyle) {
		::RedrawWindow(WinUtil::mdiClient, NULL, NULL, RDW_INVALIDATE | RDW_ALLCHILDREN);
		::SetWindowLong(WinUtil::mdiClient, GWL_EXSTYLE, dwNewStyle);
		::SetWindowPos(WinUtil::mdiClient, NULL, 0, 0, 0, 0, SWP_FRAMECHANGED | SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_NOCOPYBITS);

		if(lpRect != NULL) {
			::GetClientRect(WinUtil::mdiClient, lpRect);
        }

		return true;
	}

	return false;
}

void MDITabChildWindow::onWindowPosChanged(LPARAM lParam) {
	LPWINDOWPOS lpWndPos = (LPWINDOWPOS)lParam;

	if(!(lpWndPos->flags & SWP_NOSIZE)) {
		RECT rectClient;
		if(UpdateClientEdge(&rectClient) && ((::GetWindowLong(m_hWnd, GWL_STYLE) & WS_MAXIMIZE) != 0)) {
			::AdjustWindowRectEx(&rectClient, ::GetWindowLong(m_hWnd, GWL_STYLE), FALSE, ::GetWindowLong(m_hWnd, GWL_EXSTYLE));
			lpWndPos->x = rectClient.left;
			lpWndPos->y = rectClient.top;
			lpWndPos->cx = rectClient.right - rectClient.left;
			lpWndPos->cy = rectClient.bottom - rectClient.top;
		}
	}
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
