/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2009 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"
#include "MagnetDlg.h"
#include "WinUtil.h"
#include "WinApiUtil.h"

INT_PTR CALLBACK MagnetDlg::MagnetDlgProc(HWND hWndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam){
	MagnetDlg * pParent;

	if(uMsg == WM_INITDIALOG) {
		pParent = (MagnetDlg *)lParam;
		SetWindowLongPtr(hWndDlg, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWndDlg;
	} else {
		pParent = (MagnetDlg *)GetWindowLongPtr(hWndDlg, GWLP_USERDATA);
		if(pParent == NULL) {
			return FALSE;
		}
	}

    SetWindowLongPtr(hWndDlg, DWLP_MSGRESULT, 0);

	return pParent->MagnetProc(hWndDlg, uMsg, wParam, lParam);
}

LRESULT MagnetDlg::MagnetProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_INITDIALOG:
            onInitDialog();
            return TRUE;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDOK:
                case IDCANCEL:
                    onCloseCmd(LOWORD(wParam));
                    return TRUE;
                case IDC_MAGNET_QUEUE:
                case IDC_MAGNET_NOTHING:
                case IDC_MAGNET_SEARCH:
                    onRadioButton(LOWORD(wParam));
                    return TRUE;
            }
            break;
    }

	return FALSE;
}

void MagnetDlg::onInitDialog() {
	// zombies.
	::SetWindowText(m_hWnd, CTSTRING(MAGNET_DLG_TITLE));
	CenterWindow(m_hWnd, ::GetParent(m_hWnd));

	// fill in dialog bits
	::SetDlgItemText(m_hWnd, IDC_MAGNET_HASH, CTSTRING(MAGNET_DLG_HASH));
	::SetDlgItemText(m_hWnd, IDC_MAGNET_NAME, CTSTRING(MAGNET_DLG_FILE));
	::SetDlgItemText(m_hWnd, IDC_MAGNET_SIZE, CTSTRING(MAGNET_DLG_SIZE));
	::SetDlgItemText(m_hWnd, IDC_MAGNET_QUEUE, CTSTRING(MAGNET_DLG_QUEUE));
	::SetDlgItemText(m_hWnd, IDC_MAGNET_SEARCH, CTSTRING(MAGNET_DLG_SEARCH));
	::SetDlgItemText(m_hWnd, IDC_MAGNET_NOTHING, CTSTRING(MAGNET_DLG_NOTHING));
	::SetDlgItemText(m_hWnd, IDC_MAGNET_REMEMBER, CTSTRING(MAGNET_DLG_REMEMBER));
	if(mSize <= 0 || mFileName.length() <= 0) {
		::ShowWindow(::GetDlgItem(m_hWnd, IDC_MAGNET_QUEUE), FALSE);
		::ShowWindow(::GetDlgItem(m_hWnd, IDC_MAGNET_REMEMBER), FALSE);
	}
	::SetDlgItemText(m_hWnd, IDC_MAGNET_TEXT, CTSTRING(MAGNET_DLG_TEXT_GOOD));

	// file details
	::SetDlgItemText(m_hWnd, IDC_MAGNET_DISP_HASH, mHash.c_str());
	::SetDlgItemText(m_hWnd, IDC_MAGNET_DISP_NAME, mFileName.length() > 0 ? mFileName.c_str() : _T("N/A"));
	char buf[32];
	_i64toa_s(mSize, buf, 32, 10);
	::SetDlgItemText(m_hWnd, IDC_MAGNET_DISP_SIZE, mSize > 0 ? Text::toT(buf).c_str() : _T("N/A"));
		//search->minFileSize > 0 ? _i64toa(search->minFileSize, buf, 10) : ""

	// radio button
	::CheckRadioButton(m_hWnd, IDC_MAGNET_QUEUE, IDC_MAGNET_NOTHING, IDC_MAGNET_SEARCH);

	// focus
	::SetFocus(::GetDlgItem(m_hWnd, IDC_MAGNET_SEARCH));
}

void MagnetDlg::onCloseCmd(WORD wID) {
	if(wID == IDOK) {
		if(::IsDlgButtonChecked(m_hWnd, IDC_MAGNET_REMEMBER) == BST_CHECKED) {
			SettingsManager::getInstance()->set(SettingsManager::MAGNET_ASK,  false);
			if(::IsDlgButtonChecked(m_hWnd, IDC_MAGNET_QUEUE)) {
				SettingsManager::getInstance()->set(SettingsManager::MAGNET_ACTION, SettingsManager::MAGNET_AUTO_DOWNLOAD);
			} else if(::IsDlgButtonChecked(m_hWnd, IDC_MAGNET_SEARCH)) {
				SettingsManager::getInstance()->set(SettingsManager::MAGNET_ACTION, SettingsManager::MAGNET_AUTO_SEARCH);
			}
		}

		if(::IsDlgButtonChecked(m_hWnd, IDC_MAGNET_SEARCH)) {
			TTHValue tmphash(Text::fromT(mHash));
			WinUtil::searchHash(&tmphash); 
		} else if(::IsDlgButtonChecked(m_hWnd, IDC_MAGNET_QUEUE)) {
            try {
                QueueManager::getInstance()->add(Text::fromT(mFileName), mSize, Text::fromT(mHash));
			} catch(const Exception& e) {
				LogManager::getInstance()->message(e.getError());
			}
		} 
	}
	::EndDialog(m_hWnd, wID);
}

void MagnetDlg::onRadioButton(WORD wID) {
	switch(wID) {
		case IDC_MAGNET_QUEUE:
		case IDC_MAGNET_SEARCH:
			if(mSize > 0 && mFileName.length() > 0) {
				::EnableWindow(::GetDlgItem(m_hWnd, IDC_MAGNET_REMEMBER), TRUE);
			}
			break;
		case IDC_MAGNET_NOTHING:
			if(::IsDlgButtonChecked(m_hWnd, IDC_MAGNET_REMEMBER) == BST_CHECKED) {
				::CheckDlgButton(m_hWnd, IDC_MAGNET_REMEMBER, FALSE);
			}
			::EnableWindow(::GetDlgItem(m_hWnd, IDC_MAGNET_REMEMBER), FALSE);
			break;
	}
}

void MagnetDlg::DoModal(HWND hWndParent) {
	::DialogBoxParam(g_hInstance, MAKEINTRESOURCE(IDD_MAGNET), hWndParent, MagnetDlgProc, (LPARAM)this);
}
