/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "MainFrm.h"
#include "AboutDlg.h"
#include "HubFrame.h"
#include "SearchFrm.h"
#include "PublicHubsFrm.h"
#include "PropertiesDlg.h"
#include "UsersFrame.h"
#include "DirectoryListingFrm.h"
#include "FavoritesFrm.h"
#include "NotepadFrame.h"
#include "QueueFrame.h"
#include "SpyFrame.h"
#include "FinishedFrame.h"
#include "ADLSearchFrame.h"
#include "FinishedULFrame.h"
#include "TextFrame.h"
#include "StatsFrame.h"
#include "LineDlg.h"
#include "HashProgressDlg.h"
#include "UPnP.h"
#include "SystemFrame.h"
#include "RecentsFrm.h"
#include "WaitingUsersFrame.h"
#include "PrivateFrame.h"
#include "DebugFrame.h"
#include "CountryFlags.h"

#include "../client/ConnectionManager.h"
#include "../client/DownloadManager.h"
#include "../client/UploadManager.h"
#include "../client/SimpleXML.h"
#include "../client/ShareManager.h"
#include "../client/LogManager.h"

MainFrame* MainFrame::pMainFrame = NULL;
bool MainFrame::bShutdown = false;
time_t MainFrame::iCurrentShutdownTime = 0;
bool MainFrame::isShutdownStatus = false;
bool MainFrame::closing = false;
HWND MainFrame::m_hWnd = NULL;
static WORD wMenuBitmapIds[] = { ID_FILE_CONNECT, ID_FILE_RECONNECT, IDC_FOLLOW, IDC_RECENTS, IDC_FAVORITES, IDC_FAVUSERS, IDC_QUEUE, IDC_FINISHED, IDC_UPLOAD_QUEUE,
    IDC_FINISHED_UL, ID_FILE_SEARCH, IDC_FILE_ADL_SEARCH, IDC_SEARCH_SPY, IDC_NET_STATS, IDC_SYSTEM_LOG, IDC_OPEN_FILE_LIST, ID_FILE_SETTINGS,
    IDC_NOTEPAD, ID_WINDOW_CASCADE, ID_WINDOW_TILE_HORZ, ID_WINDOW_TILE_VERT, ID_WINDOW_MINIMIZE_ALL, ID_WINDOW_RESTORE_ALL };

LRESULT CALLBACK MainFrame::StaticWindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	MainFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (MainFrame *)((LPCREATESTRUCT)lParam)->lpCreateParams;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		MainFrame::m_hWnd = hWnd;
	} else {
		pParent = (MainFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefFrameProc(MainFrame::m_hWnd, WinUtil::mdiClient, uMsg, wParam, lParam);
		}
	}

	return pParent->MainWindowProc(uMsg, wParam, lParam);
}
//------------------------------------------------------------------------------

LRESULT MainFrame::MainWindowProc(UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CREATE:
            OnCreate();
            return 0;
        case WM_ERASEBKGND:
            if(OnEraseBackground() == true) {
                return 1;
            }
            break;
        case WM_CLOSE:
            if(OnClose() == true) {
                return 0;
            }
            break;
        case WM_SPEAKER:
            onSpeaker(wParam, lParam);
            return 0;
        case FTM_SELECTED:
            onSelected(wParam);
            return 0;
        case FTM_ROWS_CHANGED:
            onRowsChanged();
            return 0;
        case WM_APP+242:
            onTrayIcon(lParam);
            return 0;
        case WM_DESTROY:
            onDestroy();
            break;
        case WM_SIZE:
            onSize(wParam);
            return 0;
		case WM_GETMINMAXINFO: {
			MINMAXINFO *mminfo = (MINMAXINFO*)lParam;
			mminfo->ptMinTrackSize.x = 600;
			mminfo->ptMinTrackSize.y = 400;
			return 0;
		}
        case WM_ENDSESSION:
            onEndSession();
            return 0;
        case WM_COPYDATA:
            onCopyData(lParam);
            return 0;
        case WM_ACTIVATEAPP:
            onActivateApp(wParam);
            break;
        case WM_APPCOMMAND:
            if(onAppCommand(lParam) == true) {
                return TRUE;
            }
            break;
        case IDC_REBUILD_TOOLBAR:
            createToolbar();
            return 0;
        case WM_REFRESH_SETTINGS:
            lastSave = GET_TICK();
            return 0;
        case WM_SETFOCUS:
            onSetFocus();
            break;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case ID_APP_EXIT:
                    ::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
                    return 0;
                case ID_FILE_SETTINGS:
                    OnFileSettings();
                    return 0;
                case IDC_MATCH_ALL:
                    onMatchAll();
                    return 0;
                case ID_VIEW_TOOLBAR:
                    OnViewToolBar();
                    return 0;
                case ID_VIEW_STATUS_BAR:
                    OnViewStatusBar();
                    return 0;
                case ID_VIEW_TRANSFER_VIEW:
                    OnViewTransferView();
                    return 0;
                case ID_APP_ABOUT:
                    OnAppAbout();
                    return 0;
                case ID_WINDOW_CASCADE:
                    ::SendMessage(WinUtil::mdiClient, WM_MDICASCADE, (WPARAM)0, 0);
                    return 0;
                case ID_WINDOW_TILE_HORZ:
                    ::SendMessage(WinUtil::mdiClient, WM_MDITILE, (WPARAM)MDITILE_HORIZONTAL, 0);
                    return 0;
                case ID_WINDOW_TILE_VERT:
                    ::SendMessage(WinUtil::mdiClient, WM_MDITILE, (WPARAM)MDITILE_VERTICAL, 0);
                    return 0;
                case ID_WINDOW_ARRANGE:
                    ::SendMessage(WinUtil::mdiClient, WM_MDIICONARRANGE, 0, 0);
                    return 0;
                case ID_FILE_CONNECT:
                case ID_FILE_SEARCH:
                case IDC_FAVORITES:
                case IDC_FAVUSERS:
                case IDC_NOTEPAD:
                case IDC_QUEUE:
                case IDC_SEARCH_SPY:
                case IDC_FILE_ADL_SEARCH:
                case IDC_NET_STATS:
                case IDC_FINISHED:
                case IDC_FINISHED_UL:
                case IDC_UPLOAD_QUEUE:
                case IDC_SYSTEM_LOG:
                case IDC_RECENTS:
                case IDC_DEBUG_WINDOW:
                    onOpenWindows(LOWORD(wParam));
                    return 0;
                case IDC_HELP_HOMEPAGE:
                case IDC_HELP_DISCUSS:
                    onLink(LOWORD(wParam));
                    return 0;
                case IDC_OPEN_FILE_LIST:
                    onOpenFileList();
                    return 0;
                case IDC_OPEN_OWN_LIST:
                    onOpenOwnList();
                    return 0;
                case IDC_TRAY_SHOW:
                    onAppShow();
                    return 0;
                case ID_WINDOW_MINIMIZE_ALL:
                    onWindowMinimizeAll();
                    return 0;
                case ID_WINDOW_RESTORE_ALL:
                    onWindowRestoreAll();
                    return 0;
                case IDC_CLOSE_DISCONNECTED:
                case IDC_CLOSE_DISCONNECTED_PM:
                case IDC_CLOSE_ALL_PM:
                case IDC_CLOSE_ALL_SEARCHS:
                case IDC_CLOSE_ALL_FILELISTS:
                    onCloseWindows(LOWORD(wParam));
                    return 0;
                case IDC_OPEN_DOWNLOADS:
                    WinUtil::openFile(Text::toT(SETTING(DOWNLOAD_DIRECTORY)));
                    return 0;
                case IDC_REFRESH_FILE_LIST:
                    onRefreshFileList();
                    return 0;
                case ID_FILE_QUICK_CONNECT:
                    onQuickConnect();
                    return 0;
                case IDC_HASH_PROGRESS:
                    HashProgressDlg(false).DoModal(m_hWnd);
                    return 0;
                case IDC_TOOLBAR_AWAY:
                    onToolbarAway();
                    return 0;
                case IDC_SHUTDOWN:
                    setShutDown(!getShutDown());
                    return 0;
                case IDC_ALTERNATE_LIMIT_TRAY:
                    onAlternateLimitTray();
                    return 0;
                case IDC_MUTE_SOUNDS_TRAY:
                    onMuteSounds();
                    return 0;
            }

            {
        		HWND hWndChild = (HWND)::SendMessage(WinUtil::mdiClient, WM_MDIGETACTIVE, 0, (LPARAM)NULL);
        		if(hWndChild != NULL) {
        			::SendMessage(hWndChild, uMsg, wParam, lParam);
                }
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->code == TTN_GETDISPINFO) {
                onGetToolTip((int)wParam, (LPNMHDR)lParam);
                return 0;
            } else if(((LPNMHDR)lParam)->code == RBN_CHEVRONPUSHED) {
                OnChevronPushed((LPNMREBARCHEVRON)lParam);
                return 0;
            } else if(((LPNMHDR)lParam)->code == RBN_AUTOSIZE) {
                UpdateLayout(false);
                return 0;
            }

            break;
        case WM_MEASUREITEM: {
            LPMEASUREITEMSTRUCT lpMeasureItemStruct = (LPMEASUREITEMSTRUCT)lParam;

            if(lpMeasureItemStruct->CtlType == ODT_MENU) {
                HDC dc = ::GetWindowDC(NULL);
                HFONT m_fontMenu;

                NONCLIENTMETRICS info = { 0 };
				info.cbSize = sizeof(NONCLIENTMETRICS);
                BOOL bRet = ::SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(info), &info, 0);
                if(bRet == FALSE) {
                    ::ReleaseDC(NULL, dc);
                    break;
                }

                HFONT hFontMenu = ::CreateFontIndirect(&info.lfMenuFont);
                if(hFontMenu == NULL) {
                    ::ReleaseDC(NULL, dc);
                    break;
                }

                m_fontMenu = hFontMenu;

                HFONT hOldFont = (HFONT)::SelectObject(dc, m_fontMenu);

                HMENU hMainMenu = ::GetMenu(m_hWnd);
                TCHAR lpstrText[256];
                ::GetMenuString(hMainMenu, lpMeasureItemStruct->itemID, lpstrText, 256, MF_BYCOMMAND);

                RECT rcText = { 0, 0, 0, 0 };
                ::DrawText(dc, lpstrText, -1, &rcText, DT_SINGLELINE | DT_LEFT | DT_VCENTER | DT_CALCRECT);

                LOGFONT logfont = { 0 };
                ::GetObject(m_fontMenu, sizeof(LOGFONT), &logfont);
                int cy = logfont.lfHeight;

                ::SelectObject(dc, hOldFont);
                DeleteObject(m_fontMenu);

                if(cy < 0) {
                    cy = -cy;
                }

                lpMeasureItemStruct->itemHeight = max(cy+8, 16);

                lpMeasureItemStruct->itemWidth = (((rcText.right - rcText.left) + 37) - (::GetSystemMetrics(SM_CXMENUCHECK) - 1));

                ::ReleaseDC(NULL, dc);
                
                return TRUE;
            }

            break;
        }
        case WM_DRAWITEM: {
            LPDRAWITEMSTRUCT lpDrawItemStruct = (LPDRAWITEMSTRUCT)lParam;

            if(lpDrawItemStruct->CtlType == ODT_MENU) {
                HDC dc = lpDrawItemStruct->hDC;
                const RECT& rcItem = lpDrawItemStruct->rcItem;

                BOOL bSelected = lpDrawItemStruct->itemState & ODS_SELECTED;

                if(bSelected == TRUE) {
                    ::FillRect(dc, &rcItem, ::GetSysColorBrush(COLOR_MENUHILIGHT));
                    ::FrameRect(dc, &rcItem, ::GetSysColorBrush(COLOR_HIGHLIGHT));
                } else if(lpDrawItemStruct->itemAction == ODA_SELECT) {
                    ::FillRect(dc, &rcItem, ::GetSysColorBrush(COLOR_MENU));
                }

                RECT rcButn = { rcItem.left, rcItem.top, rcItem.left + 16, rcItem.top + 16 };
                ::OffsetRect(&rcButn, 0, ((rcItem.bottom - rcItem.top) - (rcButn.bottom - rcButn.top)) / 2);

				::ImageList_Draw(images, (uint8_t)lpDrawItemStruct->itemData, dc, rcButn.left + ((rcButn.right - rcButn.left - 16) / 2),
                    rcButn.top + ((rcButn.bottom - rcButn.top - 16) / 2), ILD_TRANSPARENT);

                RECT rcText = rcItem;
                rcText.left += 19;
                rcText.right -= 16;
                ::SetBkMode(dc, TRANSPARENT);

                HMENU hMainMenu = (HMENU)lpDrawItemStruct->hwndItem;
                TCHAR lpstrText[256];
                ::GetMenuString(hMainMenu, lpDrawItemStruct->itemID, lpstrText, 256, MF_BYCOMMAND);

                size_t szTab = -1;
                for(size_t i = 0; i < _tcslen(lpstrText); i++) {
                    if(lpstrText[i] == _T('\t')) {
                        szTab = i;
                        break;
                    }
                }

                ::SetTextColor(dc, ::GetSysColor(bSelected ? COLOR_HIGHLIGHTTEXT : COLOR_MENUTEXT));
                ::DrawText(dc, lpstrText, (int)szTab, &rcText, DT_SINGLELINE | DT_LEFT | DT_VCENTER | DT_HIDEPREFIX);
                if(szTab != -1) {
                    ::DrawText(dc, &lpstrText[szTab + 1], -1, &rcText, DT_SINGLELINE | DT_RIGHT | DT_VCENTER | DT_HIDEPREFIX);
                }

                return TRUE;
            }

            break;
        }
        case WM_SETTINGCHANGE:
            ListViewArrows::rebuildArrows();
            break;
    }

	if(uMsg == trayMessage) {
		onTray();
		return 0;
	} else if(uMsg == WMU_WHERE_ARE_YOU) {
		return WMU_WHERE_ARE_YOU;
	}

    LRESULT lResult = 0;
    if(uMsg == WM_MOUSEACTIVATE) {
        lResult = ::DefFrameProc(m_hWnd, WinUtil::mdiClient, uMsg, wParam, lParam);
    }

    if(CSplitterProc(uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

	return ::DefFrameProc(m_hWnd, WinUtil::mdiClient, uMsg, wParam, lParam);
}
//------------------------------------------------------------------------------

MainFrame::MainFrame() : trayMessage(0), maximized(false), lastUpload(-1), lastUpdate(0), lastUp(0), 
lastDown(0), oldshutdown(false), stopperThread(NULL), c(new HttpConnection()), missedAutoConnect(false), 
awaybyminimize(false), alternatelimitingfromtray(false), tabsontop(false), bTrayIcon(false), bAppMinimized(false), 
bIsPM(false), UPnP_TCPConnection(NULL), UPnP_UDPConnection(NULL), trayMenu(NULL) {
    m_hWndToolBar = NULL;
    m_hWndStatusBar = NULL;
    m_hAccel = NULL;
	memset(statusSizes, 0, sizeof(statusSizes));
	pMainFrame = this;
	lastSave = GET_TICK();
}

MainFrame::~MainFrame() {
    if(trayMenu != NULL) {
        ::DestroyMenu(trayMenu);
    }

    if(Util::OsMajor >= 6) {
        for(uint8_t i = 0; i < 22; i++) {
            if(m_MenuBitmaps[i] != NULL) {
                ::DeleteObject(m_MenuBitmaps[i]);
            }
        }
	}

	ImageList_Destroy(images);
	ImageList_Destroy(largeImages);
	ImageList_Destroy(largeImagesHot);

	WinUtil::uninit();
	ListViewArrows::Uninit();
	CountryFlags::Uninit();
}

DWORD WINAPI MainFrame::stopper(void* p) {
	HWND wnd, wnd2 = NULL;

	while((wnd = ::GetWindow(WinUtil::mdiClient, GW_CHILD)) != NULL) {
		if(wnd == wnd2) {
			Sleep(100);
		} else {
			::PostMessage(wnd, WM_CLOSE, 0, 0);
			wnd2 = wnd;
		}
	}

	::PostMessage(((MainFrame*)p)->m_hWnd, WM_CLOSE, 0, 0);	
	return 0;
}

class ListMatcher : public Thread {
public:
	ListMatcher(StringList files_) : files(files_) {

	}

	int run() {
		for(StringIter i = files.begin(); i != files.end(); ++i) {
			UserPtr u = DirectoryListing::getUserFromFilename(*i);
			if(!u)
				continue;
			DirectoryListing dl(u);
			try {
				dl.loadFile(*i);
				const size_t BUF_SIZE = STRING(MATCHED_FILES).size() + 16;
				AutoArray<char> tmp(BUF_SIZE);
				sprintf_s(tmp, BUF_SIZE, CSTRING(MATCHED_FILES), QueueManager::getInstance()->matchListing(dl));
				LogManager::getInstance()->message(Util::toString(ClientManager::getInstance()->getNicks(u->getCID())) + ": " + string(tmp));
			} catch(const Exception&) {

			}
		}
		delete this;
		return 0;
	}
	StringList files;
};

void MainFrame::onMatchAll() {
	ListMatcher* matcher = new ListMatcher(File::findFiles(Util::getListPath(), "*.xml*"));
	try {
		matcher->start();
	} catch(const ThreadException&) {
		///@todo add error message
		delete matcher;
	}
}

void MainFrame::OnCreate() {
	TimerManager::getInstance()->addListener(this);
	QueueManager::getInstance()->addListener(this);
	LogManager::getInstance()->addListener(this);
	SettingsManager::getInstance()->addListener(this);

	WinUtil::init(m_hWnd);
	ListViewArrows::Init();
	CountryFlags::Init();

	trayMessage = RegisterWindowMessage(_T("TaskbarCreated"));

	TimerManager::getInstance()->start();

	hShutdownIcon = (HICON)::LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDB_SHUTDOWN), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);

	// Get main menu handle
	HMENU m_hMenu = ::GetMenu(m_hWnd);

	// Translate main menu popup strings
	const TCHAR * pMenuPopupStrings[] = { CTSTRING(MENU_FILE), CTSTRING(MENU_VIEW), CTSTRING(MENU_TRANSFERS), CTSTRING(MENU_WINDOW), CTSTRING(MENU_HELP) };
	for(UINT i = 0; i < 5; i++) {
		::ModifyMenu(m_hMenu, i, MF_BYPOSITION, 0, pMenuPopupStrings[i]);
	}

	// Translate main menu strings
	const UINT uiMenuIds[] = { IDC_OPEN_FILE_LIST, IDC_OPEN_OWN_LIST, IDC_MATCH_ALL, IDC_REFRESH_FILE_LIST, 
		IDC_OPEN_DOWNLOADS, ID_FILE_QUICK_CONNECT, IDC_FOLLOW, ID_FILE_RECONNECT, ID_FILE_SETTINGS, 
		ID_APP_EXIT, ID_FILE_CONNECT, IDC_RECENTS, IDC_FAVORITES, IDC_FAVUSERS, 
		ID_FILE_SEARCH, IDC_FILE_ADL_SEARCH, IDC_SEARCH_SPY, IDC_NOTEPAD, IDC_HASH_PROGRESS, 
		IDC_SYSTEM_LOG, ID_VIEW_TOOLBAR, ID_VIEW_STATUS_BAR, ID_VIEW_TRANSFER_VIEW, IDC_QUEUE, 
		IDC_FINISHED, IDC_UPLOAD_QUEUE, IDC_FINISHED_UL, IDC_NET_STATS, ID_WINDOW_CASCADE, 
		ID_WINDOW_TILE_HORZ, ID_WINDOW_TILE_VERT, ID_WINDOW_ARRANGE, ID_WINDOW_MINIMIZE_ALL, ID_WINDOW_RESTORE_ALL, 
		IDC_CLOSE_DISCONNECTED, IDC_CLOSE_DISCONNECTED_PM, IDC_CLOSE_ALL_PM, IDC_CLOSE_ALL_SEARCHS, IDC_CLOSE_ALL_FILELISTS, 
		ID_APP_ABOUT, IDC_HELP_HOMEPAGE, IDC_HELP_DISCUSS
	};
	const TCHAR * pMenuStrings[] = { CTSTRING(MENU_OPEN_FILE_LIST), CTSTRING(MENU_OPEN_OWN_LIST), CTSTRING(MENU_OPEN_MATCH_ALL), CTSTRING(MENU_REFRESH_FILE_LIST), 
		CTSTRING(MENU_OPEN_DOWNLOADS_DIR), CTSTRING(MENU_QUICK_CONNECT), CTSTRING(MENU_FOLLOW_REDIRECT), CTSTRING(MENU_RECONNECT), CTSTRING(MENU_SETTINGS), 
		CTSTRING(MENU_EXIT), CTSTRING(MENU_PUBLIC_HUBS), CTSTRING(MENU_FILE_RECENT_HUBS), CTSTRING(MENU_FAVORITE_HUBS), CTSTRING(MENU_FAVORITE_AND_IGNORED_USERS),
		CTSTRING(MENU_SEARCH), CTSTRING(MENU_ADL_SEARCH), CTSTRING(MENU_SEARCH_SPY), CTSTRING(MENU_NOTEPAD), CTSTRING(MENU_HASH_PROGRESS), 
		CTSTRING(MENU_SYSTEM_LOG), CTSTRING(MENU_TOOLBAR), CTSTRING(MENU_STATUS_BAR), CTSTRING(MENU_TRANSFER_VIEW), CTSTRING(MENU_DOWNLOAD_QUEUE), 
		CTSTRING(FINISHED_DOWNLOADS), CTSTRING(UPLOAD_QUEUE), CTSTRING(FINISHED_UPLOADS), CTSTRING(MENU_NETWORK_STATISTICS), CTSTRING(MENU_CASCADE), 
		CTSTRING(MENU_HORIZONTAL_TILE), CTSTRING(MENU_VERTICAL_TILE), CTSTRING(MENU_ARRANGE), CTSTRING(MENU_MINIMIZE_ALL), CTSTRING(MENU_RESTORE_ALL), 
		CTSTRING(MENU_CLOSE_DISCONNECTED), CTSTRING(CLOSE_DISCONNECTED_PM), CTSTRING(CLOSE_ALL_PM), CTSTRING(CLOSE_ALL_SEARCHS), CTSTRING(CLOSE_ALL_FILELISTS), 
		CTSTRING(MENU_ABOUT), CTSTRING(MENU_HOMEPAGE), CTSTRING(MENU_DISCUSS)
	};
	for(UINT i = 0; i < (sizeof(uiMenuIds) / sizeof(uiMenuIds[0])); i++) {
		::ModifyMenu(m_hMenu, uiMenuIds[i], MF_BYCOMMAND, uiMenuIds[i], pMenuStrings[i]);
	}

	// load command bar images
	images = ImageList_LoadImage(g_hInstance, MAKEINTRESOURCE(IDB_TOOLBAR), 16, 4, CLR_NONE, IMAGE_BITMAP, LR_CREATEDIBSECTION);

    if(Util::OsMajor >= 6) {
        // Create display compatible memory DC
        HDC hDC = ::GetDC(NULL);
        HDC dcMem = ::CreateCompatibleDC(hDC);
        HBITMAP hBitmapSave = (HBITMAP)::GetCurrentObject(dcMem, OBJ_BITMAP);

        for(uint8_t i = 0; i < 22; i++) {
            // Create 32-bit bitmap
            BITMAPINFO bi = { 0 };
            bi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
            bi.bmiHeader.biWidth = 16;
            bi.bmiHeader.biHeight = 16;
            bi.bmiHeader.biPlanes = 1;
            bi.bmiHeader.biBitCount = 32;
            bi.bmiHeader.biCompression = BI_RGB;
            m_MenuBitmaps[i] = ::CreateDIBSection(hDC, &bi, DIB_RGB_COLORS, NULL, NULL, 0);

            // Select bitmap into target DC and draw from image list to it
            if(m_MenuBitmaps[i] != NULL) {
                ::SelectObject(dcMem, m_MenuBitmaps[i]);

                IMAGELISTDRAWPARAMS ildp = { 0 };
                ildp.cbSize = sizeof(IMAGELISTDRAWPARAMS);
                ildp.himl = images;
                ildp.i = i;
                ildp.hdcDst = dcMem;
                ildp.fStyle = ILD_TRANSPARENT;
                ildp.fState = ILS_ALPHA;
                ildp.Frame = 255;
                ::ImageList_DrawIndirect(&ildp);
            }
		}

        ::SelectObject(dcMem, hBitmapSave);
        ::DeleteDC(dcMem);
        ::ReleaseDC(NULL, hDC);
	}

	for(uint8_t i = 0; i < 22; i++) {
		MENUITEMINFO mii = { 0 };
		mii.cbSize = sizeof(MENUITEMINFO);
		if(Util::OsMajor >= 6) {
            mii.fMask = MIIM_BITMAP;
            mii.hbmpItem = m_MenuBitmaps[i];
            ::SetMenuItemInfo(m_hMenu, wMenuBitmapIds[i], FALSE, &mii);
        } else {
			mii.fMask = MIIM_FTYPE | MIIM_DATA;
			mii.fType = MFT_OWNERDRAW;
			mii.dwItemData = (ULONG_PTR)i;
			::SetMenuItemInfo(m_hMenu, wMenuBitmapIds[i], FALSE, &mii);
        }
	}

	m_hWndToolBar = ::CreateWindowEx(0, REBARCLASSNAME, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | RBS_VARHEIGHT |
        RBS_BANDBORDERS | RBS_AUTOSIZE | CCS_NODIVIDER, 0, 0, 100, 100, m_hWnd, NULL, g_hInstance, NULL);

	// Initialize and send the REBARINFO structure
	REBARINFO rbi = { 0 };
	rbi.cbSize = sizeof(REBARINFO);
	rbi.fMask  = 0;
	::SendMessage(m_hWndToolBar, RB_SETBARINFO, 0, (LPARAM)&rbi);

    largeImages = ImageList_LoadImage(g_hInstance, MAKEINTRESOURCE(IDB_TOOLBAR24), 24, 4, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);
    largeImagesHot = ImageList_LoadImage(g_hInstance, MAKEINTRESOURCE(IDB_TOOLBAR24_HOT), 24, 4, CLR_DEFAULT, IMAGE_BITMAP, LR_CREATEDIBSECTION | LR_SHARED);

	hwndToolbar = ::CreateWindowEx(0, TOOLBARCLASSNAME, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | CCS_NODIVIDER |
        CCS_NORESIZE | TBSTYLE_FLAT | TBSTYLE_TOOLTIPS, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, NULL, g_hInstance, NULL);
	::SendMessage(hwndToolbar, TB_SETEXTENDEDSTYLE, 0, TBSTYLE_EX_HIDECLIPPEDBUTTONS);
	::SendMessage(hwndToolbar, TB_SETIMAGELIST, 0, (LPARAM)largeImages);
	::SendMessage(hwndToolbar, TB_SETHOTIMAGELIST, 0, (LPARAM)largeImagesHot);

	createToolbar();
	::SendMessage(hwndToolbar, TB_CHECKBUTTON, IDC_TOOLBAR_AWAY, MAKELPARAM(Util::getAway(), 0));
	::SendMessage(hwndToolbar, TB_CHECKBUTTON, IDC_MUTE_SOUNDS_TRAY, MAKELPARAM(BOOLSETTING(MUTE_SOUND), 0));

    RECT rcTmp = { 0 };
	::SendMessage(hwndToolbar, TB_GETITEMRECT, ((int)::SendMessage(hwndToolbar, TB_BUTTONCOUNT, 0, 0L)) - 1, (LPARAM)&rcTmp);

	REBARBANDINFO rbBand = { 0 };
	rbBand.cbSize = sizeof(REBARBANDINFO);
	rbBand.fMask = RBBIM_CHILD | RBBIM_CHILDSIZE | RBBIM_STYLE | RBBIM_ID | RBBIM_SIZE | RBBIM_IDEALSIZE;
	rbBand.hwndChild = hwndToolbar;
	rbBand.cyMinChild = rcTmp.bottom - rcTmp.top;
	rbBand.fStyle = RBBS_CHILDEDGE | RBBS_USECHEVRON | RBBS_NOGRIPPER;
	rbBand.wID = 0xEB00;
    rbBand.cx = rcTmp.right;
    rbBand.cxIdeal = rcTmp.right;

	::SendMessage(hwndToolbar, TB_GETITEMRECT, 0, (LPARAM)&rcTmp);
	rbBand.cxMinChild = rcTmp.right;

	::SendMessage(m_hWndToolBar, RB_INSERTBAND, (WPARAM)-1, (LPARAM)&rbBand);

    m_hWndStatusBar = CreateWindowEx(0, STATUSCLASSNAME, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | SBARS_SIZEGRIP,
        0, 0, 0, 0, m_hWnd, (HMENU)NULL, g_hInstance, NULL);

	::SendMessage(m_hWndStatusBar, SB_SIMPLE, FALSE, 0L);
	int w[10] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
	::SendMessage(m_hWndStatusBar, SB_SETPARTS, 10, (LPARAM)w);
	statusSizes[0] = WinUtil::getTextWidth(TSTRING(AWAY), m_hWndStatusBar); // for "AWAY" segment

	TOOLINFO ti;
    ToolInfoInit(ti, TTF_SUBCLASS, m_hWndStatusBar, 0, NULL, LPSTR_TEXTCALLBACK);

	hwndLastLines = ::CreateWindowEx(WS_EX_TOPMOST, TOOLTIPS_CLASS, NULL, WS_POPUP | TTS_NOPREFIX | TTS_ALWAYSTIP, CW_USEDEFAULT, CW_USEDEFAULT,
        CW_USEDEFAULT, CW_USEDEFAULT, m_hWndStatusBar, NULL, g_hInstance, NULL);
	::SetWindowPos(hwndLastLines, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
	::SendMessage(hwndLastLines, TTM_ADDTOOL, 0, (LPARAM)&ti);
	::SendMessage(hwndLastLines, TTM_SETDELAYTIME, TTDT_AUTOPOP, MAKELPARAM(30000, 0));

    // Create MDI Client
	CLIENTCREATESTRUCT ccs = { 0 };
	ccs.hWindowMenu = ::GetSubMenu(::GetMenu(m_hWnd), 3);
	ccs.idFirstChild = 0xFF00;

	// Create MDICLIENT window
	WinUtil::mdiClient = ::CreateWindowEx(WS_EX_CLIENTEDGE, _T("MDICLIENT"), NULL, MDIS_ALLCHILDSTYLES | WS_VISIBLE | WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
        0, 0, 0, 0, m_hWnd, NULL, g_hInstance, &ccs);

	// Move it to the top of z-order
	::BringWindowToTop(WinUtil::mdiClient);

	// update to proper size
	UpdateLayout();

	ctrlTab.Create(m_hWnd);
	WinUtil::tabCtrl = &ctrlTab;
	tabsontop = BOOLSETTING(TABS_ON_TOP);

	transferView.Create(m_hWnd);

    m_hSplitWnd = m_hWnd;
    SetSplitterHorCursor();
	SetSplitterPanes(WinUtil::mdiClient, transferView.m_hWnd);
	iProportionalPos = SETTING(TRANSFER_SPLIT_SIZE);
	SetCheck(ID_VIEW_TOOLBAR, true);
	SetCheck(ID_VIEW_STATUS_BAR, true);
	SetCheck(ID_VIEW_TRANSFER_VIEW, true);

	trayMenu = ::CreatePopupMenu();
    ::AppendMenu(trayMenu, MF_STRING, IDC_TRAY_SHOW, CTSTRING(MENU_SHOW));
    ::AppendMenu(trayMenu, MF_STRING, IDC_OPEN_DOWNLOADS, CTSTRING(MENU_OPEN_DOWNLOADS_DIR));
    ::AppendMenu(trayMenu, MF_STRING, ID_FILE_SETTINGS, CTSTRING(MENU_SETTINGS));
	::AppendMenu(trayMenu, MF_SEPARATOR, 0, NULL);
    ::AppendMenu(trayMenu, MF_STRING, IDC_REFRESH_FILE_LIST, CTSTRING(MENU_REFRESH_FILE_LIST));
	::AppendMenu(trayMenu, MF_SEPARATOR, 0, NULL);
    ::AppendMenu(trayMenu, MF_STRING, IDC_ALTERNATE_LIMIT_TRAY, CTSTRING(USE_ALTERNATE_LIMIT_TRAY));
    ::AppendMenu(trayMenu, MF_STRING, IDC_MUTE_SOUNDS_TRAY, CTSTRING(MUTE_SOUNDS_TRAY));
	::AppendMenu(trayMenu, MF_SEPARATOR, 0, NULL);
    ::AppendMenu(trayMenu, MF_STRING, ID_APP_ABOUT, CTSTRING(MENU_ABOUT));
    ::AppendMenu(trayMenu, MF_STRING, ID_APP_EXIT, CTSTRING(MENU_EXIT));

	if(BOOLSETTING(MUTE_SOUND) == true) {
        ::CheckMenuItem(trayMenu, IDC_MUTE_SOUNDS_TRAY, MF_CHECKED | MF_BYCOMMAND);
    }

	c->addListener(this);
	c->downloadFile("http://www.czdc.org/update.xml");

	if(BOOLSETTING(OPEN_SYSTEM_LOG)) ::PostMessage(m_hWnd, WM_COMMAND, IDC_SYSTEM_LOG, 0);
	if(BOOLSETTING(OPEN_PUBLIC)) ::PostMessage(m_hWnd, WM_COMMAND, ID_FILE_CONNECT, 0);
	if(BOOLSETTING(OPEN_FAVORITE_HUBS)) ::PostMessage(m_hWnd, WM_COMMAND, IDC_FAVORITES, 0);
	if(BOOLSETTING(OPEN_USERS)) ::PostMessage(m_hWnd, WM_COMMAND, IDC_FAVUSERS, 0);
	if(BOOLSETTING(OPEN_QUEUE)) ::PostMessage(m_hWnd, WM_COMMAND, IDC_QUEUE, 0);
	if(BOOLSETTING(OPEN_FINISHED_DOWNLOADS)) ::PostMessage(m_hWnd, WM_COMMAND, IDC_FINISHED, 0);
	if(BOOLSETTING(OPEN_FINISHED_UPLOADS)) ::PostMessage(m_hWnd, WM_COMMAND, IDC_FINISHED_UL, 0);
	if(BOOLSETTING(OPEN_SEARCH_SPY)) ::PostMessage(m_hWnd, WM_COMMAND, IDC_SEARCH_SPY, 0);
	if(BOOLSETTING(OPEN_NOTEPAD)) ::PostMessage(m_hWnd, WM_COMMAND, IDC_NOTEPAD, 0);
	if(BOOLSETTING(OPEN_NETWORK_STATISTIC)) ::PostMessage(m_hWnd, WM_COMMAND, IDC_NET_STATS, 0);
	if(BOOLSETTING(OPEN_RECENT_HUBS)) ::PostMessage(m_hWnd, WM_COMMAND, IDC_RECENTS, 0);
	if(BOOLSETTING(OPEN_UPLOAD_QUEUE)) ::PostMessage(m_hWnd, WM_COMMAND, IDC_UPLOAD_QUEUE, 0);

	if(BOOLSETTING(SHOW_STATUSBAR) == false) ::PostMessage(m_hWnd, WM_COMMAND, ID_VIEW_STATUS_BAR, 0);
	if(BOOLSETTING(SHOW_TOOLBAR) == false) ::PostMessage(m_hWnd, WM_COMMAND, ID_VIEW_TOOLBAR, 0);
	if(BOOLSETTING(SHOW_TRANSFERVIEW) == false) ::PostMessage(m_hWnd, WM_COMMAND, ID_VIEW_TRANSFER_VIEW, 0);

	if(!WinUtil::isShift()) {
		::PostMessage(m_hWnd, WM_SPEAKER, AUTO_CONNECT, 0);
	}

	::PostMessage(m_hWnd, WM_SPEAKER, PARSE_COMMAND_LINE, 0);

	File::ensureDirectory(SETTING(LOG_DIRECTORY));

	startSocket();

	normalicon.hIcon = (HICON)::LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDR_MAINFRAME), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
	pmicon.hIcon = (HICON)::LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDR_TRAY_PM), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);

	updateTray(BOOLSETTING(MINIMIZE_TRAY));

	if(SETTING(NICK).empty() == true) {
		::PostMessage(m_hWnd, WM_COMMAND, ID_FILE_SETTINGS, 0);
	}
}

void MainFrame::startSocket() {
	SearchManager::getInstance()->disconnect();
	ConnectionManager::getInstance()->disconnect();

	if(ClientManager::getInstance()->isActive()) {
		try {
			ConnectionManager::getInstance()->listen();
		} catch(const Exception&) {
			::MessageBox(m_hWnd, CTSTRING(TCP_PORT_BUSY), _T(CzDC_VERSION_STRING), MB_ICONSTOP | MB_OK);
		}
		try {
			SearchManager::getInstance()->listen();
		} catch(const Exception&) {
			::MessageBox(m_hWnd, CTSTRING(TCP_PORT_BUSY), _T(CzDC_VERSION_STRING), MB_ICONSTOP | MB_OK);
		}
	}

	startUPnP();
}

void MainFrame::startUPnP() {
	stopUPnP();

	if(BOOLSETTING(SETTINGS_USE_UPNP) == true && SETTING(CONNECTION_TYPE) == SettingsManager::CONNECTION_ACTIVE) {
		UPnP_TCPConnection = new UPnP( Util::getLocalIp(), "TCP", CzDC_VERSION_STRING " Download Port (" + Util::toString(ConnectionManager::getInstance()->getPort()) + " TCP)", ConnectionManager::getInstance()->getPort() );
		UPnP_UDPConnection = new UPnP( Util::getLocalIp(), "UDP", CzDC_VERSION_STRING " Search Port (" + Util::toString(SearchManager::getInstance()->getPort()) + " UDP)", SearchManager::getInstance()->getPort() );

		if ( FAILED(UPnP_UDPConnection->OpenPorts()) || FAILED(UPnP_TCPConnection->OpenPorts()) )
		{
			LogManager::getInstance()->message(STRING(UPNP_FAILED_TO_CREATE_MAPPINGS));
			::MessageBox(m_hWnd, CTSTRING(UPNP_FAILED_TO_CREATE_MAPPINGS), _T(CzDC_VERSION_STRING), MB_OK | MB_ICONWARNING);

			// We failed! thus reset the objects
			delete UPnP_TCPConnection;
			delete UPnP_UDPConnection;
			UPnP_TCPConnection = UPnP_UDPConnection = NULL;
		}
		else
		{
			if(!BOOLSETTING(NO_IP_OVERRIDE)) {
				// now lets configure the external IP (connect to me) address
				string ExternalIP = UPnP_TCPConnection->GetExternalIP();
				if ( !ExternalIP.empty() ) {
					// woohoo, we got the external IP from the UPnP framework
					SettingsManager::getInstance()->set(SettingsManager::EXTERNAL_IP, ExternalIP );
				} else {
					//:-( Looks like we have to rely on the user setting the external IP manually
					// no need to do cleanup here because the mappings work
					LogManager::getInstance()->message(STRING(UPNP_FAILED_TO_GET_EXTERNAL_IP));
					::MessageBox(m_hWnd, CTSTRING(UPNP_FAILED_TO_GET_EXTERNAL_IP), _T(CzDC_VERSION_STRING), MB_OK | MB_ICONWARNING);
				}
			}
		}
	}
}

void MainFrame::stopUPnP() {
	// Just check if the port mapping objects are initialized (NOT NULL)
	if ( UPnP_TCPConnection != NULL )
	{
		if (FAILED(UPnP_TCPConnection->ClosePorts()) )
		{
			LogManager::getInstance()->message(STRING(UPNP_FAILED_TO_REMOVE_MAPPINGS));
		}
		delete UPnP_TCPConnection;
	}
	if ( UPnP_UDPConnection != NULL )
	{
		if (FAILED(UPnP_UDPConnection->ClosePorts()) )
		{
			LogManager::getInstance()->message(STRING(UPNP_FAILED_TO_REMOVE_MAPPINGS));
		}
		delete UPnP_UDPConnection;
	}
	// Not sure this is required (i.e. Objects are checked later in execution)
	// But its better being on the save side :P
	UPnP_TCPConnection = UPnP_UDPConnection = NULL;
}

void MainFrame::createToolbar() {
	while((int)::SendMessage(hwndToolbar, TB_BUTTONCOUNT, 0, 0L) > 0) {
		::SendMessage(hwndToolbar, TB_DELETEBUTTON, 0, 0L);
    }

	::SendMessage(hwndToolbar, TB_BUTTONSTRUCTSIZE, sizeof(TBBUTTON), 0L);

	StringTokenizer<string> t(SETTING(TOOLBAR), ',');
	StringList& l = t.getTokens();

	for(StringList::const_iterator k = l.begin(); k != l.end(); ++k) {
		int i = Util::toInt(*k);		

		TBBUTTON nTB;
		memset(&nTB, 0, sizeof(TBBUTTON));

		if(i < 0 || i > 21) {
			nTB.fsStyle = TBSTYLE_SEP;	
		} else {
			nTB.iBitmap = WinUtil::ToolbarButtons[i].image;
			nTB.idCommand = WinUtil::ToolbarButtons[i].id;
			nTB.fsState = TBSTATE_ENABLED;
			nTB.fsStyle = TBSTYLE_AUTOSIZE | ((WinUtil::ToolbarButtons[i].check == true)? TBSTYLE_CHECK : TBSTYLE_BUTTON);
		}

		::SendMessage(hwndToolbar, TB_ADDBUTTONS, 1, (LPARAM)&nTB);
	}	

	::SendMessage(hwndToolbar, TB_AUTOSIZE, 0, 0L);
}

void MainFrame::onSpeaker(WPARAM wParam, LPARAM lParam) {
	switch(wParam) {
        case DOWNLOAD_LISTING: {
			auto_ptr<DirectoryListInfo> i(reinterpret_cast<DirectoryListInfo*>(lParam));
			DirectoryListingFrame::openWindow(i->file, i->user, i->speed);
    		break;
    	}
        case VIEW_FILE_AND_DELETE: {
    		auto_ptr<tstring> file(reinterpret_cast<tstring*>(lParam));
    		TextFrame::openWindow(*file);
    		File::deleteFile(Text::fromT(*file));
    		break;
	   }
       case STATS: {
			auto_ptr<TStringList> pstr(reinterpret_cast<TStringList*>(lParam));
			const TStringList& str = *pstr;
			if(::IsWindow(m_hWndStatusBar)) {
    			HDC dc = ::GetDC(m_hWndStatusBar);
    			bool u = false;
    			::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)str[0].c_str());
    			for(int i = 1; i < 8; i++) {
    				int w = WinUtil::getTextWidth(str[i], dc);

    				if(statusSizes[i] < w) {
    					statusSizes[i] = w;
    					u = true;
    				}
    				::SendMessage(m_hWndStatusBar, SB_SETTEXT, (i+1 | 0), (LPARAM)str[i].c_str());
    			}
    			::ReleaseDC(m_hWndStatusBar, dc);
    			if(u)
    				UpdateLayout(true);
    		}
    		
    		if(bShutdown) {
    			time_t iSec = GET_TICK() / 1000;
				if (::IsWindow(m_hWndStatusBar)) {
    				if(!isShutdownStatus) {
    					::SendMessage(m_hWndStatusBar, SB_SETICON, 9, (LPARAM)hShutdownIcon);
    					isShutdownStatus = true;
    				}
    				if(DownloadManager::getInstance()->getDownloadCount() > 0) {
    					iCurrentShutdownTime = iSec;
    					::SendMessage(m_hWndStatusBar, SB_SETTEXT, (9 | 0), (LPARAM)_T(""));
    				} else {
                        ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (9 | SBT_POPOUT), 
                            (LPARAM)Text::toT(' ' + Util::toTime(SETTING(SHUTDOWN_TIMEOUT) - (uint32_t)(iSec - iCurrentShutdownTime))).c_str());
    					if (iCurrentShutdownTime + SETTING(SHUTDOWN_TIMEOUT) <= iSec) {
    						bool bDidShutDown = false;
    						bDidShutDown = shutDown();
    						if (bDidShutDown) {
    							// Should we go faster here and force termination?
    							// We "could" do a manual shutdown of this app...
    						} else {
                                ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (0 | 0), (LPARAM)CTSTRING(FAILED_TO_SHUTDOWN));
    							::SendMessage(m_hWndStatusBar, SB_SETTEXT, (9 | 0), (LPARAM)_T(""));
    						}
    						// We better not try again. It WON'T work...
    						bShutdown = false;
    					}
    				}
    			}
    		} else {
				if (::IsWindow(m_hWndStatusBar)) {
    				if(isShutdownStatus) {
                        ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (9 | 0), (LPARAM)_T(""));
                        ::SendMessage(m_hWndStatusBar, SB_SETICON, 9, (LPARAM)NULL);
    					isShutdownStatus = false;
    				}
    			}
    		}
    		break;
	   }
       case AUTO_CONNECT:
            autoConnect(FavoriteManager::getInstance()->getFavoriteHubs());
            break;
       case PARSE_COMMAND_LINE:
            parseCommandLine(GetCommandLine());
            break;
       case STATUS_MESSAGE: {
    		auto_ptr<pair<time_t, tstring> > msg((pair<time_t, tstring>*)lParam);
			if(::IsWindow(m_hWndStatusBar)) {
    			tstring line = Text::toT("[" + Util::getShortTimeString(msg->first) + "] ") + msg->second;

    			::SendMessage(m_hWndStatusBar, SB_SETTEXT, (0 | 0), (LPARAM)line.c_str());
    			while(lastLinesList.size() + 1 > MAX_CLIENT_LINES)
    				lastLinesList.erase(lastLinesList.begin());
    			if(_tcschr(line.c_str(), _T('\r')) == NULL) {
    				lastLinesList.push_back(line);
    			} else {
    				lastLinesList.push_back(line.substr(0, line.find(_T('\r'))));
    			}
    		}
    		break;
	   } 
       case SET_NORMAL_TRAY_ICON:
    		if(bIsPM && bTrayIcon == true) {
    			NOTIFYICONDATA nid;
    			nid.cbSize = sizeof(NOTIFYICONDATA);
    			nid.hWnd = m_hWnd;
    			nid.uID = 0;
    			nid.uFlags = NIF_ICON;
    			nid.hIcon = normalicon.hIcon;
    			::Shell_NotifyIcon(NIM_MODIFY, &nid);
    			bIsPM = false;
    		}
    		break;
        case SET_PM_TRAY_ICON:
    		if(bIsPM == false && (!WinUtil::isAppActive || bAppMinimized) && bTrayIcon == true) {
    			NOTIFYICONDATA nid;
    			nid.cbSize = sizeof(NOTIFYICONDATA);
    			nid.hWnd = m_hWnd;
    			nid.uID = 0;
    			nid.uFlags = NIF_ICON;
    			nid.hIcon = pmicon.hIcon;
    			::Shell_NotifyIcon(NIM_MODIFY, &nid);
    			bIsPM = true;
    		}
    		break;
        default:
            break;
    }
}

void MainFrame::parseCommandLine(const tstring& cmdLine) {
	string::size_type i = 0;
	string::size_type j;

	if((j = cmdLine.find(_T("dchub://"), i)) != string::npos) {
		WinUtil::parseDchubUrl(cmdLine.substr(j));
	}
	if( (j = cmdLine.find(_T("magnet:?"), i)) != string::npos) {
		WinUtil::parseMagnetUri(cmdLine.substr(j));
	}
}

void MainFrame::onCopyData(LPARAM lParam) {
	tstring cmdLine = (LPCTSTR)(((COPYDATASTRUCT *)lParam)->lpData);
	parseCommandLine(Text::toT(WinUtil::getAppName() + " ") + cmdLine);
}

void MainFrame::OnAppAbout() {
	AboutDlg dlg;
	dlg.DoModal(m_hWnd);
}

void MainFrame::onOpenWindows(WORD wID) {
	switch(wID) {
		case ID_FILE_SEARCH:
            SearchFrame::openWindow();
            break;
		case ID_FILE_CONNECT:
            PublicHubsFrame::openWindow();
            break;
		case IDC_FAVORITES:
            FavoriteHubsFrame::openWindow();
            break;
		case IDC_FAVUSERS:
            UsersFrame::openWindow();
            break;
		case IDC_NOTEPAD:
            NotepadFrame::openWindow();
            break;
		case IDC_QUEUE:
            QueueFrame::openWindow();
            break;
		case IDC_SEARCH_SPY:
            SpyFrame::openWindow();
            break;
		case IDC_FILE_ADL_SEARCH:
            ADLSearchFrame::openWindow();
            break;
		case IDC_NET_STATS:
            StatsFrame::openWindow();
            break;
		case IDC_FINISHED:
            FinishedFrame::openWindow();
            break;
		case IDC_FINISHED_UL:
            FinishedULFrame::openWindow();
            break;
		case IDC_UPLOAD_QUEUE:
            WaitingUsersFrame::openWindow();
            break;
		case IDC_RECENTS:
            RecentHubsFrame::openWindow();
            break;
		case IDC_DEBUG_WINDOW:
            DebugFrame::openWindow();
            break;
		case IDC_SYSTEM_LOG:
            SystemFrame::openWindow();
            break;
		default:
            break;
	}
}

void MainFrame::OnFileSettings() {
	PropertiesDlg dlg(m_hWnd, SettingsManager::getInstance());

	unsigned short lastPort = (unsigned short)SETTING(TCP_PORT);
	unsigned short lastUDP = (unsigned short)SETTING(UDP_PORT);
	int lastConn = SETTING(CONNECTION_TYPE);

	bool lastSortFavUsersFirst = BOOLSETTING(SORT_FAVUSERS_FIRST);

	if(dlg.DoModal(m_hWnd) == IDOK) {
		SettingsManager::getInstance()->save();
 		if(missedAutoConnect && !SETTING(NICK).empty()) {
			::PostMessage(m_hWnd, WM_SPEAKER, AUTO_CONNECT, 0);
 		}
		if(SETTING(CONNECTION_TYPE) != lastConn || SETTING(TCP_PORT) != lastPort || SETTING(UDP_PORT) != lastUDP) {
			startSocket();
		}
		ClientManager::getInstance()->infoUpdated(false);

		if(BOOLSETTING(SORT_FAVUSERS_FIRST) != lastSortFavUsersFirst)
			HubFrame::resortUsers();

		if(BOOLSETTING(URL_HANDLER)) {
			WinUtil::registerDchubHandler();
			WinUtil::urlDcRegistered = true;
		} else if(WinUtil::urlDcRegistered) {
			WinUtil::unRegisterDchubHandler();
			WinUtil::urlDcRegistered = false;
		}
		if(BOOLSETTING(MAGNET_REGISTER)) {
			WinUtil::registerMagnetHandler();
			WinUtil::urlMagnetRegistered = true;
		} else if(WinUtil::urlMagnetRegistered) {
			WinUtil::unRegisterMagnetHandler();
			WinUtil::urlMagnetRegistered = false;
		}

		if(Util::getAway() && (BOOL)::SendMessage(hwndToolbar, TB_ISBUTTONCHECKED, IDC_TOOLBAR_AWAY, 0L) == false) {
			::SendMessage(hwndToolbar, TB_CHECKBUTTON, IDC_TOOLBAR_AWAY, MAKELPARAM(true, 0));
		}
		if(getShutDown() && (BOOL)::SendMessage(hwndToolbar, TB_ISBUTTONCHECKED, IDC_SHUTDOWN, 0L) == false) {
            ::SendMessage(hwndToolbar, TB_CHECKBUTTON, IDC_SHUTDOWN, MAKELPARAM(true, 0));
		}
		if(BOOLSETTING(MUTE_SOUND) && (BOOL)::SendMessage(hwndToolbar, TB_ISBUTTONCHECKED, IDC_MUTE_SOUNDS_TRAY, 0L) == false) {
            ::SendMessage(hwndToolbar, TB_CHECKBUTTON, IDC_MUTE_SOUNDS_TRAY, MAKELPARAM(true, 0));
		}
		updateTray(BOOLSETTING(MINIMIZE_TRAY));
		if(tabsontop != BOOLSETTING(TABS_ON_TOP)) {
			tabsontop = BOOLSETTING(TABS_ON_TOP);
			UpdateLayout();
		}
	}
}

void MainFrame::on(HttpConnectionListener::Complete, HttpConnection* /*aConn*/, const string&) throw() {
	try {
		SimpleXML xml;
		xml.fromXML(versionInfo);
		xml.stepIn();

		string url;
		if(xml.findChild("URL")) {
			url = xml.getChildData();
		}
		xml.resetCurrentChild();
		if(xml.findChild("Version")) {
			int version = atoi(xml.getChildAttrib("ActualVersion").c_str());
			int clientversion = (int)(VERSIONFLOAT * 100000);
			int subversion = (char)xml.getChildAttrib("SubVersion")[0];
			int clientsubversion = (char)CZDCVERSIONSTRING[0];
			if(BOOLSETTING(GET_UPDATE_INFO) && (version > clientversion || (version == clientversion &&
				subversion > clientsubversion))) {
				xml.resetCurrentChild();
				xml.resetCurrentChild(); 
				if(xml.findChild("Title")) {
					const string& title = xml.getChildData();
					xml.resetCurrentChild();
					if(xml.findChild("Message")) {
						if(url.empty()) {
							const string& msg = xml.getChildData();
							::MessageBox(m_hWnd, Text::toT(msg).c_str(), Text::toT(title).c_str(), MB_OK);
						} else {
							string msg = xml.getChildData() + "\r\n" + STRING(OPEN_DOWNLOAD_PAGE);
							if(::MessageBox(m_hWnd, Text::toT(msg).c_str(), Text::toT(title).c_str(), MB_YESNO) == IDYES) {
								WinUtil::openLink(Text::toT(url));
							}
						}
					}
				}
			} else {
				xml.resetCurrentChild();
				if(xml.findChild("VeryOldVersion")) {
					int version = atoi(xml.getChildAttrib("Version").c_str());
					int clientversion = (int)(VERSIONFLOAT * 100000);
					int subversion = (char)xml.getChildAttrib("SubVersion")[0];
					int clientsubversion = (char)CZDCVERSIONSTRING[0];
					if(version >= clientversion && subversion >= clientsubversion) {
						string msg = xml.getChildAttrib("Message", "Your version of CzDC is very old.");
						::MessageBox(m_hWnd, Text::toT(msg + "\r\nPlease get a new one at " + url).c_str(), _T("Old CzDC version"), MB_OK);
						oldshutdown = true;
						::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
					}
				}
			}

			xml.resetCurrentChild();
			if(xml.findChild("BadVersions")) {
				xml.stepIn();
				while(xml.findChild("BadVersion")) {
					int version = atoi(xml.getChildAttrib("Version").c_str());
					int clientversion = (int)(VERSIONFLOAT * 100000);
					int subversion = (char)xml.getChildAttrib("SubVersion")[0];
					int clientsubversion = (char)CZDCVERSIONSTRING[0];
					if(version == clientversion && subversion == clientsubversion) {
						string msg = xml.getChildAttrib("Message", "Your version of CzDC contains a serious bug that affects all users of the DC network or the security of your computer.");
						::MessageBox(m_hWnd, Text::toT(msg + "\r\nPlease get a new one at " + url).c_str(), _T("Bad CzDC version"), MB_OK | MB_ICONEXCLAMATION);
						oldshutdown = true;
						::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
					}
				}
				xml.stepOut();
			}
		}
		xml.stepOut();
	} catch (const Exception&) {
		// ...
	}
}

void MainFrame::onGetToolTip(int idCtrl, LPNMHDR pnmh) {
	LPNMTTDISPINFO pDispInfo = (LPNMTTDISPINFO)pnmh;
	pDispInfo->szText[0] = 0;

	if(idCtrl != 0 && !(pDispInfo->uFlags & TTF_IDISHWND)) {
		for(int i = 0; WinUtil::ToolbarButtons[i].id != 0; i++) {
			if(WinUtil::ToolbarButtons[i].id == idCtrl) {
                _tcscpy_s(pDispInfo->szText, 79, CTSTRING_I((ResourceManager::Strings)WinUtil::ToolbarButtons[i].tooltip));
                pDispInfo->uFlags |= TTF_DI_SETITEM;
				break;
			}
		}
	} else { // if we're really in the status bar, this should be detected intelligently
		lastLines.clear();
		for(TStringIter i = lastLinesList.begin(); i != lastLinesList.end(); ++i) {
			lastLines += *i;
			lastLines += _T("\r\n");
		}
		if(lastLines.size() > 2) {
			lastLines.erase(lastLines.size() - 2);
		}
		pDispInfo->lpszText = (LPTSTR)lastLines.c_str();
	}
}

void MainFrame::autoConnect(const FavoriteHubEntry::List& fl) {
 	missedAutoConnect = false;
	for(FavoriteHubEntry::List::const_iterator i = fl.begin(); i != fl.end(); ++i) {
		FavoriteHubEntry* entry = *i;
		if(entry->getConnect()) {
			if(!entry->getNick().empty() || !SETTING(NICK).empty()) {
				RecentHubEntry r;
				r.setName(entry->getName());
				r.setDescription(entry->getDescription());
				r.setUsers("*");
				r.setShared("*");
				r.setServer(entry->getServer());
				FavoriteManager::getInstance()->addRecent(r);
				HubFrame::openWindow(Text::toT(entry->getServer()), 
					entry->getWindowPosX(), entry->getWindowPosY(), entry->getWindowSizeX(), entry->getWindowSizeY(), 
					entry->getWindowType(), entry->getChatUserSplit(), entry->getUserListState(), entry->getColumsOrder(), 
                    entry->getColumsWidth(), entry->getColumsVisible());
			} else
				missedAutoConnect = true;
		}
	}
}

void MainFrame::updateTray(bool add /* = true */) {
	if(add) {
		if(bTrayIcon == false) {
			NOTIFYICONDATA nid;
			nid.cbSize = sizeof(NOTIFYICONDATA);
			nid.hWnd = m_hWnd;
			nid.uID = 0;
			nid.uFlags = NIF_ICON | NIF_TIP | NIF_MESSAGE;
			nid.uCallbackMessage = WM_APP + 242;
			nid.hIcon = normalicon.hIcon;
			_tcscpy_s(nid.szTip, 64, _T(CzDC_VERSION_STRING));
			lastMove = GET_TICK() - 1000;
			::Shell_NotifyIcon(NIM_ADD, &nid);
			bTrayIcon = true;
		}
	} else {
		if(bTrayIcon) {
			NOTIFYICONDATA nid;
			nid.cbSize = sizeof(NOTIFYICONDATA);
			nid.hWnd = m_hWnd;
			nid.uID = 0;
			nid.uFlags = 0;
			::Shell_NotifyIcon(NIM_DELETE, &nid);
			::ShowWindow(m_hWnd, SW_SHOW);
			bTrayIcon = false;
		}
	}
}

void MainFrame::onSize(WPARAM wParam) {
	if(wParam == SIZE_MINIMIZED) {
		if(BOOLSETTING(AUTO_AWAY)) {
			if(bAppMinimized == false) {
				if(Util::getAway() == true) {
					awaybyminimize = false;
				} else {
					awaybyminimize = true;
					Util::setAway(true);
					WinUtil::changeToolbarAway(true);
					ClientManager::getInstance()->infoUpdated(true);
				}
            }
		}

		if(BOOLSETTING(MINIMIZE_TRAY) != WinUtil::isShift()) {
			updateTray(true);
			::ShowWindow(m_hWnd, SW_HIDE);
		} else {
			updateTray(false);
		}

		bAppMinimized = true;
		maximized = ::IsZoomed(m_hWnd) > 0;

        return;
	} else if( (wParam == SIZE_RESTORED || wParam == SIZE_MAXIMIZED) ) {
		if(BOOLSETTING(AUTO_AWAY)) {
			if(awaybyminimize == true) {
				awaybyminimize = false;
				Util::setAway(false);
				WinUtil::changeToolbarAway(false);
				ClientManager::getInstance()->infoUpdated(true);
			}
		}

		setNormalTrayIcon();
		bAppMinimized = false;
	}

    UpdateLayout();
}

void MainFrame::onEndSession() {
	if(c != NULL) {
		c->removeListener(this);
		delete c;
		c = NULL;
	}

	WINDOWPLACEMENT wp;
	wp.length = sizeof(wp);
	::GetWindowPlacement(m_hWnd, &wp);

	RECT rc;
	::GetWindowRect(m_hWnd, &rc);

	if(wp.showCmd == SW_SHOW || wp.showCmd == SW_SHOWNORMAL) {
		SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_POS_X, rc.left);
		SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_POS_Y, rc.top);
		SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_SIZE_X, rc.right - rc.left);
		SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_SIZE_Y, rc.bottom - rc.top);
	}

	if(wp.showCmd == SW_SHOWNORMAL || wp.showCmd == SW_SHOW || wp.showCmd == SW_SHOWMAXIMIZED || wp.showCmd == SW_MAXIMIZE) {
		SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_STATE, (int)wp.showCmd);
    }

	QueueManager::getInstance()->saveQueue();
	SettingsManager::getInstance()->save();
}

bool MainFrame::OnClose() {
	if(c != NULL) {
		c->removeListener(this);
		delete c;
		c = NULL;
	}

	if(closing == false) {
		if(oldshutdown || BOOLSETTING(CONFIRM_EXIT) == false ||
            ::MessageBox(m_hWnd, CTSTRING(REALLY_EXIT), _T(CzDC_VERSION_STRING), MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2) == IDYES) {
			updateTray(false);
			string tmp1;
			string tmp2;

			WINDOWPLACEMENT wp;
			wp.length = sizeof(wp);
			::GetWindowPlacement(m_hWnd, &wp);

			RECT rc;
			::GetWindowRect(m_hWnd, &rc);
			if(BOOLSETTING(SHOW_TRANSFERVIEW)) {
				SettingsManager::getInstance()->set(SettingsManager::TRANSFER_SPLIT_SIZE, iProportionalPos);
			}
			if(wp.showCmd == SW_SHOW || wp.showCmd == SW_SHOWNORMAL) {
				SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_POS_X, rc.left);
				SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_POS_Y, rc.top);
				SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_SIZE_X, rc.right - rc.left);
				SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_SIZE_Y, rc.bottom - rc.top);
			}
			if(wp.showCmd == SW_SHOWNORMAL || wp.showCmd == SW_SHOW || wp.showCmd == SW_SHOWMAXIMIZED || wp.showCmd == SW_MAXIMIZE)
				SettingsManager::getInstance()->set(SettingsManager::MAIN_WINDOW_STATE, (int)wp.showCmd);

			::ShowWindow(m_hWnd, SW_HIDE);
			transferView.prepareClose();

			TimerManager::getInstance()->removeListener(this);
			QueueManager::getInstance()->removeListener(this);
			LogManager::getInstance()->removeListener(this);
			SettingsManager::getInstance()->removeListener(this);
			SearchManager::getInstance()->disconnect();
			ConnectionManager::getInstance()->disconnect();

			stopUPnP();

			DWORD id;
			stopperThread = CreateThread(NULL, 0, stopper, this, 0, &id);
			closing = true;
		}
		return true;
	} else {
		// This should end immediately, as it only should be the stopper that sends another WM_CLOSE
		WaitForSingleObject(stopperThread, 60*1000);
		CloseHandle(stopperThread);
		stopperThread = NULL;
		DestroyIcon(normalicon.hIcon);
		DestroyIcon(hShutdownIcon);
		DestroyIcon(pmicon.hIcon);
		return false;
	}
}

void MainFrame::onLink(WORD wID) {
	switch(wID) {
        case IDC_HELP_HOMEPAGE:
            WinUtil::openLink(_T("http://www.czdc.org"));
            break;
        case IDC_HELP_DISCUSS:
            WinUtil::openLink(_T("http://www.czdc.org/forum"));
            break;
        default:
            break;
	}
}

void MainFrame::UpdateLayout(bool bResizeBars /* = true */) {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);

	// resize toolbar
	if(m_hWndToolBar != NULL && ((DWORD)::GetWindowLong(m_hWndToolBar, GWL_STYLE) & WS_VISIBLE)) {
		if(bResizeBars) {
			::SendMessage(m_hWndToolBar, WM_SIZE, 0, 0);
			::InvalidateRect(m_hWndToolBar, NULL, FALSE);
		}

		RECT rectTB = { 0 };
		::GetWindowRect(m_hWndToolBar, &rectTB);
		rect.top += rectTB.bottom - rectTB.top;
	}

	// resize status bar
	if(m_hWndStatusBar != NULL && ((DWORD)::GetWindowLong(m_hWndStatusBar, GWL_STYLE) & WS_VISIBLE)) {
		if(bResizeBars) {
			::SendMessage(m_hWndStatusBar, WM_SIZE, 0, 0);
        }
		RECT rectSB = { 0 };
		::GetWindowRect(m_hWndStatusBar, &rectSB);
		rect.bottom -= rectSB.bottom - rectSB.top;
	}
	
	if(::IsWindow(m_hWndStatusBar) && ::IsWindow(hwndLastLines)) {
		RECT sr;
		int w[10];
		::GetClientRect(m_hWndStatusBar, &sr);
		w[9] = sr.right - 16;
		w[8] = w[9] - 60;
#define setw(x) w[x] = max(w[x+1] - statusSizes[x], 0)
		setw(7); setw(6); setw(5); setw(4); setw(3); setw(2); setw(1); setw(0);

        ::SendMessage(m_hWndStatusBar, SB_SETPARTS, 10, (LPARAM)w);
		::SendMessage(hwndLastLines, TTM_SETMAXTIPWIDTH, 0, w[0]);
	}
	RECT rc = rect;
	if(tabsontop == false)
		rc.top = rc.bottom - ctrlTab.getHeight();
	else
		rc.bottom = rc.top + ctrlTab.getHeight();
		
	if(::IsWindow(ctrlTab.m_hWnd)) {
		::MoveWindow(ctrlTab.m_hWnd, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
	}
	
	RECT rc2 = rect;
	if(tabsontop == false)
		rc2.bottom = rc.top;
	else
		rc2.top = rc.bottom;
		
	SetSplitterRect(&rc2);
}

static const TCHAR types[] = _T("File Lists\0*.DcLst;*.xml.bz2\0All Files\0*.*\0");

void MainFrame::onOpenFileList() {
	tstring file;
	if(WinUtil::browseFile(file, m_hWnd, false, Text::toT(Util::getListPath()), types)) {
		UserPtr u = DirectoryListing::getUserFromFilename(Text::fromT(file));
		if(u) {
			DirectoryListingFrame::openWindow(file, u, 0);
		} else {
			::MessageBox(m_hWnd, CTSTRING(INVALID_LISTNAME), _T(CzDC_VERSION_STRING), MB_OK);
		}
	}
}

void MainFrame::onOpenOwnList() {
	if(ShareManager::getInstance()->getOwnListFile().empty() == false){
		DirectoryListingFrame::openWindow(Text::toT(ShareManager::getInstance()->getOwnListFile()), ClientManager::getInstance()->getMe(), 0);
	}
}

void MainFrame::onRefreshFileList() {
	ShareManager::getInstance()->setDirty();
	ShareManager::getInstance()->refresh(true);
}

void MainFrame::onTrayIcon(LPARAM lParam) {
	switch(lParam) {
        case WM_LBUTTONUP: {
    		if(bAppMinimized) {
				::ShowWindow(m_hWnd, SW_SHOW);
				::ShowWindow(m_hWnd, maximized ? SW_MAXIMIZE : SW_RESTORE);
    		} else {
    			SetForegroundWindow(m_hWnd);
    		}
    		break;
        }
    	case WM_MOUSEMOVE: {
            if((lastMove + 1000) < GET_TICK()) {
        		NOTIFYICONDATA nid;
        		nid.cbSize = sizeof(NOTIFYICONDATA);
        		nid.hWnd = m_hWnd;
        		nid.uID = 0;
        		nid.uFlags = NIF_TIP;
        		_tcsncpy_s(nid.szTip, Text::toT("D: " + Util::formatBytes(DownloadManager::getInstance()->getRunningAverage()) + "/s (" + 
        			Util::toString((uint32_t)DownloadManager::getInstance()->getTotalDownloadCount()) + ")\r\nU: " +
        			Util::formatBytes(UploadManager::getInstance()->getRunningAverage()) + "/s (" + 
        			Util::toString((uint32_t)UploadManager::getInstance()->getUploadCount()) + ")"
        			+ "\r\nUptime: " + Util::formatSeconds(Util::getUptime())
        			).c_str(), 64);
        		nid.szTip[63] = NULL;
        		::Shell_NotifyIcon(NIM_MODIFY, &nid);
        		lastMove = GET_TICK();
            }
            break;
    	} 
        case WM_RBUTTONUP: {
            DWORD dwPoint = GetMessagePos();
    		POINT pt = { (short)LOWORD(dwPoint), (short)HIWORD(dwPoint) };
    		SetForegroundWindow(m_hWnd);
    		::TrackPopupMenu(trayMenu, TPM_RIGHTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, 0, m_hWnd, NULL);
			::PostMessage(m_hWnd, WM_NULL, 0, 0);
    		break;
    	}
    	default:
            break;
    }
}

void MainFrame::OnViewToolBar() {
	static bool bVisible = TRUE;	// initially visible
	bVisible = !bVisible;
	int nBandIndex = (int)::SendMessage(m_hWndToolBar, RB_IDTOINDEX, 0xEB00, 0L);
	::SendMessage(m_hWndToolBar, RB_SHOWBAND, nBandIndex, bVisible);
	SetCheck(ID_VIEW_TOOLBAR, bVisible);
	UpdateLayout();
	SettingsManager::getInstance()->set(SettingsManager::SHOW_TOOLBAR, bVisible);
}

void MainFrame::OnViewStatusBar() {
	bool bVisible = !::IsWindowVisible(m_hWndStatusBar);
	::ShowWindow(m_hWndStatusBar, bVisible ? SW_SHOWNOACTIVATE : SW_HIDE);
	SetCheck(ID_VIEW_STATUS_BAR, bVisible);
	UpdateLayout();
	SettingsManager::getInstance()->set(SettingsManager::SHOW_STATUSBAR, bVisible);
}

void MainFrame::OnViewTransferView() {
	bool bVisible = !::IsWindowVisible(transferView.m_hWnd);
	if(bVisible == false) {
		if(i8SinglePane == SPLIT_PANE_NONE)
			SetSinglePaneMode(SPLIT_PANE_LEFT_OR_TOP);
	} else {
		if(i8SinglePane != SPLIT_PANE_NONE)
			SetSinglePaneMode(SPLIT_PANE_NONE);
	}
	SetCheck(ID_VIEW_TRANSFER_VIEW, bVisible);
	UpdateLayout();
	SettingsManager::getInstance()->set(SettingsManager::SHOW_TRANSFERVIEW, bVisible);
}

void MainFrame::onQuickConnect() {
	LineDlg dlg;
	dlg.description = TSTRING(HUB_ADDRESS);
	dlg.title = TSTRING(QUICK_CONNECT);
	if(dlg.DoModal(m_hWnd) == IDOK){
		if(SETTING(NICK).empty() == true) {
			return;
        }

		tstring tmp = dlg.line;
		// Strip out all the spaces
		string::size_type i;
		while((i = tmp.find(' ')) != string::npos) {
			tmp.erase(i, 1);
        }

		RecentHubEntry r;
		r.setName("*");
		r.setDescription("*");
		r.setUsers("*");
		r.setShared("*");
		r.setServer(Text::fromT(tmp));
		FavoriteManager::getInstance()->addRecent(r);
		HubFrame::openWindow(tmp);
	}
}

void MainFrame::on(TimerManagerListener::Second, uint64_t aTick) throw() {
	Util::increaseUptime();
	int64_t diff = (int64_t)((lastUpdate == 0) ? aTick - 1000 : aTick - lastUpdate);
	uint64_t updiff = Socket::getTotalUp() - lastUp;
	uint64_t downdiff = Socket::getTotalDown() - lastDown;

	// Limitery sem a tam, vsude kam se podivam :o)
	if(BOOLSETTING(THROTTLE_ENABLE) || alternatelimitingfromtray) {
		if( SETTING(MAX_UPLOAD_SPEED_LIMIT_NORMAL) > 0) {
			if( SETTING(MAX_UPLOAD_SPEED_LIMIT_NORMAL) < ((2 * Util::getSlots()) + 3) ) {
				SettingsManager::getInstance()->set(SettingsManager::MAX_UPLOAD_SPEED_LIMIT_NORMAL, ((2 * Util::getSlots()) + 3) );
			}
			if ( (SETTING(MAX_DOWNLOAD_SPEED_LIMIT_NORMAL) > ( SETTING(MAX_UPLOAD_SPEED_LIMIT_NORMAL) * 7)) || ( SETTING(MAX_DOWNLOAD_SPEED_LIMIT_NORMAL) == 0) ) {
				SettingsManager::getInstance()->set(SettingsManager::MAX_DOWNLOAD_SPEED_LIMIT_NORMAL, (SETTING(MAX_UPLOAD_SPEED_LIMIT_NORMAL)*7) );
			}
		}
	
		if( SETTING(MAX_UPLOAD_SPEED_LIMIT_TIME) > 0) {
			if( SETTING(MAX_UPLOAD_SPEED_LIMIT_TIME) < ((2 * Util::getSlots()) + 3) ) {
				SettingsManager::getInstance()->set(SettingsManager::MAX_UPLOAD_SPEED_LIMIT_TIME, ((2 * Util::getSlots()) + 3) );
			}
			if ( (SETTING(MAX_DOWNLOAD_SPEED_LIMIT_TIME) > ( SETTING(MAX_UPLOAD_SPEED_LIMIT_TIME) * 7)) || ( SETTING(MAX_DOWNLOAD_SPEED_LIMIT_TIME) == 0) ) {
				SettingsManager::getInstance()->set(SettingsManager::MAX_DOWNLOAD_SPEED_LIMIT_TIME, (SETTING(MAX_UPLOAD_SPEED_LIMIT_TIME)*7) );
			}
		}
	
		time_t currentTime; tm _tm;
		time(&currentTime); localtime_s(&_tm, &currentTime);
		int currentHour = _tm.tm_hour;

		if((SETTING(TIME_DEPENDENT_THROTTLE) &&
			((SETTING(BANDWIDTH_LIMIT_START) < SETTING(BANDWIDTH_LIMIT_END) &&
				currentHour >= SETTING(BANDWIDTH_LIMIT_START) && currentHour < SETTING(BANDWIDTH_LIMIT_END)) ||
			(SETTING(BANDWIDTH_LIMIT_START) > SETTING(BANDWIDTH_LIMIT_END) &&
				(currentHour >= SETTING(BANDWIDTH_LIMIT_START) || currentHour < SETTING(BANDWIDTH_LIMIT_END))))) ||
			alternatelimitingfromtray) {
			SettingsManager::getInstance()->set(SettingsManager::MAX_UPLOAD_SPEED_LIMIT, SETTING(MAX_UPLOAD_SPEED_LIMIT_TIME));
			SettingsManager::getInstance()->set(SettingsManager::MAX_DOWNLOAD_SPEED_LIMIT, SETTING(MAX_DOWNLOAD_SPEED_LIMIT_TIME));
			if(SETTING(ALTERNATE_UPLOAD_SLOTS) > 0)
				Util::setSlots(SETTING(ALTERNATE_UPLOAD_SLOTS));
		} else {
			SettingsManager::getInstance()->set(SettingsManager::MAX_UPLOAD_SPEED_LIMIT, SETTING(MAX_UPLOAD_SPEED_LIMIT_NORMAL));
			SettingsManager::getInstance()->set(SettingsManager::MAX_DOWNLOAD_SPEED_LIMIT, SETTING(MAX_DOWNLOAD_SPEED_LIMIT_NORMAL));
			Util::setSlots(SETTING(SLOTS));
		}
	} else {
		SettingsManager::getInstance()->set(SettingsManager::MAX_UPLOAD_SPEED_LIMIT, 0);
		SettingsManager::getInstance()->set(SettingsManager::MAX_DOWNLOAD_SPEED_LIMIT, 0);
	}		

	TStringList* str = new TStringList();
	str->push_back(Util::getAway() ? TSTRING(AWAY) : _T(""));
	str->push_back(Text::toT(STRING(SHARED) + ": " + Util::formatBytes(ShareManager::getInstance()->getShareSizeString())));
	str->push_back(Text::toT("H: " + Client::getCounts()));
	str->push_back(Text::toT(STRING(SLOTS) + ": " + Util::toString(UploadManager::getInstance()->getFreeSlots()) + '/' + Util::toString(Util::getSlots()) + " (" + Util::toString(UploadManager::getInstance()->getFreeExtraSlots()) + '/' + Util::toString(SETTING(EXTRA_SLOTS)) + ")"));
	str->push_back(Text::toT("D: " + Util::formatBytes(Socket::getTotalDown())));
	str->push_back(Text::toT("U: " + Util::formatBytes(Socket::getTotalUp())));
	str->push_back(Text::toT("D: [" + Util::toString((uint32_t)DownloadManager::getInstance()->getDownloadCount()) + "/" + Util::toString(DownloadManager::getInstance()->getFileListDownloads()) + "][" + (SETTING(MAX_DOWNLOAD_SPEED_LIMIT) == 0 ? string("N") : Util::toString((int)SETTING(MAX_DOWNLOAD_SPEED_LIMIT)) + "k") + "] " + Util::formatBytes(downdiff*1000I64/diff) + "/s"));
	str->push_back(Text::toT("U: [" + Util::toString((uint32_t)(UploadManager::getInstance()->getUploadCount()-UploadManager::getInstance()->getExtra())) + "/" + Util::toString(UploadManager::getInstance()->getExtra()) + "][" + (SETTING(MAX_UPLOAD_SPEED_LIMIT) == 0 ? string("N") : Util::toString((int)SETTING(MAX_UPLOAD_SPEED_LIMIT)) + "k") + "] " + Util::formatBytes(updiff*1000I64/diff) + "/s"));
	::PostMessage(m_hWnd, WM_SPEAKER, STATS, (LPARAM)str);
	SettingsManager::getInstance()->set(SettingsManager::TOTAL_UPLOAD, SETTING(TOTAL_UPLOAD) + updiff);
	SettingsManager::getInstance()->set(SettingsManager::TOTAL_DOWNLOAD, SETTING(TOTAL_DOWNLOAD) + downdiff);
	SettingsManager::getInstance()->set(SettingsManager::TOTAL_UPTIME, SETTING(TOTAL_UPTIME) + 1);
	lastUpdate = aTick;
	lastUp = Socket::getTotalUp();
	lastDown = Socket::getTotalDown();
}

void MainFrame::on(HttpConnectionListener::Data, HttpConnection* /*conn*/, const uint8_t* buf, size_t len) throw() {
	versionInfo += string((const char*)buf, len);
}

void MainFrame::on(QueueManagerListener::Finished, QueueItem* qi, int64_t speed) throw() {
	if(qi->isSet(QueueItem::FLAG_CLIENT_VIEW)) {
		if(qi->isSet(QueueItem::FLAG_USER_LIST)) {
			// This is a file listing, show it...
			DirectoryListInfo* i = new DirectoryListInfo(qi->getCurrent(), Text::toT(qi->getListName()), speed);

			::PostMessage(m_hWnd, WM_SPEAKER, DOWNLOAD_LISTING, (LPARAM)i);
		} else if(qi->isSet(QueueItem::FLAG_TEXT)) {
			::PostMessage(m_hWnd, WM_SPEAKER, VIEW_FILE_AND_DELETE, (LPARAM) new tstring(Text::toT(qi->getTarget())));
		}
	}
}

void MainFrame::onDestroy() {
	LogManager::getInstance()->removeListener(this);
	QueueManager::getInstance()->removeListener(this);
	TimerManager::getInstance()->removeListener(this);

	if(bTrayIcon) {
		updateTray(false);
	}

    ::PostQuitMessage(1);
}

void MainFrame::onActivateApp(WPARAM wParam) {
	WinUtil::isAppActive = (wParam == FALSE ? false : true);
	if(wParam == TRUE) {
		setNormalTrayIcon();
	}
}

bool MainFrame::onAppCommand(LPARAM lParam) {
	if(GET_APPCOMMAND_LPARAM(lParam) == APPCOMMAND_BROWSER_FORWARD) {
		ctrlTab.SwitchTo();
		return true;
    } else if(GET_APPCOMMAND_LPARAM(lParam) == APPCOMMAND_BROWSER_BACKWARD) {
		ctrlTab.SwitchTo(false);
		return true;
    }

    return false;
}

void MainFrame::onToolbarAway() {
	Util::getAway() ? Util::setAway(false) : Util::setAway(true);
	ClientManager::getInstance()->infoUpdated(true);
	WinUtil::changeToolbarAway(Util::getAway());
}

void MainFrame::clearSearches() {
	SearchFrame::lastSearches.clear();
}

void MainFrame::onCloseWindows(WORD wID) {
    switch(wID) {
        case IDC_CLOSE_DISCONNECTED:
            HubFrame::closeDisconnected();
            break;
        case IDC_CLOSE_DISCONNECTED_PM:
            PrivateFrame::closeDisconnected();
            break;
        case IDC_CLOSE_ALL_PM:
            PrivateFrame::closeAll();
            break;
        case IDC_CLOSE_ALL_SEARCHS:
            SearchFrame::closeAll();
            break;
        case IDC_CLOSE_ALL_FILELISTS:
            DirectoryListingFrame::closeAll();
            break;
    }
}

bool MainFrame::shutDown() {
	// Prepare for shutdown
	UINT iForceIfHung = 0;
	OSVERSIONINFO osvi;
	osvi.dwOSVersionInfoSize = sizeof(osvi);
	if(GetVersionEx(&osvi) != 0 && osvi.dwPlatformId == VER_PLATFORM_WIN32_NT) {
		iForceIfHung = 0x00000010;
		HANDLE hToken;
		OpenProcessToken(GetCurrentProcess(), (TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY), &hToken);
		LUID luid;
		LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, &luid);
		TOKEN_PRIVILEGES tp;
		tp.PrivilegeCount = 1;
		tp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
		tp.Privileges[0].Luid = luid;
		AdjustTokenPrivileges(hToken, FALSE, &tp, 0, (PTOKEN_PRIVILEGES)NULL, (PDWORD)NULL);
		CloseHandle(hToken);
	}
	// Shutdown
	if (ExitWindowsEx(EWX_POWEROFF | iForceIfHung, 0) == 0) {
		return false;
	} else {
		return true;
	}
}

void MainFrame::on(TimerManagerListener::Minute, uint64_t /*aTick*/) throw() {
//	SetProcessWorkingSetSize(GetCurrentProcess(), 0xffffffff, 0xffffffff);
	if(lastSave+3600000 < GET_TICK())
		SettingsManager::getInstance()->save();
}

void MainFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void MainFrame::SetCheck(const int &iID, const bool &bChecked) {
	if(bChecked == true) {
        ::CheckMenuItem(::GetMenu(m_hWnd), iID, MF_BYCOMMAND | MF_CHECKED);
    } else {
        ::CheckMenuItem(::GetMenu(m_hWnd), iID, MF_BYCOMMAND | MF_UNCHECKED);
    }
}

void MainFrame::OnChevronPushed(LPNMREBARCHEVRON nmrcRebarChevron) {
	REBARBANDINFO rbbi = { 0 };
	rbbi.cbSize = sizeof(REBARBANDINFO);
    rbbi.fMask = RBBIM_CHILD;
    ::SendMessage(nmrcRebarChevron->hdr.hwndFrom, RB_GETBANDINFO, nmrcRebarChevron->uBand, (LPARAM)&rbbi);

    int iCount = (int)::SendMessage(rbbi.hwndChild, TB_BUTTONCOUNT, 0, 0);

    HMENU hMenu = ::CreatePopupMenu();

    RECT rcClient = { 0 };
    ::GetClientRect(rbbi.hwndChild, &rcClient);

    for(int i = 0; i < iCount; i++) {
        TBBUTTON tbb = { 0 };
        ::SendMessage(rbbi.hwndChild, TB_GETBUTTON, i, (LPARAM)&tbb);

		RECT rcButton = { 0 };
		::SendMessage(rbbi.hwndChild, TB_GETITEMRECT, i, (LPARAM)&rcButton);

		if(rcButton.right > rcClient.right) {
			if(tbb.fsStyle & BTNS_SEP) {
				if(::GetMenuItemCount(hMenu) > 0) {
					::AppendMenu(hMenu, MF_SEPARATOR, 0, NULL);
                }
			} else {
				const int cchBuff = 200;
				TCHAR szBuff[cchBuff] = { 0 };
				LPTSTR lpstrText = szBuff;
				TBBUTTONINFO tbbi = { 0 };
				tbbi.cbSize = sizeof(TBBUTTONINFO);
				tbbi.dwMask = TBIF_TEXT;
				tbbi.pszText = szBuff;
				tbbi.cchText = cchBuff;
				if(::SendMessage(rbbi.hwndChild, TB_GETBUTTONINFO, tbb.idCommand, (LPARAM)&tbbi) == -1 || lstrlen(szBuff) == 0) {
                    for(int i = 0; WinUtil::ToolbarButtons[i].id != 0; i++) {
                        if(WinUtil::ToolbarButtons[i].id == tbb.idCommand) {
                            _tcscpy_s(szBuff, 200, CTSTRING_I((ResourceManager::Strings)WinUtil::ToolbarButtons[i].tooltip));
                            break;
                        }
                    }
				}

				::AppendMenu(hMenu, MF_STRING | (((tbb.fsState & TBSTATE_CHECKED) != 0) ? MF_CHECKED : MF_UNCHECKED), tbb.idCommand, lpstrText);
			}
		}
	}

	for(uint8_t i = 0; i < 22; i++) {
		MENUITEMINFO mii = { 0 };
		mii.cbSize = sizeof(MENUITEMINFO);
		if(Util::OsMajor >= 6) {
            mii.fMask = MIIM_BITMAP;
            mii.hbmpItem = m_MenuBitmaps[i];
            ::SetMenuItemInfo(hMenu, wMenuBitmapIds[i], FALSE, &mii);
        } else {
			mii.fMask = MIIM_FTYPE | MIIM_DATA;
			mii.fType = MFT_OWNERDRAW;
			mii.dwItemData = (ULONG_PTR)i;
			::SetMenuItemInfo(hMenu, wMenuBitmapIds[i], FALSE, &mii);
        }
	}

    POINT pt = { nmrcRebarChevron->rc.left, nmrcRebarChevron->rc.bottom };
    ::MapWindowPoints(nmrcRebarChevron->hdr.hwndFrom, NULL, &pt, 1);
    RECT rc = nmrcRebarChevron->rc;
    ::MapWindowPoints(nmrcRebarChevron->hdr.hwndFrom, NULL, (LPPOINT)&rc, 2);

    UINT uMenuFlags = TPM_LEFTBUTTON | TPM_VERTICAL | TPM_LEFTALIGN | TPM_TOPALIGN | TPM_VERPOSANIMATION;
    TPMPARAMS TPMParams = { 0 };
    TPMParams.cbSize = sizeof(TPMPARAMS);
    TPMParams.rcExclude = rc;

	::TrackPopupMenuEx(hMenu, uMenuFlags, pt.x, pt.y, m_hWnd, &TPMParams);

    ::DestroyMenu(hMenu);
}

HWND MainFrame::CreateEx() {
    WNDCLASSEX m_wc;
    memset(&m_wc, 0, sizeof(WNDCLASSEX));
    m_wc.cbSize = sizeof(WNDCLASSEX);
    m_wc.lpfnWndProc = StaticWindowProc;
    m_wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
    m_wc.lpszClassName = _T(CzDC_VERSION_STRING);
    m_wc.lpszMenuName = MAKEINTRESOURCE(IDR_MAINMENU);
    m_wc.hInstance = g_hInstance;
	m_wc.hCursor = ::LoadCursor(m_wc.hInstance, IDC_ARROW);
	m_wc.style = CS_HREDRAW | CS_VREDRAW;
	m_wc.hIcon = (HICON)::LoadImage(m_wc.hInstance, MAKEINTRESOURCE(IDR_MAINFRAME), IMAGE_ICON, 32, 32, LR_DEFAULTCOLOR);
	m_wc.hIconSm = (HICON)::LoadImage(m_wc.hInstance, MAKEINTRESOURCE(IDR_MAINFRAME), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);

	ATOM atom = ::RegisterClassEx(&m_wc);

    int rtl = ResourceManager::getInstance()->isRTL() ? WS_EX_RTLREADING : 0;

	if(SETTING(MAIN_WINDOW_POS_X) != CW_USEDEFAULT && SETTING(MAIN_WINDOW_POS_X) > 0 &&
		SETTING(MAIN_WINDOW_POS_Y) != CW_USEDEFAULT && SETTING(MAIN_WINDOW_POS_Y) > 0 &&
		SETTING(MAIN_WINDOW_SIZE_X) != CW_USEDEFAULT && SETTING(MAIN_WINDOW_SIZE_X) > 0 &&
		SETTING(MAIN_WINDOW_SIZE_Y) != CW_USEDEFAULT && SETTING(MAIN_WINDOW_SIZE_Y) > 0) {
        m_hWnd = ::CreateWindowEx(rtl | WS_EX_APPWINDOW | WS_EX_WINDOWEDGE, MAKEINTATOM(atom), _T(CzDC_VERSION_STRING), WS_OVERLAPPEDWINDOW |
            WS_CLIPCHILDREN | WS_CLIPSIBLINGS, SETTING(MAIN_WINDOW_POS_X), SETTING(MAIN_WINDOW_POS_Y), SETTING(MAIN_WINDOW_SIZE_X),
            SETTING(MAIN_WINDOW_SIZE_Y), NULL, NULL, g_hInstance, this);
	} else {
        m_hWnd = ::CreateWindowEx(rtl | WS_EX_APPWINDOW | WS_EX_WINDOWEDGE, MAKEINTATOM(atom), _T(CzDC_VERSION_STRING), WS_OVERLAPPEDWINDOW |
            WS_CLIPCHILDREN | WS_CLIPSIBLINGS, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, g_hInstance, this);
    }

	if(m_hWnd != NULL){
		m_hAccel = ::LoadAccelerators(g_hInstance, MAKEINTRESOURCE(IDR_MAINFRAME));
    }

	return m_hWnd;
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
