/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "NotepadFrame.h"
#include "WinUtil.h"
#include "../client/File.h"
#include "WinApiUtil.h"
#include "FlatTabCtrl.h"

ATOM NotepadFrame::m_aAtom = 0;
UINT NotepadFrame::m_uiResourceID = IDR_NOTEPAD;
static WNDPROC DefEditProc = NULL;

NotepadFrame::NotepadFrame() : dirty(false), closed(false) {
	SettingsManager::getInstance()->addListener(this);

    RegisterWindowClass(m_aAtom, NotepadFrameWndProc, _T("NotepadFrame"), m_uiResourceID);
}

LRESULT CALLBACK NotepadFrame::NotepadFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	NotepadFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (NotepadFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (NotepadFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->NotepadFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT NotepadFrame::NotepadFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_SETFOCUS:
            OnFocus();
            break;
        case WM_CREATE:
            OnCreate();
            return 0;
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
            break;
        case WM_CTLCOLOREDIT:
        case WM_CTLCOLORSTATIC: {
            LRESULT lResult = onCtlColor(wParam, lParam);
            if(lResult != 0) {
                return lResult;
            }
            break;
        }
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

    LRESULT lResult = 0;
    if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
    	return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK NotepadFrame::EditWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == WM_LBUTTONDBLCLK) {
		NotepadFrame * pParent = (NotepadFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
            if(pParent->onLButton(lParam) == true) {
                return 0;
            }
        }
    }

    return DefEditProc(hWnd, uMsg, wParam, lParam);
}

void NotepadFrame::OnCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(0, 0, 0), IDR_NOTEPAD);
    created = true;

    hWndPad = CreateWindowEx(WS_EX_CLIENTEDGE, _T("EDIT"), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_VSCROLL |
        ES_AUTOVSCROLL | ES_MULTILINE | ES_NOHIDESEL, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);

	::SendMessage(hWndPad, EM_LIMITTEXT, 0, 0L);
	::SendMessage(hWndPad, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	string tmp;
	try {
		tmp = File(Util::getNotepadFile(), File::READ, File::OPEN).read();
	} catch(const FileException&) {
		// ...
	}

	::SetWindowText(hWndPad, Text::toT(tmp).c_str());
	::SendMessage(hWndPad, EM_EMPTYUNDOBUFFER, 0, 0L);
    ::SetWindowLongPtr(hWndPad, GWLP_USERDATA, (LONG_PTR)this);
    DefEditProc = (WNDPROC)::SetWindowLongPtr(hWndPad, GWLP_WNDPROC, (LONG_PTR)EditWndProc);;
}

bool NotepadFrame::onClose() {
	if(!closed) {
		SettingsManager::getInstance()->removeListener(this);
		if(dirty || (BOOL)::SendMessage(hWndPad, EM_GETMODIFY, 0, 0L)) {
			AutoArray<TCHAR> buf(::GetWindowTextLength(hWndPad) + 1);
			::GetWindowText(hWndPad, buf, ::GetWindowTextLength(hWndPad) + 1);
			try {
                string tmp(Text::fromT(tstring(buf, ::GetWindowTextLength(hWndPad))));
                File(Util::getNotepadFile(), File::WRITE, File::CREATE | File::TRUNCATE).write(tmp);
			} catch(const FileException&) {
				// Oops...
			}
		}

		closed = true;
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);

		return true;
	} else {
		return false;
	}
}

void NotepadFrame::UpdateLayout() {
	RECT rc;
	::GetClientRect(m_hWnd, &rc);

	rc.bottom -= 1;
	rc.top += 1;
	rc.left +=1;
	rc.right -=1;
	::MoveWindow(hWndPad, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
}

bool NotepadFrame::onLButton(LPARAM lParam) {
	if(GetFocus() == hWndPad) {
		POINT pt = { (LONG)LOWORD(lParam), (LONG)HIWORD(lParam) };
		tstring x;
		tstring::size_type start = (tstring::size_type)WinUtil::textUnderCursor(pt, hWndPad, x);
		tstring::size_type end = x.find(_T(" "), start);

		if(end == string::npos)
			end = x.length();

		return WinUtil::parseDBLClick(x, start, end);
	}

	return false;
}

void NotepadFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void NotepadFrame::onRefreshSettings() {
	::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
