/* 
 * Copyright (C) 2003 Twink,  spm7@waikato.ac.nz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "PreviewDlg.h"
#include "WinUtil.h"
#include "WinApiUtil.h"

INT_PTR CALLBACK PreviewDlg::PreviewDlgProc(HWND hWndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam){
	PreviewDlg * pParent;

	if(uMsg == WM_INITDIALOG) {
		pParent = (PreviewDlg *)lParam;
		SetWindowLongPtr(hWndDlg, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWndDlg;
	} else {
		pParent = (PreviewDlg *)GetWindowLongPtr(hWndDlg, GWLP_USERDATA);
		if(pParent == NULL) {
			return FALSE;
		}
	}

    SetWindowLongPtr(hWndDlg, DWLP_MSGRESULT, 0);

	return pParent->PreviewProc(hWndDlg, uMsg, wParam, lParam);
}

LRESULT PreviewDlg::PreviewProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_INITDIALOG:
            OnInitDialog();
            return TRUE;
        case WM_SETFOCUS:
            onFocus();
            return TRUE;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDOK:
                case IDCANCEL:
                    OnCloseCmd(LOWORD(wParam));
                    return TRUE;
                case IDC_PREVIEW_BROWSE:
                    OnBrowse();
                    return TRUE;
            }
            break;
    }

	return FALSE;
}

void PreviewDlg::onFocus() {
	::SetFocus(::GetDlgItem(m_hWnd, IDC_PREVIEW_NAME));
}

void PreviewDlg::OnInitDialog() {
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_PREVIEW_NAME), name.c_str());
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_PREVIEW_APPLICATION), application.c_str());
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_PREVIEW_ARGUMENTS), argument.c_str());
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_PREVIEW_EXTENSION), extensions.c_str());

	CenterWindow(m_hWnd, ::GetParent(m_hWnd));
}

void PreviewDlg::OnBrowse() {
	TCHAR buf[MAX_PATH];

	::GetDlgItemText(m_hWnd, IDC_PREVIEW_APPLICATION, buf, MAX_PATH);
	tstring x = buf;

	if(WinUtil::browseFile(x, m_hWnd, false,  Util::emptyStringT, _T("Programs\0*.exe")) == IDOK) {
		::SetDlgItemText(m_hWnd, IDC_PREVIEW_APPLICATION, x.c_str());
	}
}

void PreviewDlg::OnCloseCmd(WORD wID) {
	if(wID == IDOK) {
		TCHAR buf[256];

		if(::GetWindowTextLength(::GetDlgItem(m_hWnd, IDC_PREVIEW_NAME)) == 0 || 
			::GetWindowTextLength(::GetDlgItem(m_hWnd, IDC_PREVIEW_APPLICATION)) == 0) {
			::MessageBox(m_hWnd, _T("Name and command must not be empty"), _T(""), MB_OK);
			return;
		}

		::GetDlgItemText(m_hWnd, IDC_PREVIEW_NAME, buf, 256);
		name = buf;

		::GetDlgItemText(m_hWnd, IDC_PREVIEW_APPLICATION, buf, 256);
		application = buf;

		::GetDlgItemText(m_hWnd, IDC_PREVIEW_ARGUMENTS, buf, 256);
		argument = buf;

		::GetDlgItemText(m_hWnd, IDC_PREVIEW_EXTENSION, buf, 256);
		extensions = buf;
	}

	::EndDialog(m_hWnd, wID);
}

INT_PTR PreviewDlg::DoModal(HWND hWndParent) {
	return ::DialogBoxParam(g_hInstance, MAKEINTRESOURCE(IDD_PREVIEW), hWndParent, PreviewDlgProc, (LPARAM)this);
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
