/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef PRIVATE_FRAME_H
#define PRIVATE_FRAME_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../client/User.h"
#include "../client/ClientManagerListener.h"
#include "../client/ResourceManager.h"

#include "MDIChildWindow.h"
#include "WinUtil.h"
#include "UCHandler.h"

#include "ChatCtrl.h"

class PrivateFrame : public MDITabChildWindow, private ClientManagerListener, private UCHandler<PrivateFrame>, private SettingsManagerListener {
public:
	static void gotMessage(Identity& aUser, Identity& byUser, const tstring& aMessage);
	static void openWindow(const UserPtr& aUser, const tstring& aMessage = Util::emptyStringT);
	static bool isOpen(const UserPtr u) { return frames.find(u) != frames.end(); }

	enum {
		USER_UPDATED
	};

    static ATOM m_aAtom;
    static UINT m_uiResourceID;

    static LRESULT CALLBACK PrivateFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    LRESULT PrivateFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void OnCreate();
	BOOL onChar(UINT uMsg, WPARAM wParam);
	void onGetList();
	void onMatchQueue();
	void onGrantSlot();
	void onGrantSlotHour();
	void onGrantSlotDay();
	void onGrantSlotWeek();
	void onUnGrantSlot();
	void onAddToFavorites();
	void onTabContextMenu(LPARAM lParam);
	bool onContextMenu(LPARAM lParam);
	void onEditCopy();
	void onEditSelectAll();
	void onEditClearAll();
	void onCopyActualLine();
	void onClientEnLink(LPNMHDR pnmh);
	void onOpenUserLog();
	void onCopyURL();
	bool onClose();
	void onSearch(WORD wID);
	void onRefreshSettings();

	void addLine(const Identity& byUser, const tstring& aLine, CHARFORMAT2& cf);
	void addStatus(const tstring& aLine, CHARFORMAT2& cf);
	void onEnter();
	void UpdateLayout();	
	void runUserCommand(UserCommand& uc);
	void readLog();

	void onCloseWindow() {
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
	}

	LRESULT onCtlColor(WPARAM wParam, LPARAM lParam) {
		HWND hWnd = (HWND)lParam;
		HDC hDC = (HDC)wParam;
		if(hWnd == ctrlClient.m_hWnd || hWnd == hWndMessage) {
			::SetBkColor(hDC, WinUtil::bgColor);
			::SetTextColor(hDC, WinUtil::textColor);
			return (LRESULT)WinUtil::bgBrush;
		}

		return 0;
	}

	void onFocus() {
		::SetFocus(hWndMessage);
		ctrlClient.GoToEnd();
	}

	void addClientLine(const tstring& aLine);
	void sendMessage(const tstring& msg);

	UserPtr& getUser() { return user; }
	static void closeDisconnected();
	static void closeAll();
	static size_t getPmCount() { return frames.size(); }
private:
	PrivateFrame(const UserPtr& aUser);
	~PrivateFrame();

	typedef HASH_MAP<UserPtr, PrivateFrame*, User::HashFunction> FrameMap;
	typedef FrameMap::iterator FrameIter;
	static FrameMap frames;
	ChatCtrl ctrlClient;
	HWND hWndMessage;

	HMENU textMenu;
	HMENU tabMenu;

	UserPtr user;	

	bool closed;
	bool isoffline;

	StringMap ucLineParams;

	void updateTitle();

	TStringList prevCommands;
	tstring currentCommand;
	TStringList::size_type curCommandPosition;

	// ClientManagerListener
	void on(ClientManagerListener::UserUpdated, const OnlineUser& aUser) throw() {
		if(aUser.getUser() == user) {
			::PostMessage(m_hWnd, WM_SPEAKER, USER_UPDATED, 0);
	}
	}
	void on(ClientManagerListener::UserConnected, const UserPtr& aUser) throw() {
		if(aUser == user) {
			::PostMessage(m_hWnd, WM_SPEAKER, USER_UPDATED, 0);
		}
	}
	void on(ClientManagerListener::UserDisconnected, const UserPtr& aUser) throw() {
		if(aUser == user) {
			::PostMessage(m_hWnd, WM_SPEAKER, USER_UPDATED, 0);
		}
	}
	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();

    static LRESULT CALLBACK MessageWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
};

#endif // PRIVATE_FRAME_H

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
