/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "PublicHubsFrm.h"
#include "HubFrame.h"
#include "WinUtil.h"
#include "CountryFlags.h"

#include "../client/Client.h"
#include "../client/version.h"

int PublicHubsFrame::columnIndexes[] = { COLUMN_NAME, COLUMN_DESCRIPTION, COLUMN_USERS, COLUMN_SERVER, COLUMN_COUNTRY, COLUMN_SHARED, COLUMN_MINSHARE,
	COLUMN_MINSLOTS, COLUMN_MAXHUBS, COLUMN_MAXUSERS, COLUMN_RELIABILITY, COLUMN_RATING
};
int PublicHubsFrame::columnSizes[] = { 200, 290, 50, 100, 100, 100, 100, 100, 100, 100, 100, 100 };
static ResourceManager::Strings columnNames[] = { ResourceManager::HUB_NAME, ResourceManager::DESCRIPTION, ResourceManager::USERS,
    ResourceManager::HUB_ADDRESS, ResourceManager::COUNTRY, ResourceManager::SHARED, ResourceManager::MIN_SHARE, ResourceManager::MIN_SLOTS,
	ResourceManager::MAX_HUBS, ResourceManager::MAX_USERS, ResourceManager::RELIABILITY, ResourceManager::RATING,
};
ATOM PublicHubsFrame::m_aAtom = 0;
UINT PublicHubsFrame::m_uiResourceID = IDR_PUBLICHUBS;
static WNDPROC DefEditProc = NULL;

PublicHubsFrame::PublicHubsFrame() : users(0), hubs(0), closed(false), hubsMenu(NULL) {
    RegisterWindowClass(m_aAtom, PublicHubsFrameWndProc, _T("PublicHubsFrame"), m_uiResourceID);
}

PublicHubsFrame::~PublicHubsFrame() {
    if(hubsMenu != NULL) {
        ::DestroyMenu(hubsMenu);
    }
}

LRESULT CALLBACK PublicHubsFrame::PublicHubsFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	PublicHubsFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (PublicHubsFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (PublicHubsFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->PublicHubsFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT PublicHubsFrame::PublicHubsFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CREATE:
            onCreate();
            return 0;
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
            break;
        case WM_CONTEXTMENU:
            if(onContextMenu(wParam, lParam) == true) {
                return 0;
            }
            break;
        case WM_SPEAKER:
            onSpeaker(wParam, lParam);
            return 0;
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case WM_CTLCOLOREDIT:
        case WM_CTLCOLORSTATIC: {
            LRESULT lResult = onCtlColor(wParam, lParam);
            if(lResult != 0) {
                return lResult;
            }
            break;
        }
        case WM_SETFOCUS:
            onSetFocus();
            break;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDC_FILTER_FOCUS:
                    onFilterFocus();
                    return 0;
                case IDC_ADD:
                    onAdd();
                    return 0;
                case IDC_REFRESH:
                    onClickedRefresh();
                    return 0;
                case IDC_CONNECT:
                    onClickedConnect();
                    return 0;
                case IDC_COPY_HUB:
                    onCopyHub();
                    return 0;
                case IDC_PUB_LIST_DROPDOWN:
                    if(HIWORD(wParam) == CBN_SELCHANGE) {
                        onListSelChanged();
                    }
                    break;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_HUBLIST) {
                switch(((LPNMHDR)lParam)->code) {
                    case LVN_COLUMNCLICK:
                        onColumnClickHublist((LPNMHDR)lParam);
                        return 0;
                    case NM_RETURN:
                        onEnter();
                        return 0;
                    case NM_DBLCLK:
                        onDoubleClickHublist((LPNMHDR)lParam);
                        return 0;
                }
            }

            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

    LRESULT lResult = 0;
	if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK PublicHubsFrame::FilterWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == WM_CHAR) {
		PublicHubsFrame * pParent = (PublicHubsFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
			if(pParent->onFilterChar(wParam) == true) {
                return 0;
            }
        }
    }

    return DefEditProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK PublicHubsFrame::HubAddressWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == WM_CHAR) {
		PublicHubsFrame * pParent = (PublicHubsFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
			if(pParent->onChar(wParam) == true) {
                return 0;
            }
        }
    }

    return DefEditProc(hWnd, uMsg, wParam, lParam);
}

void PublicHubsFrame::onCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(0, 0, 0), IDR_PUBLICHUBS);
    created = true;

    m_hWndStatusBar = CreateWindowEx(0, STATUSCLASSNAME, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | SBARS_SIZEGRIP,
        0, 0, 0, 0, m_hWnd, (HMENU)NULL, g_hInstance, NULL);

	int w[3] = { 0, 0, 0};
	::SendMessage(m_hWndStatusBar, SB_SETPARTS, 3, (LPARAM)w);

	ctrlHubs.Create(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_SHOWSELALWAYS |
        LVS_SINGLESEL | LVS_SHAREIMAGELISTS, WS_EX_CLIENTEDGE, IDC_HUBLIST, LVS_EX_LABELTIP | LVS_EX_HEADERDRAGDROP | LVS_EX_FULLROWSELECT);
	::SendMessage(ctrlHubs.m_hWnd, LVM_SETIMAGELIST, LVSIL_SMALL, (LPARAM)CountryFlags::flagImages);

	// Create listview columns
	WinUtil::splitTokens(columnIndexes, SETTING(PUBLICHUBSFRAME_ORDER), COLUMN_LAST);
	WinUtil::splitTokens(columnSizes, SETTING(PUBLICHUBSFRAME_WIDTHS), COLUMN_LAST);

	for(int j=0; j<COLUMN_LAST; j++) {
		int fmt = (j == COLUMN_USERS) ? LVCFMT_RIGHT : LVCFMT_LEFT;
		ListViewInsertColumn(ctrlHubs.m_hWnd, j, CTSTRING_I(columnNames[j]), fmt, columnSizes[j], j);
	}

	::SendMessage(ctrlHubs.m_hWnd, LVM_SETCOLUMNORDERARRAY, COLUMN_LAST, (LPARAM)columnIndexes);

	ctrlHubs.setSort(COLUMN_USERS, ExListViewCtrl::SORT_INT, false);
	::SetFocus(ctrlHubs.m_hWnd);

    hWndHubAddress = CreateWindowEx(WS_EX_CLIENTEDGE, _T("EDIT"), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | ES_AUTOHSCROLL,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
	::SendMessage(hWndHubAddress, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
    ::SetWindowLongPtr(hWndHubAddress, GWLP_USERDATA, (LONG_PTR)this);
    DefEditProc = (WNDPROC)::SetWindowLongPtr(hWndHubAddress, GWLP_WNDPROC, (LONG_PTR)HubAddressWndProc);

	hWndConnect = CreateWindowEx(0, _T("BUTTON"), CTSTRING(CONNECT), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | BS_PUSHBUTTON,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_CONNECT, g_hInstance, NULL);
	::SendMessage(hWndConnect, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	hWndRefresh = CreateWindowEx(0, _T("BUTTON"), CTSTRING(REFRESH), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | BS_PUSHBUTTON,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_REFRESH, g_hInstance, NULL);
	::SendMessage(hWndRefresh, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	hWndAddress = CreateWindowEx(WS_EX_TRANSPARENT, _T("BUTTON"), CTSTRING(MANUAL_ADDRESS), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
        BS_GROUPBOX, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
	::SendMessage(hWndAddress, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	hWndLists = CreateWindowEx(WS_EX_TRANSPARENT, _T("BUTTON"), CTSTRING(SETTINGS_PUBLIC_HUB_LIST), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
		BS_GROUPBOX, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
	::SendMessage(hWndLists, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	hWndPubLists = CreateWindowEx(WS_EX_CLIENTEDGE, _T("COMBOBOX"), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL |
        WS_VSCROLL | CBS_DROPDOWNLIST, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_PUB_LIST_DROPDOWN, g_hInstance, NULL);
	::SendMessage(hWndPubLists, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	// populate with values from the settings
	updateDropDown();

    hWndFilter = CreateWindowEx(WS_EX_CLIENTEDGE, _T("EDIT"), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | ES_AUTOHSCROLL,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
    ::SendMessage(hWndFilter, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
    ::SetWindowLongPtr(hWndFilter, GWLP_USERDATA, (LONG_PTR)this);
    DefEditProc = (WNDPROC)::SetWindowLongPtr(hWndFilter, GWLP_WNDPROC, (LONG_PTR)FilterWndProc);

	hWndFilterSel = CreateWindowEx(WS_EX_CLIENTEDGE, _T("COMBOBOX"), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL |
        WS_VSCROLL | CBS_DROPDOWNLIST, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
	::SendMessage(hWndFilterSel, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	//populate the filter list with the column names
	for(int j=0; j<COLUMN_LAST; j++) {
		::SendMessage(hWndFilterSel, CB_ADDSTRING, 0, (LPARAM)CTSTRING_I(columnNames[j]));
	}
	::SendMessage(hWndFilterSel, CB_ADDSTRING, 0, (LPARAM)CTSTRING(ANY));
	::SendMessage(hWndFilterSel, CB_SETCURSEL, COLUMN_LAST, 0L);

	hWndFilterDesc = CreateWindowEx(WS_EX_TRANSPARENT, _T("BUTTON"), CTSTRING(FILTER), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
		BS_GROUPBOX, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
	::SendMessage(hWndFilterDesc, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	FavoriteManager::getInstance()->addListener(this);
	SettingsManager::getInstance()->addListener(this);

	hubs = FavoriteManager::getInstance()->getPublicHubs();
	if(FavoriteManager::getInstance()->isDownloading()) {
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (0 | 0), (LPARAM)CTSTRING(DOWNLOADING_HUB_LIST));
	} else if(hubs.empty()) {
		FavoriteManager::getInstance()->refresh();
	}

	updateList();

	hubsMenu = ::CreatePopupMenu();
	::AppendMenu(hubsMenu, MF_STRING, IDC_CONNECT, CTSTRING(CONNECT));
	::AppendMenu(hubsMenu, MF_STRING, IDC_ADD, CTSTRING(ADD_TO_FAVORITES));
	::AppendMenu(hubsMenu, MF_STRING, IDC_COPY_HUB, CTSTRING(COPY_HUB));
	::SetMenuDefaultItem(hubsMenu, IDC_CONNECT, 0);
}

void PublicHubsFrame::onColumnClickHublist(LPNMHDR pnmh) {
	NMLISTVIEW* l = (NMLISTVIEW*)pnmh;
	if(l->iSubItem == ctrlHubs.getSortColumn()) {
		if(!ctrlHubs.isAscending()) {
			ctrlHubs.setSort(-1, ctrlHubs.getSortType());
		} else {
			ctrlHubs.setSortDirection(false);
        }
	} else {
		// BAH, sorting on bytes will break of course...oh well...later...
		if(l->iSubItem == COLUMN_USERS || l->iSubItem == COLUMN_MINSLOTS ||l->iSubItem == COLUMN_MAXHUBS || l->iSubItem == COLUMN_MAXUSERS) {
			ctrlHubs.setSort(l->iSubItem, ExListViewCtrl::SORT_INT);
		} else if(l->iSubItem == COLUMN_RELIABILITY) {
			ctrlHubs.setSort(l->iSubItem, ExListViewCtrl::SORT_FLOAT);
		} else if(l->iSubItem == COLUMN_SHARED || l->iSubItem == COLUMN_MINSHARE) {
			ctrlHubs.setSort(l->iSubItem, ExListViewCtrl::SORT_BYTES);
		} else {
			ctrlHubs.setSort(l->iSubItem, ExListViewCtrl::SORT_STRING_NOCASE);
		}
	}
}

void PublicHubsFrame::onDoubleClickHublist(LPNMHDR pnmh) {
	if(!checkNick()) {
		return;
    }

	NMITEMACTIVATE* item = (NMITEMACTIVATE*) pnmh;

	if(item->iItem != -1) {
		TCHAR buf[256];

		RecentHubEntry r;
		ListViewGetItemText(ctrlHubs.m_hWnd, item->iItem, COLUMN_NAME, buf, 256);
		r.setName(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, item->iItem, COLUMN_DESCRIPTION, buf, 256);
		r.setDescription(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, item->iItem, COLUMN_USERS, buf, 256);
		r.setUsers(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, item->iItem, COLUMN_SHARED, buf, 256);
		r.setShared(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, item->iItem, COLUMN_SERVER, buf, 256);
		r.setServer(Text::fromT(buf));
		FavoriteManager::getInstance()->addRecent(r);
		HubFrame::openWindow(buf);
	}
}

void PublicHubsFrame::onEnter() {
	if(!checkNick()) {
		return;
    }

	int item = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_FOCUSED, 0));
	if(item != -1) {
		TCHAR buf[256];
		RecentHubEntry r;
		ListViewGetItemText(ctrlHubs.m_hWnd, item, COLUMN_NAME, buf, 256);
		r.setName(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, item, COLUMN_DESCRIPTION, buf, 256);
		r.setDescription(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, item, COLUMN_USERS, buf, 256);
		r.setUsers(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, item, COLUMN_SHARED, buf, 256);
		r.setShared(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, item, COLUMN_SERVER, buf, 256);
		r.setServer(Text::fromT(buf));
		FavoriteManager::getInstance()->addRecent(r);
		HubFrame::openWindow(buf);
	}
}

void PublicHubsFrame::onClickedRefresh() {
	::SendMessage(m_hWndStatusBar, SB_SETTEXT, (0 | 0), (LPARAM)CTSTRING(DOWNLOADING_HUB_LIST));
	FavoriteManager::getInstance()->refresh(true);
	updateDropDown();
}

void PublicHubsFrame::onClickedConnect() {
	if(!checkNick()) {
		return;
    }

	if(::GetWindowTextLength(hWndHubAddress) > 0) {
		AutoArray<TCHAR> hub(::GetWindowTextLength(hWndHubAddress)+1);
		::GetWindowText(hWndHubAddress, hub, ::GetWindowTextLength(hWndHubAddress)+1);
		::SetWindowText(hWndHubAddress, _T(""));
		tstring tmp = hub;
		string::size_type i;
		while((i = tmp.find(' ')) != string::npos) {
			tmp.erase(i, 1);
        }

		RecentHubEntry r;
		r.setName("***");
		r.setDescription("***");
		r.setUsers("*");
		r.setShared("*");
		r.setServer(Text::fromT(tmp));
		FavoriteManager::getInstance()->addRecent(r);

		HubFrame::openWindow(tmp);
	} else if((UINT)::SendMessage(ctrlHubs.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0L) == 1) {
		TCHAR buf[256];
		int i = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_SELECTED, 0));
		ListViewGetItemText(ctrlHubs.m_hWnd, i, COLUMN_SERVER, buf, 256);

		RecentHubEntry r;
		ListViewGetItemText(ctrlHubs.m_hWnd, i, COLUMN_NAME, buf, 256);
		r.setName(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, i, COLUMN_DESCRIPTION, buf, 256);
		r.setDescription(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, i, COLUMN_USERS, buf, 256);
		r.setUsers(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, i, COLUMN_SHARED, buf, 256);
		r.setShared(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, i, COLUMN_SERVER, buf, 256);
		r.setServer(Text::fromT(buf));
		FavoriteManager::getInstance()->addRecent(r);

		HubFrame::openWindow(buf);
	}
}

void PublicHubsFrame::onFilterFocus() {
	::SetFocus(hWndFilter);
}

void PublicHubsFrame::onAdd() {
	if(!checkNick()) {
		return;
    }
	
	if((UINT)::SendMessage(ctrlHubs.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0L) == 1) {
		int i = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_SELECTED, 0));
		FavoriteHubEntry e;
		TCHAR buf[256];
		ListViewGetItemText(ctrlHubs.m_hWnd, i, COLUMN_NAME, buf, 256);
		e.setName(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, i, COLUMN_DESCRIPTION, buf, 256);
		e.setDescription(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, i, COLUMN_SERVER, buf, 256);
		e.setServer(Text::fromT(buf));
		FavoriteManager::getInstance()->addFavorite(e);
	}
}

bool PublicHubsFrame::onChar(WPARAM wParam) {
	if(wParam == VK_RETURN && ::GetWindowTextLength(hWndHubAddress) > 0) {
		if(!checkNick()) {
			return false;
		}

		AutoArray<TCHAR> hub(::GetWindowTextLength(hWndHubAddress)+1);
		::GetWindowText(hWndHubAddress, hub, ::GetWindowTextLength(hWndHubAddress)+1);
		::SetWindowText(hWndHubAddress, _T(""));
		tstring tmp = hub;
		tstring::size_type i;
		while((i = tmp.find(' ')) != tstring::npos)
			tmp.erase(i, 1);

		RecentHubEntry r;
		r.setName("***");
		r.setDescription("***");
		r.setUsers("*");
		r.setShared("*");
		r.setServer(Text::fromT(tmp));
		FavoriteManager::getInstance()->addRecent(r);

		HubFrame::openWindow(tmp);

		return true;
	} else {
		return false;
	}
}

bool PublicHubsFrame::onClose() {
	if(!closed) {
		FavoriteManager::getInstance()->removeListener(this);
		SettingsManager::getInstance()->removeListener(this);
		closed = true;
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
		return true;
	} else {
		WinUtil::saveHeaderOrder(ctrlHubs.m_hWnd, SettingsManager::PUBLICHUBSFRAME_ORDER,
			SettingsManager::PUBLICHUBSFRAME_WIDTHS, COLUMN_LAST, columnIndexes, columnSizes);

		return false;
	}
}

void PublicHubsFrame::onListSelChanged() {
	FavoriteManager::getInstance()->setHubList((int)::SendMessage(hWndPubLists, CB_GETCURSEL, 0, 0L));
	hubs = FavoriteManager::getInstance()->getPublicHubs();
	updateList();
}

void PublicHubsFrame::UpdateLayout() {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);
	// position bars and offset their dimensions
	UpdateBarsPosition(rect);

	if(::IsWindow(m_hWndStatusBar)) {
		RECT sr;
		int w[3];
		::GetClientRect(m_hWndStatusBar, &sr);
		int tmp = (sr.right - sr.left) > 316 ? 216 : (((sr.right - sr.left) > 116) ? (sr.right - sr.left) - 100 : 16);

		w[0] = sr.right - tmp;
		w[1] = w[0] + (tmp-16)/2;
		w[2] = w[0] + (tmp-16);

		::SendMessage(m_hWndStatusBar, SB_SETPARTS, 3, (LPARAM)w);
	}

	RECT rc = rect;
	rc.top += 2;
	rc.bottom -=(56);
	::MoveWindow(ctrlHubs.m_hWnd, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	rc = rect;
	rc.top = rc.bottom - 52;
	rc.bottom = rc.top + 46;
	rc.right -= 100;
	rc.right -= (2 * ((rc.right - rc.left) / 3)) + 3;
	::MoveWindow(hWndFilterDesc, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	int iHalf = ((rc.right - rc.left) - 16) / 2;
	rc.top += 16;
	rc.bottom -= 8;
	rc.right -= (iHalf+10);
	rc.left += 8;
	::MoveWindow(hWndFilter, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	//filter sel
	rc.bottom += 140;
	rc.right += 2 + iHalf;
	rc.left += 3 + iHalf;
	::MoveWindow(hWndFilterSel, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	rc = rect;
	rc.top = rc.bottom - 52;
	rc.bottom = rc.top + 46;
	rc.right -= 100;
	rc.left += ((rc.right - rc.left) / 3) + 1;
	rc.right = rc.left + ((rc.right - rect.left) / 3) + 1;
	::MoveWindow(hWndAddress, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
	
	rc.top += 16;
	rc.bottom -= 8;
	rc.right -= 8;
	rc.left += 8;
	::MoveWindow(hWndHubAddress, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	rc = rect;
	rc.top = rc.bottom - 52;
	rc.bottom = rc.top + 46;
	rc.right -= 100;
	rc.left += (2 * (((rc.right - rc.left) / 3))) + 4;
	::MoveWindow(hWndLists, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	rc.top += 16;
	rc.bottom += 132;
	rc.right -= 8;
	rc.left += 8;
	::MoveWindow(hWndPubLists, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	rc = rect;
	rc.bottom -= 2;
	rc.top = rc.bottom - 22;
	rc.left = rc.right - 96;
	rc.right -= 2;
	::MoveWindow(hWndConnect, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	rc.top -= 24;
	rc.bottom -= 24;
	::MoveWindow(hWndRefresh, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
}

bool PublicHubsFrame::checkNick() {
	if(SETTING(NICK).empty()) {
		::MessageBox(m_hWnd, CTSTRING(ENTER_NICK), _T(CzDC_VERSION_STRING), MB_ICONSTOP | MB_OK);
		return false;
	}
	return true;
}

void PublicHubsFrame::updateList() {
	::SendMessage(ctrlHubs.m_hWnd, LVM_DELETEALLITEMS, 0, 0L);
	users = 0;
	visibleHubs = 0;

	::SendMessage(ctrlHubs.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);

	double size = -1;
	FilterModes mode = NONE;

	int sel = (int)::SendMessage(hWndFilterSel, CB_GETCURSEL, 0, 0L);

	bool doSizeCompare = parseFilter(mode, size);

	for(HubEntry::List::const_iterator i = hubs.begin(); i != hubs.end(); ++i) {
		if(matchFilter(*i, sel, doSizeCompare, mode, size)) {

			TStringList l;
			l.resize(COLUMN_LAST);
			l[COLUMN_NAME] = Text::toT(i->getName());
			l[COLUMN_DESCRIPTION] = Text::toT(i->getDescription());
			l[COLUMN_USERS] = Text::toT(Util::toString(i->getUsers()));
			l[COLUMN_SERVER] = Text::toT(i->getServer());
			l[COLUMN_COUNTRY] = Text::toT(i->getCountry());
			l[COLUMN_SHARED] = Text::toT(Util::formatBytes(i->getShared()));
			l[COLUMN_MINSHARE] = Text::toT(Util::formatBytes(i->getMinShare()));
			l[COLUMN_MINSLOTS] = Text::toT(Util::toString(i->getMinSlots()));
			l[COLUMN_MAXHUBS] = Text::toT(Util::toString(i->getMaxHubs()));
			l[COLUMN_MAXUSERS] = Text::toT(Util::toString(i->getMaxUsers()));
			l[COLUMN_RELIABILITY] = Text::toT(Util::toString(i->getReliability()));
			l[COLUMN_RATING] = Text::toT(i->getRating());
			ctrlHubs.insert((int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMCOUNT, 0, 0L), l, CountryFlags::GetFlagImageIndex(i->getCountry().c_str(), true));
			visibleHubs++;
			users += i->getUsers();
		}
	}

	::SendMessage(ctrlHubs.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
	ctrlHubs.resort();

	updateStatus();
}

void PublicHubsFrame::updateStatus() {
    ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)Text::toT(STRING(HUBS) + ": " + Util::toString(visibleHubs)).c_str());
    ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0), (LPARAM)Text::toT(STRING(USERS) + ": " + Util::toString(users)).c_str());
}

void PublicHubsFrame::onSpeaker(WPARAM wParam, LPARAM lParam) {
	if(wParam == FINISHED || wParam == LOADED_FROM_CACHE) {
		hubs = FavoriteManager::getInstance()->getPublicHubs();
		updateList();
		tstring* x = (tstring*)lParam;
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (0 | 0),
            (LPARAM)(((wParam == LOADED_FROM_CACHE) ? TSTRING(HUB_LIST_LOADED_FROM_CACHE) : TSTRING(HUB_LIST_DOWNLOADED)) + _T(" (") + (*x) + _T(")")).c_str());
		delete x;
	} else if(wParam == STARTING) {
		tstring* x = (tstring*)lParam;
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (0 | 0), (LPARAM)(TSTRING(DOWNLOADING_HUB_LIST) + _T(" (") + (*x) + _T(")")).c_str());
		delete x;
	} else if(wParam == FAILED) {
		tstring* x = (tstring*)lParam;
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (0 | 0), (LPARAM)(TSTRING(DOWNLOAD_FAILED) + (*x) ).c_str());
		delete x;
	}
}

bool PublicHubsFrame::onFilterChar(WPARAM wParam) {
	if(wParam == VK_RETURN) {
		AutoArray<TCHAR> str(::GetWindowTextLength(hWndFilter)+1);
		::GetWindowText(hWndFilter, str, ::GetWindowTextLength(hWndFilter)+1);
		filter = Text::fromT(tstring(str, ::GetWindowTextLength(hWndFilter)));
		updateList();
		return true;
	} else {
		return false;
	}
}

bool PublicHubsFrame::onContextMenu(WPARAM wParam, LPARAM lParam) {
	if((HWND)wParam == ctrlHubs.m_hWnd && (UINT)::SendMessage(ctrlHubs.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0L) == 1) {
		POINT pt = { (LONG)LOWORD(lParam), (LONG)HIWORD(lParam) };

		if(pt.x == -1 && pt.y == -1) {
			WinUtil::getListContextMenuPos(ctrlHubs.m_hWnd, pt);
		}

        ::TrackPopupMenu(hubsMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, 0, m_hWnd, NULL);

		return true;
	}

	return false;
}

void PublicHubsFrame::onCopyHub() {
	if((UINT)::SendMessage(ctrlHubs.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0L) == 1) {
		TCHAR buf[256];
		int i = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_SELECTED, 0));
		ListViewGetItemText(ctrlHubs.m_hWnd, i, COLUMN_SERVER, buf, 256);
		WinUtil::setClipboard(buf);
	}
}

void PublicHubsFrame::updateDropDown() {
	::SendMessage(hWndPubLists, CB_RESETCONTENT, 0, 0L);
	StringList lists(FavoriteManager::getInstance()->getHubLists());
	for(StringList::iterator idx = lists.begin(); idx != lists.end(); ++idx) {
		::SendMessage(hWndPubLists, CB_ADDSTRING, 0, (LPARAM)Text::toT(*idx).c_str());
	}
	::SendMessage(hWndPubLists, CB_SETCURSEL, FavoriteManager::getInstance()->getSelectedHubList(), 0L);
}

bool PublicHubsFrame::parseFilter(FilterModes& mode, double& size) {
	string::size_type start = (string::size_type)string::npos;
	string::size_type end = (string::size_type)string::npos;
	int64_t multiplier = 1;

	if(filter.compare(0, 2, ">=") == 0) {
		mode = GREATER_EQUAL;
		start = 2;
	} else if(filter.compare(0, 2, "<=") == 0) {
		mode = LESS_EQUAL;
		start = 2;
	} else if(filter.compare(0, 2, "==") == 0) {
		mode = EQUAL;
		start = 2;
	} else if(filter.compare(0, 2, "!=") == 0) {
		mode = NOT_EQUAL;
		start = 2;
	} else if(filter[0] == _T('<')) {
		mode = LESS;
		start = 1;
	} else if(filter[0] == _T('>')) {
		mode = GREATER;
		start = 1;
	} else if(filter[0] == _T('=')) {
		mode = EQUAL;
		start = 1;
	}

	if(start == string::npos)
		return false;
	if(filter.length() <= start)
		return false;

	if((end = Util::findSubString(filter, "TiB")) != tstring::npos) {
		multiplier = 1024LL * 1024LL * 1024LL * 1024LL;
	} else if((end = Util::findSubString(filter, "GiB")) != tstring::npos) {
		multiplier = 1024*1024*1024;
	} else if((end = Util::findSubString(filter, "MiB")) != tstring::npos) {
		multiplier = 1024*1024;
	} else if((end = Util::findSubString(filter, "KiB")) != tstring::npos) {
		multiplier = 1024;
	} else if((end = Util::findSubString(filter, "TB")) != tstring::npos) {
		multiplier = 1000LL * 1000LL * 1000LL * 1000LL;
	} else if((end = Util::findSubString(filter, "GB")) != tstring::npos) {
		multiplier = 1000*1000*1000;
	} else if((end = Util::findSubString(filter, "MB")) != tstring::npos) {
		multiplier = 1000*1000;
	} else if((end = Util::findSubString(filter, "kB")) != tstring::npos) {
		multiplier = 1000;
	} else if((end = Util::findSubString(filter, "B")) != tstring::npos) {
		multiplier = 1;
	}


	if(end == string::npos) {
		end = filter.length();
	}

	string tmpSize = filter.substr(start, end-start);
	size = Util::toDouble(tmpSize) * multiplier;

	return true;
}

bool PublicHubsFrame::matchFilter(const HubEntry& entry, const int& sel, bool doSizeCompare, const FilterModes& mode, const double& size) {
	if(filter.empty())
		return true;

	double entrySize = 0;
	string entryString = "";

	switch(sel) {
		case COLUMN_NAME: entryString = entry.getName(); doSizeCompare = false; break;
		case COLUMN_DESCRIPTION: entryString = entry.getDescription(); doSizeCompare = false; break;
		case COLUMN_USERS: entrySize = entry.getUsers(); break;
		case COLUMN_SERVER: entryString = entry.getServer(); doSizeCompare = false; break;
		case COLUMN_COUNTRY: entryString = entry.getCountry(); doSizeCompare = false; break;
		case COLUMN_SHARED: entrySize = (double)entry.getShared(); break;
		case COLUMN_MINSHARE: entrySize = (double)entry.getMinShare(); break;
		case COLUMN_MINSLOTS: entrySize = entry.getMinSlots(); break;
		case COLUMN_MAXHUBS: entrySize = entry.getMaxHubs(); break;
		case COLUMN_MAXUSERS: entrySize = entry.getMaxUsers(); break;
		case COLUMN_RELIABILITY: entrySize = entry.getReliability(); break;
		case COLUMN_RATING: entryString = entry.getRating(); doSizeCompare = false; break;
		default: break;
	}

	bool insert = false;
	if(doSizeCompare) {
		switch(mode) {
			case EQUAL: insert = (size == entrySize); break;
			case GREATER_EQUAL: insert = (size <= entrySize); break;
			case LESS_EQUAL: insert = (size >= entrySize); break;
			case GREATER: insert = (size < entrySize); break;
			case LESS: insert = (size > entrySize); break;
			case NOT_EQUAL: insert = (size != entrySize); break;
		}
	} else {
		if(sel >= COLUMN_LAST) {
			if( Util::findSubString(entry.getName(), filter) != string::npos ||
				Util::findSubString(entry.getDescription(), filter) != string::npos ||
				Util::findSubString(entry.getServer(), filter) != string::npos ||
				Util::findSubString(entry.getCountry(), filter) != string::npos ||
				Util::findSubString(entry.getRating(), filter) != string::npos ) {
					insert = true;
				}
		}
		if(Util::findSubString(entryString, filter) != string::npos)
			insert = true;
	}

	return insert;
}

void PublicHubsFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void PublicHubsFrame::onRefreshSettings() {
	bool refresh = false;

	if((COLORREF)::SendMessage(ctrlHubs.m_hWnd, LVM_GETBKCOLOR, 0, 0L) != WinUtil::bgColor) {
		::SendMessage(ctrlHubs.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlHubs.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
		refresh = true;
	}
	if((COLORREF)::SendMessage(ctrlHubs.m_hWnd, LVM_GETTEXTCOLOR, 0, 0L) != WinUtil::textColor) {
		::SendMessage(ctrlHubs.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
		refresh = true;
	}

	if(refresh == true) {
		::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
	}

	updateDropDown();
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
