/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef PUBLIC_HUBS_FRM_H
#define PUBLIC_HUBS_FRM_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MDIChildWindow.h"
#include "ExListViewCtrl.h"
#include "Resource.h"

#include "../client/FavoriteManager.h"
#include "../client/StringSearch.h"

#include "WinUtil.h"

class PublicHubsFrame : public MDITabChildWindow, public StaticFrame<PublicHubsFrame, ResourceManager::PUBLIC_HUBS, ID_FILE_CONNECT>, 
	private FavoriteManagerListener, private SettingsManagerListener {
public:
	PublicHubsFrame();
	~PublicHubsFrame();

    static ATOM m_aAtom;
    static UINT m_uiResourceID;

    static LRESULT CALLBACK PublicHubsFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    LRESULT PublicHubsFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
		
	bool onChar(WPARAM wParam);
	bool onFilterChar(WPARAM wParam);
	void onCreate();
	void onDoubleClickHublist(LPNMHDR pnmh);
	void onEnter();
	void onFilterFocus();
	void onAdd();
	void onClickedRefresh();
	void onClickedConnect();
	void onSpeaker(WPARAM wParam, LPARAM lParam);
	bool onContextMenu(WPARAM wParam, LPARAM lParam);
	void onCopyHub();
	bool onClose();
	void onListSelChanged();
	void onColumnClickHublist(LPNMHDR pnmh);
	void onRefreshSettings();
	
	void UpdateLayout();
	bool checkNick();

	LRESULT onCtlColor(WPARAM wParam, LPARAM lParam) {
		HWND hWnd = (HWND)lParam;
		HDC hDC = (HDC)wParam;
		if(hWnd == hWndHubAddress || hWnd == hWndFilter || hWnd == hWndPubLists || hWnd == hWndFilterSel) {
			::SetBkColor(hDC, WinUtil::bgColor);
			::SetTextColor(hDC, WinUtil::textColor);
			return (LRESULT)WinUtil::bgBrush;
		}

		return 0;
	}

	void onSetFocus() {
		::SetFocus(ctrlHubs.m_hWnd);
	}
private:
	enum {
		COLUMN_FIRST,
		COLUMN_NAME = COLUMN_FIRST,
		COLUMN_DESCRIPTION,
		COLUMN_USERS,
		COLUMN_SERVER,
		COLUMN_COUNTRY,
		COLUMN_SHARED,
		COLUMN_MINSHARE,
		COLUMN_MINSLOTS,
		COLUMN_MAXHUBS,
		COLUMN_MAXUSERS,
		COLUMN_RELIABILITY,
		COLUMN_RATING,
		COLUMN_LAST
	};

	enum {
		FINISHED,
		LOADED_FROM_CACHE,
		STARTING,
		FAILED
	};

	enum FilterModes{
		NONE,
		EQUAL,
		GREATER_EQUAL,
		LESS_EQUAL,
		GREATER,
		LESS,
		NOT_EQUAL
	};

	int visibleHubs;
	int users;

	HWND hWndConnect, hWndRefresh, hWndAddress, hWndLists, hWndFilterDesc, hWndFilter;

	HMENU hubsMenu;
	
	HWND hWndPubLists, hWndFilterSel, hWndHubAddress;

	ExListViewCtrl ctrlHubs;

	HubEntry::List hubs;
	string filter;

	bool closed;

	static int columnIndexes[];
	static int columnSizes[];

	void on(DownloadStarting, const string& l) throw() { speak(STARTING, l); }
	void on(DownloadFailed, const string& l) throw() { speak(FAILED, l); }
	void on(DownloadFinished, const string& l) throw() { speak(FINISHED, l); }
	void on(LoadedFromCache, const string& l) throw() { speak(LOADED_FROM_CACHE, l); }
	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();

	void speak(int x, const string& l) {
		::PostMessage(m_hWnd, WM_SPEAKER, x, (LPARAM)new tstring(Text::toT(l)));
	}

	void updateStatus();
	void updateList();
	void updateDropDown();
	bool parseFilter(FilterModes& mode, double& size);
	bool matchFilter(const HubEntry& entry, const int& sel, bool doSizeCompare, const FilterModes& mode, const double& size);

    static LRESULT CALLBACK FilterWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    static LRESULT CALLBACK HubAddressWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
};

#endif // PUBLIC_HUBS_FRM_H

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
