/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "PublicHubsListDlg.h"

#include "../Client/FavoriteManager.h"
#include "../client/StringTokenizer.h"
#include "LineDlg.h"

PublicHubListDlg::PublicHubListDlg() {
    m_hWnd = NULL;
}

PublicHubListDlg::~PublicHubListDlg() {
    ctrlList.m_hWnd = NULL;
}

INT_PTR CALLBACK PublicHubListDlg::PublicHubListDlgProc(HWND hWndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    PublicHubListDlg * pParent;

    if(uMsg == WM_INITDIALOG) {
        pParent = (PublicHubListDlg *)lParam;
        SetWindowLongPtr(hWndDlg, GWLP_USERDATA, (LONG_PTR)pParent);

        pParent->m_hWnd = hWndDlg;
    } else {
        pParent = (PublicHubListDlg *)GetWindowLongPtr(hWndDlg, GWLP_USERDATA);
        if(pParent == NULL) {
            return FALSE;
        }
    }

    SetWindowLongPtr(hWndDlg, DWLP_MSGRESULT, 0);

    return pParent->PublicHubListProc(hWndDlg, uMsg, wParam, lParam);
}

LRESULT PublicHubListDlg::PublicHubListProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_INITDIALOG:
            onInitDialog();
            return TRUE;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDOK:
                case IDCANCEL:
                    onCloseCmd(LOWORD(wParam));
                    return TRUE;
                case IDC_LIST_ADD:
                    onAdd();
                    return TRUE;
                case IDC_LIST_UP:
                    onMoveUp();
                    return TRUE;
                case IDC_LIST_DOWN:
                    onMoveDown();
                    return TRUE;
                case IDC_LIST_EDIT:
                    onEdit();
                    return TRUE;
                case IDC_LIST_REMOVE:
                    onRemove();
                    return TRUE;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_LIST_LIST && ((LPNMHDR)lParam)->code == LVN_ITEMCHANGED) {
                onItemchanged((LPNMHDR)lParam);
                return FALSE;
            }

            break;
    }

    return FALSE;
}

void PublicHubListDlg::onInitDialog() {
	// set the title, center the window
	::SetWindowText(m_hWnd, CTSTRING(CONFIGURED_HUB_LISTS));
	CenterWindow(m_hWnd, ::GetParent(m_hWnd));

	// fill in translatable strings
	::SetDlgItemText(m_hWnd, IDC_LIST_DESC, _T(""));
	::SetDlgItemText(m_hWnd, IDC_LIST_ADD, CTSTRING(ADD));
	::SetDlgItemText(m_hWnd, IDC_LIST_UP, CTSTRING(MOVE_UP));
	::SetDlgItemText(m_hWnd, IDC_LIST_DOWN, CTSTRING(MOVE_DOWN));
	::SetDlgItemText(m_hWnd, IDC_LIST_EDIT, CTSTRING(EDIT_ACCEL));
	::SetDlgItemText(m_hWnd, IDC_LIST_REMOVE, CTSTRING(REMOVE));

	// set up the list of lists
	RECT rc;
	ctrlList.m_hWnd = ::GetDlgItem(m_hWnd, IDC_LIST_LIST);
	::SendMessage(ctrlList.m_hWnd, LVM_SETEXTENDEDLISTVIEWSTYLE, 0, LVS_EX_LABELTIP | LVS_EX_FULLROWSELECT);
	::GetClientRect(ctrlList.m_hWnd, &rc);
	ListViewInsertColumn(ctrlList.m_hWnd, 0, CTSTRING(SETTINGS_NAME), LVCFMT_LEFT, (rc.right - rc.left) - 4, 0);
	StringList lists(FavoriteManager::getInstance()->getHubLists());
	for(StringList::iterator idx = lists.begin(); idx != lists.end(); ++idx) {
		ctrlList.insert((int)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMCOUNT, 0, 0L), Text::toT(*idx));
	}

	// set the initial focus
	::SetFocus(::GetDlgItem(m_hWnd, IDC_LIST_EDIT_BOX));
}

void PublicHubListDlg::onAdd() {
	TCHAR* buf = new TCHAR[::GetWindowTextLength(::GetDlgItem(m_hWnd, IDC_LIST_EDIT_BOX)) + 1];
	::GetWindowText(::GetDlgItem(m_hWnd, IDC_LIST_EDIT_BOX), buf, ::GetWindowTextLength(::GetDlgItem(m_hWnd, IDC_LIST_EDIT_BOX)) + 1);
	tstring contents(buf, ::GetWindowTextLength(::GetDlgItem(m_hWnd, IDC_LIST_EDIT_BOX)));
	delete[] buf;

	StringTokenizer<tstring> t(contents, ';');
	for(TStringList::reverse_iterator i = t.getTokens().rbegin(); i != t.getTokens().rend(); ++i) {
		if(!i->empty()) {
			ctrlList.insert(0, *i);
        }
    }

	::SetDlgItemText(m_hWnd, IDC_LIST_EDIT_BOX, _T(""));
}

void PublicHubListDlg::onMoveUp() {
	int j = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMCOUNT, 0, 0L);
	for(int i = 1; i < j; ++i) {
		if((UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMSTATE, i, LVIS_SELECTED)) {
			ctrlList.moveItem(i, i-1);
		    ListViewSelectItem(ctrlList.m_hWnd, i-1);
			::SetFocus(ctrlList.m_hWnd);
		}
	}
}

void PublicHubListDlg::onMoveDown() {
	int j = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMCOUNT, 0, 0L) - 2;
	for(int i = j; i >= 0; --i) {
		if((UINT)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMSTATE, i, LVIS_SELECTED)) {
			ctrlList.moveItem(i, i+1);
			ListViewSelectItem(ctrlList.m_hWnd, i+1);
			::SetFocus(ctrlList.m_hWnd);
		}
	}
}

void PublicHubListDlg::onEdit() {
	int i = -1;
	TCHAR buf[256];
	while( (i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, i, MAKELPARAM(LVNI_SELECTED, 0))) != -1) {
		LineDlg hublist;
		hublist.title = CTSTRING(HUB_LIST);
		hublist.description = CTSTRING(HUB_LIST_EDIT);
		ListViewGetItemText(ctrlList.m_hWnd, i, 0, buf, 256);
		hublist.line = tstring(buf);
		if(hublist.DoModal(m_hWnd) == IDOK) {
			ListViewSetItem(ctrlList.m_hWnd, i, 0, LVIF_TEXT, hublist.line.c_str(), 0);
		}
	}
}

void PublicHubListDlg::onRemove() {
	int i = -1;
	while((i = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_SELECTED, 0))) != -1) {
		::SendMessage(ctrlList.m_hWnd, LVM_DELETEITEM, i, 0L);
	}
}

void PublicHubListDlg::onCloseCmd(WORD wID) {
	if(wID == IDOK) {
		TCHAR buf[512];
		string tmp;
		int j = (int)::SendMessage(ctrlList.m_hWnd, LVM_GETITEMCOUNT, 0, 0L);
		for(int i = 0; i < j; ++i) {
			ListViewGetItemText(ctrlList.m_hWnd, i, 0, buf, 512);
			tmp += Text::fromT(buf) + ';';
		}
		if(j > 0) {
			tmp.erase(tmp.size()-1);
		}
		SettingsManager::getInstance()->set(SettingsManager::HUBLIST_SERVERS, tmp);
	}
	::EndDialog(m_hWnd, wID);
}

void PublicHubListDlg::onItemchanged(LPNMHDR pnmh) {
	NM_LISTVIEW* lv = (NM_LISTVIEW*)pnmh;
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_LIST_UP), (lv->uNewState & LVIS_FOCUSED));
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_LIST_DOWN), (lv->uNewState & LVIS_FOCUSED));
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_LIST_EDIT), (lv->uNewState & LVIS_FOCUSED));
	::EnableWindow(::GetDlgItem(m_hWnd, IDC_LIST_REMOVE), (lv->uNewState & LVIS_FOCUSED));
}

void PublicHubListDlg::DoModal(HWND hWndParent) {
	::DialogBoxParam(g_hInstance, MAKEINTRESOURCE(IDD_HUB_LIST), hWndParent, PublicHubListDlgProc, (LPARAM)this);
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */