/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef QUEUE_FRAME_H
#define QUEUE_FRAME_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MDIChildWindow.h"
#include "WinApiUtil.h"
#include "TypedListViewCtrl.h"
#include "SplitterWindow.h"

#include "../client/QueueManager.h"
#include "../client/TaskQueue.h"

class QueueFrame : public MDITabChildWindow, public StaticFrame<QueueFrame, ResourceManager::DOWNLOAD_QUEUE, IDC_QUEUE>, private QueueManagerListener,
    private CSplitter, private SettingsManagerListener {
public:
	QueueFrame();
    ~QueueFrame();

    static ATOM m_aAtom;
    static UINT m_uiResourceID;

    static LRESULT CALLBACK QueueFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    LRESULT QueueFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void onPriority(WORD wID);
	void onBrowseList(WORD wID);
	void onRemoveSource(WORD wID);
	void onRemoveSources(WORD wID);
	void onPM(WORD wID);
	void onReadd(WORD wID);
	void onSearchAlternates();
	bool onContextMenu(WPARAM wParam, LPARAM lParam);
	void onSpeaker();
	void OnCreate();
	bool onClose();
	bool onKeyDown(LPNMHDR pnmh);
	void onPreviewCommand(WORD wID);
	LRESULT onCustomDraw(LPNMHDR pnmh, bool& bHandled);
	void onRemoveOffline();
	void onClearErrors();
	void onCopy(WORD wID);
	void onRefreshSettings();
	void UpdateLayout(bool bResizeBars = true);
	void removeDir(HTREEITEM ht);
	void setPriority(HTREEITEM ht, const QueueItem::Priority& p);
	void changePriority(bool inc);
	void clearErrors(HTREEITEM ht);

	void onItemChangedQueue(LPNMHDR pnmh) {
		NMLISTVIEW* lv = (NMLISTVIEW*)pnmh;
		if((lv->uNewState & LVIS_SELECTED) != (lv->uOldState & LVIS_SELECTED)) {
			updateStatus();
        }
	}

	void onSetFocus() {
		::SetFocus(ctrlQueue.m_hWnd);
	}

	void onRemove() {
		if(BOOLSETTING(CONFIRM_ITEM_REMOVAL) == false ||
			::MessageBox(m_hWnd, CTSTRING(CONFIRM_REMOVE), _T(CzDC_VERSION_STRING), MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2) == IDYES) {
			usingDirMenu ? removeSelectedDir() : removeSelected();
        }
	}

	void onMove() {
		usingDirMenu ? moveSelectedDir() : moveSelected();
	}

	bool onKeyDownDirs(LPNMHDR pnmh) {
		NMTVKEYDOWN* kd = (NMTVKEYDOWN*) pnmh;
		if(kd->wVKey == VK_DELETE) {
			if(BOOLSETTING(CONFIRM_ITEM_REMOVAL) == false ||
				::MessageBox(m_hWnd, CTSTRING(CONFIRM_REMOVE), _T(CzDC_VERSION_STRING), MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2) == IDYES) {
				removeSelectedDir();
            }
            return true;
		} else if(kd->wVKey == VK_TAB) {
			onTab();
			return true;
		}

		return false;
	}

	void onTab();

	void onShowTree(WPARAM wParam) {
		showTree = (wParam == BST_CHECKED);
		UpdateLayout(false);
	}
private:

	enum {
		COLUMN_FIRST,
		COLUMN_TARGET = COLUMN_FIRST,
		COLUMN_STATUS,
		COLUMN_SIZE,
		COLUMN_DOWNLOADED,
		COLUMN_PRIORITY,
		COLUMN_USERS,
		COLUMN_PATH,
		COLUMN_EXACT_SIZE,
		COLUMN_ERRORS,
		COLUMN_ADDED,
		COLUMN_TTH,
		COLUMN_TYPE,
		COLUMN_LAST
	};
	enum Tasks {
		ADD_ITEM,
		REMOVE_ITEM,
		UPDATE_ITEM
	};

	class QueueItemInfo;
	friend class QueueItemInfo;

	class QueueItemInfo: public Flags<uint16_t> {
	public:
		struct Display {
			tstring columns[COLUMN_LAST];
		};

		enum {
			MASK_TARGET = 1 << COLUMN_TARGET,
			MASK_STATUS = 1 << COLUMN_STATUS,
			MASK_SIZE = 1 << COLUMN_SIZE,
			MASK_DOWNLOADED = 1 << COLUMN_DOWNLOADED,
			MASK_PRIORITY = 1 << COLUMN_PRIORITY,
			MASK_USERS = 1 << COLUMN_USERS,
			MASK_PATH = 1 << COLUMN_PATH,
			MASK_ERRORS = 1 << COLUMN_ERRORS,
			MASK_ADDED = 1 << COLUMN_ADDED,
			MASK_TTH = 1 << COLUMN_TTH,
			MASK_TYPE = 1 << COLUMN_TYPE
		};

		QueueItemInfo(const QueueItem& aQI) : Flags(aQI), target(aQI.getTarget()),
			path(Util::getFilePath(aQI.getTarget())), size(aQI.getSize()), 
			downloadedBytes(aQI.getDownloadedBytes()), added(aQI.getAdded()), 
			tth(aQI.getTTH()), priority(aQI.getPriority()), status(aQI.getStatus()),
			updateMask((uint16_t)-1), display(NULL), sources(aQI.getSources()), badSources(aQI.getBadSources()) {
		}

		~QueueItemInfo() { if(display != NULL) delete display; }

		void update();

		void remove() { QueueManager::getInstance()->remove(getTarget()); }

		// TypedListViewCtrl functions
		const TCHAR* getText(const int &col, TCHAR* buf) {
            return getDisplay()->columns[col].c_str();
		}

		const tstring& getErrTxt() {
            return getDisplay()->columns[COLUMN_ERRORS];
        }

		static int compareItems(QueueItemInfo* a, QueueItemInfo* b, int col) {
			switch(col) {
				case COLUMN_SIZE: case COLUMN_EXACT_SIZE: return compare(a->getSize(), b->getSize());
				case COLUMN_PRIORITY: return compare((int)a->getPriority(), (int)b->getPriority());
				case COLUMN_DOWNLOADED: return compare(a->getDownloadedBytes(), b->getDownloadedBytes());
				case COLUMN_ADDED: return compare(a->getAdded(), b->getAdded());
				default:
                    return lstrcmpi(a->getDisplay()->columns[col].c_str(), b->getDisplay()->columns[col].c_str());
			}
		}

		int imageIndex() {
			return WinUtil::getIconIndex(Text::toT(target));
		}

		const tstring& getTargetFileName() { return getDisplay()->columns[COLUMN_TARGET]; }

		QueueItem::SourceList& getSources() { return sources; }
		QueueItem::SourceList& getBadSources() { return badSources; }

		Display* getDisplay() {
			if(display == NULL) {
				display = new Display;
				update();
			}
			return display;
		}

		bool isSource(const UserPtr& u) {
			return find(sources.begin(), sources.end(), u) != sources.end();
		}
		bool isBadSource(const UserPtr& u) {
			return find(badSources.begin(), badSources.end(), u) != badSources.end();
		}

		GETSET(string, target, Target);
		GETSET(string, path, Path);
		GETSET(int64_t, size, Size);
		GETSET(int64_t, downloadedBytes, DownloadedBytes);
		GETSET(time_t, added, Added);
		GETSET(QueueItem::Priority, priority, Priority);
		GETSET(QueueItem::Status, status, Status);
		GETSET(TTHValue*, tth, TTH);
		GETSET(QueueItem::SourceList, sources, Sources);
		GETSET(QueueItem::SourceList, badSources, BadSources);
		uint16_t updateMask;

	private:

		Display* display;

		QueueItemInfo(const QueueItemInfo&);
		QueueItemInfo& operator=(const QueueItemInfo&);
	};

	struct QueueItemInfoTask {
		QueueItemInfoTask(QueueItemInfo* ii_) : ii(ii_) { }
		QueueItemInfo* ii;
	};

	struct UpdateTask {
		UpdateTask(const QueueItem& source) : target(source.getTarget()), priority(source.getPriority()), status(source.getStatus()), 
			downloadedBytes(source.getDownloadedBytes()), sources(source.getSources()), badSources(source.getBadSources()) { }

		string target;
		QueueItem::Priority priority;
		QueueItem::Status status;
		int64_t downloadedBytes;

		QueueItem::SourceList sources;
		QueueItem::SourceList badSources;
	};

	TaskQueue tasks;
	bool spoken;

	/** Single selection in the queue part */
	HMENU singleMenu;
	/** Multiple selection in the queue part */
	HMENU multiMenu;
	/** Tree part menu */
	HMENU browseMenu;

	HMENU removeMenu;
	HMENU removeAllMenu;
	HMENU pmMenu;
	HMENU readdMenu;
	HMENU dirMenu;
	HMENU previewMenu;
	HMENU copyMenu;

	int PreviewAppsSize;

	HWND hWndShowTree;

	bool showTree;
	bool usingDirMenu;
	bool dirty;

	int menuItems;
	int readdItems;

	HTREEITEM fileLists;

	typedef HASH_MULTIMAP_X(string, QueueItemInfo*, noCaseStringHash, noCaseStringEq, noCaseStringLess) DirectoryMap;
	typedef DirectoryMap::iterator DirectoryIter;
	typedef pair<DirectoryIter, DirectoryIter> DirectoryPair;
	DirectoryMap directories;
	string curDir;

	TypedListViewCtrl<QueueItemInfo, IDC_QUEUE> ctrlQueue;
	HWND hWndDirs;
	HICON hIcPaused, hIcRunning, hIcUserOffline, hIcUsersOffline, hIcWaiting, hIcNoUser;

	int statusSizes[6];

	int64_t queueSize;
	int queueItems;

	bool closed;

	static int columnIndexes[COLUMN_LAST];
	static int columnSizes[COLUMN_LAST];

	void addQueueList(const QueueItem::StringMap& l);
	void addQueueItem(QueueItemInfo* qi, bool noSort);
	HTREEITEM addDirectory(const string& dir, bool isFileList = false, HTREEITEM startAt = NULL);
	void removeDirectory(const string& dir, bool isFileList = false);
	void removeDirectories(HTREEITEM ht);

	void updateQueue();
	void updateStatus();

	/**
	 * This one is different from the others because when a lot of files are removed
	 * at the same time, the WM_SPEAKER messages seem to get lost in the handling or
	 * something, they're not correctly processed anyway...thanks windows.
	 */
	void speak(Tasks t, void * p) {
        tasks.add(t, p);
		if(spoken == false) {
			spoken = true;
			::PostMessage(m_hWnd, WM_SPEAKER, 0, 0);
		}
	}

	bool isCurDir(const string& aDir) const { return Util::stricmp(curDir, aDir) == 0; }

	void moveSelected();
	void moveSelectedDir();
	void moveDir(HTREEITEM ht, const string& target);

	void moveNode(HTREEITEM item, HTREEITEM parent);

	void clearTree(HTREEITEM item);

	QueueItemInfo* getItemInfo(const string& target);

	void removeSelected();
	void removeSelectedDir();

	const string& getSelectedDir() {
		HTREEITEM ht = (HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CARET, 0L);
		return ht == NULL ? Util::emptyString : getDir((HTREEITEM)::SendMessage(hWndDirs, TVM_GETNEXTITEM, TVGN_CARET, 0L));
	}

	const string& getDir(HTREEITEM ht) {
        dcassert(ht != NULL);
        return *(string*)(TreeGetItemData(hWndDirs, ht));
    }

	void on(QueueManagerListener::Added, QueueItem* aQI) throw();
	void on(QueueManagerListener::Moved, QueueItem* aQI, const string& oldTarget) throw();
	void on(QueueManagerListener::Removed, QueueItem* aQI) throw();
	void on(QueueManagerListener::SourcesUpdated, QueueItem* aQI) throw();
	void on(QueueManagerListener::StatusUpdated, QueueItem* aQI) throw() { on(QueueManagerListener::SourcesUpdated(), aQI); }
	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();

    static LRESULT CALLBACK ShowTreeWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
};

#endif //QUEUE_FRAME_H

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
