/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "QueuePage.h"
#include "CommandDlg.h"

#include "../client/SettingsManager.h"
#include "WinUtil.h"

QueuePage * pQueuePage = NULL;

QueuePage::QueuePage(SettingsManager *s) : PropPage(s, IDD_QUEUEPAGE) {
    pQueuePage = this;
	title = _tcsdup((TSTRING(SETTINGS_DOWNLOADS) + _T('\\') + TSTRING(SETTINGS_QUEUE)).c_str());
	SetTitle(title);
	m_psp.pfnDlgProc = (DLGPROC)QueuePageProc;
}

QueuePage::~QueuePage() {
    pQueuePage = NULL;
	free(title);
}

BOOL CALLBACK QueuePage::QueuePageProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == WM_INITDIALOG) {
        pQueuePage->m_hWnd = hWnd;
        pQueuePage->onInitDialog();
        return TRUE;
    } else if(uMsg == WM_COMMAND) {
        if(LOWORD(wParam) == IDC_ANTIVIR_BROWSE && HIWORD(wParam) == BN_CLICKED) {
            pQueuePage->onBrowse();
            return TRUE;
        }
    }

	return FALSE;
}

PropPage::TextItem QueuePage::texts[] = {
	{ IDC_SETTINGS_AUTOPRIO, ResourceManager::SETTINGS_PRIO_AUTOPRIO },
	{ IDC_SETTINGS_PRIO_HIGHEST, ResourceManager::SETTINGS_PRIO_HIGHEST },
	{ IDC_SETTINGS_KB3, ResourceManager::KB },
	{ IDC_SETTINGS_PRIO_HIGH, ResourceManager::SETTINGS_PRIO_HIGH },
	{ IDC_SETTINGS_KB4, ResourceManager::KB },
	{ IDC_SETTINGS_PRIO_NORMAL, ResourceManager::SETTINGS_PRIO_NORMAL },
	{ IDC_SETTINGS_KB5, ResourceManager::KB },
	{ IDC_SETTINGS_PRIO_LOW, ResourceManager::SETTINGS_PRIO_LOW },
	{ IDC_SETTINGS_KB6, ResourceManager::KB },
	{ IDC_SETTINGS_OTHER_QUEUE_OPTIONS, ResourceManager::SETTINGS_OTHER_QUEUE_OPTIONS },
	{ IDC_CZDC_ANTI_VIR, ResourceManager::SETCZDC_ANTI_VIR },
	{ IDC_ANTIVIR_BROWSE, ResourceManager::BROWSE },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

PropPage::Item QueuePage::items[] = {
	{ IDC_PRIO_HIGHEST_SIZE, SettingsManager::PRIO_HIGHEST_SIZE, PropPage::T_INT },
	{ IDC_PRIO_HIGH_SIZE, SettingsManager::PRIO_HIGH_SIZE, PropPage::T_INT },
	{ IDC_PRIO_NORMAL_SIZE, SettingsManager::PRIO_NORMAL_SIZE, PropPage::T_INT },
	{ IDC_PRIO_LOW_SIZE, SettingsManager::PRIO_LOW_SIZE, PropPage::T_INT },

	{ 0, 0, PropPage::T_END }
};

PropPage::ListItem QueuePage::optionItems[] = {
	{ SettingsManager::PRIO_LOWEST, ResourceManager::SETTINGS_PRIO_LOWEST },
	{ SettingsManager::AUTO_SEARCH, ResourceManager::SETTINGS_AUTO_SEARCH },
	{ SettingsManager::AUTO_SEARCH_ON_AWAY, ResourceManager::SETTINGS_AUTO_SEARCH_ON_AWAY },
	{ SettingsManager::AUTO_SEARCH_AUTO_MATCH, ResourceManager::SETTINGS_AUTO_SEARCH_AUTO_MATCH },
	{ SettingsManager::SKIP_ZERO_BYTE, ResourceManager::SETTINGS_SKIP_ZERO_BYTE },
	{ SettingsManager::DONT_DL_ALREADY_SHARED, ResourceManager::SETTINGS_DONT_DL_ALREADY_SHARED },
	{ SettingsManager::ANTI_FRAG, ResourceManager::SETTINGS_ANTI_FRAG },
	{ SettingsManager::ONLY_DL_TTH_FILES, ResourceManager::SETTINGS_ONLY_DL_TTH_FILES },
	{ SettingsManager::DONT_DL_ALREADY_QUEUED, ResourceManager::SETTINGS_DONT_DL_ALREADY_QUEUED },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

void QueuePage::onInitDialog() {
	PropPage::translate(m_hWnd, texts);
	PropPage::read(m_hWnd, items, 0, 0);
	PropPage::read(m_hWnd, items, optionItems, ::GetDlgItem(m_hWnd, IDC_OTHER_QUEUE_OPTIONS));

	// Do specialized reading here
	::SetDlgItemText(m_hWnd, IDC_ANTIVIR_PATH, Text::toT(SETTING(ANTIVIR_PATH)).c_str());
}

void QueuePage::write() {
	PropPage::write(m_hWnd, items, 0, 0);
	PropPage::write(m_hWnd, items, optionItems, ::GetDlgItem(m_hWnd, IDC_OTHER_QUEUE_OPTIONS));
	
	TCHAR buf[256];
	::GetDlgItemText(m_hWnd, IDC_ANTIVIR_PATH, buf, 256);
	settings->set(SettingsManager::ANTIVIR_PATH, Text::fromT(buf));
}

void QueuePage::onBrowse() {
	TCHAR buf[MAX_PATH];

	::GetDlgItemText(m_hWnd, IDC_ANTIVIR_PATH, buf, MAX_PATH);
	tstring x = buf;

	if(WinUtil::browseFile(x, m_hWnd, false) == IDOK) {
		::SetDlgItemText(m_hWnd, IDC_ANTIVIR_PATH, x.c_str());
	}
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
