/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2009 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "RecentsFrm.h"
#include "HubFrame.h"
#include "LineDlg.h"

#include "../client/ClientManager.h"

int RecentHubsFrame::columnIndexes[] = { COLUMN_NAME, COLUMN_DESCRIPTION, COLUMN_USERS, COLUMN_SHARED, COLUMN_SERVER };
int RecentHubsFrame::columnSizes[] = { 200, 290, 50, 50, 100 };
static ResourceManager::Strings columnNames[] = { ResourceManager::HUB_NAME, ResourceManager::DESCRIPTION, 
    ResourceManager::USERS, ResourceManager::SHARED, ResourceManager::HUB_ADDRESS
};
ATOM RecentHubsFrame::m_aAtom = 0;
UINT RecentHubsFrame::m_uiResourceID = IDR_RECENTS;

RecentHubsFrame::RecentHubsFrame() : closed(false), hubsMenu(NULL) {
    RegisterWindowClass(m_aAtom, RecentHubsFrameWndProc, _T("RecentHubsFrame"), m_uiResourceID);
}

RecentHubsFrame::~RecentHubsFrame() {
    if(hubsMenu != NULL) {
        ::DestroyMenu(hubsMenu);
    }
}

LRESULT CALLBACK RecentHubsFrame::RecentHubsFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	RecentHubsFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (RecentHubsFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (RecentHubsFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->RecentHubsFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT RecentHubsFrame::RecentHubsFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CREATE:
            onCreate();
            return 0;
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
            break;
        case WM_CONTEXTMENU:
            if(onContextMenu(wParam, lParam) == true) {
                return 0;
            }
            break;
        case WM_SETFOCUS:
            onSetFocus();
            break;
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDC_CONNECT:
                    onClickedConnect();
                    return 0;
                case IDC_ADD:
                    onAdd();
                    return 0;
                case IDC_REMOVE:
                    onRemove();
                    return 0;
                case IDC_REMOVE_ALL:
                    onRemoveAll();
                    return 0;
                case IDC_EDIT:
                    onEdit();
                    return 0;
                case IDC_OPEN_HUB_LOG:
                    onOpenHubLog();
                    return 0;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_RECENTS) {
                switch(((LPNMHDR)lParam)->code) {
                    case LVN_COLUMNCLICK:
                        onColumnClickHublist((LPNMHDR)lParam);
                        return 0;
                    case NM_DBLCLK:
                        onDoubleClickHublist((LPNMHDR)lParam);
                        return 0;
                    case NM_RETURN:
                        onEnter();
                        return 0;
                    case LVN_KEYDOWN:
                        if(onKeyDown((LPNMHDR)lParam) == true) {
                            return 0;
                        }
                        break;
                    case LVN_ITEMCHANGED:
                        onItemchanged((LPNMHDR)lParam);
                        return 0;
                }
            }

            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

    LRESULT lResult = 0;
	if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

void RecentHubsFrame::onCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(0, 0, 0), IDR_RECENTS);
    created = true;

	ctrlHubs.Create(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_SHOWSELALWAYS,
        WS_EX_CLIENTEDGE, IDC_RECENTS, LVS_EX_LABELTIP | LVS_EX_HEADERDRAGDROP | LVS_EX_FULLROWSELECT);

	// Create listview columns
	WinUtil::splitTokens(columnIndexes, SETTING(RECENTHUBSFRAME_ORDER), COLUMN_LAST);
	WinUtil::splitTokens(columnSizes, SETTING(RECENTHUBSFRAME_WIDTHS), COLUMN_LAST);
	
	for(int j=0; j<COLUMN_LAST; j++) {
		int fmt = LVCFMT_LEFT;
		ListViewInsertColumn(ctrlHubs.m_hWnd, j, CTSTRING_I(columnNames[j]), fmt, columnSizes[j], j);
	}
	
	::SendMessage(ctrlHubs.m_hWnd, LVM_SETCOLUMNORDERARRAY, COLUMN_LAST, (LPARAM)columnIndexes);

	hWndConnect = CreateWindowEx(0, _T("BUTTON"), CTSTRING(CONNECT), WS_CHILD | WS_VISIBLE | WS_DISABLED | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
        BS_PUSHBUTTON, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_CONNECT, g_hInstance, NULL);
	::SendMessage(hWndConnect, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	hWndRemove = CreateWindowEx(0, _T("BUTTON"), CTSTRING(REMOVE), WS_CHILD | WS_VISIBLE | WS_DISABLED | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
        BS_PUSHBUTTON, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_REMOVE, g_hInstance, NULL);
	::SendMessage(hWndRemove, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	hWndRemoveAll = CreateWindowEx(0, _T("BUTTON"), CTSTRING(REMOVE_ALL), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | BS_PUSHBUTTON,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_REMOVE_ALL, g_hInstance, NULL);
	::SendMessage(hWndRemoveAll, WM_SETFONT, (WPARAM)WinUtil::font, MAKELPARAM(TRUE, 0));

	FavoriteManager::getInstance()->addListener(this);
	SettingsManager::getInstance()->addListener(this);
	updateList(FavoriteManager::getInstance()->getRecentHubs());
	
	hubsMenu = ::CreatePopupMenu();
	::AppendMenu(hubsMenu, MF_STRING, IDC_OPEN_HUB_LOG, CTSTRING(OPEN_HUB_LOG));
	::AppendMenu(hubsMenu, MF_SEPARATOR, 0, NULL);
	::AppendMenu(hubsMenu, MF_STRING, IDC_CONNECT, CTSTRING(CONNECT));
	::AppendMenu(hubsMenu, MF_STRING, IDC_ADD, CTSTRING(ADD_TO_FAVORITES));
	::AppendMenu(hubsMenu, MF_STRING, IDC_EDIT, CTSTRING(PROPERTIES));
	::AppendMenu(hubsMenu, MF_STRING, IDC_REMOVE, CTSTRING(REMOVE));
	::AppendMenu(hubsMenu, MF_STRING, IDC_REMOVE_ALL, CTSTRING(REMOVE_ALL));
	::SetMenuDefaultItem(hubsMenu, IDC_CONNECT, 0);
}

bool RecentHubsFrame::onClose() {
	if(!closed) {
		FavoriteManager::getInstance()->removeListener(this);
		SettingsManager::getInstance()->removeListener(this);
		closed = true;
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);

		return true;
	} else {
		WinUtil::saveHeaderOrder(ctrlHubs.m_hWnd, SettingsManager::RECENTHUBSFRAME_ORDER,
			SettingsManager::RECENTHUBSFRAME_WIDTHS, COLUMN_LAST, columnIndexes, columnSizes);

		return false;
	}
}

void RecentHubsFrame::onDoubleClickHublist(LPNMHDR pnmh) {
	NMITEMACTIVATE* item = (NMITEMACTIVATE*)pnmh;

	if(item->iItem != -1) {
		RecentHubEntry* entry = (RecentHubEntry*)ListViewGetItem(ctrlHubs.m_hWnd, item->iItem);
		HubFrame::openWindow(Text::toT(entry->getServer()));
	}
}

void RecentHubsFrame::onEnter() {
	int item = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_FOCUSED, 0));

	if(item != -1) {
		RecentHubEntry* entry = (RecentHubEntry*)ListViewGetItem(ctrlHubs.m_hWnd, item);
		HubFrame::openWindow(Text::toT(entry->getServer()));
	}
}

void RecentHubsFrame::onAdd() {
	TCHAR buf[256];

	if((UINT)::SendMessage(ctrlHubs.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0L) == 1) {
		int i = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_SELECTED, 0));
		FavoriteHubEntry e;
		ListViewGetItemText(ctrlHubs.m_hWnd, i, COLUMN_NAME, buf, 256);
		e.setName(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, i, COLUMN_DESCRIPTION, buf, 256);
		e.setDescription(Text::fromT(buf));
		ListViewGetItemText(ctrlHubs.m_hWnd, i, COLUMN_SERVER, buf, 256);
		e.setServer(Text::fromT(buf));
		FavoriteManager::getInstance()->addFavorite(e);
	}
}

void RecentHubsFrame::onClickedConnect() {
	int i = -1;
	while( (i = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, i, MAKELPARAM(LVNI_SELECTED, 0))) != -1) {
		RecentHubEntry* entry = (RecentHubEntry*)ListViewGetItem(ctrlHubs.m_hWnd, i);
		HubFrame::openWindow(Text::toT(entry->getServer()));
	}
}

void RecentHubsFrame::onRemove() {
	int i = -1;
	while( (i = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_SELECTED, 0))) != -1) {
		FavoriteManager::getInstance()->removeRecent((RecentHubEntry*)ListViewGetItem(ctrlHubs.m_hWnd, i));
	}
}

void RecentHubsFrame::onRemoveAll() {
	::SendMessage(ctrlHubs.m_hWnd, LVM_DELETEALLITEMS, 0, 0L);
	FavoriteManager::getInstance()->removeallRecent();
}

void RecentHubsFrame::onEdit() {
	int i = -1;
	if((i = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, i, MAKELPARAM(LVNI_SELECTED, 0))) != -1) {
		RecentHubEntry* r = (RecentHubEntry*)ListViewGetItem(ctrlHubs.m_hWnd, i);
		dcassert(r != NULL);
		LineDlg dlg;
		dlg.description = TSTRING(DESCRIPTION);
		dlg.title = Text::toT(r->getName());
		dlg.line = Text::toT(r->getDescription());
		if(dlg.DoModal(m_hWnd) == IDOK) {
			r->setDescription(Text::fromT(dlg.line));
			ListViewSetItem(ctrlHubs.m_hWnd, i, COLUMN_DESCRIPTION, LVIF_TEXT, Text::toT(r->getDescription()).c_str(), 0);
			FavoriteManager::getInstance()->recentsave();
		}
	}
}

void RecentHubsFrame::onOpenHubLog() {
	if((UINT)::SendMessage(ctrlHubs.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0L) == 1) {
		int i = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_SELECTED, 0));
		RecentHubEntry* entry = (RecentHubEntry*)ListViewGetItem(ctrlHubs.m_hWnd, i);
		StringMap params;
		params["hubNI"] = entry->getName();
		params["hubURL"] = entry->getServer();
		tstring filename = Text::toT(Util::validateFileName(SETTING(LOG_DIRECTORY) + Util::formatParams(SETTING(LOG_FILE_MAIN_CHAT), params, true)));
		if(File::FileExist(Text::fromT(filename))){
			ShellExecute(NULL, NULL, filename.c_str(), NULL, NULL, SW_SHOWNORMAL);
		} else {
			::MessageBox(m_hWnd, CTSTRING(NO_LOG_FOR_HUB), CTSTRING(NO_LOG_FOR_HUB), MB_OK);
		}
	}
}

void RecentHubsFrame::onRefreshSettings() {
	bool refresh = false;

	if((COLORREF)::SendMessage(ctrlHubs.m_hWnd, LVM_GETBKCOLOR, 0, 0L) != WinUtil::bgColor) {
		::SendMessage(ctrlHubs.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlHubs.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
		refresh = true;
	}
	if((COLORREF)::SendMessage(ctrlHubs.m_hWnd, LVM_GETTEXTCOLOR, 0, 0L) != WinUtil::textColor) {
		::SendMessage(ctrlHubs.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
		refresh = true;
	}

	if(refresh == true) {
		::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
	}
}

void RecentHubsFrame::onItemchanged(LPNMHDR pnmh) {
	NM_LISTVIEW* lv = (NM_LISTVIEW*)pnmh;
	::EnableWindow(hWndConnect, (lv->uNewState & LVIS_FOCUSED));
	::EnableWindow(hWndRemove, (lv->uNewState & LVIS_FOCUSED));
}

void RecentHubsFrame::UpdateLayout() {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);
	// position bars and offset their dimensions
	UpdateBarsPosition(rect);

	RECT rc = rect;
	rc.bottom -=28;
	::MoveWindow(ctrlHubs.m_hWnd, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	const long bwidth = 90;
	const long bspace = 10;

	rc = rect;
	rc.bottom -= 2;
	rc.top = rc.bottom - 22;

	rc.left = 2;
	rc.right = rc.left + bwidth;
	::MoveWindow(hWndConnect, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	::OffsetRect(&rc, bspace + bwidth + 2, 0);
	::MoveWindow(hWndRemove, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

	::OffsetRect(&rc, bwidth + 2, 0);
	::MoveWindow(hWndRemoveAll, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
}

bool RecentHubsFrame::onContextMenu(WPARAM wParam, LPARAM lParam) {
	if((HWND)wParam == ctrlHubs.m_hWnd && (UINT)::SendMessage(ctrlHubs.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0L) > 0) {
		POINT pt = { (LONG)LOWORD(lParam), (LONG)HIWORD(lParam) };        // location of mouse click
	    if(pt.x == -1 && pt.y == -1) {
			WinUtil::getListContextMenuPos(ctrlHubs.m_hWnd, pt);
        }

        ::TrackPopupMenu(hubsMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, 0, m_hWnd, NULL);

        return true;
	}

	return false;
}

void RecentHubsFrame::onSetFocus() {
	::SetFocus(ctrlHubs.m_hWnd);
}

void RecentHubsFrame::onColumnClickHublist(LPNMHDR pnmh) {
	NMLISTVIEW* l = (NMLISTVIEW*)pnmh;
	if(l->iSubItem == ctrlHubs.getSortColumn()) {
		if(!ctrlHubs.isAscending()) {
			ctrlHubs.setSort(-1, ctrlHubs.getSortType());
		} else {
			ctrlHubs.setSortDirection(false);
        }
	} else {
		if(l->iSubItem == 2 || l->iSubItem == 3) {
			ctrlHubs.setSort(l->iSubItem, ExListViewCtrl::SORT_INT);
		} else {
			ctrlHubs.setSort(l->iSubItem, ExListViewCtrl::SORT_STRING_NOCASE);
		}
	}
}

void RecentHubsFrame::updateList(const RecentHubEntry::List& fl) {
	::SendMessage(ctrlHubs.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
	for(RecentHubEntry::List::const_iterator i = fl.begin(); i != fl.end(); ++i) {
		addEntry(*i, (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMCOUNT, 0, 0L));
	}
	::SendMessage(ctrlHubs.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
	::InvalidateRect(ctrlHubs.m_hWnd, NULL, TRUE);
}

void RecentHubsFrame::addEntry(const RecentHubEntry* entry, int pos) {
	TStringList l;
	l.push_back(Text::toT(entry->getName()));
	l.push_back(Text::toT(entry->getDescription()));
	l.push_back(Text::toT(entry->getUsers()));
	l.push_back(Text::toT(Util::formatBytes(entry->getShared())));
	l.push_back(Text::toT(entry->getServer()));

	ctrlHubs.insert(pos, l, 0, (LPARAM)entry);
}

bool RecentHubsFrame::onKeyDown(LPNMHDR pnmh) {
	NMLVKEYDOWN* kd = (NMLVKEYDOWN*)pnmh;
	if(kd->wVKey == VK_DELETE) {
		int i = -1;
		while( (i = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_SELECTED, 0))) != -1) {
			FavoriteManager::getInstance()->removeRecent((RecentHubEntry*)ListViewGetItem(ctrlHubs.m_hWnd, i));
		}
		return true;
	}

	return false;
}

void RecentHubsFrame::on(RecentAdded, const RecentHubEntry* entry) throw() {
	addEntry(entry, (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMCOUNT, 0, 0L));
}

void RecentHubsFrame::on(RecentRemoved, const RecentHubEntry* entry) throw() {
	::SendMessage(ctrlHubs.m_hWnd, LVM_DELETEITEM, ctrlHubs.find((LPARAM)entry), 0L);
}

void RecentHubsFrame::on(RecentUpdated, const RecentHubEntry* entry) throw() {
	int i = -1;
	if((i = ctrlHubs.find((LPARAM)entry)) != -1) {
        ListViewSetItem(ctrlHubs.m_hWnd, i, COLUMN_NAME, LVIF_TEXT, Text::toT(entry->getName()).c_str(), 0);
		ListViewSetItem(ctrlHubs.m_hWnd, i, COLUMN_DESCRIPTION, LVIF_TEXT, Text::toT(entry->getDescription()).c_str(), 0);
		ListViewSetItem(ctrlHubs.m_hWnd, i, COLUMN_USERS, LVIF_TEXT, Text::toT(entry->getUsers()).c_str(), 0);
		ListViewSetItem(ctrlHubs.m_hWnd, i, COLUMN_SHARED, LVIF_TEXT, Text::toT(Util::formatBytes(entry->getShared())).c_str(), 0);
		ListViewSetItem(ctrlHubs.m_hWnd, i, COLUMN_SERVER, LVIF_TEXT, Text::toT(entry->getServer()).c_str(), 0);
	}
}

void RecentHubsFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}
