/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "SearchFrm.h"
#include "LineDlg.h"
#include "FlatTabCtrl.h"
#include "CountryFlags.h"

#include "../client/QueueManager.h"
#include "../client/StringTokenizer.h"
#include "../client/ClientManager.h"
#include "../client/TimerManager.h"

TStringList SearchFrame::lastSearches;
SearchFrame::FrameMap SearchFrame::frames;
int64_t SearchFrame::resultsCount = 0;

int SearchFrame::columnIndexes[] = { COLUMN_FILENAME, COLUMN_HITS, COLUMN_NICK, COLUMN_TYPE, COLUMN_SIZE,
	COLUMN_PATH, COLUMN_SLOTS, COLUMN_CONNECTION, COLUMN_HUB, COLUMN_EXACT_SIZE, COLUMN_IP, COLUMN_TTH
};
int SearchFrame::columnSizes[] = { 200, 80, 100, 50, 80, 100, 40, 70, 150, 80, 100, 125 };
static ResourceManager::Strings columnNames[] = { ResourceManager::FILE, ResourceManager::HIT_COUNT, ResourceManager::USER, ResourceManager::TYPE, ResourceManager::SIZE,
	ResourceManager::PATH, ResourceManager::SLOTS, ResourceManager::CONNECTION,
	ResourceManager::HUB, ResourceManager::EXACT_SIZE, ResourceManager::COUNTRY, ResourceManager::TTH_ROOT
};
ATOM SearchFrame::m_aAtom = 0;
UINT SearchFrame::m_uiResourceID = IDR_SEARCH;
static WNDPROC DefButtonProc = NULL;
static WNDPROC DefComboBoxProc = NULL;
static WNDPROC DefEditProc = NULL;

SearchFrame::SearchFrame() :
	initialSize(0), initialMode(SearchManager::SIZE_ATLEAST), 
    initialType(SearchManager::TYPE_ANY), showUI(true), onlyFree(BOOLSETTING(FREE_SLOTS_DEFAULT)), closed(false), isHash(false), droppedResults(0),
    onlyTTH(BOOLSETTING(SEARCH_ONLY_TTH)), timerID(0), bWarned(false), expandSR(BOOLSETTING(SEARCH_EXPAND)), resultsMenu(NULL), targetMenu(NULL),
    useGrouping(BOOLSETTING(GROUP_SEARCH_RESULTS)), targetDirMenu(NULL), needresort(false) {
	TimerManager::getInstance()->addListener(this);
	SearchManager::getInstance()->addListener(this);

    RegisterWindowClass(m_aAtom, SearchFrameWndProc, _T("SearchFrame"), m_uiResourceID);
}

SearchFrame::~SearchFrame() {
    if(resultsMenu != NULL) {
        ::DestroyMenu(resultsMenu);
    }
}

LRESULT CALLBACK SearchFrame::SearchFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	SearchFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (SearchFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (SearchFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->SearchFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT SearchFrame::SearchFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_CREATE:
            onCreate();
            return 0;
        case WM_SETFOCUS:
            onFocus();
            break;
        case WM_CONTEXTMENU:
            if(onContextMenu(wParam, lParam) == true) {
                return 0;
            }
            break;
        case WM_SPEAKER:
            onSpeaker(wParam, lParam);
            return 0;
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case WM_CTLCOLOREDIT:
        case WM_CTLCOLORSTATIC:
        case WM_CTLCOLORLISTBOX:
            return onCtlColor(wParam, lParam);
        case WM_CLOSE:
            if(onClose() == true) {
                return 0;
            }
            break;
        case WM_TIMER:
            onTimer();
            return 0;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDC_DOWNLOAD:
                    onDownload();
                    return 0;
                case IDC_DOWNLOADTO:
                    onDownloadTo();
                    return 0;
                case IDC_DOWNLOADDIR:
                    onDownloadWhole();
                    return 0;
                case IDC_DOWNLOADDIRTO:
                    onDownloadWholeTo();
                    return 0;
                case IDC_VIEW_AS_TEXT:
                    onViewAsText();
                    return 0;
                case IDC_REMOVE:
                    onRemove();
                    return 0;
                case IDC_SEARCH:
                    onEnter();
                    return 0;
                case IDC_SEARCH_PAUSE:
                    onPause();
                    return 0;
                case IDC_FREESLOTS:
                    onFreeSlots();
                    return 0;
                case IDC_ONLYTTH:
                    onTTH();
                    return 0;
                case IDC_GETLIST:
                    onGetList();
                    return 0;
                case IDC_SEARCH_ALTERNATES:
                    onSearchByTTH();
                    return 0;
                case IDC_COLLAPSED:
                    onCollapsed();
                    return 0;
                case IDC_COPY_NICK:
                case IDC_COPY_FILENAME:
                case IDC_COPY_PATH:
                case IDC_COPY_SIZE:
                case IDC_COPY_LINK:
                case IDC_COPY_TTH:
                    onCopy(LOWORD(wParam));
                    return 0;
            }

            if(LOWORD(wParam) >= IDC_DOWNLOAD_FAVORITE_DIRS && LOWORD(wParam) <= IDC_DOWNLOAD_FAVORITE_DIRS + FavoriteManager::getInstance()->getFavoriteDirs().size()) {
                onDownloadFavoriteDirs(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS && LOWORD(wParam) <= IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS + FavoriteManager::getInstance()->getFavoriteDirs().size()) {
                onDownloadWholeFavoriteDirs(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_DOWNLOAD_TARGET && LOWORD(wParam) <= IDC_DOWNLOAD_TARGET + targets.size() + WinUtil::lastDirs.size()) {
                onDownloadTarget(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_DOWNLOAD_WHOLE_TARGET && LOWORD(wParam) <= IDC_DOWNLOAD_WHOLE_TARGET + WinUtil::lastDirs.size()) {
                onDownloadWholeTarget(LOWORD(wParam));
                return 0;
            } else if(LOWORD(wParam) >= IDC_PRIORITY_PAUSED && LOWORD(wParam) <= IDC_PRIORITY_HIGHEST) {
                onDownloadWithPrio(LOWORD(wParam));
                return 0;
            }

            break;
        case WM_NOTIFY:
            if(((LPNMHDR)lParam)->idFrom == IDC_RESULTS) {
                switch(((LPNMHDR)lParam)->code) {
                    case LVN_GETDISPINFO: {
                        ctrlResults.onGetDispInfo((LPNMHDR)lParam);
                        return 0;
                    }
                    case LVN_COLUMNCLICK: {
                        ctrlResults.onColumnClick((LPNMHDR)lParam);
                        return 0;
					}
                    case NM_DBLCLK:
                        onDoubleClickResults((LPNMHDR)lParam);
                        return 0;
                    case LVN_KEYDOWN:
                        if(onKeyDown((LPNMHDR)lParam) == true) {
                            return 0;
                        }
                        break;
                    case NM_CUSTOMDRAW:
                        return onCustomDraw((LPNMHDR)lParam);
                }
            } else if(((LPNMHDR)lParam)->idFrom == IDC_HUB) {
                if(((LPNMHDR)lParam)->code == LVN_GETDISPINFO) {
                    ctrlHubs.onGetDispInfo((LPNMHDR)lParam);
                    return 0;
                } else if(((LPNMHDR)lParam)->code == LVN_ITEMCHANGED) {
                    onItemChangedHub((LPNMHDR)lParam);
                    return 0;
                }
            }

            break;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

	if(UCHandlerProc(uMsg, wParam) == true) {
		return 0;
	}

	if(UserInfoBaseHandlerProc(uMsg, wParam) == true) {
		return 0;
    }

	LRESULT lResult = 0;
	if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK SearchFrame::SearchWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    SearchFrame * pParent = (SearchFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
    if(pParent == NULL) { // should never happen, but ... :-D
        return 0;
    }

    switch(uMsg) {
        case WM_CHAR:
        case WM_KEYDOWN:
        case WM_KEYUP:
            if(pParent->onChar(uMsg, wParam) == true) {
                return 0;
            }
    }

    if(hWnd == pParent->ctrlResults.m_hWnd) {
        return pParent->ctrlResults.TypedTreeListViewProc(hWnd, uMsg, wParam, lParam);
    } else if(hWnd == pParent->ctrlHubs.m_hWnd) {
		return pParent->ctrlHubs.TypedListViewProc(hWnd, uMsg, wParam, lParam);
    } else if(hWnd == pParent->hWndDoSearch || hWnd == pParent->hWndPauseSearch || hWnd == pParent->hWndSlots || hWnd == pParent->hWndTTH ||
        hWnd == pParent->hWndCollapsed) {
        return DefButtonProc(hWnd, uMsg, wParam, lParam);
    } else if(hWnd == pParent->hWndSearchBox || hWnd == pParent->hWndMode || hWnd == pParent->hWndSizeMode || hWnd == pParent->hWndFiletype) {
        return DefComboBoxProc(hWnd, uMsg, wParam, lParam);
    } else if(hWnd == pParent->hWndSearch || hWnd == pParent->hWndSize) {
        return DefEditProc(hWnd, uMsg, wParam, lParam);
    }

    return 0;
}

LRESULT CALLBACK SearchFrame::ShowUIWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == BM_SETCHECK) {
		SearchFrame * pParent = (SearchFrame *)::GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent != NULL) {
			pParent->onShowUI(wParam);
        }
    }

    return DefButtonProc(hWnd, uMsg, wParam, lParam);
}


void SearchFrame::openWindow(const tstring& str /* = Util::emptyString */, LONGLONG size /* = 0 */, SearchManager::SizeModes mode /* = SearchManager::SIZE_ATLEAST */, SearchManager::TypeModes type /* = SearchManager::TYPE_ANY ( 0 ) */) {
	SearchFrame* pChild = new SearchFrame();
	pChild->setInitial(str, size, mode, type);
	pChild->CreateEx(pChild, m_aAtom, m_uiResourceID);

	frames.insert(FramePair(pChild->m_hWnd, pChild));
}

void SearchFrame::onCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(127, 127, 255), IDR_SEARCH);
    created = true;

    m_hWndStatusBar = CreateWindowEx(0, STATUSCLASSNAME, _T(""), WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | SBARS_SIZEGRIP,
      0, 0, 0, 0, m_hWnd, (HMENU)NULL, g_hInstance, NULL);

    hWndSearchBox = CreateWindowEx(0, _T("COMBOBOX"), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
		WS_VSCROLL | CBS_DROPDOWN | CBS_AUTOHSCROLL, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
    ::SendMessage(hWndSearchBox, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
    ::SendMessage(hWndSearchBox, CB_SETEXTENDEDUI, TRUE, 0L);

	if(lastSearches.size() == 0) {
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_01)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_02)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_03)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_04)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_05)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_06)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_07)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_08)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_09)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_10)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_11)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_12)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_13)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_14)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_15)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_16)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_17)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_18)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_19)));
		lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_20)));
	}

	for(TStringIter i = lastSearches.begin(); i != lastSearches.end(); ++i) {
		::SendMessage(hWndSearchBox, CB_INSERTSTRING, 0, (LPARAM)i->c_str());
	}

    ::SetWindowLongPtr(hWndSearchBox, GWLP_USERDATA, (LONG_PTR)this);
    DefComboBoxProc = (WNDPROC)::SetWindowLongPtr(hWndSearchBox, GWLP_WNDPROC, (LONG_PTR)SearchWndProc);
	
    hWndMode = CreateWindowEx(WS_EX_CLIENTEDGE, _T("COMBOBOX"), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
		WS_HSCROLL | WS_VSCROLL | CBS_DROPDOWNLIST, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
    ::SendMessage(hWndMode, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
    ::SetWindowLongPtr(hWndMode, GWLP_USERDATA, (LONG_PTR)this);
    DefComboBoxProc = (WNDPROC)::SetWindowLongPtr(hWndMode, GWLP_WNDPROC, (LONG_PTR)SearchWndProc);

	::SendMessage(hWndMode, CB_ADDSTRING, 0, (LPARAM)CTSTRING(NORMAL));
	::SendMessage(hWndMode, CB_ADDSTRING, 0, (LPARAM)CTSTRING(AT_LEAST));
	::SendMessage(hWndMode, CB_ADDSTRING, 0, (LPARAM)CTSTRING(AT_MOST));
	::SendMessage(hWndMode, CB_SETCURSEL, 1, 0L);

    hWndSize = CreateWindowEx(WS_EX_CLIENTEDGE, _T("EDIT"), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
		ES_AUTOHSCROLL | ES_NUMBER, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
	::SendMessage(hWndSize, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
    ::SetWindowLongPtr(hWndSize, GWLP_USERDATA, (LONG_PTR)this);
    DefEditProc = (WNDPROC)::SetWindowLongPtr(hWndSize, GWLP_WNDPROC, (LONG_PTR)SearchWndProc);
	
    hWndSizeMode = CreateWindowEx(WS_EX_CLIENTEDGE, _T("COMBOBOX"), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
		WS_HSCROLL | WS_VSCROLL | CBS_DROPDOWNLIST, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
    ::SendMessage(hWndSizeMode, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
    ::SetWindowLongPtr(hWndSizeMode, GWLP_USERDATA, (LONG_PTR)this);
    DefComboBoxProc = (WNDPROC)::SetWindowLongPtr(hWndSizeMode, GWLP_WNDPROC, (LONG_PTR)SearchWndProc);

	::SendMessage(hWndSizeMode, CB_ADDSTRING, 0, (LPARAM)CTSTRING(B));
	::SendMessage(hWndSizeMode, CB_ADDSTRING, 0, (LPARAM)CTSTRING(KB));
	::SendMessage(hWndSizeMode, CB_ADDSTRING, 0, (LPARAM)CTSTRING(MB));
	::SendMessage(hWndSizeMode, CB_ADDSTRING, 0, (LPARAM)CTSTRING(GB));

    hWndFiletype = CreateWindowEx(WS_EX_CLIENTEDGE, _T("COMBOBOX"), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
		WS_HSCROLL | WS_VSCROLL | CBS_DROPDOWNLIST, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
    ::SendMessage(hWndFiletype, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
    ::SetWindowLongPtr(hWndFiletype, GWLP_USERDATA, (LONG_PTR)this);
    DefComboBoxProc = (WNDPROC)::SetWindowLongPtr(hWndFiletype, GWLP_WNDPROC, (LONG_PTR)SearchWndProc);

	::SendMessage(hWndFiletype, CB_ADDSTRING, 0, (LPARAM)CTSTRING(ANY));
	::SendMessage(hWndFiletype, CB_ADDSTRING, 0, (LPARAM)CTSTRING(AUDIO));
	::SendMessage(hWndFiletype, CB_ADDSTRING, 0, (LPARAM)CTSTRING(COMPRESSED));
	::SendMessage(hWndFiletype, CB_ADDSTRING, 0, (LPARAM)CTSTRING(DOCUMENT));
	::SendMessage(hWndFiletype, CB_ADDSTRING, 0, (LPARAM)CTSTRING(EXECUTABLE));
	::SendMessage(hWndFiletype, CB_ADDSTRING, 0, (LPARAM)CTSTRING(PICTURE));
	::SendMessage(hWndFiletype, CB_ADDSTRING, 0, (LPARAM)CTSTRING(VIDEO));
	::SendMessage(hWndFiletype, CB_ADDSTRING, 0, (LPARAM)CTSTRING(DIRECTORY));
	::SendMessage(hWndFiletype, CB_ADDSTRING, 0, (LPARAM)_T("TTH"));
	::SendMessage(hWndFiletype, CB_SETCURSEL, SETTING(LAST_SEARCH_TYPE), 0L);

	ctrlResults.Create(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | 
		WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SHAREIMAGELISTS, WS_EX_CLIENTEDGE, IDC_RESULTS, 
		LVS_EX_LABELTIP | LVS_EX_HEADERDRAGDROP | LVS_EX_FULLROWSELECT);

	::SendMessage(ctrlResults.m_hWnd, LVM_SETIMAGELIST, LVSIL_SMALL, (LPARAM)WinUtil::fileImages);
    ::SetWindowLongPtr(ctrlResults.m_hWnd, GWLP_USERDATA, (LONG_PTR)this);
    ::SetWindowLongPtr(ctrlResults.m_hWnd, GWLP_WNDPROC, (LONG_PTR)SearchWndProc);

	ctrlHubs.Create(m_hWnd, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | 
		WS_HSCROLL | WS_VSCROLL | LVS_REPORT | LVS_NOCOLUMNHEADER, WS_EX_CLIENTEDGE, IDC_HUB, 
		LVS_EX_LABELTIP | LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT, false);

    ::SetWindowLongPtr(ctrlHubs.m_hWnd, GWLP_USERDATA, (LONG_PTR)this);
    ::SetWindowLongPtr(ctrlHubs.m_hWnd, GWLP_WNDPROC, (LONG_PTR)SearchWndProc);

	hWndSearchLabel = CreateWindowEx(0, _T("STATIC"), CTSTRING(SEARCH_FOR), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
	::SendMessage(hWndSearchLabel, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	hWndSizeLabel = CreateWindowEx(0, _T("STATIC"), CTSTRING(SIZE), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
	::SendMessage(hWndSizeLabel, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	hWndTypeLabel = CreateWindowEx(0, _T("STATIC"), CTSTRING(FILE_TYPE), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
	::SendMessage(hWndTypeLabel, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	hWndOptionLabel = CreateWindowEx(0, _T("STATIC"), CTSTRING(SEARCH_OPTIONS), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
	::SendMessage(hWndOptionLabel, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

	hWndHubsLabel = CreateWindowEx(0, _T("STATIC"), CTSTRING(HUBS), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);
	::SendMessage(hWndHubsLabel, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));

    hWndSlots = CreateWindowEx(0, _T("BUTTON"), CTSTRING(ONLY_FREE_SLOTS), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_FREESLOTS, g_hInstance, NULL);
	::SendMessage(hWndSlots, BM_SETSTYLE, BS_AUTOCHECKBOX, (LPARAM)FALSE);
	::SendMessage(hWndSlots, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
	::SendMessage(hWndSlots, BM_SETCHECK, (onlyFree == true ? BST_CHECKED : BST_UNCHECKED), 0L);
    ::SetWindowLongPtr(hWndSlots, GWLP_USERDATA, (LONG_PTR)this);
    DefButtonProc = (WNDPROC)::SetWindowLongPtr(hWndSlots, GWLP_WNDPROC, (LONG_PTR)SearchWndProc);

    hWndTTH = CreateWindowEx(0, _T("BUTTON"), CTSTRING(ONLY_TTH), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_ONLYTTH, g_hInstance, NULL);
	::SendMessage(hWndTTH, BM_SETSTYLE, BS_AUTOCHECKBOX, (LPARAM)FALSE);
	::SendMessage(hWndTTH, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
	::SendMessage(hWndTTH, BM_SETCHECK, (onlyTTH == true ? BST_CHECKED : BST_UNCHECKED), 0L);
    ::SetWindowLongPtr(hWndTTH, GWLP_USERDATA, (LONG_PTR)this);
    DefButtonProc = (WNDPROC)::SetWindowLongPtr(hWndTTH, GWLP_WNDPROC, (LONG_PTR)SearchWndProc);

    hWndCollapsed = CreateWindowEx(0, _T("BUTTON"), CTSTRING(EXPANDED_RESULTS), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_COLLAPSED, g_hInstance, NULL);
	::SendMessage(hWndCollapsed, BM_SETSTYLE, BS_AUTOCHECKBOX, (LPARAM)FALSE);
	::SendMessage(hWndCollapsed, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
	::SendMessage(hWndCollapsed, BM_SETCHECK, (expandSR == true ? BST_CHECKED : BST_UNCHECKED), 0L);
    ::SetWindowLongPtr(hWndCollapsed, GWLP_USERDATA, (LONG_PTR)this);
    DefButtonProc = (WNDPROC)::SetWindowLongPtr(hWndCollapsed, GWLP_WNDPROC, (LONG_PTR)SearchWndProc);

    hWndShowUI = CreateWindowEx(0, _T("BUTTON"), _T("+/-"), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWndStatusBar, 0U, g_hInstance, NULL);
	::SendMessage(hWndShowUI, BM_SETSTYLE, BS_AUTOCHECKBOX, (LPARAM)FALSE);
	::SendMessage(hWndShowUI, BM_SETCHECK, BST_CHECKED, 0L);
    ::SetWindowLongPtr(hWndShowUI, GWLP_USERDATA, (LONG_PTR)this);
    DefButtonProc = (WNDPROC)::SetWindowLongPtr(hWndShowUI, GWLP_WNDPROC, (LONG_PTR)ShowUIWndProc);

    hWndDoSearch = CreateWindowEx(0, _T("BUTTON"), CTSTRING(SEARCH), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | BS_PUSHBUTTON,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_SEARCH, g_hInstance, NULL);
	::SendMessage(hWndDoSearch, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
    ::SetWindowLongPtr(hWndDoSearch, GWLP_USERDATA, (LONG_PTR)this);
    DefButtonProc = (WNDPROC)::SetWindowLongPtr(hWndDoSearch, GWLP_WNDPROC, (LONG_PTR)SearchWndProc);

	hWndPauseSearch = CreateWindowEx(0, _T("BUTTON"), CTSTRING(PAUSE_SEARCH), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
		BS_PUSHBUTTON, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, (HMENU)IDC_SEARCH_PAUSE, g_hInstance, NULL);
	::SendMessage(hWndPauseSearch, WM_SETFONT, (WPARAM)WinUtil::systemFont, MAKELPARAM(TRUE, 0));
    ::SetWindowLongPtr(hWndPauseSearch, GWLP_USERDATA, (LONG_PTR)this);
    DefButtonProc = (WNDPROC)::SetWindowLongPtr(hWndPauseSearch, GWLP_WNDPROC, (LONG_PTR)SearchWndProc);

	if(initialSize == 0) {
		::SendMessage(hWndSizeMode, CB_SETCURSEL, 2, 0L);
	} else {
		::SendMessage(hWndSizeMode, CB_SETCURSEL, 0, 0L);
    }

	// Create listview columns
	WinUtil::splitTokens(columnIndexes, SETTING(SEARCHFRAME_ORDER), COLUMN_LAST);
	WinUtil::splitTokens(columnSizes, SETTING(SEARCHFRAME_WIDTHS), COLUMN_LAST);

	for(int j=0; j<COLUMN_LAST; j++) {
		int fmt = (j == COLUMN_SIZE || j == COLUMN_EXACT_SIZE) ? LVCFMT_RIGHT : LVCFMT_LEFT;
		ctrlResults.InsertColumn(j, CTSTRING_I(columnNames[j]), fmt, columnSizes[j], j);
	}

	ctrlResults.setColumnOrderArray(COLUMN_LAST, columnIndexes);
	ctrlResults.setAscending(false);
	ctrlResults.setSortColumn(COLUMN_HITS);
	ctrlResults.setVisible(SETTING(SEARCHFRAME_VISIBLE));

	ctrlHubs.InsertColumn(0, _T("Dummy"), LVCFMT_LEFT, LVSCW_AUTOSIZE, 0);

	initHubs();

    targetMenu = ::CreatePopupMenu();
    targetDirMenu = ::CreatePopupMenu();

    HMENU copyMenu = ::CreatePopupMenu();
    ::AppendMenu(copyMenu, MF_STRING, IDC_COPY_NICK, CTSTRING(COPY_NICK));
	::AppendMenu(copyMenu, MF_STRING, IDC_COPY_FILENAME, CTSTRING(FILENAME));
	::AppendMenu(copyMenu, MF_STRING, IDC_COPY_PATH, CTSTRING(PATH));
	::AppendMenu(copyMenu, MF_STRING, IDC_COPY_SIZE, CTSTRING(SIZE));
	::AppendMenu(copyMenu, MF_STRING, IDC_COPY_TTH, CTSTRING(TTH_ROOT));
	::AppendMenu(copyMenu, MF_STRING, IDC_COPY_LINK, CTSTRING(MAGNET_LINK));

    resultsMenu = ::CreatePopupMenu();
    ::AppendMenu(resultsMenu, MF_STRING, IDC_DOWNLOAD, CTSTRING(DOWNLOAD));
    ::AppendMenu(resultsMenu, MF_POPUP, (UINT_PTR)targetMenu, CTSTRING(DOWNLOAD_TO));

    WinUtil::AddPrioritySubMenu(resultsMenu, TSTRING(DOWNLOAD_WITH_PRIORITY));

    ::AppendMenu(resultsMenu, MF_STRING, IDC_DOWNLOADDIR, CTSTRING(DOWNLOAD_WHOLE_DIR));
    ::AppendMenu(resultsMenu, MF_POPUP, (UINT_PTR)targetDirMenu, CTSTRING(DOWNLOAD_WHOLE_DIR_TO));
    ::AppendMenu(resultsMenu, MF_STRING, IDC_VIEW_AS_TEXT, CTSTRING(VIEW_AS_TEXT));
	::AppendMenu(resultsMenu, MF_STRING, IDC_SEARCH_ALTERNATES, CTSTRING(SEARCH_FOR_ALTERNATES));
	::AppendMenu(resultsMenu, MF_SEPARATOR, 0, NULL);
    ::AppendMenu(resultsMenu, MF_STRING, IDC_GETLIST, CTSTRING(GET_FILE_LIST));
	::AppendMenu(resultsMenu, MF_STRING, IDC_MATCH_QUEUE, CTSTRING(MATCH_QUEUE));
	::AppendMenu(resultsMenu, MF_STRING, IDC_PRIVATEMESSAGE, CTSTRING(SEND_PRIVATE_MESSAGE));
	::AppendMenu(resultsMenu, MF_STRING, IDC_ADD_TO_FAVORITES, CTSTRING(ADD_TO_FAVORITES));
	::AppendMenu(resultsMenu, MF_STRING, IDC_REMOVEALL, CTSTRING(REMOVE_FROM_ALL));
	::AppendMenu(resultsMenu, MF_SEPARATOR, 0, NULL);
    ::AppendMenu(resultsMenu, MF_POPUP, (UINT_PTR)copyMenu, CTSTRING(COPY));

    WinUtil::AddGrantSubMenu(resultsMenu);

	::SetMenuDefaultItem(resultsMenu, IDC_DOWNLOAD, 0);

	UpdateLayout();

	if(!initialString.empty()) {
		lastSearches.push_back(initialString);
		::SendMessage(hWndSearchBox, CB_INSERTSTRING, 0, (LPARAM)initialString.c_str());
		::SendMessage(hWndSearchBox, CB_SETCURSEL, 0, 0L);
		::SendMessage(hWndMode, CB_SETCURSEL, initialMode, 0L);
		::SetWindowText(hWndSize, Text::toT(Util::toString(initialSize)).c_str());
		::SendMessage(hWndFiletype, CB_SETCURSEL, initialType, 0L);
		onEnter();
	} else {
		::SetWindowText(m_hWnd, CTSTRING(SEARCH));
		::EnableWindow(hWndPauseSearch, FALSE);
	}
}

void SearchFrame::onEnter() {
	StringList clients;

	if(!(::GetWindowTextLength(hWndSearch) > 0))
		return;

	int n = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMCOUNT, 0, 0L);
	for(int i = 0; i < n; i++) {
		if((((UINT)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMSTATE, i, LVIS_STATEIMAGEMASK) >> 12) - 1)) {
			clients.push_back(Text::fromT(ctrlHubs.getItemData(i)->url));
		}
	}

	if(!clients.size())
		return;

	if(SearchManager::getInstance()->okToSearch()) {
		tstring s(::GetWindowTextLength(hWndSearch) + 1, _T('\0'));
		::GetWindowText(hWndSearch, &s[0], (int)s.size());
		s.resize(s.size()-1);

		tstring size(::GetWindowTextLength(hWndSize) + 1, _T('\0'));
		::GetWindowText(hWndSize, &size[0], (int)size.size());
		size.resize(size.size()-1);

		double lsize = Util::toDouble(Text::fromT(size));
		switch((int)::SendMessage(hWndSizeMode, CB_GETCURSEL, 0, 0L)) {
			case 1:
				lsize *= 1024.0;
                break;
			case 2:
				lsize *= 1024.0 * 1024.0;
                break;
			case 3:
				lsize *= 1024.0 * 1024.0 * 1024.0;
                break;
		}

		int64_t llsize = (int64_t)lsize;

		ctrlResults.deleteAllItems();

		{
			Lock l(cs);
			search = StringTokenizer<tstring>(s, ' ').getTokens();
			s.clear();
			//strip out terms beginning with -
			for(TStringList::iterator si = search.begin(); si != search.end(); ) {
				if(si->empty()) {
					si = search.erase(si);
					continue;
				}
				if ((*si)[0] != _T('-')) 
					s += *si + _T(' ');	
				++si;
			}
	
			s = s.substr(0, max(s.size(), static_cast<tstring::size_type>(1)) - 1);
		}

		SearchManager::SizeModes mode((SearchManager::SizeModes)((int)::SendMessage(hWndMode, CB_GETCURSEL, 0, 0L)));
		if(llsize == 0)
			mode = SearchManager::SIZE_DONTCARE;

		int ftype = (int)::SendMessage(hWndFiletype, CB_GETCURSEL, 0, 0L);


		// Add new searches to the last-search dropdown list
		if(find(lastSearches.begin(), lastSearches.end(), s) == lastSearches.end()) {
			if((int)::SendMessage(hWndSearchBox, CB_GETCOUNT, 0, 0L) > 19)
				::SendMessage(hWndSearchBox, CB_DELETESTRING, 19, 0L);

			::SendMessage(hWndSearchBox, CB_INSERTSTRING, 0, (LPARAM)s.c_str());

			while(lastSearches.size() > (TStringList::size_type)19) {
				lastSearches.erase(lastSearches.begin());
			}
			lastSearches.push_back(s);
		}

		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)(TSTRING(SEARCHING_FOR) + s + _T("...")).c_str());
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0), (LPARAM)_T(""));
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (3 | 0), (LPARAM)_T(""));

		droppedResults = 0;
		isHash = (ftype == SearchManager::TYPE_TTH);

		::SetWindowText(m_hWnd, (TSTRING(SEARCH) + _T(" - ") + s).c_str());

		for(SearchInfo::Iter i = PausedResults.begin(); i != PausedResults.end(); ++i) {
			delete *i;
		}

		resultsCount = 0;

		bWarned = false;
		PausedResults.clear();
		bPaused = false;
		::EnableWindow(hWndPauseSearch, TRUE);
		::SetWindowText(hWndPauseSearch, CTSTRING(PAUSE_SEARCH));
		SearchManager::getInstance()->search(clients, Text::fromT(s), llsize, 
			(SearchManager::TypeModes)ftype, mode);
		if(BOOLSETTING(CLEAR_SEARCH)) // Only clear if the search was sent
			::SetWindowText(hWndSearch, _T(""));
	} else {
		bWarned = true;
		int32_t waitFor = SearchManager::getInstance()->timeToSearch();
		TCHAR * buf = new TCHAR[TSTRING(SEARCHING_WAIT).size() + 16];
		_stprintf_s(buf, TSTRING(SEARCHING_WAIT).size() + 16, CTSTRING(SEARCHING_WAIT), waitFor);

		::SetWindowText(m_hWnd, (TSTRING(SEARCH) + _T(" - ") + tstring(buf)).c_str());
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)buf);

		delete[] buf;

		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0), (LPARAM)_T(""));
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (3 | 0), (LPARAM)_T(""));

		// Start the countdown timer
		timerID = ::SetTimer(m_hWnd, 1, 1000, NULL);
	}
}

void SearchFrame::on(SearchManagerListener::SR, SearchResult* aResult) throw() {
	// Check that this is really a relevant search result...
	{
		Lock l(cs);

		if(search.empty()) {
			return;
		}

		if(isHash) {
			if(aResult->getTTH() == NULL)
				return;
			if(Util::stricmp(Text::toT(aResult->getTTH()->toBase32()), search[0]) != 0)
				return;
		} else {
			// match all here
			for(TStringIter j = search.begin(); j != search.end(); ++j) {
				if((*j->begin() != _T('-') && Util::findSubString(aResult->getUtf8() ? aResult->getFile() : Text::acpToUtf8(aResult->getFile()), Text::fromT(*j)) == -1) ||
					(*j->begin() == _T('-') && j->size() != 1 && Util::findSubString(aResult->getUtf8() ? aResult->getFile() : Text::acpToUtf8(aResult->getFile()), Text::fromT(j->substr(1))) != -1)
					)
				{
					droppedResults++;
					::PostMessage(m_hWnd, WM_SPEAKER, FILTER_RESULT, NULL);
					return;
				}
			}
		}
	}

	// Reject results without free slots or tth if selected
	// but always show directories
	if((onlyFree && aResult->getFreeSlots() < 1) || ((onlyTTH && aResult->getTTH() == NULL) && (aResult->getType() != SearchResult::TYPE_DIRECTORY))) {
		droppedResults++;
		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (3 | 0), (LPARAM)Text::toT(Util::toString((uint32_t)droppedResults) + ' ' + STRING(FILTERED)).c_str());
		return;
	}

	SearchInfo* i = new SearchInfo(aResult);
	::PostMessage(m_hWnd, WM_SPEAKER, ADD_RESULT, (LPARAM)i);
}

void SearchFrame::onTimer() {
	int32_t waitFor = SearchManager::getInstance()->timeToSearch();
	if(waitFor > 0) {
		TCHAR * buf = new TCHAR[TSTRING(SEARCHING_WAIT).size() + 16];
		_stprintf_s(buf, TSTRING(SEARCHING_WAIT).size() + 16, CTSTRING(SEARCHING_WAIT), waitFor);

		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)buf);
		::SetWindowText(m_hWnd, (TSTRING(SEARCH) + _T(" - ") + tstring(buf)).c_str());

		delete[] buf;
	} else {
		if(timerID != 0) {
			::KillTimer(m_hWnd, timerID);
			timerID = 0;
		}

		::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)(TSTRING(SEARCHING_READY)).c_str());
		::SetWindowText(m_hWnd, (TSTRING(SEARCH) + _T(" - ") + TSTRING(SEARCHING_READY)).c_str());
	}

	::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0), (LPARAM)_T(""));
	::SendMessage(m_hWndStatusBar, SB_SETTEXT, (3 | 0), (LPARAM)_T(""));
}

void SearchFrame::SearchInfo::view() {
	try {
		if(sr->getType() == SearchResult::TYPE_FILE) {
			QueueManager::getInstance()->add(Util::getTempPath() + Text::fromT(columns[COLUMN_FILENAME]), 
				sr->getSize(), sr->getTTH(), sr->getUser(), sr->getFile(), 
				sr->getUtf8(), QueueItem::FLAG_CLIENT_VIEW | QueueItem::FLAG_TEXT);
		}
	} catch(const Exception&) {
	}
}

void SearchFrame::SearchInfo::Download::operator()(SearchInfo* si) {
	try {
		if(si->sr->getType() == SearchResult::TYPE_FILE) {
			string target = Text::fromT(tgt + si->columns[COLUMN_FILENAME]);
			QueueManager::getInstance()->add(target, si->sr->getSize(), 
				si->sr->getTTH(), si->sr->getUser(), si->sr->getFile(), 
				si->sr->getUtf8());

			for(SearchInfo::Iter i = si->subItems.begin(); i != si->subItems.end(); i++) {
				SearchInfo* j = *i;
				try {
					QueueManager::getInstance()->add(target, j->sr->getSize(), j->sr->getTTH(), j->sr->getUser(), 
						j->sr->getFile(), j->sr->getUtf8()); 
				} catch(const Exception&) {
					// ...
				}
			}

			if(WinUtil::isShift() || prio != QueueItem::DEFAULT)
                QueueManager::getInstance()->setPriority(target, WinUtil::isShift() ? QueueItem::HIGHEST : prio);
		} else {
			QueueManager::getInstance()->addDirectory(si->sr->getFile(), si->sr->getUser(), Text::fromT(tgt),
				WinUtil::isShift() ? QueueItem::HIGHEST : prio);
		}
	} catch(const Exception&) {
        // ...
	}
}

void SearchFrame::SearchInfo::DownloadWhole::operator()(SearchInfo* si) {
	try {
		if(si->sr->getType() == SearchResult::TYPE_FILE) {
			QueueManager::getInstance()->addDirectory(Text::fromT(si->columns[COLUMN_PATH]), si->sr->getUser(), Text::fromT(tgt),
				WinUtil::isShift() ? QueueItem::HIGHEST : QueueItem::DEFAULT);
		} else {
			QueueManager::getInstance()->addDirectory(si->sr->getFile(), si->sr->getUser(), Text::fromT(tgt),
				WinUtil::isShift() ? QueueItem::HIGHEST : QueueItem::DEFAULT);
		}
	} catch(const Exception&) {
	}
}

void SearchFrame::SearchInfo::DownloadTarget::operator()(SearchInfo* si) {
		try {
		if(si->sr->getType() == SearchResult::TYPE_FILE) {
			string target = Text::fromT(tgt);
			QueueManager::getInstance()->add(target, si->sr->getSize(),
				si->sr->getTTH(), si->sr->getUser(), si->sr->getFile(),
				si->sr->getUtf8());

			if(WinUtil::isShift())
				QueueManager::getInstance()->setPriority(target, QueueItem::HIGHEST);
		} else {
			QueueManager::getInstance()->addDirectory(si->sr->getFile(), si->sr->getUser(), Text::fromT(tgt),
				WinUtil::isShift() ? QueueItem::HIGHEST : QueueItem::DEFAULT);
		}
	} catch(const Exception&) {
	}
}

void SearchFrame::SearchInfo::getList() {
	try {
		WinUtil::addInitalDir(sr->getUser(), Text::fromT(columns[COLUMN_PATH]));
		QueueManager::getInstance()->addList(sr->getUser(), QueueItem::FLAG_CLIENT_VIEW);
	} catch(const Exception&) {
		// Ignore for now...
	}
}

void SearchFrame::SearchInfo::CheckSize::operator()(SearchInfo* si) {
	if(!si->columns[COLUMN_TTH].empty()) {
		if(firstTTH) {
			tth = si->columns[COLUMN_TTH];
			hasTTH = true;
			firstTTH = false;
		} else if(hasTTH) {
			if(tth != si->columns[COLUMN_TTH]) {
				hasTTH = false;
			}
		} 
	} else {
		firstTTH = false;
		hasTTH = false;
	}

	if(si->sr->getType() == SearchResult::TYPE_FILE) {
		if(ext.empty()) {
			ext = Util::getFileExt(si->columns[COLUMN_FILENAME]);
			size = si->sr->getSize();
		} else if(size != -1) {
			if((si->sr->getSize() != size) || (Util::stricmp(ext, Util::getFileExt(si->columns[COLUMN_FILENAME])) != 0)) {
				size = -1;
		}
	}
	} else {
		size = -1;
	}

	StringList userhubs = ClientManager::getInstance()->getHubs(si->sr->getUser()->getCID());
	string userhub;
	if(userhubs.size() > 0) userhub = userhubs[0];

	if(oneHub && hub.empty()) {
		hub = userhub;
	} else if(hub != userhub) {
		oneHub = false;
		hub.clear();
	}
	if(op)
		op = userhub.empty() ? false : ClientManager::getInstance()->isOp(ClientManager::getInstance()->getMe(), userhub);
}

void SearchFrame::onDownloadTo() {
	if((UINT)::SendMessage(ctrlResults.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0L) == 1) {
		int i = (int)::SendMessage(ctrlResults.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_SELECTED, 0));
		dcassert(i != -1);
		SearchInfo* si = ctrlResults.getItemData(i);
		SearchResult* sr = si->sr;

		if(sr->getType() == SearchResult::TYPE_FILE) {
    		tstring target = Text::toT(SETTING(DOWNLOAD_DIRECTORY)) + si->columns[COLUMN_FILENAME];
    		if(WinUtil::browseFile(target, m_hWnd)) {
    			WinUtil::addLastDir(Util::getFilePath(target));
    			ctrlResults.forEachSelectedT(SearchInfo::DownloadTarget(target));
    		}
		} else {
    		tstring target = Text::toT(SETTING(DOWNLOAD_DIRECTORY));
    		if(WinUtil::browseDirectory(target, m_hWnd)) {
    			WinUtil::addLastDir(target);
    			ctrlResults.forEachSelectedT(SearchInfo::Download(target));
    		}
		}
	} else {
		tstring target = Text::toT(SETTING(DOWNLOAD_DIRECTORY));
		if(WinUtil::browseDirectory(target, m_hWnd)) {
			WinUtil::addLastDir(target);
			ctrlResults.forEachSelectedT(SearchInfo::Download(target));
		}
	}
}

void SearchFrame::onDownloadWholeTo() {
	tstring target = Text::toT(SETTING(DOWNLOAD_DIRECTORY));
	if(WinUtil::browseDirectory(target, m_hWnd)) {
		WinUtil::addLastDir(target);
		ctrlResults.forEachSelectedT(SearchInfo::DownloadWhole(target));
	}
}

void SearchFrame::onDownloadTarget(WORD wID) {
	dcassert(wID >= IDC_DOWNLOAD_TARGET);
	size_t newId = (size_t)wID - IDC_DOWNLOAD_TARGET;

	if(newId < WinUtil::lastDirs.size()) {
		ctrlResults.forEachSelectedT(SearchInfo::Download(WinUtil::lastDirs[newId]));
	} else {
		dcassert((newId - WinUtil::lastDirs.size()) < targets.size());
		ctrlResults.forEachSelectedT(SearchInfo::DownloadTarget(Text::toT(targets[newId - WinUtil::lastDirs.size()])));
	}
}

void SearchFrame::onDownloadWholeTarget(WORD wID) {
	dcassert((wID-IDC_DOWNLOAD_WHOLE_TARGET) < (int)WinUtil::lastDirs.size());
	ctrlResults.forEachSelectedT(SearchInfo::DownloadWhole(WinUtil::lastDirs[wID-IDC_DOWNLOAD_WHOLE_TARGET]));
}

void SearchFrame::onDownloadFavoriteDirs(WORD wID) {
	dcassert(wID >= IDC_DOWNLOAD_FAVORITE_DIRS);
	size_t newId = (size_t)wID - IDC_DOWNLOAD_FAVORITE_DIRS;

	StringPairList spl = FavoriteManager::getInstance()->getFavoriteDirs();
	if(newId < spl.size()) {
		ctrlResults.forEachSelectedT(SearchInfo::Download(Text::toT(spl[newId].first)));
	} else {
		dcassert((newId - spl.size()) < targets.size());
		ctrlResults.forEachSelectedT(SearchInfo::DownloadTarget(Text::toT(targets[newId - spl.size()])));
	}
}

void SearchFrame::onDownloadWholeFavoriteDirs(WORD wID) {
	StringPairList spl = FavoriteManager::getInstance()->getFavoriteDirs();
	dcassert((wID-IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS) < (int)spl.size());
	ctrlResults.forEachSelectedT(SearchInfo::DownloadWhole(Text::toT(spl[wID-IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS].first)));
}

void SearchFrame::onDoubleClickResults(LPNMHDR pnmh) {
	LPNMITEMACTIVATE item = (LPNMITEMACTIVATE)pnmh;
	
	if(item->iItem != -1) {
		RECT rect;
		rect.left = LVIR_ICON;
		::SendMessage(ctrlResults.m_hWnd, LVM_GETITEMRECT, (WPARAM)item->iItem, (LPARAM)&rect);

		// if double click on state icon, ignore...
		if(item->ptAction.x < rect.left) {
			return;
        }

		ctrlResults.forEachSelectedT(SearchInfo::Download(Text::toT(SETTING(DOWNLOAD_DIRECTORY))));
	}
}

bool SearchFrame::onClose() {
	if(!closed) {
        SettingsManager::getInstance()->removeListener(this);
		SearchManager::getInstance()->removeListener(this);
 		ClientManager::getInstance()->removeListener(this);
 		TimerManager::getInstance()->removeListener(this);

		frames.erase(m_hWnd);

		closed = true;
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);

		return true;
	} else {
        if(!initialType && (int)::SendMessage(hWndFiletype, CB_GETCURSEL, 0, 0L) != SETTING(LAST_SEARCH_TYPE)) {
            SettingsManager::getInstance()->set(SettingsManager::LAST_SEARCH_TYPE, (int)::SendMessage(hWndFiletype, CB_GETCURSEL, 0, 0L));
        }

		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_01, Text::fromT(lastSearches[0]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_02, Text::fromT(lastSearches[1]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_03, Text::fromT(lastSearches[2]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_04, Text::fromT(lastSearches[3]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_05, Text::fromT(lastSearches[4]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_06, Text::fromT(lastSearches[5]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_07, Text::fromT(lastSearches[6]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_08, Text::fromT(lastSearches[7]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_09, Text::fromT(lastSearches[8]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_10, Text::fromT(lastSearches[9]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_11, Text::fromT(lastSearches[10]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_12, Text::fromT(lastSearches[11]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_13, Text::fromT(lastSearches[12]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_14, Text::fromT(lastSearches[13]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_15, Text::fromT(lastSearches[14]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_16, Text::fromT(lastSearches[15]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_17, Text::fromT(lastSearches[16]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_18, Text::fromT(lastSearches[17]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_19, Text::fromT(lastSearches[18]));
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_MSG_RECENT_20, Text::fromT(lastSearches[19]));
		ctrlResults.deleteAllItems();
		for(SearchInfo::Iter i = PausedResults.begin(); i != PausedResults.end(); ++i) {
			delete *i;
		}
		PausedResults.clear();

		for(int i = 0; i < (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMCOUNT, 0, 0L); i++) {
			delete ctrlHubs.getItemData(i);
		}

		::SendMessage(ctrlHubs.m_hWnd, LVM_DELETEALLITEMS, 0, 0L);

		ctrlResults.saveHeaderOrder(SettingsManager::SEARCHFRAME_ORDER, SettingsManager::SEARCHFRAME_WIDTHS, 
			SettingsManager::SEARCHFRAME_VISIBLE);

        return false;
	}
}

void SearchFrame::UpdateLayout(bool bResizeBars /* = true*/) {
	RECT rect;
	::GetClientRect(m_hWnd, &rect);
	// position bars and offset their dimensions
	UpdateBarsPosition(rect, bResizeBars);

	if(::IsWindow(m_hWndStatusBar)) {
		RECT sr;
		int w[3];
		::GetClientRect(m_hWndStatusBar, &sr);
		int tmp = (sr.right - sr.left) > 316 ? 216 : (((sr.right - sr.left) > 116) ? (sr.right - sr.left) - 100 : 16);

		w[0] = 15;
		w[1] = sr.right - tmp/2;
		w[2] = w[1] + (tmp-16);
		
		::SendMessage(m_hWndStatusBar, SB_SETPARTS, 3, (LPARAM)w);

		// Layout showUI button in statusbar part #0
		::SendMessage(m_hWndStatusBar, SB_GETRECT, 0, (LPARAM)&sr);
		::MoveWindow(hWndShowUI, sr.left, sr.top, sr.right - sr.left, sr.bottom - sr.top, TRUE);
	}

	if(showUI)
	{
		int const width = 220, spacing = 50, labelH = 16, comboH = 140, lMargin = 2, rMargin = 4;
		RECT rc = rect;

		rc.left += width;
		::MoveWindow(ctrlResults.m_hWnd, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

		// "Search for"
		rc.right = width - rMargin;
		rc.left = lMargin;
		rc.top += 25;
		rc.bottom = rc.top + comboH + 21;
		::MoveWindow(hWndSearchBox, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

		::MoveWindow(hWndSearchLabel, rc.left + lMargin, rc.top - labelH, width - rMargin, labelH-1, TRUE);

		// "Size"
		int w2 = width - rMargin - lMargin;
		rc.top += spacing;
		rc.bottom += spacing;
		rc.right = w2/3;
		::MoveWindow(hWndMode, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

		::MoveWindow(hWndSizeLabel, rc.left + lMargin, rc.top - labelH, width - rMargin, labelH-1, TRUE);

		rc.left = rc.right + lMargin;
		rc.right += w2/3;
		rc.bottom -= comboH;
		::MoveWindow(hWndSize, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

		rc.left = rc.right + lMargin;
		rc.right = width - rMargin;
		rc.bottom += comboH;
		::MoveWindow(hWndSizeMode, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
		rc.bottom -= comboH;

		// "File type"
		rc.left = lMargin;
		rc.right = width - rMargin;
		rc.top += spacing;
		rc.bottom = rc.top + comboH + 21;
		::MoveWindow(hWndFiletype, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
		rc.bottom -= comboH;

		::MoveWindow(hWndTypeLabel, rc.left + lMargin, rc.top - labelH, width - rMargin, labelH-1, TRUE);

		// "Search options"
		rc.left = lMargin;
		rc.right = width - rMargin;
		rc.top += spacing;
		rc.bottom += spacing;
		::MoveWindow(hWndSlots, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

		::MoveWindow(hWndOptionLabel, rc.left + lMargin, rc.top - labelH, width - rMargin, labelH-1, TRUE);

		rc.top += 21;
		rc.bottom += 21;
		::MoveWindow(hWndTTH, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

		rc.top += labelH + 3;
		rc.bottom += labelH + 3;
		::MoveWindow(hWndCollapsed, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

		// "Hubs"
		rc.left = lMargin;
		rc.right = width - rMargin;
		rc.top += spacing;
		rc.bottom = rc.top + comboH;
		if (rc.bottom + labelH + 21 > rect.bottom) {
			rc.bottom = rect.bottom - labelH - 21;
			if (rc.bottom < rc.top + (labelH*3)/2)
				rc.bottom = rc.top + (labelH*3)/2;
		}

		::MoveWindow(ctrlHubs.m_hWnd, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

		::MoveWindow(hWndHubsLabel, rc.left + lMargin, rc.top - labelH, width - rMargin, labelH-1, TRUE);

		// "Pause Search"
		rc.right = width - rMargin;
		rc.left = rc.right - 110;
		rc.top = rc.bottom + labelH;
		rc.bottom = rc.top + 21;
		::MoveWindow(hWndPauseSearch, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

		// "Search"
		rc.left = lMargin;
		rc.right = rc.left + 100;
		::MoveWindow(hWndDoSearch, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
	}
	else
	{
		RECT rc = rect;
		::MoveWindow(ctrlResults.m_hWnd, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);

		::SetRect(&rc, 0, 0, 0, 0);
		::MoveWindow(hWndSearchBox, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
		::MoveWindow(hWndMode, 0, 0, 0, 0, TRUE);
		::MoveWindow(hWndSize, 0, 0, 0, 0, TRUE);
		::MoveWindow(hWndSizeMode, 0, 0, 0, 0, TRUE);
		::MoveWindow(hWndFiletype, 0, 0, 0, 0, TRUE);
	}

	POINT pt;
	pt.x = 10;
	pt.y = 10;
	HWND hWnd = ::ChildWindowFromPoint(hWndSearchBox, pt);
	if(hWnd != NULL && !::IsWindow(hWndSearch) && hWnd != hWndSearchBox) {
		hWndSearch = hWnd;
        ::SetWindowLongPtr(hWndSearch, GWLP_USERDATA, (LONG_PTR)this);
        DefEditProc = (WNDPROC)::SetWindowLongPtr(hWndSearch, GWLP_WNDPROC, (LONG_PTR)SearchWndProc);
	}
}

void SearchFrame::runUserCommand(UserCommand& uc) {
	if(!WinUtil::getUCParams(m_hWnd, uc, ucLineParams))
		return;

	StringMap ucParams = ucLineParams;

	set<CID> users;

	int sel = -1;
	while((sel = (int)::SendMessage(ctrlResults.m_hWnd, LVM_GETNEXTITEM, sel, MAKELPARAM(LVNI_SELECTED, 0))) != -1) {
		SearchResult* sr = ctrlResults.getItemData(sel)->sr;

		if(!sr->getUser()->isOnline())
			continue;

		if(uc.getType() == UserCommand::TYPE_RAW_ONCE) {
			if(users.find(sr->getUser()->getCID()) != users.end())
			continue;
			users.insert(sr->getUser()->getCID());
		}

		ucParams["fileFN"] = sr->getFile();
		ucParams["fileSI"] = Util::toString(sr->getSize());
		ucParams["fileSIshort"] = Util::formatBytes(sr->getSize());
		if(sr->getTTH() != NULL) {
			ucParams["fileTR"] = sr->getTTH()->toBase32();
		} else {
			ucParams["fileTR"] = "NONE";
		}

		// compatibility with 0.674 and earlier
		ucParams["file"] = ucParams["fileFN"];
		ucParams["filesize"] = ucParams["fileSI"];
		ucParams["filesizeshort"] = ucParams["fileSIshort"];
		ucParams["tth"] = ucParams["fileTR"];

		StringMap tmp = ucParams;
		ClientManager::getInstance()->userCommand(sr->getUser(), uc, tmp, true);
	}
}

LRESULT SearchFrame::onCtlColor(WPARAM wParam, LPARAM lParam) {
	HWND hWnd = (HWND)lParam;
	HDC hDC = (HDC)wParam;

	if(hWnd == hWndSearchLabel || hWnd == hWndSizeLabel || hWnd == hWndOptionLabel || hWnd == hWndTypeLabel ||
		hWnd == hWndHubsLabel || hWnd == hWndSlots || hWnd == hWndTTH || hWnd == hWndCollapsed) {
		::SetBkColor(hDC, ::GetSysColor(COLOR_3DFACE));
		::SetTextColor(hDC, ::GetSysColor(COLOR_BTNTEXT));
		return (LRESULT)::GetSysColorBrush(COLOR_3DFACE);
	} else {
		::SetBkColor(hDC, WinUtil::bgColor);
		::SetTextColor(hDC, WinUtil::textColor);
		return (LRESULT)WinUtil::bgBrush;
	}
}

bool SearchFrame::onChar(UINT uMsg, WPARAM wParam) {
    switch(wParam) {
        case VK_TAB:
            if(uMsg == WM_KEYDOWN) {
                onTab(WinUtil::isShift());
            }
            break;
        case VK_RETURN:
            if(WinUtil::isShift() || WinUtil::isCtrl() || WinUtil::isAlt()) {
                return false;
            } else {
                if(uMsg == WM_KEYDOWN) {
                    onEnter();
                }
            }
            break;
        default:
            return false;
	}

	return true;
}

void SearchFrame::onTab(bool shift) {
	HWND wnds[] = {
		hWndSearch, hWndMode, hWndSize, hWndSizeMode, hWndFiletype, hWndSlots, hWndTTH, hWndCollapsed, ctrlHubs.m_hWnd, hWndDoSearch,
        hWndPauseSearch, ctrlResults.m_hWnd
	};

	HWND focus = GetFocus();
	if(focus == hWndSearchBox)
		focus = hWndSearch;

	static const int size = sizeof(wnds) / sizeof(wnds[0]);
	int i;
	for(i = 0; i < size; i++) {
		if(wnds[i] == focus)
			break;
	}

	::SetFocus(wnds[(i + (shift ? -1 : 1)) % size]);
}

void SearchFrame::onSearchByTTH() {
	if((UINT)::SendMessage(ctrlResults.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0L) == 1) {
		int i = (int)::SendMessage(ctrlResults.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_SELECTED, 0));
		SearchResult* sr = ctrlResults.getItemData(i)->sr;

		if(sr->getTTH() != NULL) {
			SearchFrame::openWindow(Text::toT(sr->getTTH()->toBase32()), 0, SearchManager::SIZE_DONTCARE, SearchManager::TYPE_TTH);
		}
	}
}

void SearchFrame::onSpeaker(WPARAM wParam, LPARAM lParam) {
 	switch(wParam) {
		case ADD_RESULT: {
			SearchInfo* si = (SearchInfo*)lParam;
			SearchResult* sr = si->sr;
			// Check previous search results for dupes
			if(BOOLSETTING(FILTER_SEARCH) && si->columns[COLUMN_TTH].empty() == false) {
				SearchInfo* main = ctrlResults.findMainItem(si->columns[COLUMN_TTH]);
				if(main) {
				    if((sr->getUser()->getCID() == main->sr->getUser()->getCID()) && (sr->getFile() == main->sr->getFile())) {
						delete si;
						return;
					}
					for(SearchInfo::Iter k = main->subItems.begin(); k != main->subItems.end(); k++){
						if((sr->getUser()->getCID() == (*k)->getUser()->getCID()) && (sr->getFile() == (*k)->sr->getFile())) {
							delete si;
							return;
						}
					}
				}
			}
			if(bPaused == false) {
    			if(si->columns[COLUMN_TTH].empty() == false && useGrouping) {
					ctrlResults.insertGroupedItem(si, expandSR);
    			} else {
					ctrlResults.insertItem(si, si->imageIndex());
					ctrlResults.mainItems.push_back(si);
                }
				if(BOOLSETTING(BOLD_SEARCH))
					setDirty();

				::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0), (LPARAM)Text::toT(Util::toString(resultsCount) + " " + STRING(FILES)).c_str());

				needresort = true;
			} else {
				PausedResults.push_back(si);
				::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0),
                    (LPARAM)Text::toT(Util::toString(resultsCount-PausedResults.size()) + "/" + Util::toString(resultsCount) + " " + STRING(FILES)).c_str());
			}
			break;
		}
		case FILTER_RESULT:
			::SendMessage(m_hWndStatusBar, SB_SETTEXT, (3 | 0), (LPARAM)Text::toT(Util::toString((uint32_t)droppedResults) + ' ' + STRING(FILTERED)).c_str());
			break;
 		case HUB_ADDED:
 			onHubAdded((HubInfo*)(lParam));
			break;
 		case HUB_CHANGED:
 			onHubChanged((HubInfo*)(lParam));
			break;
 		case HUB_REMOVED:
 			onHubRemoved((HubInfo*)(lParam));
 			break;
		case STATS: {
			tstring* buf = (tstring*)(lParam);
			::SendMessage(m_hWndStatusBar, SB_SETTEXT, (1 | 0), (LPARAM)(*buf).c_str());
			::SetWindowText(m_hWnd, (*buf).c_str());
			delete buf;
			break;
		}
		case RESORT: {
			::SendMessage(ctrlResults.m_hWnd, WM_SETREDRAW, (WPARAM)FALSE, 0);
			ctrlResults.resort();
			::SendMessage(ctrlResults.m_hWnd, WM_SETREDRAW, (WPARAM)TRUE, 0);
			needresort = false;
			break;
		}
 	}
}

bool SearchFrame::onContextMenu(WPARAM wParam, LPARAM lParam) {
	if((HWND)wParam == ctrlResults.m_hWnd) {
		RECT rc;                    // client area of window 
		POINT pt = { (LONG)LOWORD(lParam), (LONG)HIWORD(lParam) };

		::GetWindowRect((HWND)::SendMessage(ctrlResults.m_hWnd, LVM_GETHEADER, 0, 0L), &rc);
		if(PtInRect(&rc, pt)){
			ctrlResults.showMenu(pt);
			return true;
		}

		if(pt.x == -1 && pt.y == -1) {
			WinUtil::getListContextMenuPos(ctrlResults.m_hWnd, pt);
		}

		if((UINT)::SendMessage(ctrlResults.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0L) > 0) {
			while(::GetMenuItemCount(targetMenu) > 0) {
                ::RemoveMenu(targetMenu, 0, MF_BYPOSITION);
			}
			while(::GetMenuItemCount(targetDirMenu) > 0) {
                ::RemoveMenu(targetDirMenu, 0, MF_BYPOSITION);
			}
	
			int n = 0;
	
			//Append favorite download dirs
			StringPairList spl = FavoriteManager::getInstance()->getFavoriteDirs();
			if (spl.size() > 0) {
				for(StringPairIter i = spl.begin(); i != spl.end(); i++) {
                    ::AppendMenu(targetMenu, MF_STRING, IDC_DOWNLOAD_FAVORITE_DIRS + n, Text::toT(i->second).c_str());
					n++;
				}
				::AppendMenu(targetMenu, MF_SEPARATOR, 0, NULL);
			}
	
			n = 0;
			::AppendMenu(targetMenu, MF_STRING, IDC_DOWNLOADTO, CTSTRING(BROWSE));
			if(WinUtil::lastDirs.size() > 0) {
				::AppendMenu(targetMenu, MF_SEPARATOR, 0, NULL);
				for(TStringIter i = WinUtil::lastDirs.begin(); i != WinUtil::lastDirs.end(); ++i) {
                    ::AppendMenu(targetMenu, MF_STRING, IDC_DOWNLOAD_TARGET + n, i->c_str());
					n++;
				}
			}
	
			SearchInfo::CheckSize cs = ctrlResults.forEachSelectedT(SearchInfo::CheckSize());

			if(cs.size != -1 || cs.hasTTH) {
				targets.clear();
				if(cs.hasTTH) {
					QueueManager::getInstance()->getTargetsByRoot(targets, TTHValue(Text::fromT(cs.tth)));
				} else if(BOOLSETTING(USE_EXTENSION_DOWNTO)) { 
					QueueManager::getInstance()->getTargetsBySize(targets, cs.size, Text::fromT(cs.ext));
				} else {
					QueueManager::getInstance()->getTargetsBySize(targets, cs.size, Util::emptyString);
				}

				if(targets.size() > 0) {
					::AppendMenu(targetMenu, MF_SEPARATOR, 0, NULL);
					for(StringIter i = targets.begin(); i != targets.end(); ++i) {
                        ::AppendMenu(targetMenu, MF_STRING, IDC_DOWNLOAD_TARGET + n, Text::toT(*i).c_str());
						n++;
					}
				}
			}

			n = 0;
			//Append favorite download dirs
			if (spl.size() > 0) {
				for(StringPairIter i = spl.begin(); i != spl.end(); ++i) {
                    ::AppendMenu(targetDirMenu, MF_STRING, IDC_DOWNLOAD_WHOLE_FAVORITE_DIRS + n, Text::toT(i->second).c_str());
					n++;
				}
				::AppendMenu(targetDirMenu, MF_SEPARATOR, 0, NULL);
			}

			n = 0;
			::AppendMenu(targetDirMenu, MF_STRING, IDC_DOWNLOADDIRTO, CTSTRING(BROWSE));
			if(WinUtil::lastDirs.size() > 0) {
    			::AppendMenu(targetDirMenu, MF_SEPARATOR, 0, NULL);
    			for(TStringIter i = WinUtil::lastDirs.begin(); i != WinUtil::lastDirs.end(); ++i) {
                    ::AppendMenu(targetDirMenu, MF_STRING, IDC_DOWNLOAD_WHOLE_TARGET + n, i->c_str());
					n++;
				}
			}

			// magnet link
			if((UINT)::SendMessage(ctrlResults.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0L) == 1){
				SearchResult* sr = ctrlResults.getSelectedItem()->sr;
				if(sr->getType() == SearchResult::TYPE_FILE && sr->getTTH()) {
                    ::EnableMenuItem(resultsMenu, IDC_SEARCH_ALTERNATES, MF_BYCOMMAND | MFS_ENABLED);
                    ::EnableMenuItem(resultsMenu, IDC_COPY_TTH, MF_BYCOMMAND | MFS_ENABLED);
                    ::EnableMenuItem(resultsMenu, IDC_COPY_LINK, MF_BYCOMMAND | MFS_ENABLED);
				} else {
                    ::EnableMenuItem(resultsMenu, IDC_SEARCH_ALTERNATES, MF_BYCOMMAND | MFS_DISABLED);
                    ::EnableMenuItem(resultsMenu, IDC_COPY_TTH, MF_BYCOMMAND | MFS_DISABLED);
                    ::EnableMenuItem(resultsMenu, IDC_COPY_LINK, MF_BYCOMMAND | MFS_DISABLED);
				}
			}
	
			prepareMenu(resultsMenu, UserCommand::CONTEXT_SEARCH, cs.hub, cs.op);
			if(!(::GetMenuState(resultsMenu, ::GetMenuItemCount(resultsMenu)-1, MF_BYPOSITION) & MF_SEPARATOR)) {	
				::AppendMenu(resultsMenu, MF_SEPARATOR, 0, NULL);
			}
			::AppendMenu(resultsMenu, MF_STRING, IDC_REMOVE, CTSTRING(REMOVE));
			::TrackPopupMenu(resultsMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, 0, m_hWnd, NULL);
			::RemoveMenu(resultsMenu, ::GetMenuItemCount(resultsMenu)-1, MF_BYPOSITION);
			::RemoveMenu(resultsMenu, ::GetMenuItemCount(resultsMenu)-1, MF_BYPOSITION);
			cleanMenu(resultsMenu);
			return true;
		}
	}

	return false;
}

void SearchFrame::initHubs() {
	ctrlHubs.insertItem(new HubInfo(Util::emptyStringT, TSTRING(ONLY_WHERE_OP), false), 0);
	ListViewSetItemState(ctrlHubs.m_hWnd, 0, INDEXTOSTATEIMAGEMASK(1), LVIS_STATEIMAGEMASK);

	ClientManager* clientMgr = ClientManager::getInstance();
	clientMgr->lock();
	clientMgr->addListener(this);

	Client::List& clients = clientMgr->getClients();

	Client::List::iterator it;
	Client::List::iterator endIt = clients.end();
	for(it = clients.begin(); it != endIt; ++it) {
		Client* client = *it;
		if (!client->isConnected())
			continue;

		onHubAdded(new HubInfo(Text::toT(client->getHubUrl()), Text::toT(client->getHubName()), client->getMyIdentity().isOp()));
	}

	clientMgr->unlock();
	::SendMessage(ctrlHubs.m_hWnd, LVM_SETCOLUMNWIDTH, 0, MAKELPARAM(LVSCW_AUTOSIZE, 0));

}

void SearchFrame::onHubAdded(HubInfo* info) {
	int nItem = ctrlHubs.insertItem(info, 0);
	ListViewSetItemState(ctrlHubs.m_hWnd, nItem, 
		INDEXTOSTATEIMAGEMASK(((((UINT)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMSTATE, 0, LVIS_STATEIMAGEMASK) >> 12) - 1) ? info->op : true) ? 2 : 1), 
		LVIS_STATEIMAGEMASK);
	::SendMessage(ctrlHubs.m_hWnd, LVM_SETCOLUMNWIDTH, 0, MAKELPARAM(LVSCW_AUTOSIZE, 0));
}

void SearchFrame::onHubChanged(HubInfo* info) {
	int nItem = 0;
	int n = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMCOUNT, 0, 0L);
	for(; nItem < n; nItem++) {
		if(ctrlHubs.getItemData(nItem)->url == info->url)
			break;
	}
	if (nItem == n)
		return;

	delete ctrlHubs.getItemData(nItem);

	LVITEM lvi = { 0 };
	lvi.mask = LVIF_PARAM;
	lvi.iItem = nItem;
	lvi.iSubItem = 0;
	lvi.stateMask = 0;
	lvi.state = 0;
	lvi.pszText = NULL;
	lvi.iImage = 0;
	lvi.lParam = (LPARAM)(DWORD_PTR)info;
	::SendMessage(ctrlHubs.m_hWnd, LVM_SETITEM, 0, (LPARAM)&lvi);

	ctrlHubs.updateItem(nItem);

	if (((UINT)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMSTATE, 0, LVIS_STATEIMAGEMASK) >> 12) - 1) {
		ListViewSetItemState(ctrlHubs.m_hWnd, nItem, INDEXTOSTATEIMAGEMASK(info->op ? 2 : 1), LVIS_STATEIMAGEMASK);
	}

	::SendMessage(ctrlHubs.m_hWnd, LVM_SETCOLUMNWIDTH, 0, MAKELPARAM(LVSCW_AUTOSIZE, 0));
}

void SearchFrame::onHubRemoved(HubInfo* info) {
	int nItem = 0;
	int n = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETITEMCOUNT, 0, 0L);
	for(; nItem < n; nItem++) {
		if(ctrlHubs.getItemData(nItem)->url == info->url) {
            delete info;
			break;
        }
	}
	
	if(nItem == n)
		return;

	delete ctrlHubs.getItemData(nItem);
	::SendMessage(ctrlHubs.m_hWnd, LVM_DELETEITEM, nItem, 0L);
	::SendMessage(ctrlHubs.m_hWnd, LVM_SETCOLUMNWIDTH, 0, MAKELPARAM(LVSCW_AUTOSIZE, 0));
}

void SearchFrame::onGetList() {
	ctrlResults.forEachSelected(&SearchInfo::getList);
}

void SearchFrame::onItemChangedHub(LPNMHDR pnmh) {
	NMLISTVIEW* lv = (NMLISTVIEW*)pnmh;

	if(lv->iItem == 0 && (lv->uNewState ^ lv->uOldState) & LVIS_STATEIMAGEMASK) {
		if(((lv->uNewState & LVIS_STATEIMAGEMASK) >> 12) - 1) {
			for(int iItem = 0; (iItem = (int)::SendMessage(ctrlHubs.m_hWnd, LVM_GETNEXTITEM, iItem, MAKELPARAM(LVNI_ALL, 0))) != -1;) {
				HubInfo* client = ctrlHubs.getItemData(iItem);
				if(!client->op) {
					ListViewSetItemState(ctrlHubs.m_hWnd, iItem, INDEXTOSTATEIMAGEMASK(1), LVIS_STATEIMAGEMASK);
				}
			}
		}
	}
}

void SearchFrame::SearchInfo::update() {
	if(sr->getType() == SearchResult::TYPE_FILE) {
		if(sr->getFile().rfind(_T('\\')) == tstring::npos) {
			columns[COLUMN_FILENAME] = Text::toT(sr->getUtf8() ? sr->getFile() : Text::acpToUtf8(sr->getFile()));
		} else {
			columns[COLUMN_FILENAME] = Text::toT(Util::getFileName(sr->getUtf8() ? sr->getFile() : Text::acpToUtf8(sr->getFile())));
			columns[COLUMN_PATH] = Text::toT(Util::getFilePath(sr->getUtf8() ? sr->getFile() : Text::acpToUtf8(sr->getFile())));
		}

		columns[COLUMN_TYPE] = Text::toT(Util::getFileExt(Text::fromT(columns[COLUMN_FILENAME])));
		if(!columns[COLUMN_TYPE].empty() && columns[COLUMN_TYPE][0] == _T('.'))
			columns[COLUMN_TYPE].erase(0, 1);
		columns[COLUMN_SIZE] = Text::toT(Util::formatBytes(sr->getSize()));
		columns[COLUMN_EXACT_SIZE] = Text::toT(Util::formatExactSize(sr->getSize()));
	} else {
		columns[COLUMN_FILENAME] = Text::toT(sr->getUtf8() ? sr->getFileName() : Text::acpToUtf8(sr->getFileName()));
		columns[COLUMN_PATH] = Text::toT(sr->getUtf8() ? sr->getFile() : Text::acpToUtf8(sr->getFile()));
		columns[COLUMN_TYPE] = TSTRING(DIRECTORY);
		if(sr->getSize() > 0) {
			columns[COLUMN_SIZE] = Text::toT(Util::formatBytes(sr->getSize()));
			columns[COLUMN_EXACT_SIZE] = Text::toT(Util::formatExactSize(sr->getSize()));
		}
	}
	columns[COLUMN_NICK] = WinUtil::getNicks(sr->getUser());
	columns[COLUMN_CONNECTION] = Text::toT(Text::toUtf8(ClientManager::getInstance()->getConnection(sr->getUser()->getCID())));
	columns[COLUMN_HUB] = Text::toT(sr->getHubName());
	columns[COLUMN_SLOTS] = Text::toT(sr->getSlotString());

	if(sr->getUser()->isOnline()) {
		OnlineUser& ou = ClientManager::getInstance()->getOnlineUser(sr->getUser());
		if(&ou && ou.getClient().isOp()) {
			columns[COLUMN_IP] = Text::toT(sr->getIP());
		}
	}

	if(sr->getTTH() != NULL)
		columns[COLUMN_TTH] = Text::toT(sr->getTTH()->toBase32());

	flagimage = CountryFlags::GetFlagImageIndex(Util::getIpCountry(sr->getIP().c_str()).c_str());
}

void SearchFrame::onCopy(WORD wID) {
	int pos = (int)::SendMessage(ctrlResults.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_SELECTED, 0));
	if(pos >= 0) {
		SearchResult* sr = ctrlResults.getItemData(pos)->sr;
		switch(wID) {
			case IDC_COPY_NICK:
				WinUtil::setClipboard(Text::toT(sr->getUser()->getFirstNick()));
				break;
			case IDC_COPY_FILENAME:
				WinUtil::setClipboard(Text::toT(Util::getFileName(sr->getUtf8() ? sr->getFile() : Text::acpToUtf8(sr->getFile()))));
				break;
			case IDC_COPY_PATH:
				WinUtil::setClipboard(Text::toT(Util::getFilePath(sr->getUtf8() ? sr->getFile() : Text::acpToUtf8(sr->getFile()))));
				break;
			case IDC_COPY_SIZE:
				WinUtil::setClipboard(Text::toT(Util::formatBytes(sr->getSize())));
				break;
			case IDC_COPY_LINK:
				if(sr->getType() == SearchResult::TYPE_FILE) {
					WinUtil::copyMagnet(sr->getTTH(), sr->getSize(), Text::toT(sr->getUtf8() ? sr->getFileName() : Text::acpToUtf8(sr->getFileName())));
                }
				break;
			case IDC_COPY_TTH:
				if(sr->getType() == SearchResult::TYPE_FILE && sr->getTTH()) {
					WinUtil::setClipboard(Text::toT(sr->getTTH()->toBase32()));
                }
				break;
			default:
				break;
		}
	}
}

LRESULT SearchFrame::onCustomDraw(LPNMHDR pnmh) {
	LPNMLVCUSTOMDRAW cd = (LPNMLVCUSTOMDRAW)pnmh;

	switch(cd->nmcd.dwDrawStage) {
        case CDDS_PREPAINT:
            return CDRF_NOTIFYITEMDRAW;
        case CDDS_ITEMPREPAINT:
            return CDRF_NOTIFYSUBITEMDRAW;
        case CDDS_SUBITEM | CDDS_ITEMPREPAINT: {
            RECT rc;
            int colIndex = ctrlResults.findColumn(cd->iSubItem);

            if(colIndex == COLUMN_NICK) {
                SearchInfo* si = (SearchInfo*)cd->nmcd.lItemlParam;
                rc.top = cd->iSubItem;
                rc.left = LVIR_BOUNDS;
                ::SendMessage(ctrlResults.m_hWnd, LVM_GETSUBITEMRECT, (int)cd->nmcd.dwItemSpec, (LPARAM)&rc);
                COLORREF color;
                if((UINT)::SendMessage(ctrlResults.m_hWnd, LVM_GETITEMSTATE, (int)cd->nmcd.dwItemSpec, LVIS_SELECTED) & LVIS_SELECTED) {
				    if(ctrlResults.m_hWnd == ::GetFocus()) {
                        color = GetSysColor(COLOR_HIGHLIGHT);
                        SetBkColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHT));
                        SetTextColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHTTEXT));
                    } else {
                        color = GetBkColor(cd->nmcd.hdc);
                        SetBkColor(cd->nmcd.hdc, color);
                    }
                } else {
				    color = WinUtil::bgColor;
				    SetBkColor(cd->nmcd.hdc, WinUtil::bgColor);
				    SetTextColor(cd->nmcd.hdc, WinUtil::textColor);
                }
                RECT rc2 = rc;
                rc2.left += 2;
                HGDIOBJ oldpen = ::SelectObject(cd->nmcd.hdc, CreatePen(PS_SOLID,0, color));
                HGDIOBJ oldbr = ::SelectObject(cd->nmcd.hdc, CreateSolidBrush(color));
                Rectangle(cd->nmcd.hdc,rc.left, rc.top, rc.right, rc.bottom);
                DeleteObject(::SelectObject(cd->nmcd.hdc, oldpen));
                DeleteObject(::SelectObject(cd->nmcd.hdc, oldbr));
                TCHAR buf[256];
                ListViewGetItemText(ctrlResults.m_hWnd, (int)cd->nmcd.dwItemSpec, cd->iSubItem, buf, 255);
                buf[255] = 0;
                POINT p = { rc2.left, rc2.top };
                string country = Text::fromT(buf);
                OnlineUser& ou = ClientManager::getInstance()->getOnlineUser(si->sr->getUser());
                if(&ou != NULL) {
                    ImageList_Draw(WinUtil::userImages, WinUtil::getImage(ou.getIdentity(), true), cd->nmcd.hdc, p.x, p.y, LVSIL_SMALL);
                } else {
                    ImageList_Draw(WinUtil::userImages, 23, cd->nmcd.hdc, p.x, p.y, LVSIL_SMALL);
                }
                if(FavoriteManager::getInstance()->isFavoriteUser(si->sr->getUser())) SetTextColor(cd->nmcd.hdc, SETTING(TEXT_FAV_FORE_COLOR));
                LONG top = rc2.top + ((rc2.bottom - rc2.top) - WinUtil::getTextHeight(cd->nmcd.hdc) - 1)/2;
                ::ExtTextOut(cd->nmcd.hdc, rc2.left + 20, top, ETO_CLIPPED, &rc2, buf, (uint32_t)_tcslen(buf), NULL);
                return CDRF_NEWFONT | CDRF_SKIPDEFAULT;
            } else if(colIndex == COLUMN_CONNECTION) {
                SearchInfo* si = (SearchInfo*)cd->nmcd.lItemlParam;
                rc.top = cd->iSubItem;
                rc.left = LVIR_BOUNDS;
                ::SendMessage(ctrlResults.m_hWnd, LVM_GETSUBITEMRECT, (int)cd->nmcd.dwItemSpec, (LPARAM)&rc);
                COLORREF color;
                if((UINT)::SendMessage(ctrlResults.m_hWnd, LVM_GETITEMSTATE, (int)cd->nmcd.dwItemSpec, LVIS_SELECTED) & LVIS_SELECTED) {
                    if(ctrlResults.m_hWnd == ::GetFocus()) {
                        color = GetSysColor(COLOR_HIGHLIGHT);
                        SetBkColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHT));
                        SetTextColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHTTEXT));
				    } else {
                        color = GetBkColor(cd->nmcd.hdc);
                        SetBkColor(cd->nmcd.hdc, color);
                    }
                } else {
				    color = WinUtil::bgColor;
				    SetBkColor(cd->nmcd.hdc, WinUtil::bgColor);
				    SetTextColor(cd->nmcd.hdc, WinUtil::textColor);
                }
                RECT rc2 = rc;
                rc2.left += 2;
                HGDIOBJ oldpen = ::SelectObject(cd->nmcd.hdc, CreatePen(PS_SOLID,0, color));
                HGDIOBJ oldbr = ::SelectObject(cd->nmcd.hdc, CreateSolidBrush(color));
                Rectangle(cd->nmcd.hdc,rc.left, rc.top, rc.right, rc.bottom);
                DeleteObject(::SelectObject(cd->nmcd.hdc, oldpen));
                DeleteObject(::SelectObject(cd->nmcd.hdc, oldbr));
                POINT p = { rc2.left, rc2.top };
                OnlineUser& ou = ClientManager::getInstance()->getOnlineUser(si->sr->getUser());
                if(&ou != NULL) {
                    ImageList_Draw(WinUtil::speedImages, WinUtil::getSpeedImage(ou.getIdentity().getcType()), cd->nmcd.hdc, p.x, p.y, LVSIL_SMALL);
                } else {
                    ImageList_Draw(WinUtil::speedImages, WinUtil::getSpeedImage(11), cd->nmcd.hdc, p.x, p.y, LVSIL_SMALL);
                }
                LONG top = rc2.top + ((rc2.bottom - rc2.top) - WinUtil::getTextHeight(cd->nmcd.hdc) - 1)/2;
                if(&ou != NULL) {
                    tstring tsConn = Text::toT(Text::toUtf8(ou.getIdentity().getConnection()));
                    ::ExtTextOut(cd->nmcd.hdc, rc2.left + 20, top, ETO_CLIPPED, &rc2, tsConn.c_str(), (uint32_t)tsConn.size(), NULL);
                }
                return CDRF_NEWFONT | CDRF_SKIPDEFAULT;
            } else if(colIndex == COLUMN_IP) {
                SearchInfo* si = (SearchInfo*)cd->nmcd.lItemlParam;
                if(si->sr->getIP() != "" && BOOLSETTING(GET_USER_COUNTRY)) {
                    rc.top = cd->iSubItem;
                    rc.left = LVIR_BOUNDS;
                    ::SendMessage(ctrlResults.m_hWnd, LVM_GETSUBITEMRECT, (int)cd->nmcd.dwItemSpec, (LPARAM)&rc);
                    COLORREF color;
                    if((UINT)::SendMessage(ctrlResults.m_hWnd, LVM_GETITEMSTATE, (int)cd->nmcd.dwItemSpec, LVIS_SELECTED) & LVIS_SELECTED) {
                        if(ctrlResults.m_hWnd == ::GetFocus()) {
                            color = GetSysColor(COLOR_HIGHLIGHT);
                            SetBkColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHT));
                            SetTextColor(cd->nmcd.hdc, GetSysColor(COLOR_HIGHLIGHTTEXT));
                        } else {
                            color = GetBkColor(cd->nmcd.hdc);
                            SetBkColor(cd->nmcd.hdc, color);
                        }
                    } else {
                        color = WinUtil::bgColor;
                        SetBkColor(cd->nmcd.hdc, WinUtil::bgColor);
                        SetTextColor(cd->nmcd.hdc, cd->clrText);
                    }
                    RECT rc2 = rc;
                    rc2.left += 2;

                    HGDIOBJ oldpen = ::SelectObject(cd->nmcd.hdc, CreatePen(PS_SOLID,0, color));
                    HGDIOBJ oldbr = ::SelectObject(cd->nmcd.hdc, CreateSolidBrush(color));
                    Rectangle(cd->nmcd.hdc,rc.left, rc.top, rc.right, rc.bottom);
                    DeleteObject(::SelectObject(cd->nmcd.hdc, oldpen));
                    DeleteObject(::SelectObject(cd->nmcd.hdc, oldbr));
                    LONG top = rc2.top + ((rc2.bottom - rc2.top) - 15)/2;
                    if((top - rc2.top) < 2)
                        top = rc2.top + 1;
                    TCHAR buf[256];
                    ListViewGetItemText(ctrlResults.m_hWnd, (int)cd->nmcd.dwItemSpec, cd->iSubItem, buf, 255);
                    buf[255] = 0;
                    POINT p = { rc2.left, top };
                    ImageList_Draw(CountryFlags::flagImages, si->getflagImage(), cd->nmcd.hdc, p.x, p.y, LVSIL_SMALL);
                    top = rc2.top + ((rc2.bottom - rc2.top) - WinUtil::getTextHeight(cd->nmcd.hdc) - 1)/2;
                    ::ExtTextOut(cd->nmcd.hdc, rc2.left + 29, top + 1, ETO_CLIPPED, &rc2, buf, (uint32_t)_tcslen(buf), NULL);
                    return CDRF_SKIPDEFAULT;
                }
            }
        }
        default:
            return CDRF_DODEFAULT;
	}
}

void SearchFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void SearchFrame::onRefreshSettings() {
	bool refresh = false;

	if((COLORREF)::SendMessage(ctrlResults.m_hWnd, LVM_GETBKCOLOR, 0, 0L) != WinUtil::bgColor) {
		::SendMessage(ctrlResults.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlResults.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlHubs.m_hWnd, LVM_SETBKCOLOR, 0, WinUtil::bgColor);
		::SendMessage(ctrlHubs.m_hWnd, LVM_SETTEXTBKCOLOR, 0, WinUtil::bgColor);
		refresh = true;
	}
	if((COLORREF)::SendMessage(ctrlResults.m_hWnd, LVM_GETTEXTCOLOR, 0, 0L) != WinUtil::textColor) {
		::SendMessage(ctrlResults.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
		::SendMessage(ctrlHubs.m_hWnd, LVM_SETTEXTCOLOR, 0, WinUtil::textColor);
		refresh = true;
	}

	if(refresh == true) {
		::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
	}

	::SendMessage(hWndSearchBox, CB_RESETCONTENT, 0, 0L);

	lastSearches.clear();

	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_01)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_02)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_03)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_04)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_05)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_06)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_07)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_08)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_09)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_10)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_11)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_12)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_13)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_14)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_15)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_16)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_17)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_18)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_19)));
	lastSearches.push_back(Text::toT(SETTING(SEARCH_MSG_RECENT_20)));

	for(TStringIter i = lastSearches.begin(); i != lastSearches.end(); ++i) {
		::SendMessage(hWndSearchBox, CB_INSERTSTRING, 0, (LPARAM)i->c_str());
	}

	::SendMessage(hWndSearchBox, CB_SETCURSEL, 0, 0L);
}

void SearchFrame::closeAll() {
    for(FrameIter i=frames.begin(); i!= frames.end(); ++i) {
		PostMessage(i->second->m_hWnd, WM_CLOSE, 0, 0);
	}
}

void SearchFrame::onDownloadWithPrio(WORD wID) {
	QueueItem::Priority p;

	switch(wID) {
		case IDC_PRIORITY_PAUSED:
            p = QueueItem::PAUSED;
            break;
		case IDC_PRIORITY_LOWEST:
            p = QueueItem::LOWEST;
            break;
		case IDC_PRIORITY_LOW:
            p = QueueItem::LOW;
            break;
		case IDC_PRIORITY_NORMAL:
            p = QueueItem::NORMAL;
            break;
		case IDC_PRIORITY_HIGH:
            p = QueueItem::HIGH;
            break;
		case IDC_PRIORITY_HIGHEST:
            p = QueueItem::HIGHEST;
            break;
		default:
            p = QueueItem::DEFAULT;
            break;
	}

	ctrlResults.forEachSelectedT(SearchInfo::Download(Text::toT(SETTING(DOWNLOAD_DIRECTORY)), p));
}

void SearchFrame::on(TimerManagerListener::Second, uint64_t /*aTick*/) throw() {
	if(needresort == true) {
		PostMessage(m_hWnd, WM_SPEAKER, RESORT, 0);
	}
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
