/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef SEARCH_FRM_H
#define SEARCH_FRM_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "MDIChildWindow.h"
#include "TypedListViewCtrl.h"
#include "WinUtil.h"

#include "../client/Client.h"
#include "../client/SearchManager.h"
#include "../client/CriticalSection.h"
#include "../client/ClientManagerListener.h"
#include "../client/FavoriteManager.h"
#include "../client/QueueItem.h"

#include "UCHandler.h"

class SearchFrame : private MDITabChildWindow, private SearchManagerListener, private ClientManagerListener, private UCHandler<SearchFrame>,
    private UserInfoBaseHandler<SearchFrame>, private SettingsManagerListener, private TimerManagerListener {
public:
    SearchFrame();
	~SearchFrame();

	static void openWindow(const tstring& str = Util::emptyStringW, LONGLONG size = 0, SearchManager::SizeModes mode = SearchManager::SIZE_ATLEAST, SearchManager::TypeModes type = SearchManager::TYPE_ANY);

    static ATOM m_aAtom;
    static UINT m_uiResourceID;

    static LRESULT CALLBACK SearchFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    LRESULT SearchFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	bool onChar(UINT uMsg, WPARAM wParam);
	bool onClose();
	bool onContextMenu(WPARAM wParam, LPARAM lParam);
	void onCreate();
	LRESULT onCtlColor(WPARAM wParam, LPARAM lParam);
	void onDoubleClickResults(LPNMHDR pnmh);
	void onDownloadTarget(WORD wID);
	void onDownloadTo();
	void onDownloadWholeTarget(WORD wID);
	void onDownloadWholeTo();
	void onDownloadFavoriteDirs(WORD wID);
	void onDownloadWholeFavoriteDirs(WORD wID);
	void onSpeaker(WPARAM wParam, LPARAM lParam);
	void onSearchByTTH();
	void onCopy(WORD wID);
	LRESULT onCustomDraw(LPNMHDR pnmh);
	void onTimer();
	void onGetList();
	void onDownloadWithPrio(WORD wID);
	void onRefreshSettings();
	void UpdateLayout(bool bResizeBars = true);
	void runUserCommand(UserCommand& uc);

	void removeSelected() {
		int i = -1;
		Lock l(cs);
		while( (i = (int)::SendMessage(ctrlResults.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_SELECTED, 0))) != -1) {
			ctrlResults.removeGroupedItem(ctrlResults.getItemData(i));
		}
	}

	void onDownload() {
		ctrlResults.forEachSelectedT(SearchInfo::Download(Text::toT(SETTING(DOWNLOAD_DIRECTORY))));
	}

	void onViewAsText() {
		ctrlResults.forEachSelected(&SearchInfo::view);
	}

	void onDownloadWhole() {
		ctrlResults.forEachSelectedT(SearchInfo::DownloadWhole(Text::toT(SETTING(DOWNLOAD_DIRECTORY))));
	}

	void onRemove() {
		removeSelected();
	}

	void onFreeSlots() {
		onlyFree = ((int)::SendMessage(hWndSlots, BM_GETCHECK, 0, 0L) == 1);
		SettingsManager::getInstance()->set(SettingsManager::FREE_SLOTS_DEFAULT, onlyFree);
	}

	void onTTH() {
		onlyTTH = ((int)::SendMessage(hWndTTH, BM_GETCHECK, 0, 0L) == 1);
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_ONLY_TTH, onlyTTH);
	}

	void onCollapsed() {
		expandSR = ((int)::SendMessage(hWndCollapsed, BM_GETCHECK, 0, 0L) == 1);
		SettingsManager::getInstance()->set(SettingsManager::SEARCH_EXPAND, expandSR);
	}

	bool onKeyDown(LPNMHDR pnmh) {
		NMLVKEYDOWN* kd = (NMLVKEYDOWN*) pnmh;

		if(kd->wVKey == VK_DELETE) {
			removeSelected();
			return true;
		}

		return false;
	}

	void onFocus() {
		if(::IsWindow(hWndSearch)) {
			::SetFocus(hWndSearch);
        }
	}

	void onShowUI(WPARAM wParam) {
		showUI = (wParam == BST_CHECKED);
		UpdateLayout(false);
	}

	void SearchFrame::setInitial(const tstring& str, LONGLONG size, SearchManager::SizeModes mode, SearchManager::TypeModes type) {
		initialString = str; initialSize = size; initialMode = mode; initialType = type; bPaused = false;
	}

	void onPause() {
		if(bPaused == true) {
			bPaused = false;
			for(SearchInfo::Iter i = PausedResults.begin(); i != PausedResults.end(); ++i) {
				::PostMessage(m_hWnd, WM_SPEAKER, ADD_RESULT, (LPARAM)(*i));
			}
			PausedResults.clear();
			::SendMessage(m_hWndStatusBar, SB_SETTEXT, (2 | 0),
                (LPARAM)Text::toT(Util::toString((int)::SendMessage(ctrlResults.m_hWnd, LVM_GETITEMCOUNT, 0, 0L)) + " " + STRING(FILES)).c_str());
			::SetWindowText(hWndPauseSearch, CTSTRING(PAUSE_SEARCH));
		} else {
			bPaused = true;
			::SetWindowText(hWndPauseSearch, CTSTRING(CONTINUE_SEARCH));
		}
	}

	static TStringList lastSearches;
	static void closeAll();
private:
	class SearchInfo;
public:
	TypedTreeListViewCtrl<SearchInfo, IDC_RESULTS>& getUserList() { return ctrlResults; }

private:
	enum {
		COLUMN_FIRST,
		COLUMN_FILENAME = COLUMN_FIRST,
		COLUMN_HITS,
		COLUMN_NICK,
		COLUMN_TYPE,
		COLUMN_SIZE,
		COLUMN_PATH,
		COLUMN_SLOTS,
		COLUMN_CONNECTION,
		COLUMN_HUB,
		COLUMN_EXACT_SIZE,
		COLUMN_IP,
		COLUMN_TTH,
		COLUMN_LAST
	};

	class SearchInfo : public UserInfoBase {
	public:
		typedef vector<SearchInfo*> List;
		typedef List::iterator Iter;

		typedef SearchInfo* Ptr;
		typedef vector<Ptr> List;
		typedef List::iterator Iter;

		SearchInfo::List subItems;

		SearchInfo(SearchResult* aSR) : UserInfoBase(aSR->getUser()), sr(aSR), collapsed(true), main(NULL) {
            resultsCount++;
			sr->incRef();
            update();
		}
		~SearchInfo() {
            resultsCount--;
			sr->decRef(); 
		}

		bool collapsed;
		SearchInfo* main;

		void getList();
		void view();
		struct Download {
			Download(const tstring& aTarget, QueueItem::Priority Priority = QueueItem::DEFAULT) : tgt(aTarget), prio(Priority) { }
			void operator()(SearchInfo* si);
			const tstring& tgt;
			const QueueItem::Priority prio;
		};
		struct DownloadWhole {
			DownloadWhole(const tstring& aTarget) : tgt(aTarget) { }
			void operator()(SearchInfo* si);
			const tstring& tgt;
		};
		struct DownloadTarget {
			DownloadTarget(const tstring& aTarget) : tgt(aTarget) { }
			void operator()(SearchInfo* si);
			const tstring& tgt;
		};
		struct CheckSize {
			CheckSize() : size(-1), op(true), oneHub(true), hasTTH(false), firstTTH(true) { }
			void operator()(SearchInfo* si);
			tstring ext;
			int64_t size;
			bool oneHub;
			string hub;
			bool op;
			bool hasTTH;
			bool firstTTH;
			tstring tth;
		};

		const TCHAR* getText(const int &col, TCHAR* buf) const {
            return columns[col].c_str();
        }

		static int compareItems(SearchInfo* a, SearchInfo* b, int col) {
			if(!a->sr || !b->sr) return 0;

			switch(col) {
				case COLUMN_TYPE:
					if(a->sr->getType() == b->sr->getType()) {
                        return lstrcmpi(a->columns[COLUMN_TYPE].c_str(), b->columns[COLUMN_TYPE].c_str());
					} else {
						return(a->sr->getType() == SearchResult::TYPE_DIRECTORY) ? -1 : 1;
					}
				case COLUMN_SLOTS: 
					if(a->sr->getFreeSlots() == b->sr->getFreeSlots()) {
						return compare(a->sr->getSlots(), b->sr->getSlots());
					} else {
						return compare(a->sr->getFreeSlots(), b->sr->getFreeSlots());
					}
				case COLUMN_SIZE:
				case COLUMN_EXACT_SIZE: return compare(a->sr->getSize(), b->sr->getSize());
				case COLUMN_HITS: return compare(a->subItems.size(), b->subItems.size());
				default:
                    return lstrcmpi(a->columns[col].c_str(), b->columns[col].c_str());
			}
		}

		int imageIndex() {
			int image = sr->getType() == SearchResult::TYPE_FILE ? WinUtil::getIconIndex(Text::toT(sr->getFile())) : WinUtil::getDirIconIndex();
			return image;
		}

		void update();
		
		SearchInfo* createMainItem() { return this; }
    	const tstring& getGroupingString() { return columns[COLUMN_TTH]; }
    	void updateMainItem() {
    		size_t total = main->subItems.size();
    		if(total != 0) {
    			TCHAR buf[256];
    			_stprintf_s(buf, 256, _T("%d %s"), total + 1, CTSTRING(USERS));
    			main->columns[COLUMN_HITS] = buf;
				if(total == 1)
					main->columns[COLUMN_SIZE] = columns[COLUMN_SIZE];
    		} else {
    			main->columns[COLUMN_HITS] = Util::emptyStringT;
    		}
    	}

		GETSET(uint8_t, flagimage, flagImage);
		SearchResult* sr;
		tstring columns[COLUMN_LAST];
	};

	struct HubInfo {
		HubInfo(const tstring& aUrl, const tstring& aName, bool aOp) : url(aUrl),
			name(aName), op(aOp) { }

		const TCHAR* getText(const int &col, TCHAR* buf) const {
			return (col == 0) ? name.c_str() : Util::emptyStringT.c_str();
		}
		static int compareItems(HubInfo* a, HubInfo* b, int col) {
			return (col == 0) ? lstrcmpi(a->name.c_str(), b->name.c_str()) : 0;
		}
		uint8_t imageIndex() {
			return 0;
		}

		tstring url;
		tstring name;
		bool op;
	};

	// WM_SPEAKER
	enum Speakers {
		ADD_RESULT,
		FILTER_RESULT,
		HUB_ADDED,
		HUB_CHANGED,
		HUB_REMOVED,
		STATS, 
		RESORT
	};

	tstring initialString;
	int64_t initialSize;
	SearchManager::SizeModes initialMode;
	SearchManager::TypeModes initialType;

	HWND hWndSearchBox, hWndSearch, hWndSize, hWndMode, hWndSizeMode, hWndFiletype, hWndDoSearch, hWndPauseSearch;

	BOOL ListMeasure(HWND hwnd, UINT uCtrlId, MEASUREITEMSTRUCT *mis);
	BOOL ListDraw(HWND hwnd, UINT uCtrlId, DRAWITEMSTRUCT *dis);
	
    HWND hWndSearchLabel, hWndSizeLabel, hWndOptionLabel, hWndTypeLabel, hWndHubsLabel;
    HWND hWndSlots, hWndTTH, hWndCollapsed, hWndShowUI;

    WNDPROC DefHubsListProc;

	bool showUI;

	TypedTreeListViewCtrl<SearchInfo, IDC_RESULTS> ctrlResults;
	TypedListViewCtrl<HubInfo, IDC_HUB> ctrlHubs;	

	HMENU resultsMenu;
	HMENU targetMenu;
	HMENU targetDirMenu;

	TStringList search;
	StringList targets;
	StringList wholeTargets;
	SearchInfo::List PausedResults;

	bool onlyFree;
	bool isHash;
	bool onlyTTH;
	bool bPaused;
	bool bWarned;
	bool useGrouping;
	bool expandSR;
	static int64_t resultsCount;

	CriticalSection cs;

	size_t droppedResults;

	bool closed;
	bool needresort;

	StringMap ucLineParams;

	static int columnIndexes[];
	static int columnSizes[];

	// Timer ID, needed to turn off timer
	UINT_PTR timerID;

	typedef map<HWND, SearchFrame*> FrameMap;
	typedef FrameMap::iterator FrameIter;
	typedef pair<HWND, SearchFrame*> FramePair;
	static FrameMap frames;

	void downloadSelected(const tstring& aDir, bool view = false);
	void downloadWholeSelected(const tstring& aDir);
	void onEnter();
	void onTab(bool shift);

	void download(SearchResult* aSR, const tstring& aDir, bool view);

	void on(SearchManagerListener::SR, SearchResult* aResult) throw();

	// ClientManagerListener
	void on(ClientConnected, Client* c) throw() { speak(HUB_ADDED, c); }
	void on(ClientUpdated, Client* c) throw() { speak(HUB_CHANGED, c); }
	void on(ClientDisconnected, Client* c) throw() { speak(HUB_REMOVED, c); }
	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();

	void on(TimerManagerListener::Second, uint64_t /*aTick*/) throw();

	void initHubs();
	void onHubAdded(HubInfo* info);
	void onHubChanged(HubInfo* info);
	void onHubRemoved(HubInfo* info);

	void onItemChangedHub(LPNMHDR pnmh);

	void speak(Speakers s, Client* aClient) {
		HubInfo* hubInfo = new HubInfo(Text::toT(aClient->getHubUrl()), Text::toT(aClient->getHubName()), aClient->getMyIdentity().isOp());
		::PostMessage(m_hWnd, WM_SPEAKER, WPARAM(s), LPARAM(hubInfo));
	}

    static LRESULT CALLBACK SearchWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    static LRESULT CALLBACK ShowUIWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
};

#endif // SEARCH_FRM_H

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
