/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2009 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "SplitterWindow.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "WinUtil.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#define SPLIT_BAR_SIZE                  4
//----------------------------------------------------------------------------------------------------------------------------------------------------
static const int iPropMax = 10000;
//----------------------------------------------------------------------------------------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CSplitter

CSplitter::CSplitter() : iSplitterPos(-1), i8DefActivePane(SPLIT_PANE_NONE), iDragOffset(0), iProportionalPos(0), bUpdateProportionalPos(true),
    i8SinglePane(SPLIT_PANE_NONE), ui8ExtendedStyle(0) {
	hWndPanes[SPLIT_PANE_LEFT_OR_TOP] = NULL;
	hWndPanes[SPLIT_PANE_RIGHT_OR_BOTTOM] = NULL;
	m_hSplitWnd = NULL;

	::SetRectEmpty(&rcSplitter);

    bVertical = true;
	hCursor = WinUtil::hVerticalCursor;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

bool CSplitter::CSplitterProc(UINT uMsg, WPARAM wParam, LPARAM lParam, LRESULT& lResult) {
    switch(uMsg) {
        case WM_PAINT:
        case WM_PRINTCLIENT:
            OnPaint();
            return true;
        case WM_SETFOCUS:
            OnSetFocus();
            return false;
        case WM_MOUSEACTIVATE:
            OnMouseActivate(lResult);
            return true;
        case WM_MOUSEMOVE:
            if(((ui8ExtendedStyle & SPLIT_NONINTERACTIVE) == SPLIT_NONINTERACTIVE) == false) {
                return OnMouseMove(wParam, lParam);
            }

            break;
        case WM_LBUTTONDOWN:
            if(((ui8ExtendedStyle & SPLIT_NONINTERACTIVE) == SPLIT_NONINTERACTIVE) == false) {
                OnLButtonDown(lParam);
                return false;
            }

            break;
        case WM_LBUTTONUP:
            if(((ui8ExtendedStyle & SPLIT_NONINTERACTIVE) == SPLIT_NONINTERACTIVE) == false) {
                OnLButtonUp();
                return false;
            }

            break;
    }

    return false;
}

//----------------------------------------------------------------------------------------------------------------------------------------------------

void CSplitter::OnPaint() {
	if(i8SinglePane == SPLIT_PANE_NONE && iSplitterPos == -1) {
        int iPos = 0;
		if(bVertical == true) {
			iPos = (rcSplitter.right - rcSplitter.left - SPLIT_BAR_SIZE) / 2;
		} else {
			iPos = (rcSplitter.bottom - rcSplitter.top - SPLIT_BAR_SIZE) / 2;
        }

		SetSplitterPos(iPos);
    }

	PAINTSTRUCT m_ps;
	HDC dc = ::BeginPaint(m_hSplitWnd, &m_ps);

	DrawSplitter(dc);

	::EndPaint(m_hSplitWnd, &m_ps);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

bool CSplitter::OnMouseMove(WPARAM wParam, LPARAM lParam) {
	int ixPos = (LONG)LOWORD(lParam);
	int iyPos = (LONG)HIWORD(lParam);

	if((wParam & MK_LBUTTON) && ::GetCapture() == m_hSplitWnd) {
		int iNewSplitPos = 0;

		if(bVertical == true) {
			iNewSplitPos = ixPos - rcSplitter.left - iDragOffset;
		} else {
			iNewSplitPos = iyPos - rcSplitter.top - iDragOffset;
        }

		if(iNewSplitPos < 0) {
			iNewSplitPos = 0;
        }

		if(iSplitterPos != iNewSplitPos) {
			if(SetSplitterPos(iNewSplitPos, true) == true) {
				::UpdateWindow(m_hSplitWnd);
            }
		}

		return true;
	} else {
		if(IsOverSplitterBar(ixPos, iyPos) == true) {
			::SetCursor(hCursor);
        }

		return false;
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void CSplitter::OnLButtonDown(LPARAM lParam) {
	int ixPos = (LONG)LOWORD(lParam);
	int iyPos = (LONG)HIWORD(lParam);

	if(IsOverSplitterBar(ixPos, iyPos) == true) {
		::SetCapture(m_hSplitWnd);

		::SetCursor(hCursor);

		if(bVertical == true) {
			iDragOffset = ixPos - rcSplitter.left - iSplitterPos;
		} else {
			iDragOffset = iyPos - rcSplitter.top - iSplitterPos;
        }
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void CSplitter::OnLButtonUp() {
	::ReleaseCapture();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void CSplitter::OnSetFocus() {
	if(i8SinglePane != SPLIT_PANE_NONE) {
        ::SetFocus(hWndPanes[i8SinglePane]);
    } else {
		if(i8DefActivePane != SPLIT_PANE_NONE) {
			::SetFocus(hWndPanes[i8DefActivePane]);
        }
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void CSplitter::OnMouseActivate(LRESULT lResult) {
	if(lResult == MA_ACTIVATE || lResult == MA_ACTIVATEANDEAT) {
		DWORD dwPos = ::GetMessagePos();
		POINT pt = { (LONG)LOWORD(dwPos), (LONG)HIWORD(dwPos) };

		::ScreenToClient(m_hSplitWnd, &pt);

		RECT rcPane;

		for(uint8_t i8Pane = 0; i8Pane < 2; i8Pane++) {
			if(GetSplitterPaneRect(i8Pane, &rcPane) && ::PtInRect(&rcPane, pt)) {
				i8DefActivePane = i8Pane;
				break;
			}
		}
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void CSplitter::SetSplitterRect(const LPRECT &lpRect) {
	rcSplitter = *lpRect;

    if(((ui8ExtendedStyle & SPLIT_NONINTERACTIVE) == SPLIT_NONINTERACTIVE) == false) {
    	int iTotal = bVertical == true? (rcSplitter.right - rcSplitter.left - SPLIT_BAR_SIZE) : (rcSplitter.bottom - rcSplitter.top - SPLIT_BAR_SIZE);

    	if(iTotal > 0) {
            bUpdateProportionalPos = false;

    		int iNewPos = ::MulDiv(iProportionalPos, iTotal, iPropMax);

       		SetSplitterPos(iNewPos, false);
    	}
    }

	UpdateSplitterLayout();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

bool CSplitter::SetSplitterPos(int iPos, const bool &bUpdate/* = true*/) {
	int iMax = 0;
	if(bVertical == true) {
		iMax = rcSplitter.right - rcSplitter.left;
	} else {
		iMax = rcSplitter.bottom - rcSplitter.top;
    }

	if(iPos < 10) {
		iPos = 10;
	} else if(iPos > (iMax - SPLIT_BAR_SIZE - 5)) {
		iPos = iMax - SPLIT_BAR_SIZE - 5;
    }

	bool bRet = (iSplitterPos != iPos);

	iSplitterPos = iPos;

    if(((ui8ExtendedStyle & SPLIT_NONINTERACTIVE) == SPLIT_NONINTERACTIVE) == false) {
    	if(bUpdateProportionalPos == true) {
        	int iTotal = bVertical == true ? (rcSplitter.right - rcSplitter.left - SPLIT_BAR_SIZE) : (rcSplitter.bottom - rcSplitter.top - SPLIT_BAR_SIZE);

        	if(iTotal > 0) {
        		iProportionalPos = ::MulDiv(iSplitterPos, iPropMax, iTotal);
        	} else {
        		iProportionalPos = 0;
            }
    	} else {
    		bUpdateProportionalPos = true;
    	}
    }

	if(bUpdate == true && bRet == true) {
		UpdateSplitterLayout();
    }

	return bRet;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void CSplitter::SetSinglePaneMode(const int8_t &i8Pane) {
	if(i8Pane != SPLIT_PANE_LEFT_OR_TOP && i8Pane != SPLIT_PANE_RIGHT_OR_BOTTOM && i8Pane != SPLIT_PANE_NONE) {
		return;
    }

	if(i8Pane != SPLIT_PANE_NONE) {
		if(::IsWindowVisible(hWndPanes[i8Pane]) == FALSE) {
			::ShowWindow(hWndPanes[i8Pane], SW_SHOW);
        }

		::ShowWindow(hWndPanes[(i8Pane == SPLIT_PANE_LEFT_OR_TOP) ? SPLIT_PANE_RIGHT_OR_BOTTOM : SPLIT_PANE_LEFT_OR_TOP], SW_HIDE);

		if(i8DefActivePane != i8Pane) {
			i8DefActivePane = i8Pane;
        }
	} else if(i8SinglePane != SPLIT_PANE_NONE) {
		::ShowWindow(hWndPanes[(i8SinglePane == SPLIT_PANE_LEFT_OR_TOP) ?
            SPLIT_PANE_RIGHT_OR_BOTTOM : SPLIT_PANE_LEFT_OR_TOP], SW_SHOW);
	}

	i8SinglePane = i8Pane;

	UpdateSplitterLayout();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void CSplitter::SetSplitterExtendedStyle(const uint8_t &ui8NewExtendedStyle) {
	ui8ExtendedStyle = ui8NewExtendedStyle;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void CSplitter::SetSplitterPanes(const HWND &hWndLeftTop, const HWND &hWndRightBottom) {
	hWndPanes[SPLIT_PANE_LEFT_OR_TOP] = hWndLeftTop;
	hWndPanes[SPLIT_PANE_RIGHT_OR_BOTTOM] = hWndRightBottom;

	UpdateSplitterLayout();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void CSplitter::DrawSplitter(const HDC &dc) {
	if(i8SinglePane == SPLIT_PANE_NONE) {
    	RECT rect;
    	if(GetSplitterBarRect(rect) == true) {
    		::FillRect(dc, &rect, (HBRUSH)LongToPtr(COLOR_3DFACE + 1));

            if((ui8ExtendedStyle & SPLIT_NONINTERACTIVE) == SPLIT_NONINTERACTIVE ||
                (bVertical == true && (rect.bottom - rect.top) < 50) || (bVertical == false && (rect.right - rect.left) < 50)) {
                return;
            }

            HPEN penShadow;

            if((penShadow = ::CreatePen(PS_SOLID, 1, ::GetSysColor(COLOR_3DDKSHADOW))) == NULL) {
                return;
            }

            HPEN oldpen = (HPEN)::SelectObject(dc, penShadow);

            int iFix = ((ui8ExtendedStyle & SPLIT_FIX_PAINT) == SPLIT_FIX_PAINT ? 1 : 0);
            if(bVertical == true) {
                int iCentr = (rect.bottom - rect.top) / 2;

                ::MoveToEx(dc, rect.left+iFix, iCentr-24, NULL);
    			::LineTo(dc, rect.left+iFix+3, iCentr-24);
                ::MoveToEx(dc, rect.left+iFix, iCentr+24, NULL);
    			::LineTo(dc, rect.left+iFix+3, iCentr+24);

    			for(int i = (iCentr-24)+3; i < iCentr+24; i += 3) {
                    ::MoveToEx(dc, rect.left+iFix+1, i, NULL);
                    ::LineTo(dc, rect.left+iFix+1, i+1);
                }
            } else {
                int iCentr = (rect.right - rect.left) / 2;

                ::MoveToEx(dc, iCentr-24, rect.top+iFix, NULL);
    			::LineTo(dc, iCentr-24, rect.top+iFix+3);
                ::MoveToEx(dc, iCentr+24, rect.top+iFix, NULL);
    			::LineTo(dc, iCentr+24, rect.top+iFix+3);

    			for(int i = (iCentr-24)+3; i < iCentr+24; i += 3) {
                    ::MoveToEx(dc, i, rect.top+iFix+1, NULL);
                    ::LineTo(dc, i+1, rect.top+iFix+1);
                }
            }

            ::SelectObject(dc, oldpen);
            ::DeleteObject(penShadow);
    	}
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void CSplitter::UpdateSplitterLayout() {
	if(i8SinglePane == SPLIT_PANE_NONE && iSplitterPos == -1) {
		return;
    }

	RECT rect = { 0, 0, 0, 0 };

	if(i8SinglePane == SPLIT_PANE_NONE) {
		if(GetSplitterBarRect(rect) == true) {
			::InvalidateRect(m_hSplitWnd, &rect, TRUE);
        }

		for(uint8_t i8Pane = 0; i8Pane < 2; i8Pane++) {
			if(GetSplitterPaneRect(i8Pane, &rect) == true) {
				::SetWindowPos(hWndPanes[i8Pane], NULL, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top, SWP_NOZORDER);
			}
		}
	} else {
		if(GetSplitterPaneRect(i8SinglePane, &rect) == true) {
			::SetWindowPos(hWndPanes[i8SinglePane], NULL, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top, SWP_NOZORDER);
		}
	}

	::SendMessage(m_hSplitWnd, WM_UPDATE_FILTER_LAYOUT, 0, 0);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

bool CSplitter::GetSplitterBarRect(RECT &Rect) {
	if(i8SinglePane != SPLIT_PANE_NONE || iSplitterPos == -1) {
		return false;
    }

	if(bVertical == true) {
		Rect.left = rcSplitter.left + iSplitterPos;
		Rect.top = rcSplitter.top;
		Rect.right = rcSplitter.left + iSplitterPos + SPLIT_BAR_SIZE;
		Rect.bottom = rcSplitter.bottom;
	} else {
		Rect.left = rcSplitter.left;
		Rect.top = rcSplitter.top + iSplitterPos;
		Rect.right = rcSplitter.right;
		Rect.bottom = rcSplitter.top + iSplitterPos + SPLIT_BAR_SIZE;
	}

	return true;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

bool CSplitter::GetSplitterPaneRect(const uint8_t &i8Pane, LPRECT lpRect) {
	if(i8SinglePane != SPLIT_PANE_NONE) {
		if(i8Pane == i8SinglePane) {
			*lpRect = rcSplitter;
		} else {
			return false;
        }
	} else if(i8Pane == SPLIT_PANE_LEFT_OR_TOP) {
		if(bVertical == true) {
			lpRect->left = rcSplitter.left;
			lpRect->top = rcSplitter.top;
			lpRect->right = rcSplitter.left + iSplitterPos;
			lpRect->bottom = rcSplitter.bottom;
		} else {
			lpRect->left = rcSplitter.left;
			lpRect->top = rcSplitter.top;
			lpRect->right = rcSplitter.right;
			lpRect->bottom = rcSplitter.top + iSplitterPos;
		}
	} else if(i8Pane == SPLIT_PANE_RIGHT_OR_BOTTOM) {
		if(bVertical == true) {
			lpRect->left = rcSplitter.left + iSplitterPos + SPLIT_BAR_SIZE;
			lpRect->top = rcSplitter.top;
			lpRect->right = rcSplitter.right;
			lpRect->bottom = rcSplitter.bottom;
		} else {
			lpRect->left = rcSplitter.left;
			lpRect->top = rcSplitter.top + iSplitterPos + SPLIT_BAR_SIZE;
			lpRect->right = rcSplitter.right;
			lpRect->bottom = rcSplitter.bottom;
		}
	}

	return true;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

bool CSplitter::IsOverSplitterBar(const int &x, const int &y) {
	if(i8SinglePane != SPLIT_PANE_NONE) {
		return false;
    }

	if(iSplitterPos == -1 || x == -1 || y == -1 ||
        x < rcSplitter.left || x > rcSplitter.right ||
        y < rcSplitter.top || y > rcSplitter.bottom) {
		return false;
    }

	int iPoint = (bVertical == true ? x : y);
	int iPointOff = (bVertical == true ? rcSplitter.left : rcSplitter.top);

	return ((iPoint >= (iPointOff + iSplitterPos)) && (iPoint < iPointOff + iSplitterPos + SPLIT_BAR_SIZE));
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void CSplitter::SetSplitterHorCursor() {
    bVertical = false;
    hCursor = WinUtil::hHorizontalCursor;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CSplitterWindow

CSplitterWindow::CSplitterWindow() {
	SetSplitterHorCursor();
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

LRESULT CALLBACK CSplitterWindow::SplitterWindowWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    CSplitterWindow * pParent;

    if(uMsg == WM_NCCREATE) {
        pParent = (CSplitterWindow *)((LPCREATESTRUCT)lParam)->lpCreateParams;
        SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

        pParent->m_hWnd = hWnd;
        pParent->m_hSplitWnd = hWnd;
    } else {
        pParent = (CSplitterWindow *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
        if(pParent == NULL) {
            return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
        }
    }

    return pParent->SplitterWindowProc(hWnd, uMsg, wParam, lParam);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

LRESULT CSplitterWindow::SplitterWindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    LRESULT lResult = 0;
    if(uMsg == WM_MOUSEACTIVATE) {
        lResult = ::DefWindowProc(m_hWnd, uMsg, wParam, lParam);
    }

	if(CSplitterProc(uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

    switch(uMsg) {
        case WM_SIZE:
            OnSize(wParam);
            break;
        case WM_COMMAND:
        case WM_NOTIFY:
        case WM_PARENTNOTIFY:
        case WM_DRAWITEM:
        case WM_MEASUREITEM:
        case WM_COMPAREITEM:
        case WM_DELETEITEM:
        case WM_VKEYTOITEM:
        case WM_CHARTOITEM:
        case WM_HSCROLL:
        case WM_VSCROLL:
        case WM_CTLCOLORBTN:
        case WM_CTLCOLORDLG:
        case WM_CTLCOLOREDIT:
        case WM_CTLCOLORLISTBOX:
        case WM_CTLCOLORMSGBOX:
        case WM_CTLCOLORSCROLLBAR:
        case WM_CTLCOLORSTATIC:
            return ::SendMessage(::GetParent(m_hWnd), uMsg, wParam, lParam);
    }

    return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void CSplitterWindow::OnSize(WPARAM wParam) {
	if(wParam != SIZE_MINIMIZED) {
        RECT rect;
		::GetClientRect(m_hWnd, &rect);
		SetSplitterRect(&rect);
    }
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

static ATOM aAtom = 0;

void CSplitterWindow::Create(HWND hWndParent) {
    if(aAtom == 0) {
        WNDCLASSEX m_wc;
        memset(&m_wc, 0, sizeof(WNDCLASSEX));
        m_wc.cbSize = sizeof(WNDCLASSEX);
        m_wc.lpfnWndProc = SplitterWindowWndProc;
        m_wc.hbrBackground = (HBRUSH)(COLOR_3DFACE + 1);
        m_wc.lpszClassName =  _T("SplitterWindow");
        m_wc.hInstance = g_hInstance;
        m_wc.hCursor = ::LoadCursor(m_wc.hInstance, IDC_ARROW);
        m_wc.style &= ~CS_GLOBALCLASS;

        aAtom = ::RegisterClassEx(&m_wc);
    }

	m_hWnd = ::CreateWindowEx(0, MAKEINTATOM(aAtom), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, CW_USEDEFAULT, CW_USEDEFAULT,
        CW_USEDEFAULT, CW_USEDEFAULT, hWndParent, 0U, g_hInstance, this);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------
