/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#if !defined(SYSTEM_FRAME_H)
#define SYSTEM_FRAME_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MDIChildWindow.h"
#include "../client/Text.h"

#include "../client/LogManager.h"

class SystemFrame : public MDITabChildWindow, public StaticFrame<SystemFrame, ResourceManager::SYSTEM_LOG, IDR_SYSTEM_LOG>,
    private LogManagerListener, private SettingsManagerListener {
public:
    SystemFrame();
	~SystemFrame();

    static ATOM m_aAtom;
    static UINT m_uiResourceID;

	static LRESULT CALLBACK SystemFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    LRESULT SystemFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void OnCreate();
	void onClose();
	void onSpeaker(WPARAM wParam);
	bool onLButton(LPARAM lParam);
	void onTabContextMenu(LPARAM lParam);
	void onSystemLog();
	void onRefreshSettings();

	void UpdateLayout();

	LRESULT onCtlColor(WPARAM wParam, LPARAM lParam) {
		HWND hWnd = (HWND)lParam;
		HDC hDC = (HDC)wParam;
		if(hWnd == hWndPad) {
			::SetBkColor(hDC, WinUtil::bgColor);
			::SetTextColor(hDC, WinUtil::textColor);
			return (LRESULT)WinUtil::bgBrush;
		}

		return 0;
	}

	void OnFocus() {
		::SetFocus(hWndPad);
	}

	void onCloseWindow() {
		::PostMessage(m_hWnd, WM_CLOSE, 0, 0);
	}

private:
    HWND hWndPad;

	HMENU tabMenu;

	void addLine(time_t t, const tstring& msg);

	void on(Message, time_t t, const string& message) {
		::PostMessage(m_hWnd, WM_SPEAKER, (WPARAM)(new pair<time_t, tstring>(t, Text::toT(message))), 0);
	}
	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();

	static LRESULT CALLBACK EditWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
};

#endif // !defined(SYSTEM_FRAME_H)

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
