/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "TextFrame.h"
#include "WinUtil.h"
#include "../client/File.h"
#include "WinApiUtil.h"
#include "FlatTabCtrl.h"

#define MAX_TEXT_LEN 32768

ATOM TextFrame::m_aAtom = 0;
UINT TextFrame::m_uiResourceID = IDR_NOTEPAD;

TextFrame::TextFrame(const tstring& fileName) : file(fileName) {
	SettingsManager::getInstance()->addListener(this);

    RegisterWindowClass(m_aAtom, TextFrameWndProc, _T("TextFrame"), m_uiResourceID);
}

LRESULT CALLBACK TextFrame::TextFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	TextFrame * pParent;

	if(uMsg == WM_NCCREATE) {
		pParent = (TextFrame *)((LPMDICREATESTRUCT)((LPCREATESTRUCT)lParam)->lpCreateParams)->lParam;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)pParent);

		pParent->m_hWnd = hWnd;
	} else {
		pParent = (TextFrame *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
		if(pParent == NULL) {
			return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
		}
	}

	return pParent->TextFrameProc(hWnd, uMsg, wParam, lParam);
}

LRESULT TextFrame::TextFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch(uMsg) {
        case WM_SETFOCUS:
            OnFocus();
            break;
        case WM_CREATE:
            OnCreate();
            return 0;
        case WM_CTLCOLOREDIT:
		case WM_CTLCOLORSTATIC: {
            LRESULT lResult = onCtlColor(wParam, lParam);
            if(lResult != 0) {
                return lResult;
            }
            break;
		}
        case WM_SIZE:
            if(wParam != SIZE_MINIMIZED) {
                UpdateLayout();
            }
            break;
        case WM_CLOSE:
            onClose();
            break;
        case WM_REFRESH_SETTINGS:
            onRefreshSettings();
            return 0;
        case WM_NCDESTROY:
            delete this;
            return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
    }

    LRESULT lResult = 0;
	if(MDIWindowProc(hWnd, uMsg, wParam, lParam, lResult) == true) {
		return lResult;
    }

	return ::DefMDIChildProc(hWnd, uMsg, wParam, lParam);
}

void TextFrame::openWindow(const tstring& aFileName) {
	TextFrame* frame = new TextFrame(aFileName);
	frame->CreateEx(frame, m_aAtom, m_uiResourceID);
}

void TextFrame::OnCreate() {
    WinUtil::tabCtrl->addTab(m_hWnd, RGB(0, 0, 0), IDR_NOTEPAD);
    created = true;

	hWndPad = CreateWindowEx(WS_EX_CLIENTEDGE, _T("EDIT"), NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | 
		WS_VSCROLL | ES_AUTOVSCROLL | ES_MULTILINE | ES_NOHIDESEL | ES_READONLY, 
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, m_hWnd, 0U, g_hInstance, NULL);

	::SendMessage(hWndPad, EM_LIMITTEXT, 0, 0L);
	::SendMessage(hWndPad, WM_SETFONT, (WPARAM)WinUtil::monoFont, MAKELPARAM(TRUE, 0));
	string tmp;
	try {
		tmp = Util::toDOS(File(Text::fromT(file), File::READ, File::OPEN).read(MAX_TEXT_LEN));
		string::size_type i = 0;
		while((i = tmp.find('\n', i)) != string::npos) {
			if(i == 0 || tmp[i-1] != '\r') {
				tmp.insert(i, 1, '\r');
				i++;
			}
			i++;
		}
		::SetWindowText(hWndPad, Text::toT(Text::acpToUtf8(tmp)).c_str());
		::SendMessage(hWndPad, EM_EMPTYUNDOBUFFER, 0, 0L);
		::SetWindowText(m_hWnd, Text::toT(Util::getFileName(Text::fromT(file))).c_str());
	} catch(const FileException& e) {
		::SetWindowText(m_hWnd, Text::toT(Util::getFileName(Text::fromT(file)) + ": " + e.getError()).c_str());
	}
}

void TextFrame::onClose() {
	SettingsManager::getInstance()->removeListener(this);
}

void TextFrame::UpdateLayout() {
	RECT rc;
	::GetClientRect(m_hWnd, &rc);

	rc.bottom -= 1;
	rc.top += 1;
	rc.left +=1;
	rc.right -=1;
	::MoveWindow(hWndPad, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, TRUE);
}

void TextFrame::on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw() {
	::PostMessage(m_hWnd, WM_REFRESH_SETTINGS, 0, 0);
}

void TextFrame::onRefreshSettings() {
	::RedrawWindow(m_hWnd, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
}

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
