/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2009 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"

#include "ThemesPage.h"
#include "../client/SettingsManager.h"
#include "WinUtil.h"
#include "MainFrm.h"

ThemesPage * pThemesPage = NULL;

ThemesPage::ThemesPage(SettingsManager *s) : PropPage(s, IDD_THEMESPAGE) {
    pThemesPage = this;
	title = _tcsdup((TSTRING(SETTINGS_APPEARANCE) + _T('\\') + TSTRING(SETTINGS_THEMES)).c_str());
	SetTitle(title);
	m_psp.pfnDlgProc = (DLGPROC)ThemesPageProc;
}

ThemesPage::~ThemesPage() {
    pThemesPage = NULL;
	free(title);
}

BOOL CALLBACK ThemesPage::ThemesPageProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    if(uMsg == WM_INITDIALOG) {
        pThemesPage->m_hWnd = hWnd;
        pThemesPage->onInitDialog();
        return TRUE;
    } else if(uMsg == WM_COMMAND) {
        if(HIWORD(wParam) == BN_CLICKED) {
            switch(LOWORD(wParam)) {
                case IDC_EMOTICONBROWSE:
                    pThemesPage->onBrowse();
                    return TRUE;
                case IDC_EMOTICONREFRESH:
                    pThemesPage->onRefresh();
                    return TRUE;
                case IDC_CHANGE_COLOR:
                    if(pThemesPage->onChangeColour((HWND)lParam) == true) {
                        return TRUE;
                    }

                    return FALSE;
            }
        }
    }

	return FALSE;
}

PropPage::TextItem ThemesPage::texts[] = {
	{ IDC_ENABLE_EMOTICONS, ResourceManager::SETCZDC_USE_CMAJLIES },
	{ IDC_SETCZDC_MAX_EMOTICONS, ResourceManager::SETCZDC_MAX_EMOTICONS },
	{ IDC_CZDC_EMOTICONFILE, ResourceManager::SETCZDC_EMOTICONSFILE },
	{ IDC_EMOTICONBROWSE, ResourceManager::BROWSE },
	{ IDC_EMOTICONREFRESH, ResourceManager::RELOAD },
	{ IDC_CZDC_EMOTICONS, ResourceManager::EMOTICONS },
	{ IDC_SETTINGS_SOUNDS, ResourceManager::SETTINGS_SOUNDS },
	{ IDC_PRIVATE_MESSAGE_BEEP, ResourceManager::SETTINGS_PM_BEEP },
	{ IDC_PRIVATE_MESSAGE_BEEP_OPEN, ResourceManager::SETTINGS_PM_BEEP_OPEN },
	{ IDC_CHANGE_COLOR, ResourceManager::SETTINGS_CHANGE_COLOR }, 
	{ IDC_USERLIST_COLORING, ResourceManager::USERLIST_COLORING },
	{ IDC_ENABLE_USERLIST_COLORS, ResourceManager::ENABLE_USERLIST_COLORS },
	{ 0, ResourceManager::SETTINGS_AUTO_AWAY }
};

PropPage::Item ThemesPage::items[] = {
	{ IDC_ENABLE_EMOTICONS, SettingsManager::USE_EMOTICONS, PropPage::T_BOOL },
	{ IDC_MAX_EMOTICONS, SettingsManager::MAX_EMOTICONS, PropPage::T_INT },
	{ IDC_PRIVATE_MESSAGE_BEEP, SettingsManager::PRIVATE_MESSAGE_BEEP, PropPage::T_BOOL },
	{ IDC_PRIVATE_MESSAGE_BEEP_OPEN, SettingsManager::PRIVATE_MESSAGE_BEEP_OPEN, PropPage::T_BOOL },
	{ IDC_ENABLE_USERLIST_COLORS, SettingsManager::USERLIST_COLORING, PropPage::T_BOOL },
	{ 0, 0, PropPage::T_END }
};

void ThemesPage::onInitDialog() {
	PropPage::translate(m_hWnd, texts);
	PropPage::read(m_hWnd, items);

	::SendMessage(::GetDlgItem(m_hWnd, IDC_MAX_EMOTICONSSPIN), UDM_SETRANGE32, 1, 999);

	normalColour = SETTING(NORMAL_COLOR);
	favoriteColour = SETTING(FAVORITE_COLOR);
	reservedSlotColour = SETTING(EXTRA_SLOT_COLOR);
	ignoredColour = SETTING(IGNORED_COLOR);
	fastColour = SETTING(FIREBALL_COLOR);
	serverColour = SETTING(SERVER_COLOR);
	activeColour = SETTING(ACTIVE_COLOR);
	pasiveColour = SETTING(PASIVE_COLOR);
	opColour = SETTING(OP_COLOR);

	::SendMessage(::GetDlgItem(m_hWnd, IDC_PREVIEW), EM_SETBKGNDCOLOR, 0, WinUtil::bgColor);

	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_COLORS), LB_ADDSTRING, 0, (LPARAM)CTSTRING(COLOR_NORMAL));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_COLORS), LB_ADDSTRING, 0, (LPARAM)CTSTRING(COLOR_FAVORITE));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_COLORS), LB_ADDSTRING, 0, (LPARAM)CTSTRING(COLOR_RESERVED));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_COLORS), LB_ADDSTRING, 0, (LPARAM)CTSTRING(COLOR_IGNORED));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_COLORS), LB_ADDSTRING, 0, (LPARAM)CTSTRING(COLOR_FAST));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_COLORS), LB_ADDSTRING, 0, (LPARAM)CTSTRING(COLOR_SERVER));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_COLORS), LB_ADDSTRING, 0, (LPARAM)CTSTRING(COLOR_OP));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_COLORS), LB_ADDSTRING, 0, (LPARAM)CTSTRING(COLOR_ACTIVE));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_COLORS), LB_ADDSTRING, 0, (LPARAM)CTSTRING(COLOR_PASIVE));
	::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_COLORS), LB_SETCURSEL, 0, 0L);

	refreshPreview(); 
	
	// Do specialized reading here
	::SetDlgItemText(m_hWnd, IDC_EMOTICONFILE, Text::toT(SETTING(EMOTICONFILE)).c_str());
}

void ThemesPage::write() {
	PropPage::write(m_hWnd, items);
	SettingsManager::getInstance()->set(SettingsManager::NORMAL_COLOR, normalColour);
	SettingsManager::getInstance()->set(SettingsManager::FAVORITE_COLOR, favoriteColour);
	SettingsManager::getInstance()->set(SettingsManager::EXTRA_SLOT_COLOR, reservedSlotColour);
	SettingsManager::getInstance()->set(SettingsManager::IGNORED_COLOR, ignoredColour);
	SettingsManager::getInstance()->set(SettingsManager::FIREBALL_COLOR, fastColour);
	SettingsManager::getInstance()->set(SettingsManager::SERVER_COLOR, serverColour);
	SettingsManager::getInstance()->set(SettingsManager::ACTIVE_COLOR, activeColour);
	SettingsManager::getInstance()->set(SettingsManager::PASIVE_COLOR, pasiveColour);
	SettingsManager::getInstance()->set(SettingsManager::OP_COLOR, opColour);
	
	// Do specialized writing here
	if(SETTING(MAX_EMOTICONS) < 1) settings->set(SettingsManager::MAX_EMOTICONS, 1);
	TCHAR buf[256];
	::GetDlgItemText(m_hWnd, IDC_EMOTICONFILE, buf, 256);
	string file = Text::fromT(buf);
	if(_stricmp(file.c_str(), SETTING(EMOTICONFILE).c_str()) != 0) {
		settings->set(SettingsManager::EMOTICONFILE, file);
		WinUtil::initEmoticons();
	}
}

void ThemesPage::onBrowse() {
	TCHAR buf[MAX_PATH];

	::GetDlgItemText(m_hWnd, IDC_EMOTICONFILE, buf, MAX_PATH);
	tstring x = buf;

	if(WinUtil::browseFile(x, m_hWnd, false) == IDOK) {
		if(_tcscmp(buf, x.c_str()) != 0) {
			::EnableWindow(::GetDlgItem(m_hWnd, IDC_EMOTICONREFRESH), FALSE);
        }
		::SetDlgItemText(m_hWnd, IDC_EMOTICONFILE, x.c_str());
	}
}

void ThemesPage::onRefresh() {
	WinUtil::initEmoticons();
}

bool ThemesPage::onChangeColour(HWND hWndCtl) {
	int index = (int)::SendMessage(::GetDlgItem(m_hWnd, IDC_USERLIST_COLORS), LB_GETCURSEL, 0, 0L);
	if(index == -1) {
        return false;
    }

	int colour = 0;
	switch(index) {
		case 0: colour = normalColour; break;
		case 1: colour = favoriteColour; break;
		case 2: colour = reservedSlotColour; break;
		case 3: colour = ignoredColour; break;
		case 4: colour = fastColour; break;
		case 5: colour = serverColour; break;
		case 7: colour = activeColour; break;
		case 8: colour = pasiveColour; break;
		case 6: colour = opColour; break;
		default: break;
	}

    CHOOSECOLOR cc;
    InitializeChooseColor(cc, colour, hWndCtl);

	if(::ChooseColor(&cc) == TRUE) {
		switch(index) {
			case 0: normalColour = cc.rgbResult; break;
			case 1: favoriteColour = cc.rgbResult; break;
			case 2: reservedSlotColour = cc.rgbResult; break;
			case 3: ignoredColour = cc.rgbResult; break;
			case 4: fastColour = cc.rgbResult; break;
			case 5: serverColour = cc.rgbResult; break;
			case 7: activeColour = cc.rgbResult; break;
			case 8: pasiveColour = cc.rgbResult; break;
			case 6: opColour = cc.rgbResult; break;
			default: break;
		}
		refreshPreview();
	}

    return true;
}

void ThemesPage::AppendTxt(const TCHAR * sTxt) {
	CHARRANGE cr = { ::GetWindowTextLength(::GetDlgItem(m_hWnd, IDC_PREVIEW)), ::GetWindowTextLength(::GetDlgItem(m_hWnd, IDC_PREVIEW)) };
	::SendMessage(::GetDlgItem(m_hWnd, IDC_PREVIEW), EM_EXSETSEL, 0, (LPARAM)&cr);
	::SendMessage(::GetDlgItem(m_hWnd, IDC_PREVIEW), EM_REPLACESEL, (WPARAM)FALSE, (LPARAM)sTxt);
}

void ThemesPage::refreshPreview() {
	::SetWindowText(::GetDlgItem(m_hWnd, IDC_PREVIEW), _T(""));

	CHARFORMAT2 cf;
	cf.dwMask = CFM_COLOR;
	cf.dwEffects = 0;
	cf.cbSize = sizeof(CHARFORMAT);
	
	cf.crTextColor = normalColour;
	::SendMessage(::GetDlgItem(m_hWnd, IDC_PREVIEW), EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
	AppendTxt(CTSTRING(COLOR_NORMAL));

	cf.crTextColor = favoriteColour;
	::SendMessage(::GetDlgItem(m_hWnd, IDC_PREVIEW), EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
	AppendTxt((_T("\r\n") + TSTRING(COLOR_FAVORITE)).c_str());
	
	cf.crTextColor = reservedSlotColour;
	::SendMessage(::GetDlgItem(m_hWnd, IDC_PREVIEW), EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
	AppendTxt((_T("\r\n") + TSTRING(COLOR_RESERVED)).c_str());
	
	cf.crTextColor = ignoredColour;
	::SendMessage(::GetDlgItem(m_hWnd, IDC_PREVIEW), EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
	AppendTxt((_T("\r\n") + TSTRING(COLOR_IGNORED)).c_str());
	
	cf.crTextColor = fastColour;
	::SendMessage(::GetDlgItem(m_hWnd, IDC_PREVIEW), EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
	AppendTxt((_T("\r\n") + TSTRING(COLOR_FAST)).c_str());

	cf.crTextColor = serverColour;
	::SendMessage(::GetDlgItem(m_hWnd, IDC_PREVIEW), EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
	AppendTxt((_T("\r\n") + TSTRING(COLOR_SERVER)).c_str());

	cf.crTextColor = opColour;
	::SendMessage(::GetDlgItem(m_hWnd, IDC_PREVIEW), EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
	AppendTxt((_T("\r\n") + TSTRING(COLOR_OP)).c_str());
	
	cf.crTextColor = activeColour;
	::SendMessage(::GetDlgItem(m_hWnd, IDC_PREVIEW), EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
	AppendTxt((_T("\r\n") + TSTRING(COLOR_ACTIVE)).c_str());

	cf.crTextColor = pasiveColour;
	::SendMessage(::GetDlgItem(m_hWnd, IDC_PREVIEW), EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&cf);
	AppendTxt((_T("\r\n") + TSTRING(COLOR_PASIVE)).c_str());

	::InvalidateRect(::GetDlgItem(m_hWnd, IDC_PREVIEW), NULL, TRUE);
}
