/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef TRANSFER_VIEW_H
#define TRANSFER_VIEW_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../client/DownloadManager.h"
#include "../client/UploadManager.h"
#include "../client/ConnectionManagerListener.h"
#include "../client/TaskQueue.h"

#include "UCHandler.h"
#include "TypedListViewCtrl.h"
#include "WinUtil.h"
#include "SearchFrm.h"

class TransferView : private DownloadManagerListener, private UploadManagerListener, private ConnectionManagerListener,
    private UserInfoBaseHandler<TransferView>, private UCHandler<TransferView>, private SettingsManagerListener {
public:
	TransferView() : PreviewAppsSize(0), transferMenu(NULL) { }
	~TransferView();

    LRESULT TransferViewProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void onCreate();
	void onSpeaker();
	bool onContextMenu(WPARAM wParam, LPARAM lParam);
	void onSize();
	void onForce();
	LRESULT onCustomDraw(LPNMHDR pnmh, bool& bHandled);
	void onDoubleClickTransfers(LPNMHDR pnmh);
	void onCopyNick();
	void onCopyIP();
	void onSearchAlternates();
	void onPreviewCommand(WORD wID);
	void onWhoisIP();
	void onOpenUserLog();
	void onRefreshSettings();
	
	void runUserCommand(UserCommand& uc);
	void prepareClose();

	bool onKeyDownTransfers(LPNMHDR pnmh) {
		NMLVKEYDOWN* kd = (NMLVKEYDOWN*)pnmh;

		if(kd->wVKey == VK_DELETE) {
			ctrlTransfers.forEachSelected(&ItemInfo::disconnect);
			return true;
		}

		return false;
	}

	void onRemove() {
		ctrlTransfers.forEachSelected(&ItemInfo::disconnect);
		if((UINT)::SendMessage(ctrlTransfers.m_hWnd, LVM_GETSELECTEDCOUNT, 0, 0L) == 1) {
            ListViewSetItemState(ctrlTransfers.m_hWnd, (int)::SendMessage(ctrlTransfers.m_hWnd, LVM_GETNEXTITEM, -1, MAKELPARAM(LVNI_SELECTED, 0)), 0, LVIS_SELECTED);
        }
	}

	void onRemoveAll() {
		ctrlTransfers.forEachSelected(&ItemInfo::removeAll);
	}

	void onDestroy() {
		ctrlTransfers.forEach(&ItemInfo::deleteSelf);
	}

    void Create(HWND hWndParent);

	HWND m_hWnd;
private:
	class ItemInfo;
public:
	TypedListViewCtrl<ItemInfo, IDC_TRANSFERS>& getUserList() { return ctrlTransfers; }
private:
	int PreviewAppsSize;

	enum {
		ADD_ITEM,
		REMOVE_ITEM,
		UPDATE_ITEM
	};

	enum {
		COLUMN_FIRST,
		COLUMN_USER = COLUMN_FIRST,
		COLUMN_HUB,
		COLUMN_STATUS,
		COLUMN_TIMELEFT,
		COLUMN_SPEED,
		COLUMN_FILE,
		COLUMN_SIZE,
		COLUMN_PATH,
		COLUMN_IP,
		COLUMN_RATIO,
		COLUMN_LAST
	};

	struct UpdateInfo;
	class ItemInfo : public UserInfoBase {
	public:
		enum Status {
			STATUS_RUNNING,
			STATUS_WAITING
		};

		ItemInfo(const UserPtr& u, bool aDownload);

		bool download;
		bool transferFailed;
		Status status;
		uint8_t flagImage;
		int64_t pos;
		int64_t size;
		int64_t start;
		int64_t actual;
		int64_t speed;
		int64_t timeLeft;
		string Target;
		tstring IP;

		tstring columns[COLUMN_LAST];
		void update(const UpdateInfo& ui);

		void disconnect();
		void removeAll();
		void deleteSelf() { delete this; }

		double getRatio() { return (pos > 0) ? (double)actual / (double)pos : 1.0; }

		const TCHAR* getText(const int &col, TCHAR* buf) const {
			return columns[col].c_str();
		}

		static int compareItems(ItemInfo* a, ItemInfo* b, int col);

		uint8_t imageIndex() {
			return download == true ? 0 : 1;
		}
	};

	struct UpdateInfo {
		enum {
			MASK_POS = 1 << 0,
			MASK_SIZE = 1 << 1,
			MASK_START = 1 << 2,
			MASK_ACTUAL = 1 << 3,
			MASK_SPEED = 1 << 4,
			MASK_FILE = 1 << 5,
			MASK_STATUS = 1 << 6,
			MASK_TIMELEFT = 1 << 7,
			MASK_IP = 1 << 8,
			MASK_STATUS_STRING = 1 << 9,
			MASK_COUNTRY = 1 << 10
		};

		bool operator==(const ItemInfo& ii) { return download == ii.download && user == ii.user; }

		UpdateInfo(const UserPtr& aUser, bool isDownload, bool isTransferFailed = false) : updateMask(0), user(aUser), download(isDownload), transferFailed(isTransferFailed), sIP(Util::emptyStringT), flagImage(0) { }

		uint32_t updateMask;

		UserPtr user;
		bool download;
		bool transferFailed;
		void setStatus(ItemInfo::Status aStatus) { status = aStatus; updateMask |= MASK_STATUS; }
		ItemInfo::Status status;
		void setPos(int64_t aPos) { pos = aPos; updateMask |= MASK_POS; }
		int64_t pos;
		void setSize(int64_t aSize) { size = aSize; updateMask |= MASK_SIZE; }
		int64_t size;
		void setStart(int64_t aStart) { start = aStart; updateMask |= MASK_START; }
		int64_t start;
		void setActual(int64_t aActual) { actual = aActual; updateMask |= MASK_ACTUAL; }
		int64_t actual;
		void setSpeed(int64_t aSpeed) { speed = aSpeed; updateMask |= MASK_SPEED; }
		int64_t speed;
		void setTimeLeft(int64_t aTimeLeft) { timeLeft = aTimeLeft; updateMask |= MASK_TIMELEFT; }
		int64_t timeLeft;
		void setStatusString(const tstring& aStatusString) { statusString = aStatusString; updateMask |= MASK_STATUS_STRING; }
		tstring statusString;
		void setFile(const tstring& aFile) { file = Util::getFileName(aFile); path = Util::getFilePath(aFile); target = aFile; updateMask|= MASK_FILE; }
		tstring file;
		tstring path;
		void setIP(const tstring& aIP) { IP = aIP; updateMask |= MASK_IP; }
		tstring IP;
		tstring target;
		uint8_t flagImage;
		tstring sIP;
	};

	void speak(int type, UpdateInfo* ui) { tasks.add(type, ui); PostMessage(m_hWnd, WM_SPEAKER, 0, 0); }

	TypedListViewCtrl<ItemInfo, IDC_TRANSFERS> ctrlTransfers;
	static int columnIndexes[];
	static int columnSizes[];

	HMENU transferMenu;

	HIMAGELIST arrows;

	TaskQueue tasks;

	StringMap ucLineParams;

	void on(ConnectionManagerListener::Added, ConnectionQueueItem* aCqi) throw();
	void on(ConnectionManagerListener::Failed, ConnectionQueueItem* aCqi, const string& aReason) throw();
	void on(ConnectionManagerListener::Removed, ConnectionQueueItem* aCqi) throw();
	void on(ConnectionManagerListener::StatusChanged, ConnectionQueueItem* aCqi) throw();

	void on(DownloadManagerListener::Complete, Download* aDownload) throw() { onTransferComplete(aDownload, false); }
	void on(DownloadManagerListener::Failed, Download* aDownload, const string& aReason) throw();
	void on(DownloadManagerListener::Starting, Download* aDownload) throw();
	void on(DownloadManagerListener::Tick, const Download::List& aDownload) throw();
	void on(DownloadManagerListener::HashCheck, Download* aDownload) throw();

	void on(UploadManagerListener::Starting, Upload* aUpload) throw();
	void on(UploadManagerListener::Tick, const Upload::List& aUpload) throw();
	void on(UploadManagerListener::Complete, Upload* aUpload) throw() { onTransferComplete(aUpload, true); }

	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();

	void onTransferComplete(Transfer* aTransfer, bool isUpload);

    static LRESULT CALLBACK TransferViewWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
};

#endif // TRANSFER_VIEW_H

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
