/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2009 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "../client/FavoriteManager.h"
#include "Resource.h"
#include "UserInfo.h"

UserInfo::UserInfo(const UserTask& u, const bool &bOperator, bool bUpdate/* = true*/) : UserInfoBase(u.user) {
	for(size_t i = 0; i < COLUMN_LAST; i++) {
		columns[i] = NULL;
	}
	if(bUpdate == true)
	   update(u.identity, -1, bOperator); 
}

UserInfo::~UserInfo() {
	for(size_t i = 0; i < COLUMN_LAST; i++) {
		if(columns[i] != NULL) {
            delete [] columns[i];
			columns[i] = NULL;
		}
	}
}

const TCHAR* UserInfo::getText(const int &col, TCHAR* buf) const {
	return columns[col] != NULL ? columns[col] : Util::emptyStringT.c_str();
}

int UserInfo::compareItems(const UserInfo* a, const UserInfo* b, int col) {
	if(a == NULL || b == NULL){
		return 0;
	}
	
	if(col == COLUMN_NICK) {
		bool aOp = a->getIdentity().isOp();
		bool bOp = b->getIdentity().isOp();
		if(aOp && !bOp) {
			return -1;
		} else if(!aOp && bOp) {
			return 1;
		}
		if(BOOLSETTING(SORT_FAVUSERS_FIRST)) {
			bool a_isFav = FavoriteManager::getInstance()->isFavoriteUser(a->getIdentity().getUser());
			bool b_isFav = FavoriteManager::getInstance()->isFavoriteUser(b->getIdentity().getUser());
			if(a_isFav && !b_isFav)
				return -1;

			if(!a_isFav && b_isFav)
				return 1;
		}
	}
	
	if(col == COLUMN_SHARED || col == COLUMN_EXACT_SHARED) {
		return compare(a->identity.getBytesShared(), b->identity.getBytesShared());
	}
	
	if(a->columns[col] != NULL && b->columns[col] != NULL) {
	   return Util::userstricmp(a->columns[col], b->columns[col]);
    } else if(a->columns[col] == NULL) {
        if(b->columns[col] == NULL) {
            return 0;
        } else {
            return -1;
        }
    } else {
        return 1;
    }
}

bool UserInfo::update(const Identity& identity, int sortCol, const bool &bOperator) {
    bool bNeedSort = (getIdentity().isOp() != identity.isOp());

	tstring old;
	if(sortCol != -1 && columns[sortCol] != NULL)
		old = columns[sortCol];
	
	if(columns[COLUMN_NICK] == NULL) {
		columns[COLUMN_NICK] = Util::tsring2tcharp(columns[COLUMN_NICK], Text::toT(identity.getNick()));
	}

	columns[COLUMN_SHARED] = Util::tsring2tcharp(columns[COLUMN_SHARED], Text::toT(Util::formatBytes(identity.getBytesShared())));
	columns[COLUMN_EXACT_SHARED] = Util::tsring2tcharp(columns[COLUMN_EXACT_SHARED], Text::toT(Util::formatExactSize(identity.getBytesShared())));
	columns[COLUMN_DESCRIPTION] = Util::tsring2tcharp(columns[COLUMN_DESCRIPTION], Text::toT(Text::toUtf8(identity.getDescription())));
	columns[COLUMN_TAG] = Util::tsring2tcharp(columns[COLUMN_TAG], Text::toT(Text::toUtf8(identity.getTag())));
	columns[COLUMN_CONNECTION] = Util::tsring2tcharp(columns[COLUMN_CONNECTION], Text::toT(Text::toUtf8(identity.getConnection())));
	columns[COLUMN_EMAIL] = Util::tsring2tcharp(columns[COLUMN_EMAIL], Text::toT(Text::toUtf8(identity.getEmail())));
	columns[COLUMN_CLIENTID] = Util::tsring2tcharp(columns[COLUMN_CLIENTID], Text::toT(Text::toUtf8(identity.getClientID())));
	columns[COLUMN_VERSION] = Util::tsring2tcharp(columns[COLUMN_VERSION], Text::toT(Text::toUtf8(identity.getVersion())));
	columns[COLUMN_MODE] = Util::tsring2tcharp(columns[COLUMN_MODE], Text::toT(identity.getMode()));
	columns[COLUMN_HUBS] = Util::tsring2tcharp(columns[COLUMN_HUBS], Text::toT(identity.getHubs()));
	columns[COLUMN_SLOTS] = Util::tsring2tcharp(columns[COLUMN_SLOTS], Text::toT(Text::toUtf8(identity.getSlots())));
	columns[COLUMN_LIMITER] = Util::tsring2tcharp(columns[COLUMN_LIMITER], Text::toT(identity.getUpload()));

    if(identity.getIp().empty() == true) {
        columns[COLUMN_COUNTRY] = Util::tsring2tcharp(columns[COLUMN_COUNTRY], Util::emptyStringT);
    } else if(bOperator == true) {
        tstring country = Text::toT(Util::getIpCountry(identity.getIp())) + _T(" ") + Text::toT(identity.getIp());
        columns[COLUMN_COUNTRY] = Util::tsring2tcharp(columns[COLUMN_COUNTRY], country);
    } else {
        tstring country = Text::toT(Util::getIpCountry(identity.getIp()));
        columns[COLUMN_COUNTRY] = Util::tsring2tcharp(columns[COLUMN_COUNTRY], country);
    }

	if(sortCol != -1) {
		bNeedSort = bNeedSort || (columns[sortCol] == NULL && old.size() != 0) || (columns[sortCol] != NULL && old.size() == 0) ||
			((columns[sortCol] != NULL && old.size() != 0) && _tcscmp(old.c_str(), columns[sortCol]) != NULL);
	}
	
    setIdentity(identity);
	return bNeedSort;
}
