/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef USERS_FRAME_H
#define USERS_FRAME_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MDIChildWindow.h"
#include "TypedListViewCtrl.h"
#include "WinUtil.h"

#include "../client/FavoriteManager.h"
#include "../client/File.h"

class UsersFrame : public MDITabChildWindow, public StaticFrame<UsersFrame, ResourceManager::FAVORITE_AND_IGNORED_USERS, IDC_FAVUSERS>,
    private FavoriteManagerListener, private UserInfoBaseHandler<UsersFrame>, private SettingsManagerListener {
public:
	UsersFrame();
	~UsersFrame();

    static ATOM m_aAtom;
    static UINT m_uiResourceID;

    static LRESULT CALLBACK UsersFrameWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
    LRESULT UsersFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	void onCreate();
	bool onClose();
	void onRemove();
	void onEdit();
	void onItemChanged(LPNMHDR pnmh);
	bool onContextMenu(WPARAM wParam, LPARAM lParam);
	void onConnect();
	bool onDoubleClick(LPNMHDR pnmh);
	bool onKeyDown(LPNMHDR pnmh);
	bool onIgnKeyDown(LPNMHDR pnmh);
	LRESULT onCustomDraw(LPNMHDR pnmh);
	void onOpenUserLog();
	void onRefreshSettings();

	void UpdateLayout();
	
	void onSpeaker(WPARAM wParam, LPARAM lParam);
	
	void onSetFocus() {
		::SetFocus(ctrlUsers.m_hWnd);
	}

private:
	class UserInfo;
public:
	TypedListViewCtrl<UserInfo, IDC_USERS>& getUserList() { return ctrlUsers; }
private:
	enum {
		COLUMN_FIRST,
		COLUMN_NICK = COLUMN_FIRST,
		COLUMN_STATUS,
		COLUMN_HUB,
		COLUMN_SEEN,
		COLUMN_DESCRIPTION,
		COLUMN_LAST
	};

	enum {
		USER_UPDATED
	};

	class UserInfo : public UserInfoBase {
	public:
		UserInfo(const FavoriteUser& u) : UserInfoBase(u.getUser()), bFavoriteUser(u.isSet(FavoriteUser::FLAG_IS_FAVORITE)) {
			update(u);
		}

		const TCHAR* getText(const int &col, TCHAR* buf) const {
			return columns[col].c_str();
		}

		static int compareItems(UserInfo* a, UserInfo* b, int col) {
			return lstrcmpi(a->columns[col].c_str(), b->columns[col].c_str());
		}

		uint8_t imageIndex() {
			return 2;
		}

		void remove() {
            if(bFavoriteUser == true) {
                FavoriteManager::getInstance()->removeFavoriteUser(user);
            } else {
                FavoriteManager::getInstance()->removeIgnoredUser(user);
            }
        }

		void update(const FavoriteUser& u);

		tstring columns[COLUMN_LAST];

		bool bFavoriteUser;
	};

	HMENU usersMenu;
	HWND hWndFavUsers, hWndIgnUsers;

	TypedListViewCtrl<UserInfo, IDC_USERS> ctrlUsers;
	TypedListViewCtrl<UserInfo, IDC_USERS> ctrlIgnUsers;
	
	bool startup, closed, bFavorite;

	static int columnSizes[COLUMN_LAST];
	static int columnIndexes[COLUMN_LAST];
	static int columnSizesIgn[COLUMN_LAST];
	static int columnIndexesIgn[COLUMN_LAST];

	// FavoriteManagerListener
	void on(UserAdded, const FavoriteUser& aUser) throw() { addUser(aUser); }
	void on(UserRemoved, const FavoriteUser& aUser) throw() { removeUser(aUser); }
	void on(StatusChanged, const UserPtr& aUser) throw() {
		::PostMessage(m_hWnd, WM_SPEAKER, (WPARAM)USER_UPDATED, (LPARAM)new UserInfoBase(aUser));
	}

	void on(SettingsManagerListener::Save, SimpleXML& /*xml*/) throw();

	void addUser(const FavoriteUser& aUser);
	void updateUser(const UserPtr& aUser);
	void removeUser(const FavoriteUser& aUser);
};

#endif // USERS_FRAME_H

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
