/*
 * CzDC - client for Direct Connect peer to peer network.

 * Copyright (C) 2002-2009 Petr Kozelka, PPK at CzDC dot org

 * CzDC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3
 * as published by the Free Software Foundation.

 * CzDC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with CzDC. If not, see <http://www.gnu.org/licenses/>.
 */

//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "stdafx.h"
#include "../client/DCPlusPlus.h"
#include "Resource.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------
#include "WinApiUtil.h"
//----------------------------------------------------------------------------------------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// TreeView

DWORD_PTR TreeGetItemData(const HWND &Tree, const HTREEITEM &hItem) {
	TVITEM tvItem = { 0 };
	tvItem.hItem = hItem;
	tvItem.mask = TVIF_PARAM;

	if((BOOL)::SendMessage(Tree, TVM_GETITEM, 0, (LPARAM)&tvItem)) {
        return tvItem.lParam;
    }

    return NULL;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

BOOL TreeGetItemText(const HWND &Tree, const HTREEITEM &hItem, const LPTSTR &lpstrText, const int &iLen) {
	TVITEM tvItem = { 0 };
	tvItem.hItem = hItem;
	tvItem.mask = TVIF_TEXT;
	tvItem.pszText = lpstrText;
	tvItem.cchTextMax = iLen;

	return (BOOL)::SendMessage(Tree, TVM_GETITEM, 0, (LPARAM)&tvItem);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

HTREEITEM TreeHitTest(const HWND &Tree, const POINT &pt, UINT * uipFlags) {
	TVHITTESTINFO tvHTI = { 0 };
	tvHTI.pt = pt;
	HTREEITEM hTreeItem = (HTREEITEM)::SendMessage(Tree, TVM_HITTEST, 0, (LPARAM)&tvHTI);

	if(uipFlags != NULL) {
		*uipFlags = tvHTI.flags;
    }

	return hTreeItem;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

HTREEITEM TreeInsertItem(const HWND &Tree, const UINT &uiMask, const LPCTSTR &lpszItem, const int &iImage, const int &iSelectedImage, const UINT &uiState, 
    const UINT &uiStateMask, const LPARAM &lParam, const HTREEITEM &hParent, const HTREEITEM &hInsertAfter) {
	TVINSERTSTRUCT tvIS = { 0 };
	tvIS.hParent = hParent;
	tvIS.hInsertAfter = hInsertAfter;
	tvIS.item.mask = uiMask;
	tvIS.item.pszText = (LPTSTR)lpszItem;
	tvIS.item.iImage = iImage;
	tvIS.item.iSelectedImage = iSelectedImage;
	tvIS.item.state = uiState;
	tvIS.item.stateMask = uiStateMask;
	tvIS.item.lParam = lParam;

	return (HTREEITEM)::SendMessage(Tree, TVM_INSERTITEM, 0, (LPARAM)&tvIS);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

BOOL TreeSetItem(const HWND &Tree, const HTREEITEM &hItem, const UINT &uiMask, const LPCTSTR &lpszItem, const int &iImage, const int &iSelectedImage, 
    const UINT &uiState, const UINT &uiStateMask, const LPARAM &lParam) {
	TVITEM tvItem = { 0 };
	tvItem.hItem = hItem;
	tvItem.mask = uiMask;
	tvItem.pszText = (LPTSTR)lpszItem;
	tvItem.iImage = iImage;
	tvItem.iSelectedImage = iSelectedImage;
	tvItem.state = uiState;
	tvItem.stateMask = uiStateMask;
	tvItem.lParam = lParam;

	return (BOOL)::SendMessage(Tree, TVM_SETITEM, 0, (LPARAM)&tvItem);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// TOOLINFO

void ToolInfoInit(TOOLINFO &ti, const UINT &iFlags, const HWND &hWnd, const UINT &iIDTool, const LPRECT &lpRect, const LPTSTR &lpstrText) {
	memset(&ti, 0, sizeof(TOOLINFO));
	ti.cbSize = sizeof(TOOLINFO);
	ti.uFlags = iFlags;

	if(iIDTool == 0) {
		ti.hwnd = ::GetParent(hWnd);
		ti.uFlags |= TTF_IDISHWND;
		ti.uId = (UINT_PTR)hWnd;
	} else {
		ti.hwnd = hWnd;
		ti.uId = iIDTool;
	}

	if(lpRect != NULL) {
		ti.rect = *lpRect;
    }

	ti.hinst = g_hInstance;
	ti.lpszText = lpstrText;
	ti.lParam = NULL;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// ListView

void ListViewInsertDummyColumn(const HWND &List) {
	RECT rc;
	::GetClientRect(List, &rc);

    ListViewInsertColumn(List, 0, _T("Dummy"), LVCFMT_LEFT, rc.right - rc.left, 0);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

int ListViewInsertColumn(const HWND &List, const int &iCol, const LPCTSTR lpszColumnHeading, const int &iFormat, const int &iWidth, const int &iSubItem) {
	LVCOLUMN lvColumn = { 0 };
	lvColumn.mask = LVCF_TEXT | LVCF_FMT | LVCF_WIDTH | LVCF_SUBITEM;
	lvColumn.pszText = (LPTSTR)lpszColumnHeading;
	lvColumn.fmt = iFormat;
	lvColumn.cx = iWidth;
	lvColumn.iSubItem = iSubItem;

	return (int)::SendMessage(List, LVM_INSERTCOLUMN, iCol, (LPARAM)&lvColumn);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void ListViewSetItem(const HWND &List, const int &iItem, const int &iSubItem, const UINT &uiMask, const LPCTSTR lpszItem, const int &iImage, const DWORD_PTR &i/* = 0*/) {
	LVITEM lvItem = { 0 };
	lvItem.mask = uiMask;
	lvItem.iItem = iItem;
	lvItem.iSubItem = iSubItem;
	lvItem.stateMask = 0;
	lvItem.state = 0;
	lvItem.pszText = (LPTSTR)lpszItem;
	lvItem.iImage = iImage;
	lvItem.lParam = (LPARAM)i;

	::SendMessage(List, LVM_SETITEM, 0, (LPARAM)&lvItem);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

DWORD_PTR ListViewGetItem(const HWND &List, const int &i) {
	LVITEM lvItem = { 0 };
	lvItem.iItem = i;
	lvItem.mask = LVIF_PARAM;

	::SendMessage(List, LVM_GETITEM, 0, (LPARAM)&lvItem);

	return (DWORD_PTR)lvItem.lParam;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void ListViewGetItemText(const HWND &List, const int &iItem, const int &iSubItem, const LPTSTR lpszText, const int &iLen) {
	LVITEM lvItem = { 0 };
	lvItem.iSubItem = iSubItem;
	lvItem.cchTextMax = iLen;
	lvItem.pszText = lpszText;

	::SendMessage(List, LVM_GETITEMTEXT, (WPARAM)iItem, (LPARAM)&lvItem);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void ListViewSelectItem(const HWND &List, const int &iItem) {
	if(ListViewSetItemState(List, iItem, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED)) {
        ::SendMessage(List, LVM_ENSUREVISIBLE, iItem, MAKELPARAM(FALSE, 0));
	}
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

BOOL ListViewSetItemState(const HWND &List, const int &iItem, const UINT &uiState, const UINT &uiStateMask) {
	LVITEM lvItem = { 0 };
	lvItem.state = uiState;
	lvItem.stateMask = uiStateMask;

	return (BOOL)::SendMessage(List, LVM_SETITEMSTATE, iItem, (LPARAM)&lvItem);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

int ListViewInsertItem(const HWND &List, const UINT &uiMask, const int &iItem, const LPCTSTR lpszItem, const UINT &uiState, 
    const UINT &uiStateMask, const int &iImage, const LPARAM lParam) {
	LVITEM lvItem = { 0 };
	lvItem.mask = uiMask;
	lvItem.iItem = iItem;
	lvItem.iSubItem = 0;
	lvItem.pszText = (LPTSTR)lpszItem;
	lvItem.state = uiState;
	lvItem.stateMask = uiStateMask;
	lvItem.iImage = iImage;
	lvItem.lParam = lParam;

	return (int)::SendMessage(List, LVM_INSERTITEM, 0, (LPARAM)&lvItem);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

COLORREF CustomColors[16] = {
	RGB(255, 255, 255), RGB(255, 255, 255), 
	RGB(255, 255, 255), RGB(255, 255, 255), 
	RGB(255, 255, 255), RGB(255, 255, 255), 
	RGB(255, 255, 255), RGB(255, 255, 255), 
	RGB(255, 255, 255), RGB(255, 255, 255), 
	RGB(255, 255, 255), RGB(255, 255, 255), 
	RGB(255, 255, 255), RGB(255, 255, 255), 
	RGB(255, 255, 255), RGB(255, 255, 255), 
};

void InitializeChooseColor(CHOOSECOLOR &cc, const COLORREF &clr, const HWND &hWndParent) {
    memset(&cc, 0, sizeof(cc));
	cc.lStructSize = sizeof(cc);
	cc.lpCustColors = CustomColors;
	cc.hwndOwner = hWndParent;
	cc.rgbResult = clr;
	cc.Flags = CC_RGBINIT;
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void RegisterWindowClass(ATOM &aAtom, const WNDPROC &StartWindowProc, const LPCTSTR lpszClassName, const UINT &uiResourceID) {
    if(aAtom != 0) {
        return;
    }

    WNDCLASSEX m_wc;
    memset(&m_wc, 0, sizeof(WNDCLASSEX));
    m_wc.cbSize = sizeof(WNDCLASSEX);
    m_wc.lpfnWndProc = StartWindowProc;
    m_wc.hbrBackground = (HBRUSH)(COLOR_3DFACE + 1);
    m_wc.lpszClassName = lpszClassName;
    m_wc.hInstance = g_hInstance;
    m_wc.hCursor = ::LoadCursor(m_wc.hInstance, IDC_ARROW);
    m_wc.style &= ~CS_GLOBALCLASS;
    m_wc.hIcon = (HICON)::LoadImage(m_wc.hInstance, MAKEINTRESOURCE(uiResourceID), IMAGE_ICON, 32, 32, LR_DEFAULTCOLOR);
    m_wc.hIconSm = (HICON)::LoadImage(m_wc.hInstance, MAKEINTRESOURCE(uiResourceID), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);

    aAtom = ::RegisterClassEx(&m_wc);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------

void CenterWindow(HWND hWnd, HWND hWndParent) {
	RECT rcDlg, rcParent;
	::GetWindowRect(hWnd, &rcDlg);
	::GetClientRect(hWndParent, &rcParent);
    ::MapWindowPoints(hWndParent, NULL, (POINT*)&rcParent, 2);

	int iWidth = rcDlg.right - rcDlg.left;
	int iHeight = rcDlg.bottom - rcDlg.top;

	int iLeft = (rcParent.left + rcParent.right) / 2 - iWidth / 2;
	int iTop = (rcParent.top + rcParent.bottom) / 2 - iHeight / 2;

	if(iLeft + iWidth > rcParent.right) {
		iLeft = rcParent.right - iWidth;
    }
	if(iLeft < rcParent.left) {
		iLeft = rcParent.left;
    }

	if(iTop + iHeight > rcParent.bottom) {
		iTop = rcParent.bottom - iHeight;
    }
	if(iTop < rcParent.top) {
		iTop = rcParent.top;
    }

	::SetWindowPos(hWnd, NULL, iLeft, iTop, -1, -1, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);
}
//----------------------------------------------------------------------------------------------------------------------------------------------------
