/*
 * Copyright (C) 2001-2007 Jacek Sieka, arnetheduck on gmail point com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef WIN_UTIL_H
#define WIN_UTIL_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../client/Util.h"
#include "../client/SettingsManager.h"
#include "../client/User.h"
#include "../client/MerkleTree.h"
#include "../client/ResourceManager.h"
#include "AGEmoticonSetup.h"

#include "resource.h"

#include <wininet.h>

// Some utilities for handling HLS colors, taken from Jean-Michel LE FOL's codeproject
// article on WTL OfficeXP Menus
typedef DWORD HLSCOLOR;
#define HLS(h,l,s) ((HLSCOLOR)(((BYTE)(h)|((WORD)((BYTE)(l))<<8))|(((DWORD)(BYTE)(s))<<16)))
#define HLS_H(hls) ((BYTE)(hls))
#define HLS_L(hls) ((BYTE)(((WORD)(hls)) >> 8))
#define HLS_S(hls) ((BYTE)((hls)>>16))

HLSCOLOR RGB2HLS (COLORREF rgb);
COLORREF HLS2RGB (HLSCOLOR hls);

COLORREF HLS_TRANSFORM (COLORREF rgb, int percent_L, int percent_S);

class UserInfoBase {
public:
	UserInfoBase(const UserPtr& u) : user(u) { }

	void getList();
	void matchQueue();
	void pm();
	void grant();
	void grantSlotHour();
	void grantSlotDay();
	void grantSlotWeek();
	void ungrantSlot();
	void addFav();
	void removeAll();

	UserPtr& getUser() { return user; }
	UserPtr user;
};

template<class T>
class UserInfoBaseHandler {
public:
	UserInfoBaseHandler() { }

    bool UserInfoBaseHandlerProc(UINT uMsg, WPARAM wParam) {
        if(uMsg == WM_COMMAND) {
            switch(LOWORD(wParam)) {
                case IDC_GETLIST:
                    onGetList();
                    return true;
                case IDC_MATCH_QUEUE:
                    onMatchQueue();
                    return true;
                case IDC_PRIVATEMESSAGE:
                    onPrivateMessage();
                    return true;
                case IDC_ADD_TO_FAVORITES:
                    onAddToFavorites();
                    return true;
                case IDC_GRANTSLOT:
                    onGrantSlot();
                    return true;
                case IDC_REMOVEALL:
                    onRemoveAll();
                    return true;
                case IDC_GRANTSLOT_HOUR:
                    onGrantSlotHour();
                    return true;
                case IDC_GRANTSLOT_DAY:
                    onGrantSlotDay();
                    return true;
                case IDC_GRANTSLOT_WEEK:
                    onGrantSlotWeek();
                    return true;
                case IDC_UNGRANTSLOT:
                    onUnGrantSlot();
                    return true;
            }
        }

    	return false;
    }

	void onMatchQueue() {
		((T*)this)->getUserList().forEachSelected(&UserInfoBase::matchQueue);
	}
	void onGetList() {
		((T*)this)->getUserList().forEachSelected(&UserInfoBase::getList);
	}
	void onAddToFavorites() {
		((T*)this)->getUserList().forEachSelected(&UserInfoBase::addFav);
	}
	void onPrivateMessage() {
		((T*)this)->getUserList().forEachSelected(&UserInfoBase::pm);
	}
	void onGrantSlot() {
		((T*)this)->getUserList().forEachSelected(&UserInfoBase::grant);
	}
	void onRemoveAll() {
		((T*)this)->getUserList().forEachSelected(&UserInfoBase::removeAll);
	}
	void onGrantSlotHour() {
		((T*)this)->getUserList().forEachSelected(&UserInfoBase::grantSlotHour);
	}
	void onGrantSlotDay() {
		((T*)this)->getUserList().forEachSelected(&UserInfoBase::grantSlotDay);
	}
	void onGrantSlotWeek() {
		((T*)this)->getUserList().forEachSelected(&UserInfoBase::grantSlotWeek);
	}
	void onUnGrantSlot() {
		((T*)this)->getUserList().forEachSelected(&UserInfoBase::ungrantSlot);
	}
};

class FlatTabCtrl;
class UserCommand;

template<class T, int title, int ID = -1>
class StaticFrame {
public:
	virtual ~StaticFrame() { frame = NULL; }

	static T* frame;
	static void openWindow() {
		if(frame == NULL) {
			frame = new T();
			RECT rc = { CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT };
			frame->CreateEx(frame, T::m_aAtom, T::m_uiResourceID, &rc, CTSTRING_I(ResourceManager::Strings(title)));
		} else {
			// match the behavior of MainFrame::onSelected()
			HWND hWnd = frame->m_hWnd;
			if(isMDIChildActive(hWnd)) {
				::PostMessage(hWnd, WM_CLOSE, NULL, NULL);
			} else if((HWND)::SendMessage(WinUtil::mdiClient, WM_MDIGETACTIVE, 0, (LPARAM)NULL) != hWnd) {
				::SendMessage(WinUtil::mdiClient, WM_MDIACTIVATE, (WPARAM)hWnd, 0);
			} else if(BOOLSETTING(TOGGLE_ACTIVE_WINDOW)) {
				::SetWindowPos(hWnd, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
				::SendMessage(WinUtil::mdiClient, WM_MDINEXT, (WPARAM)hWnd, (LPARAM)FALSE);
				hWnd = (HWND)::SendMessage(WinUtil::mdiClient, WM_MDIGETACTIVE, 0, (LPARAM)NULL);
			}

			if(::IsIconic(hWnd)) {
				::ShowWindow(hWnd, SW_RESTORE);
			}
		}
	}
	static bool isMDIChildActive(HWND hWnd) {
		return (hWnd == (HWND)::SendMessage(WinUtil::mdiClient, WM_MDIGETACTIVE, 0, (LPARAM)NULL));
	}
};

template<class T, int title, int ID>
T* StaticFrame<T, title, ID>::frame = NULL;

class WinUtil {
public:
	static HIMAGELIST fileImages;
	static int fileImageCount;
	static HIMAGELIST userImages;
	static HIMAGELIST speedImages;

	typedef HASH_MAP<string, int> ImageMap;
	typedef ImageMap::iterator ImageIter;

	struct TextItem {
		WORD itemID;
		ResourceManager::Strings translatedString;
	};

	static ImageMap fileIndexes;
	static HBRUSH bgBrush;
	static COLORREF textColor;
	static COLORREF bgColor;
	static HFONT font;
	static int fontHeight;
	static HFONT boldFont;
	static HFONT systemFont;
	static HFONT monoFont;
	static int dirIconIndex;
	static TStringList lastDirs;
	static HWND mainWnd;
	static HWND mdiClient;
	static FlatTabCtrl* tabCtrl;
	static HHOOK hook;
	static tstring tth;
	static StringPairList initialDirs;
	static bool isAppActive;
	static CHARFORMAT2 m_TextStyleTimestamp;
	static CHARFORMAT2 m_ChatTextGeneral;
	static CHARFORMAT2 m_TextStyleMyNick;
	static CHARFORMAT2 m_ChatTextMyOwn;
	static CHARFORMAT2 m_ChatTextServer;
	static CHARFORMAT2 m_ChatTextSystem;
	static CHARFORMAT2 m_TextStyleBold;
	static CHARFORMAT2 m_TextStyleFavUsers;
	static CHARFORMAT2 m_TextStyleOPs;
	static CHARFORMAT2 m_TextStyleURL;
	static CHARFORMAT2 m_ChatTextPrivate;
	static CHARFORMAT2 m_ChatTextLog;
	static CAGEmoticonSetup* g_pEmoticonsSetup;
	static HCURSOR hVerticalCursor;
	static HCURSOR hHorizontalCursor;
	static tstring commands;

	static void init(HWND hWnd);
	static void uninit();

	static string getAppName() {
		TCHAR buf[MAX_PATH+1];
		DWORD x = GetModuleFileName(NULL, buf, MAX_PATH);
		return Text::fromT(tstring(buf, x));
	}

	static void initColors();
	static void initEmoticons();

	static void decodeFont(const tstring& setting, LOGFONT &dest);

	static void addInitalDir(const UserPtr& user, string dir) {
		// Clear out previos initial dirs, just in case
		getInitialDir(user);
		while(initialDirs.size() > 30) {
			initialDirs.erase(initialDirs.begin());
		}
		initialDirs.push_back(make_pair(user->getCID().toBase32(), dir));
	}

	static string getInitialDir(const UserPtr& user) {
		for(StringPairIter i = initialDirs.begin(); i != initialDirs.end(); ++i) {
			if(i->first == user->getCID().toBase32()) {
				string dir = i->second;
				initialDirs.erase(i);
				return dir;
			}
		}
		return Util::emptyString;
	}

	static bool getVersionInfo(OSVERSIONINFOEX& ver);

	/**
	 * Check if this is a common /-command.
	 * @param cmd The whole text string, will be updated to contain only the command.
	 * @param param Set to any parameters.
	 * @param message Message that should be sent to the chat.
	 * @param status Message that should be shown in the status line.
	 * @return True if the command was processed, false otherwise.
	 */
	static bool checkCommand(tstring& cmd, tstring& param, tstring& message, tstring& status);

	static int getTextWidth(const tstring& str, HWND hWnd) {
		HDC dc = ::GetDC(hWnd);
		int sz = getTextWidth(str, dc);
		::ReleaseDC(hWnd, dc);
		return sz;
	}
	static int getTextWidth(const tstring& str, HDC dc) {
		SIZE sz = { 0, 0 };
		::GetTextExtentPoint32(dc, str.c_str(), (int)str.length(), &sz);
		return sz.cx;
	}

	static int getTextHeight(HWND wnd, HFONT fnt) {
		HDC dc = ::GetDC(wnd);
		int h = getTextHeight(dc, fnt);
		::ReleaseDC(wnd, dc);
		return h;
	}

	static int getTextHeight(HDC dc, HFONT fnt) {
		HGDIOBJ old = ::SelectObject(dc, fnt);
		int h = getTextHeight(dc);
		::SelectObject(dc, old);
		return h;
	}

	static int getTextHeight(HDC dc) {
		TEXTMETRIC tm;
		::GetTextMetrics(dc, &tm);
		return tm.tmHeight;
	}

	static void setClipboard(const tstring& str);

	static void addLastDir(const tstring& dir) {
		if(find(lastDirs.begin(), lastDirs.end(), dir) != lastDirs.end()) {
			return;
		}
		if(lastDirs.size() == 10) {
			lastDirs.erase(lastDirs.begin());
		}
		lastDirs.push_back(dir);
	}

	static tstring encodeFont(LOGFONT const& font);

	static bool browseFile(tstring& target, HWND owner = NULL, bool save = true, const tstring& initialDir = Util::emptyStringW, const TCHAR* types = NULL, const TCHAR* defExt = NULL);
	static bool browseDirectory(tstring& target, HWND owner = NULL);

	// Hash related
	static void copyMagnet(const TTHValue* aHash, int64_t aSize, const tstring& aFile);
	static void searchHash(const TTHValue* aHash);

	// URL related
	static void registerDchubHandler();
	static void registerMagnetHandler();
	static void unRegisterDchubHandler();
	static void unRegisterMagnetHandler();
	static void parseDchubUrl(const tstring& /*aUrl*/);
	static void parseMagnetUri(const tstring& /*aUrl*/, bool aOverride = false);
	static bool parseDBLClick(const tstring& /*aString*/, string::size_type start, string::size_type end);
	static bool urlDcRegistered;
	static bool urlMagnetRegistered;
	static int textUnderCursor(POINT p, HWND hWnd, tstring& x);
	static void openLink(const tstring& url);
	static void openFile(const tstring& file) {
		::ShellExecute(NULL, NULL, file.c_str(), NULL, NULL, SW_SHOWNORMAL);
	}

	static void openFolder(const tstring& file);

	static int getIconIndex(const tstring& aFileName);

	static int getDirIconIndex() { return dirIconIndex; }

	static double toBytes(TCHAR* aSize);

	//returns the position where the context menu should be
	//opened if it was invoked from the keyboard.
	//aPt is relative to the screen not the control.
	static void getListContextMenuPos(HWND hWndList, POINT& aPt);
	static void getTreeContextMenuPos(HWND hWndTree, POINT& aPt);

	static bool getUCParams(HWND parent, const UserCommand& cmd, StringMap& sm) throw();

	static tstring getNicks(const CID& cid) throw();
	static tstring getNicks(const UserPtr& u) { return getNicks(u->getCID()); }
	/** @return Pair of hubnames as a string and a bool representing the user's online status */
	static pair<tstring, bool> getHubNames(const CID& cid) throw();
	static pair<tstring, bool> getHubNames(const UserPtr& u) { return getHubNames(u->getCID()); }

	static void splitTokens(int* array, const string& tokens, int maxItems = -1) throw();
	static void saveHeaderOrder(HWND hWnd, SettingsManager::StrSetting order,
		SettingsManager::StrSetting widths, int n, int* indexes, int* sizes) throw();

	static bool isShift() { return (GetKeyState(VK_SHIFT) & 0x8000) > 0; }
	static bool isAlt() { return (GetKeyState(VK_MENU) & 0x8000) > 0; }
	static bool isCtrl() { return (GetKeyState(VK_CONTROL) & 0x8000) > 0; }

	static tstring escapeMenu(tstring str) {
		string::size_type i = 0;
		while( (i = str.find(_T('&'), i)) != string::npos) {
			str.insert(str.begin()+i, 1, _T('&'));
			i += 2;
		}
		return str;
	}
	template<class T> static void hiddenCreateEx(T& p) throw() {
		HWND active = (HWND)::SendMessage(mdiClient, WM_MDIGETACTIVE, 0, 0);
		::LockWindowUpdate(mdiClient);
		p.CreateEx(&p, T::m_aAtom, T::m_uiResourceID);
		if(active && ::IsWindow(active))
			::SendMessage(mdiClient, WM_MDIACTIVATE, (WPARAM)active, 0);
		::LockWindowUpdate(NULL);
	}
	template<class T> static void hiddenCreateEx(T* p) throw() {
		hiddenCreateEx(*p);
	}

	static void translate(HWND page, TextItem* textItems)  {
		if(textItems != NULL) {
			for(int i = 0; textItems[i].itemID != 0; i++) {
				::SetDlgItemText(page, textItems[i].itemID,
					Text::toT(ResourceManager::getInstance()->getString(textItems[i].translatedString)).c_str());
			}
		}
	}

	struct tbIDImage {
		int id, image;
		bool check;
		ResourceManager::Strings tooltip;
	};
	static tbIDImage ToolbarButtons[];

	static int SetupPreviewMenu(HMENU previewMenu, string extension);
	static void RunPreviewCommand(unsigned int index, const string &target);
	static string formatTime(uint64_t rest);
	static uint8_t getImage(const Identity& u, bool NoOPs = false);
	static uint8_t getSpeedImage(int conn);
	static COLORREF blendColors(const COLORREF& cr1, const COLORREF& cr2, double balance = 0.5);
	static void changeToolbarAway(bool Away);
	static string generateStats();
	static void SearchOnDC(tstring sSearchString);
	static void SearchOnGoogle(tstring sSearchString);
	static void SearchOnYahoo(tstring sSearchString);
	static void SearchOnIMDB(tstring sSearchString);
	static void SearchOnCSFD(tstring sSearchString);
	static void SearchOnWikipedia(tstring sSearchString);
	static void AddSearchSubMenu(HMENU menu);
	static void AddPrioritySubMenu(HMENU menu, tstring tsPriority);
	static void AddGrantSubMenu(HMENU menu);
private:
	static int CALLBACK browseCallbackProc(HWND hwnd, UINT uMsg, LPARAM /*lp*/, LPARAM pData);
	static inline BYTE getRValue(const COLORREF& cr) { return (BYTE)(cr & 0xFF); }
	static inline BYTE getGValue(const COLORREF& cr) { return (BYTE)(((cr & 0xFF00) >> 8) & 0xFF); }
	static inline BYTE getBValue(const COLORREF& cr) { return (BYTE)(((cr & 0xFF0000) >> 16) & 0xFF); }
};

#endif // WIN_UTIL_H

/*
 * Some parts Copyright (C) 2002-2009 Petr Kozelka, PPK at czdc dot org
 * Licensed under GNU General Public License version 3.
 */
